"use strict";

/*
   Copyright (c) 2018 jones
 
    http://www.apache.org/licenses/LICENSE-2.0

   开源项目 https://github.com/jones2000/HQChart
 
   jones_2000@163.com

   分析家语法编译执行器 (H5版本)
*/

//日志输出类
if (!JSConsole) {
    var JSConsole = {
        Chart: { Log: console.log }, //图形日志
        Complier: { Log: console.log //编译器日志
        } };
}

//API默认地址
var g_JSComplierResource = {
    Domain: "https://opensource.zealink.com", //API域名
    CacheDomain: "https://opensourcecache.zealink.com", //缓存域名

    DrawIcon: {
        Family: 'iconfont',
        Data: new Map([[1, { Text: "\uE625", Color: 'rgb(255,106,106)' }], //向上箭头
        [2, { Text: "\uE68B", Color: 'rgb(46,139,87)' }], //向下箭头
        [11, { Text: "\uE624", Color: 'rgb(245,159,40)' }], //点赞
        [12, { Text: "\uE600", Color: 'rgb(245,159,40)' }], [13, { Text: "\uE70F", Color: 'rgb(209,37,35)' }], //B
        [14, { Text: "\uE64C", Color: 'rgb(127,209,59)' }], //S
        [9, { Text: "\uE626", Color: 'rgb(245,159,40)' }], //$
        [36, { Text: "\uE68C", Color: 'rgb(255,106,106)' }], //关闭 红色
        [37, { Text: "\uE68C", Color: 'rgb(46,139,87)' }], //关闭 绿色
        [38, { Text: "\uE68D", Color: 'rgb(238,44,44)' }], //▲
        [39, { Text: "\uE68E", Color: 'rgb(0,139,69)' }], //▼
        [46, { Text: "\uE64D", Color: 'rgb(51,51,51)' }]])
    },

    CustomDrawIcon: {
        Data: new Map() //自定义图标 key=id value={ID:, Text:, Color, Family: }
    },

    CustomFunction: //定制函数
    {
        Data: new Map() //自定义函数 key=函数名, Value:{ID:函数名, Callback: }
    },

    CustomVariant: //自定义变量
    {
        Data: new Map() //自定义函数 key=变量名, Value:{ID:变量名, Callback: }
    },

    GetDrawIcon: function GetDrawIcon(id) {
        var icon;
        if (g_JSComplierResource.CustomDrawIcon.Data.has(id)) {
            var iconfont = g_JSComplierResource.CustomDrawIcon.Data.get(id);
            icon = { Symbol: iconfont.Text, Color: iconfont.Color, Family: iconfont.Family, IconFont: true, ID: id };
            return icon;
        }

        if (g_JSComplierResource.DrawIcon.Data.has(id)) {
            var _iconfont = g_JSComplierResource.DrawIcon.Data.get(id);
            icon = { Symbol: _iconfont.Text, Color: _iconfont.Color, Family: g_JSComplierResource.DrawIcon.Family, IconFont: true, ID: id };
            return icon;
        }

        return null;
    },

    IsCustomFunction: function IsCustomFunction(name) {
        if (g_JSComplierResource.CustomFunction.Data.has(name)) return true;
        return false;
    },

    IsCustomVariant: function IsCustomVariant(name) {
        if (g_JSComplierResource.CustomVariant.Data.has(name)) return true;
        return false;
    }
};

var Messages = {
    BadGetterArity: 'Getter must not have any formal parameters',
    BadSetterArity: 'Setter must have exactly one formal parameter',
    BadSetterRestParameter: 'Setter function argument must not be a rest parameter',
    ConstructorIsAsync: 'Class constructor may not be an async method',
    ConstructorSpecialMethod: 'Class constructor may not be an accessor',
    DeclarationMissingInitializer: 'Missing initializer in %0 declaration',
    DefaultRestParameter: 'Unexpected token =',
    DuplicateBinding: 'Duplicate binding %0',
    DuplicateConstructor: 'A class may only have one constructor',
    DuplicateProtoProperty: 'Duplicate __proto__ fields are not allowed in object literals',
    ForInOfLoopInitializer: '%0 loop variable declaration may not have an initializer',
    GeneratorInLegacyContext: 'Generator declarations are not allowed in legacy contexts',
    IllegalBreak: 'Illegal break statement',
    IllegalContinue: 'Illegal continue statement',
    IllegalExportDeclaration: 'Unexpected token',
    IllegalImportDeclaration: 'Unexpected token',
    IllegalLanguageModeDirective: 'Illegal \'use strict\' directive in function with non-simple parameter list',
    IllegalReturn: 'Illegal return statement',
    InvalidEscapedReservedWord: 'Keyword must not contain escaped characters',
    InvalidHexEscapeSequence: 'Invalid hexadecimal escape sequence',
    InvalidLHSInAssignment: 'Invalid left-hand side in assignment',
    InvalidLHSInForIn: 'Invalid left-hand side in for-in',
    InvalidLHSInForLoop: 'Invalid left-hand side in for-loop',
    InvalidModuleSpecifier: 'Unexpected token',
    InvalidRegExp: 'Invalid regular expression',
    LetInLexicalBinding: 'let is disallowed as a lexically bound name',
    MissingFromClause: 'Unexpected token',
    MultipleDefaultsInSwitch: 'More than one default clause in switch statement',
    NewlineAfterThrow: 'Illegal newline after throw',
    NoAsAfterImportNamespace: 'Unexpected token',
    NoCatchOrFinally: 'Missing catch or finally after try',
    ParameterAfterRestParameter: 'Rest parameter must be last formal parameter',
    Redeclaration: '%0 \'%1\' has already been declared',
    StaticPrototype: 'Classes may not have static property named prototype',
    StrictCatchVariable: 'Catch variable may not be eval or arguments in strict mode',
    StrictDelete: 'Delete of an unqualified identifier in strict mode.',
    StrictFunction: 'In strict mode code, functions can only be declared at top level or inside a block',
    StrictFunctionName: 'Function name may not be eval or arguments in strict mode',
    StrictLHSAssignment: 'Assignment to eval or arguments is not allowed in strict mode',
    StrictLHSPostfix: 'Postfix increment/decrement may not have eval or arguments operand in strict mode',
    StrictLHSPrefix: 'Prefix increment/decrement may not have eval or arguments operand in strict mode',
    StrictModeWith: 'Strict mode code may not include a with statement',
    StrictOctalLiteral: 'Octal literals are not allowed in strict mode.',
    StrictParamDupe: 'Strict mode function may not have duplicate parameter names',
    StrictParamName: 'Parameter name eval or arguments is not allowed in strict mode',
    StrictReservedWord: 'Use of future reserved word in strict mode',
    StrictVarName: 'Variable name may not be eval or arguments in strict mode',
    TemplateOctalLiteral: 'Octal literals are not allowed in template strings.',
    UnexpectedEOS: 'Unexpected end of input',
    UnexpectedIdentifier: 'Unexpected identifier',
    UnexpectedNumber: 'Unexpected number',
    UnexpectedReserved: 'Unexpected reserved word',
    UnexpectedString: 'Unexpected string',
    UnexpectedTemplate: 'Unexpected quasi %0',
    UnexpectedToken: 'Unexpected token %0',
    UnexpectedTokenIllegal: 'Unexpected token ILLEGAL',
    UnknownLabel: 'Undefined label \'%0\'',
    UnterminatedRegExp: 'Invalid regular expression: missing /'
};

var Regex = {
    // Unicode v8.0.0 NonAsciiIdentifierStart:
    NonAsciiIdentifierStart: /[\xAA\xB5\xBA\xC0-\xD6\xD8-\xF6\xF8-\u02C1\u02C6-\u02D1\u02E0-\u02E4\u02EC\u02EE\u0370-\u0374\u0376\u0377\u037A-\u037D\u037F\u0386\u0388-\u038A\u038C\u038E-\u03A1\u03A3-\u03F5\u03F7-\u0481\u048A-\u052F\u0531-\u0556\u0559\u0561-\u0587\u05D0-\u05EA\u05F0-\u05F2\u0620-\u064A\u066E\u066F\u0671-\u06D3\u06D5\u06E5\u06E6\u06EE\u06EF\u06FA-\u06FC\u06FF\u0710\u0712-\u072F\u074D-\u07A5\u07B1\u07CA-\u07EA\u07F4\u07F5\u07FA\u0800-\u0815\u081A\u0824\u0828\u0840-\u0858\u08A0-\u08B4\u0904-\u0939\u093D\u0950\u0958-\u0961\u0971-\u0980\u0985-\u098C\u098F\u0990\u0993-\u09A8\u09AA-\u09B0\u09B2\u09B6-\u09B9\u09BD\u09CE\u09DC\u09DD\u09DF-\u09E1\u09F0\u09F1\u0A05-\u0A0A\u0A0F\u0A10\u0A13-\u0A28\u0A2A-\u0A30\u0A32\u0A33\u0A35\u0A36\u0A38\u0A39\u0A59-\u0A5C\u0A5E\u0A72-\u0A74\u0A85-\u0A8D\u0A8F-\u0A91\u0A93-\u0AA8\u0AAA-\u0AB0\u0AB2\u0AB3\u0AB5-\u0AB9\u0ABD\u0AD0\u0AE0\u0AE1\u0AF9\u0B05-\u0B0C\u0B0F\u0B10\u0B13-\u0B28\u0B2A-\u0B30\u0B32\u0B33\u0B35-\u0B39\u0B3D\u0B5C\u0B5D\u0B5F-\u0B61\u0B71\u0B83\u0B85-\u0B8A\u0B8E-\u0B90\u0B92-\u0B95\u0B99\u0B9A\u0B9C\u0B9E\u0B9F\u0BA3\u0BA4\u0BA8-\u0BAA\u0BAE-\u0BB9\u0BD0\u0C05-\u0C0C\u0C0E-\u0C10\u0C12-\u0C28\u0C2A-\u0C39\u0C3D\u0C58-\u0C5A\u0C60\u0C61\u0C85-\u0C8C\u0C8E-\u0C90\u0C92-\u0CA8\u0CAA-\u0CB3\u0CB5-\u0CB9\u0CBD\u0CDE\u0CE0\u0CE1\u0CF1\u0CF2\u0D05-\u0D0C\u0D0E-\u0D10\u0D12-\u0D3A\u0D3D\u0D4E\u0D5F-\u0D61\u0D7A-\u0D7F\u0D85-\u0D96\u0D9A-\u0DB1\u0DB3-\u0DBB\u0DBD\u0DC0-\u0DC6\u0E01-\u0E30\u0E32\u0E33\u0E40-\u0E46\u0E81\u0E82\u0E84\u0E87\u0E88\u0E8A\u0E8D\u0E94-\u0E97\u0E99-\u0E9F\u0EA1-\u0EA3\u0EA5\u0EA7\u0EAA\u0EAB\u0EAD-\u0EB0\u0EB2\u0EB3\u0EBD\u0EC0-\u0EC4\u0EC6\u0EDC-\u0EDF\u0F00\u0F40-\u0F47\u0F49-\u0F6C\u0F88-\u0F8C\u1000-\u102A\u103F\u1050-\u1055\u105A-\u105D\u1061\u1065\u1066\u106E-\u1070\u1075-\u1081\u108E\u10A0-\u10C5\u10C7\u10CD\u10D0-\u10FA\u10FC-\u1248\u124A-\u124D\u1250-\u1256\u1258\u125A-\u125D\u1260-\u1288\u128A-\u128D\u1290-\u12B0\u12B2-\u12B5\u12B8-\u12BE\u12C0\u12C2-\u12C5\u12C8-\u12D6\u12D8-\u1310\u1312-\u1315\u1318-\u135A\u1380-\u138F\u13A0-\u13F5\u13F8-\u13FD\u1401-\u166C\u166F-\u167F\u1681-\u169A\u16A0-\u16EA\u16EE-\u16F8\u1700-\u170C\u170E-\u1711\u1720-\u1731\u1740-\u1751\u1760-\u176C\u176E-\u1770\u1780-\u17B3\u17D7\u17DC\u1820-\u1877\u1880-\u18A8\u18AA\u18B0-\u18F5\u1900-\u191E\u1950-\u196D\u1970-\u1974\u1980-\u19AB\u19B0-\u19C9\u1A00-\u1A16\u1A20-\u1A54\u1AA7\u1B05-\u1B33\u1B45-\u1B4B\u1B83-\u1BA0\u1BAE\u1BAF\u1BBA-\u1BE5\u1C00-\u1C23\u1C4D-\u1C4F\u1C5A-\u1C7D\u1CE9-\u1CEC\u1CEE-\u1CF1\u1CF5\u1CF6\u1D00-\u1DBF\u1E00-\u1F15\u1F18-\u1F1D\u1F20-\u1F45\u1F48-\u1F4D\u1F50-\u1F57\u1F59\u1F5B\u1F5D\u1F5F-\u1F7D\u1F80-\u1FB4\u1FB6-\u1FBC\u1FBE\u1FC2-\u1FC4\u1FC6-\u1FCC\u1FD0-\u1FD3\u1FD6-\u1FDB\u1FE0-\u1FEC\u1FF2-\u1FF4\u1FF6-\u1FFC\u2071\u207F\u2090-\u209C\u2102\u2107\u210A-\u2113\u2115\u2118-\u211D\u2124\u2126\u2128\u212A-\u2139\u213C-\u213F\u2145-\u2149\u214E\u2160-\u2188\u2C00-\u2C2E\u2C30-\u2C5E\u2C60-\u2CE4\u2CEB-\u2CEE\u2CF2\u2CF3\u2D00-\u2D25\u2D27\u2D2D\u2D30-\u2D67\u2D6F\u2D80-\u2D96\u2DA0-\u2DA6\u2DA8-\u2DAE\u2DB0-\u2DB6\u2DB8-\u2DBE\u2DC0-\u2DC6\u2DC8-\u2DCE\u2DD0-\u2DD6\u2DD8-\u2DDE\u3005-\u3007\u3021-\u3029\u3031-\u3035\u3038-\u303C\u3041-\u3096\u309B-\u309F\u30A1-\u30FA\u30FC-\u30FF\u3105-\u312D\u3131-\u318E\u31A0-\u31BA\u31F0-\u31FF\u3400-\u4DB5\u4E00-\u9FD5\uA000-\uA48C\uA4D0-\uA4FD\uA500-\uA60C\uA610-\uA61F\uA62A\uA62B\uA640-\uA66E\uA67F-\uA69D\uA6A0-\uA6EF\uA717-\uA71F\uA722-\uA788\uA78B-\uA7AD\uA7B0-\uA7B7\uA7F7-\uA801\uA803-\uA805\uA807-\uA80A\uA80C-\uA822\uA840-\uA873\uA882-\uA8B3\uA8F2-\uA8F7\uA8FB\uA8FD\uA90A-\uA925\uA930-\uA946\uA960-\uA97C\uA984-\uA9B2\uA9CF\uA9E0-\uA9E4\uA9E6-\uA9EF\uA9FA-\uA9FE\uAA00-\uAA28\uAA40-\uAA42\uAA44-\uAA4B\uAA60-\uAA76\uAA7A\uAA7E-\uAAAF\uAAB1\uAAB5\uAAB6\uAAB9-\uAABD\uAAC0\uAAC2\uAADB-\uAADD\uAAE0-\uAAEA\uAAF2-\uAAF4\uAB01-\uAB06\uAB09-\uAB0E\uAB11-\uAB16\uAB20-\uAB26\uAB28-\uAB2E\uAB30-\uAB5A\uAB5C-\uAB65\uAB70-\uABE2\uAC00-\uD7A3\uD7B0-\uD7C6\uD7CB-\uD7FB\uF900-\uFA6D\uFA70-\uFAD9\uFB00-\uFB06\uFB13-\uFB17\uFB1D\uFB1F-\uFB28\uFB2A-\uFB36\uFB38-\uFB3C\uFB3E\uFB40\uFB41\uFB43\uFB44\uFB46-\uFBB1\uFBD3-\uFD3D\uFD50-\uFD8F\uFD92-\uFDC7\uFDF0-\uFDFB\uFE70-\uFE74\uFE76-\uFEFC\uFF21-\uFF3A\uFF41-\uFF5A\uFF66-\uFFBE\uFFC2-\uFFC7\uFFCA-\uFFCF\uFFD2-\uFFD7\uFFDA-\uFFDC]|\uD800[\uDC00-\uDC0B\uDC0D-\uDC26\uDC28-\uDC3A\uDC3C\uDC3D\uDC3F-\uDC4D\uDC50-\uDC5D\uDC80-\uDCFA\uDD40-\uDD74\uDE80-\uDE9C\uDEA0-\uDED0\uDF00-\uDF1F\uDF30-\uDF4A\uDF50-\uDF75\uDF80-\uDF9D\uDFA0-\uDFC3\uDFC8-\uDFCF\uDFD1-\uDFD5]|\uD801[\uDC00-\uDC9D\uDD00-\uDD27\uDD30-\uDD63\uDE00-\uDF36\uDF40-\uDF55\uDF60-\uDF67]|\uD802[\uDC00-\uDC05\uDC08\uDC0A-\uDC35\uDC37\uDC38\uDC3C\uDC3F-\uDC55\uDC60-\uDC76\uDC80-\uDC9E\uDCE0-\uDCF2\uDCF4\uDCF5\uDD00-\uDD15\uDD20-\uDD39\uDD80-\uDDB7\uDDBE\uDDBF\uDE00\uDE10-\uDE13\uDE15-\uDE17\uDE19-\uDE33\uDE60-\uDE7C\uDE80-\uDE9C\uDEC0-\uDEC7\uDEC9-\uDEE4\uDF00-\uDF35\uDF40-\uDF55\uDF60-\uDF72\uDF80-\uDF91]|\uD803[\uDC00-\uDC48\uDC80-\uDCB2\uDCC0-\uDCF2]|\uD804[\uDC03-\uDC37\uDC83-\uDCAF\uDCD0-\uDCE8\uDD03-\uDD26\uDD50-\uDD72\uDD76\uDD83-\uDDB2\uDDC1-\uDDC4\uDDDA\uDDDC\uDE00-\uDE11\uDE13-\uDE2B\uDE80-\uDE86\uDE88\uDE8A-\uDE8D\uDE8F-\uDE9D\uDE9F-\uDEA8\uDEB0-\uDEDE\uDF05-\uDF0C\uDF0F\uDF10\uDF13-\uDF28\uDF2A-\uDF30\uDF32\uDF33\uDF35-\uDF39\uDF3D\uDF50\uDF5D-\uDF61]|\uD805[\uDC80-\uDCAF\uDCC4\uDCC5\uDCC7\uDD80-\uDDAE\uDDD8-\uDDDB\uDE00-\uDE2F\uDE44\uDE80-\uDEAA\uDF00-\uDF19]|\uD806[\uDCA0-\uDCDF\uDCFF\uDEC0-\uDEF8]|\uD808[\uDC00-\uDF99]|\uD809[\uDC00-\uDC6E\uDC80-\uDD43]|[\uD80C\uD840-\uD868\uD86A-\uD86C\uD86F-\uD872][\uDC00-\uDFFF]|\uD80D[\uDC00-\uDC2E]|\uD811[\uDC00-\uDE46]|\uD81A[\uDC00-\uDE38\uDE40-\uDE5E\uDED0-\uDEED\uDF00-\uDF2F\uDF40-\uDF43\uDF63-\uDF77\uDF7D-\uDF8F]|\uD81B[\uDF00-\uDF44\uDF50\uDF93-\uDF9F]|\uD82C[\uDC00\uDC01]|\uD82F[\uDC00-\uDC6A\uDC70-\uDC7C\uDC80-\uDC88\uDC90-\uDC99]|\uD835[\uDC00-\uDC54\uDC56-\uDC9C\uDC9E\uDC9F\uDCA2\uDCA5\uDCA6\uDCA9-\uDCAC\uDCAE-\uDCB9\uDCBB\uDCBD-\uDCC3\uDCC5-\uDD05\uDD07-\uDD0A\uDD0D-\uDD14\uDD16-\uDD1C\uDD1E-\uDD39\uDD3B-\uDD3E\uDD40-\uDD44\uDD46\uDD4A-\uDD50\uDD52-\uDEA5\uDEA8-\uDEC0\uDEC2-\uDEDA\uDEDC-\uDEFA\uDEFC-\uDF14\uDF16-\uDF34\uDF36-\uDF4E\uDF50-\uDF6E\uDF70-\uDF88\uDF8A-\uDFA8\uDFAA-\uDFC2\uDFC4-\uDFCB]|\uD83A[\uDC00-\uDCC4]|\uD83B[\uDE00-\uDE03\uDE05-\uDE1F\uDE21\uDE22\uDE24\uDE27\uDE29-\uDE32\uDE34-\uDE37\uDE39\uDE3B\uDE42\uDE47\uDE49\uDE4B\uDE4D-\uDE4F\uDE51\uDE52\uDE54\uDE57\uDE59\uDE5B\uDE5D\uDE5F\uDE61\uDE62\uDE64\uDE67-\uDE6A\uDE6C-\uDE72\uDE74-\uDE77\uDE79-\uDE7C\uDE7E\uDE80-\uDE89\uDE8B-\uDE9B\uDEA1-\uDEA3\uDEA5-\uDEA9\uDEAB-\uDEBB]|\uD869[\uDC00-\uDED6\uDF00-\uDFFF]|\uD86D[\uDC00-\uDF34\uDF40-\uDFFF]|\uD86E[\uDC00-\uDC1D\uDC20-\uDFFF]|\uD873[\uDC00-\uDEA1]|\uD87E[\uDC00-\uDE1D]/,
    // Unicode v8.0.0 NonAsciiIdentifierPart:
    NonAsciiIdentifierPart: /[\xAA\xB5\xB7\xBA\xC0-\xD6\xD8-\xF6\xF8-\u02C1\u02C6-\u02D1\u02E0-\u02E4\u02EC\u02EE\u0300-\u0374\u0376\u0377\u037A-\u037D\u037F\u0386-\u038A\u038C\u038E-\u03A1\u03A3-\u03F5\u03F7-\u0481\u0483-\u0487\u048A-\u052F\u0531-\u0556\u0559\u0561-\u0587\u0591-\u05BD\u05BF\u05C1\u05C2\u05C4\u05C5\u05C7\u05D0-\u05EA\u05F0-\u05F2\u0610-\u061A\u0620-\u0669\u066E-\u06D3\u06D5-\u06DC\u06DF-\u06E8\u06EA-\u06FC\u06FF\u0710-\u074A\u074D-\u07B1\u07C0-\u07F5\u07FA\u0800-\u082D\u0840-\u085B\u08A0-\u08B4\u08E3-\u0963\u0966-\u096F\u0971-\u0983\u0985-\u098C\u098F\u0990\u0993-\u09A8\u09AA-\u09B0\u09B2\u09B6-\u09B9\u09BC-\u09C4\u09C7\u09C8\u09CB-\u09CE\u09D7\u09DC\u09DD\u09DF-\u09E3\u09E6-\u09F1\u0A01-\u0A03\u0A05-\u0A0A\u0A0F\u0A10\u0A13-\u0A28\u0A2A-\u0A30\u0A32\u0A33\u0A35\u0A36\u0A38\u0A39\u0A3C\u0A3E-\u0A42\u0A47\u0A48\u0A4B-\u0A4D\u0A51\u0A59-\u0A5C\u0A5E\u0A66-\u0A75\u0A81-\u0A83\u0A85-\u0A8D\u0A8F-\u0A91\u0A93-\u0AA8\u0AAA-\u0AB0\u0AB2\u0AB3\u0AB5-\u0AB9\u0ABC-\u0AC5\u0AC7-\u0AC9\u0ACB-\u0ACD\u0AD0\u0AE0-\u0AE3\u0AE6-\u0AEF\u0AF9\u0B01-\u0B03\u0B05-\u0B0C\u0B0F\u0B10\u0B13-\u0B28\u0B2A-\u0B30\u0B32\u0B33\u0B35-\u0B39\u0B3C-\u0B44\u0B47\u0B48\u0B4B-\u0B4D\u0B56\u0B57\u0B5C\u0B5D\u0B5F-\u0B63\u0B66-\u0B6F\u0B71\u0B82\u0B83\u0B85-\u0B8A\u0B8E-\u0B90\u0B92-\u0B95\u0B99\u0B9A\u0B9C\u0B9E\u0B9F\u0BA3\u0BA4\u0BA8-\u0BAA\u0BAE-\u0BB9\u0BBE-\u0BC2\u0BC6-\u0BC8\u0BCA-\u0BCD\u0BD0\u0BD7\u0BE6-\u0BEF\u0C00-\u0C03\u0C05-\u0C0C\u0C0E-\u0C10\u0C12-\u0C28\u0C2A-\u0C39\u0C3D-\u0C44\u0C46-\u0C48\u0C4A-\u0C4D\u0C55\u0C56\u0C58-\u0C5A\u0C60-\u0C63\u0C66-\u0C6F\u0C81-\u0C83\u0C85-\u0C8C\u0C8E-\u0C90\u0C92-\u0CA8\u0CAA-\u0CB3\u0CB5-\u0CB9\u0CBC-\u0CC4\u0CC6-\u0CC8\u0CCA-\u0CCD\u0CD5\u0CD6\u0CDE\u0CE0-\u0CE3\u0CE6-\u0CEF\u0CF1\u0CF2\u0D01-\u0D03\u0D05-\u0D0C\u0D0E-\u0D10\u0D12-\u0D3A\u0D3D-\u0D44\u0D46-\u0D48\u0D4A-\u0D4E\u0D57\u0D5F-\u0D63\u0D66-\u0D6F\u0D7A-\u0D7F\u0D82\u0D83\u0D85-\u0D96\u0D9A-\u0DB1\u0DB3-\u0DBB\u0DBD\u0DC0-\u0DC6\u0DCA\u0DCF-\u0DD4\u0DD6\u0DD8-\u0DDF\u0DE6-\u0DEF\u0DF2\u0DF3\u0E01-\u0E3A\u0E40-\u0E4E\u0E50-\u0E59\u0E81\u0E82\u0E84\u0E87\u0E88\u0E8A\u0E8D\u0E94-\u0E97\u0E99-\u0E9F\u0EA1-\u0EA3\u0EA5\u0EA7\u0EAA\u0EAB\u0EAD-\u0EB9\u0EBB-\u0EBD\u0EC0-\u0EC4\u0EC6\u0EC8-\u0ECD\u0ED0-\u0ED9\u0EDC-\u0EDF\u0F00\u0F18\u0F19\u0F20-\u0F29\u0F35\u0F37\u0F39\u0F3E-\u0F47\u0F49-\u0F6C\u0F71-\u0F84\u0F86-\u0F97\u0F99-\u0FBC\u0FC6\u1000-\u1049\u1050-\u109D\u10A0-\u10C5\u10C7\u10CD\u10D0-\u10FA\u10FC-\u1248\u124A-\u124D\u1250-\u1256\u1258\u125A-\u125D\u1260-\u1288\u128A-\u128D\u1290-\u12B0\u12B2-\u12B5\u12B8-\u12BE\u12C0\u12C2-\u12C5\u12C8-\u12D6\u12D8-\u1310\u1312-\u1315\u1318-\u135A\u135D-\u135F\u1369-\u1371\u1380-\u138F\u13A0-\u13F5\u13F8-\u13FD\u1401-\u166C\u166F-\u167F\u1681-\u169A\u16A0-\u16EA\u16EE-\u16F8\u1700-\u170C\u170E-\u1714\u1720-\u1734\u1740-\u1753\u1760-\u176C\u176E-\u1770\u1772\u1773\u1780-\u17D3\u17D7\u17DC\u17DD\u17E0-\u17E9\u180B-\u180D\u1810-\u1819\u1820-\u1877\u1880-\u18AA\u18B0-\u18F5\u1900-\u191E\u1920-\u192B\u1930-\u193B\u1946-\u196D\u1970-\u1974\u1980-\u19AB\u19B0-\u19C9\u19D0-\u19DA\u1A00-\u1A1B\u1A20-\u1A5E\u1A60-\u1A7C\u1A7F-\u1A89\u1A90-\u1A99\u1AA7\u1AB0-\u1ABD\u1B00-\u1B4B\u1B50-\u1B59\u1B6B-\u1B73\u1B80-\u1BF3\u1C00-\u1C37\u1C40-\u1C49\u1C4D-\u1C7D\u1CD0-\u1CD2\u1CD4-\u1CF6\u1CF8\u1CF9\u1D00-\u1DF5\u1DFC-\u1F15\u1F18-\u1F1D\u1F20-\u1F45\u1F48-\u1F4D\u1F50-\u1F57\u1F59\u1F5B\u1F5D\u1F5F-\u1F7D\u1F80-\u1FB4\u1FB6-\u1FBC\u1FBE\u1FC2-\u1FC4\u1FC6-\u1FCC\u1FD0-\u1FD3\u1FD6-\u1FDB\u1FE0-\u1FEC\u1FF2-\u1FF4\u1FF6-\u1FFC\u200C\u200D\u203F\u2040\u2054\u2071\u207F\u2090-\u209C\u20D0-\u20DC\u20E1\u20E5-\u20F0\u2102\u2107\u210A-\u2113\u2115\u2118-\u211D\u2124\u2126\u2128\u212A-\u2139\u213C-\u213F\u2145-\u2149\u214E\u2160-\u2188\u2C00-\u2C2E\u2C30-\u2C5E\u2C60-\u2CE4\u2CEB-\u2CF3\u2D00-\u2D25\u2D27\u2D2D\u2D30-\u2D67\u2D6F\u2D7F-\u2D96\u2DA0-\u2DA6\u2DA8-\u2DAE\u2DB0-\u2DB6\u2DB8-\u2DBE\u2DC0-\u2DC6\u2DC8-\u2DCE\u2DD0-\u2DD6\u2DD8-\u2DDE\u2DE0-\u2DFF\u3005-\u3007\u3021-\u302F\u3031-\u3035\u3038-\u303C\u3041-\u3096\u3099-\u309F\u30A1-\u30FA\u30FC-\u30FF\u3105-\u312D\u3131-\u318E\u31A0-\u31BA\u31F0-\u31FF\u3400-\u4DB5\u4E00-\u9FD5\uA000-\uA48C\uA4D0-\uA4FD\uA500-\uA60C\uA610-\uA62B\uA640-\uA66F\uA674-\uA67D\uA67F-\uA6F1\uA717-\uA71F\uA722-\uA788\uA78B-\uA7AD\uA7B0-\uA7B7\uA7F7-\uA827\uA840-\uA873\uA880-\uA8C4\uA8D0-\uA8D9\uA8E0-\uA8F7\uA8FB\uA8FD\uA900-\uA92D\uA930-\uA953\uA960-\uA97C\uA980-\uA9C0\uA9CF-\uA9D9\uA9E0-\uA9FE\uAA00-\uAA36\uAA40-\uAA4D\uAA50-\uAA59\uAA60-\uAA76\uAA7A-\uAAC2\uAADB-\uAADD\uAAE0-\uAAEF\uAAF2-\uAAF6\uAB01-\uAB06\uAB09-\uAB0E\uAB11-\uAB16\uAB20-\uAB26\uAB28-\uAB2E\uAB30-\uAB5A\uAB5C-\uAB65\uAB70-\uABEA\uABEC\uABED\uABF0-\uABF9\uAC00-\uD7A3\uD7B0-\uD7C6\uD7CB-\uD7FB\uF900-\uFA6D\uFA70-\uFAD9\uFB00-\uFB06\uFB13-\uFB17\uFB1D-\uFB28\uFB2A-\uFB36\uFB38-\uFB3C\uFB3E\uFB40\uFB41\uFB43\uFB44\uFB46-\uFBB1\uFBD3-\uFD3D\uFD50-\uFD8F\uFD92-\uFDC7\uFDF0-\uFDFB\uFE00-\uFE0F\uFE20-\uFE2F\uFE33\uFE34\uFE4D-\uFE4F\uFE70-\uFE74\uFE76-\uFEFC\uFF10-\uFF19\uFF21-\uFF3A\uFF3F\uFF41-\uFF5A\uFF66-\uFFBE\uFFC2-\uFFC7\uFFCA-\uFFCF\uFFD2-\uFFD7\uFFDA-\uFFDC]|\uD800[\uDC00-\uDC0B\uDC0D-\uDC26\uDC28-\uDC3A\uDC3C\uDC3D\uDC3F-\uDC4D\uDC50-\uDC5D\uDC80-\uDCFA\uDD40-\uDD74\uDDFD\uDE80-\uDE9C\uDEA0-\uDED0\uDEE0\uDF00-\uDF1F\uDF30-\uDF4A\uDF50-\uDF7A\uDF80-\uDF9D\uDFA0-\uDFC3\uDFC8-\uDFCF\uDFD1-\uDFD5]|\uD801[\uDC00-\uDC9D\uDCA0-\uDCA9\uDD00-\uDD27\uDD30-\uDD63\uDE00-\uDF36\uDF40-\uDF55\uDF60-\uDF67]|\uD802[\uDC00-\uDC05\uDC08\uDC0A-\uDC35\uDC37\uDC38\uDC3C\uDC3F-\uDC55\uDC60-\uDC76\uDC80-\uDC9E\uDCE0-\uDCF2\uDCF4\uDCF5\uDD00-\uDD15\uDD20-\uDD39\uDD80-\uDDB7\uDDBE\uDDBF\uDE00-\uDE03\uDE05\uDE06\uDE0C-\uDE13\uDE15-\uDE17\uDE19-\uDE33\uDE38-\uDE3A\uDE3F\uDE60-\uDE7C\uDE80-\uDE9C\uDEC0-\uDEC7\uDEC9-\uDEE6\uDF00-\uDF35\uDF40-\uDF55\uDF60-\uDF72\uDF80-\uDF91]|\uD803[\uDC00-\uDC48\uDC80-\uDCB2\uDCC0-\uDCF2]|\uD804[\uDC00-\uDC46\uDC66-\uDC6F\uDC7F-\uDCBA\uDCD0-\uDCE8\uDCF0-\uDCF9\uDD00-\uDD34\uDD36-\uDD3F\uDD50-\uDD73\uDD76\uDD80-\uDDC4\uDDCA-\uDDCC\uDDD0-\uDDDA\uDDDC\uDE00-\uDE11\uDE13-\uDE37\uDE80-\uDE86\uDE88\uDE8A-\uDE8D\uDE8F-\uDE9D\uDE9F-\uDEA8\uDEB0-\uDEEA\uDEF0-\uDEF9\uDF00-\uDF03\uDF05-\uDF0C\uDF0F\uDF10\uDF13-\uDF28\uDF2A-\uDF30\uDF32\uDF33\uDF35-\uDF39\uDF3C-\uDF44\uDF47\uDF48\uDF4B-\uDF4D\uDF50\uDF57\uDF5D-\uDF63\uDF66-\uDF6C\uDF70-\uDF74]|\uD805[\uDC80-\uDCC5\uDCC7\uDCD0-\uDCD9\uDD80-\uDDB5\uDDB8-\uDDC0\uDDD8-\uDDDD\uDE00-\uDE40\uDE44\uDE50-\uDE59\uDE80-\uDEB7\uDEC0-\uDEC9\uDF00-\uDF19\uDF1D-\uDF2B\uDF30-\uDF39]|\uD806[\uDCA0-\uDCE9\uDCFF\uDEC0-\uDEF8]|\uD808[\uDC00-\uDF99]|\uD809[\uDC00-\uDC6E\uDC80-\uDD43]|[\uD80C\uD840-\uD868\uD86A-\uD86C\uD86F-\uD872][\uDC00-\uDFFF]|\uD80D[\uDC00-\uDC2E]|\uD811[\uDC00-\uDE46]|\uD81A[\uDC00-\uDE38\uDE40-\uDE5E\uDE60-\uDE69\uDED0-\uDEED\uDEF0-\uDEF4\uDF00-\uDF36\uDF40-\uDF43\uDF50-\uDF59\uDF63-\uDF77\uDF7D-\uDF8F]|\uD81B[\uDF00-\uDF44\uDF50-\uDF7E\uDF8F-\uDF9F]|\uD82C[\uDC00\uDC01]|\uD82F[\uDC00-\uDC6A\uDC70-\uDC7C\uDC80-\uDC88\uDC90-\uDC99\uDC9D\uDC9E]|\uD834[\uDD65-\uDD69\uDD6D-\uDD72\uDD7B-\uDD82\uDD85-\uDD8B\uDDAA-\uDDAD\uDE42-\uDE44]|\uD835[\uDC00-\uDC54\uDC56-\uDC9C\uDC9E\uDC9F\uDCA2\uDCA5\uDCA6\uDCA9-\uDCAC\uDCAE-\uDCB9\uDCBB\uDCBD-\uDCC3\uDCC5-\uDD05\uDD07-\uDD0A\uDD0D-\uDD14\uDD16-\uDD1C\uDD1E-\uDD39\uDD3B-\uDD3E\uDD40-\uDD44\uDD46\uDD4A-\uDD50\uDD52-\uDEA5\uDEA8-\uDEC0\uDEC2-\uDEDA\uDEDC-\uDEFA\uDEFC-\uDF14\uDF16-\uDF34\uDF36-\uDF4E\uDF50-\uDF6E\uDF70-\uDF88\uDF8A-\uDFA8\uDFAA-\uDFC2\uDFC4-\uDFCB\uDFCE-\uDFFF]|\uD836[\uDE00-\uDE36\uDE3B-\uDE6C\uDE75\uDE84\uDE9B-\uDE9F\uDEA1-\uDEAF]|\uD83A[\uDC00-\uDCC4\uDCD0-\uDCD6]|\uD83B[\uDE00-\uDE03\uDE05-\uDE1F\uDE21\uDE22\uDE24\uDE27\uDE29-\uDE32\uDE34-\uDE37\uDE39\uDE3B\uDE42\uDE47\uDE49\uDE4B\uDE4D-\uDE4F\uDE51\uDE52\uDE54\uDE57\uDE59\uDE5B\uDE5D\uDE5F\uDE61\uDE62\uDE64\uDE67-\uDE6A\uDE6C-\uDE72\uDE74-\uDE77\uDE79-\uDE7C\uDE7E\uDE80-\uDE89\uDE8B-\uDE9B\uDEA1-\uDEA3\uDEA5-\uDEA9\uDEAB-\uDEBB]|\uD869[\uDC00-\uDED6\uDF00-\uDFFF]|\uD86D[\uDC00-\uDF34\uDF40-\uDFFF]|\uD86E[\uDC00-\uDC1D\uDC20-\uDFFF]|\uD873[\uDC00-\uDEA1]|\uD87E[\uDC00-\uDE1D]|\uDB40[\uDD00-\uDDEF]/
};

var Character = {
    FromCodePoint: function FromCodePoint(cp) {
        return cp < 0x10000 ? String.fromCharCode(cp) : String.fromCharCode(0xD800 + (cp - 0x10000 >> 10)) + String.fromCharCode(0xDC00 + (cp - 0x10000 & 1023));
    },

    //是否是空格 https://tc39.github.io/ecma262/#sec-white-space
    IsWhiteSpace: function IsWhiteSpace(cp) {
        return cp === 0x20 || cp === 0x09 || cp === 0x0B || cp === 0x0C || cp === 0xA0 || cp >= 0x1680 && [0x1680, 0x2000, 0x2001, 0x2002, 0x2003, 0x2004, 0x2005, 0x2006, 0x2007, 0x2008, 0x2009, 0x200A, 0x202F, 0x205F, 0x3000, 0xFEFF].indexOf(cp) >= 0;
    },

    //是否换行 https://tc39.github.io/ecma262/#sec-line-terminators
    IsLineTerminator: function IsLineTerminator(cp) {
        return cp === 0x0A || cp === 0x0D || cp === 0x2028 || cp === 0x2029;
    },

    // https://tc39.github.io/ecma262/#sec-names-and-keywords
    IsIdentifierStart: function IsIdentifierStart(cp) {
        return cp === 0x24 || cp === 0x5F || cp >= 0x41 && cp <= 0x5A || cp >= 0x61 && cp <= 0x7A || cp === 0x5C || cp >= 0x80 && Regex.NonAsciiIdentifierStart.test(Character.FromCodePoint(cp));
    },

    IsIdentifierPart: function IsIdentifierPart(cp) {
        return cp === 0x24 || cp === 0x5F || cp >= 0x41 && cp <= 0x5A || cp >= 0x61 && cp <= 0x7A || cp >= 0x30 && cp <= 0x39 || cp === 0x5C || cp >= 0x80 && Regex.NonAsciiIdentifierPart.test(Character.FromCodePoint(cp));
    },

    // https://tc39.github.io/ecma262/#sec-literals-numeric-literals
    IsDecimalDigit: function IsDecimalDigit(cp) {
        return cp >= 0x30 && cp <= 0x39; // 0..9
    },

    IsHexDigit: function IsHexDigit(cp) {
        return cp >= 0x30 && cp <= 0x39 || cp >= 0x41 && cp <= 0x46 || cp >= 0x61 && cp <= 0x66; // a..f
    },

    isOctalDigit: function isOctalDigit(cp) {
        return cp >= 0x30 && cp <= 0x37; // 0..7
    }
};

var TOKEN_NAME = {};
TOKEN_NAME[1 /* BooleanLiteral */] = 'Boolean';
TOKEN_NAME[2 /* EOF */] = '<end>';
TOKEN_NAME[3 /* Identifier */] = 'Identifier';
TOKEN_NAME[4 /* Keyword */] = 'Keyword';
TOKEN_NAME[5 /* NullLiteral */] = 'Null';
TOKEN_NAME[6 /* NumericLiteral */] = 'Numeric';
TOKEN_NAME[7 /* Punctuator */] = 'Punctuator';
TOKEN_NAME[8 /* StringLiteral */] = 'String';
TOKEN_NAME[9 /* RegularExpression */] = 'RegularExpression';
TOKEN_NAME[10 /* Template */] = 'Template';

//编译异常, 错误类
function ErrorHandler() {
    this.Error = [];

    this.RecordError = function (error) {
        this.Error.push(error);
    };

    this.ConstructError = function (msg, column) {
        var error = new Error(msg);
        //通过自己抛异常并自己截获 来获取调用堆栈信息
        try {
            throw error;
        } catch (base) {
            if (Object.create && Object.defineProperties) {
                error = Object.create(base);
                error.Column = column;
            }
        }

        return error;
    };

    this.CreateError = function (index, line, col, description) {
        var msg = 'Line ' + line + ': ' + description;
        var error = this.ConstructError(msg, col);
        error.Index = index;
        error.LineNumber = line;
        error.Description = description;
        return error;
    };

    this.ThrowError = function (index, line, col, description) {
        var error = this.CreateError(index, line, col, description);
        throw error;
    };

    //重新下载数据
    this.ThrowDownloadJob = function (index, line, col, description, job) {
        var error = this.CreateError(index, line, col, description);
        error.Job = job;
        throw error;
    };
}

//扫描类
function Scanner(code, ErrorHandler) {
    this.Source = code;
    this.ErrorHandler = ErrorHandler;
    this.Length = code.length;
    this.Index = 0;
    this.LineNumber = code.length > 0 ? 1 : 0;
    this.LineStart = 0;
    this.CurlyStack = [];

    this.SaveState = function () //保存当前扫描状态
    {
        return { Index: this.Index, LineNumber: this.LineNumber, LineStart: this.LineStart };
    };

    this.RestoreState = function (state) //还原扫描状态
    {
        this.Index = state.Index;
        this.LineNumber = state.LineNumber;
        this.LineStart = state.LineStart;
    };

    this.IsEOF = function () //否是已经结束
    {
        return this.Index >= this.Length;
    };

    this.IsKeyword = function (id) {
        return false;
    };

    this.CodePointAt = function (i) {
        var cp = this.Source.charCodeAt(i);
        if (cp >= 0xD800 && cp <= 0xDBFF) {
            var second = this.Source.charCodeAt(i + 1);
            if (second >= 0xDC00 && second <= 0xDFFF) {
                var first = cp;
                cp = (first - 0xD800) * 0x400 + second - 0xDC00 + 0x10000;
            }
        }
        return cp;
    };

    this.Lex = function () {
        if (this.IsEOF()) return { Type: 2 /*EOF*/, Value: '', LineNumber: this.LineNumber, LineStart: this.LineStart, Start: this.Index, End: this.Index };
        var cp = this.Source.charCodeAt(this.Index);

        //变量名 或 关键字
        if (Character.IsIdentifierStart(cp)) return this.ScanIdentifier();

        //( ) ; 开头 操作符扫描
        if (cp === 0x28 || cp === 0x29 || cp === 0x3B) return this.ScanPunctuator();

        //' " 开头 字符串扫描
        if (cp === 0x27 || cp === 0x22) return this.ScanStringLiteral();

        //. 开头 浮点型
        if (cp == 0x2E) {
            if (Character.IsDecimalDigit(this.Source.charCodeAt(this.Index + 1))) return this.ScanNumericLiteral();

            return this.ScanPunctuator();
        }

        //数字
        if (Character.IsDecimalDigit(cp)) return this.ScanNumericLiteral();

        if (cp >= 0xD800 && cp < 0xDFFF) {
            if (Character.IsIdentifierStart(this.CodePointAt(this.Index))) return this.ScanIdentifier();
        }

        return this.ScanPunctuator();
    };

    //关键字 变量名 https://tc39.github.io/ecma262/#sec-names-and-keywords
    this.ScanIdentifier = function () {
        var type = void 0;
        var start = this.Index;
        //0x5C 反斜杠
        var id = this.Source.charCodeAt(start) === 0x5C ? this.GetComplexIdentifier() : this.GetIdentifier();

        if (id.length) type = 3; //Identifier
        else if (this.IsKeyword(id)) type = 4; //Keyword
            else if (id == null) type = 5; //NullLiteral
                else if (id == 'true' || id == 'false') type = 1; //BooleanLiteral
                    else type = 3; //Identifier

        if (type != 3 && start + id.length != this.Index) {
            var restore = this.Index;
            this.Index = start;
            throw Messages.InvalidEscapedReservedWord;
            this.Index = restore;
        }

        if (id == 'AND' || id == 'OR') type = 7 /*Punctuator*/;

        return { Type: type, Value: id, LineNumber: this.LineNumber, LineStart: this.LineStart, Start: start, End: this.Index };
    };

    this.GetIdentifier = function () {
        var start = this.Index++; //start 保存进来的位置
        while (!this.IsEOF()) {
            var ch = this.Source.charCodeAt(this.Index);
            if (ch == 0x5C) {
                this.Index = start;
                return this.GetComplexIdentifier();
            } else if (ch >= 0xD800 && ch < 0xDFFF) {
                this.Index = start;
                return this.GetComplexIdentifier();
            }

            if (Character.IsIdentifierPart(ch)) ++this.Index;else break;
        }

        return this.Source.slice(start, this.Index);
    };

    //操作符 https://tc39.github.io/ecma262/#sec-punctuators
    this.ScanPunctuator = function () {
        var start = this.Index;
        var str = this.Source[this.Index];
        switch (str) {
            case '(':
                ++this.Index;
                break;
            case ')':
            case ';':
            case ',':
                ++this.Index;
                break;
            default:
                str = this.Source.substr(this.Index, 3);
                if (str == 'AND') {
                    this.Index += 3;
                } else {
                    str = this.Source.substr(this.Index, 2);
                    if (str === '&&' || str === '||' || str === '==' || str === '!=' || str === '<=' || str === '>=' || str === '=>' || str == ':=' || str == 'OR' || str == '<>') {
                        this.Index += 2;
                    } else {
                        str = this.Source[this.Index];
                        if ('<>=!+-*%&|^/:'.indexOf(str) >= 0) ++this.Index;
                    }
                }
        }

        if (this.Index == start) this.ThrowUnecpectedToken();

        return { Type: 7 /*Punctuator*/, Value: str, LineNumber: this.LineNumber, LineStart: this.LineStart, Start: start, End: this.Index };
    };

    //字符串 https://tc39.github.io/ecma262/#sec-literals-string-literals
    this.ScanStringLiteral = function () {
        var start = this.Index;
        var quote = this.Source[this.Index];

        ++this.Index;
        var octal = false;
        var str = '';
        while (!this.IsEOF()) {
            var ch = this.Source[this.Index++];
            if (ch == quote) {
                quote = '';
                break;
            } else if (ch == '\\') //字符串转义
                {
                    throw "not complete";
                } else if (Character.IsLineTerminator(ch.charCodeAt(0))) {
                break;
            } else {
                str += ch;
            }
        }

        if (quote != '') {
            this.Index = start;
            this.ThrowUnecpectedToken();
        }

        return { Type: 8 /*StringLiteral*/, Value: str, LineNumber: this.LineNumber, LineStart: this.LineStart, Start: start, End: this.Index };
    };

    this.ScanNumericLiteral = function () {
        var start = this.Index;
        var ch = this.Source[this.Index];
        var num = '';
        if (ch != '.') {
            num = this.Source[this.Index++];
            ch = this.Source[this.Index];
            // Hex number starts with '0x'. 16进制
            if (num == '0') {
                if (ch == 'x' || ch == 'X') {
                    ++this.Index;
                    return this.ScanHexLiteral(start);
                }
            }

            while (Character.IsDecimalDigit(this.Source.charCodeAt(this.Index))) {
                num += this.Source[this.Index++];
            }

            ch = this.Source[this.Index];
        }

        if (ch == '.') {
            num += this.Source[this.Index++];
            while (Character.IsDecimalDigit(this.Source.charCodeAt(this.Index))) {
                num += this.Source[this.Index++];
            }
            ch = this.Source[this.Index];
        }

        //科学计数法
        if (ch == 'e' || ch == 'E') {
            num += this.Source[this.Index++];
            ch = this.Source[this.Index];
            if (ch == '+' || ch == '-') num += this.Source[this.Index];
            if (Character.IsDecimalDigit(this.Source.charCodeAt(this.Index))) {
                while (Character.IsDecimalDigit(this.Source.charCodeAt(this.Index))) {
                    num += this.Source[this.Index++];
                }
            } else {
                this.ThrowUnecpectedToken();
            }
        }

        if (Character.IsIdentifierStart(this.Source.charCodeAt(this.Index))) {
            this.ThrowUnecpectedToken();
        }

        return { Type: 6 /*NumericLiteral*/, Value: parseFloat(num), LineNumber: this.LineNumber, LineStart: this.LineStart, Start: start, End: this.Index };
    };

    //空格 或 注释
    this.ScanComments = function () {
        var comments = void 0;
        var start = this.Index == 0;
        while (!this.IsEOF()) {
            var ch = this.Source.charCodeAt(this.Index);
            if (Character.IsWhiteSpace(ch)) //过滤掉空格
                {
                    ++this.Index;
                } else if (Character.IsLineTerminator(ch)) {
                ++this.Index;
                if (ch == 0x0D && this.Source.charCodeAt(this.Index) == 0x0A) ++this.Index; //回车+换行
                ++this.LineNumber;
                this.LineStart = this.Index;
                start = true;
            } else if (ch == 0x2F) // //注释
                {
                    ch = this.Source.charCodeAt(this.Index + 1);
                    if (ch == 0x2F) {
                        this.Index += 2;
                        var comment = this.SkipSingleLineComment(2);
                        start = true;
                    } else {
                        break;
                    }
                } else if (ch == 0x7B) //{ }  注释
                {
                    this.Index += 1;
                    var _comment = this.SkipMultiLineComment();
                } else {
                break;
            }
        }

        return comments;
    };

    this.SkipMultiLineComment = function () {
        var comments = [];
        while (!this.IsEOF()) {
            var ch = this.Source.charCodeAt(this.Index);
            if (Character.IsLineTerminator(ch)) {
                ++this.LineNumber;
                ++this.Index;
                this.LineStart = this.Index;
            } else if (ch == 0x7D) {
                this.Index += 1;
                return comments;
            } else {
                ++this.Index;
            }
        }

        return comments;
    };

    //单行注释 https://tc39.github.io/ecma262/#sec-comments
    this.SkipSingleLineComment = function (offset) {
        var comments = [];
        while (!this.IsEOF()) {
            var ch = this.Source.charCodeAt(this.Index);
            ++this.Index;
            if (Character.IsLineTerminator(ch)) {
                if (ch === 13 && this.Source.charCodeAt(this.Index) === 10) ++this.Index;

                ++this.LineNumber;
                this.LineStart = this.Index;
                return comments;
            }
        }

        return comments;
    };

    this.ThrowUnecpectedToken = function (message) {
        if (!message) message = Messages.UnexpectedTokenIllegal;
        return this.ErrorHandler.ThrowError(this.Index, this.LineNumber, this.Index - this.LineStart + 1, message);
    };
}

function Tokenizer(code) {
    this.ErrorHandler = new ErrorHandler(); //错误信息处理类
    this.Scanner = new Scanner(code, this.ErrorHandler);
    this.Buffer = [];

    this.GetNextToken = function () {
        if (this.Buffer.length == 0) {
            var comments = this.Scanner.ScanComments();
            if (!this.Scanner.IsEOF()) {
                var token = this.Scanner.Lex();

                var entry = { Type: TOKEN_NAME[token.Type], Value: this.Scanner.Source.slice(token.Start, token.End) };

                this.Buffer.push(entry);
            }
        }

        return this.Buffer.shift();
    };
}

var Syntax = {
    AssignmentExpression: 'AssignmentExpression',
    AssignmentPattern: 'AssignmentPattern',
    ArrayExpression: 'ArrayExpression',
    ArrayPattern: 'ArrayPattern',
    ArrowFunctionExpression: 'ArrowFunctionExpression',
    AwaitExpression: 'AwaitExpression',
    BlockStatement: 'BlockStatement',
    BinaryExpression: 'BinaryExpression',
    BreakStatement: 'BreakStatement',
    CallExpression: 'CallExpression',
    CatchClause: 'CatchClause',
    ClassBody: 'ClassBody',
    ClassDeclaration: 'ClassDeclaration',
    ClassExpression: 'ClassExpression',
    ConditionalExpression: 'ConditionalExpression',
    ContinueStatement: 'ContinueStatement',
    DoWhileStatement: 'DoWhileStatement',
    DebuggerStatement: 'DebuggerStatement',
    EmptyStatement: 'EmptyStatement',
    ExportAllDeclaration: 'ExportAllDeclaration',
    ExportDefaultDeclaration: 'ExportDefaultDeclaration',
    ExportNamedDeclaration: 'ExportNamedDeclaration',
    ExportSpecifier: 'ExportSpecifier',
    ExpressionStatement: 'ExpressionStatement',
    ForStatement: 'ForStatement',
    ForOfStatement: 'ForOfStatement',
    ForInStatement: 'ForInStatement',
    FunctionDeclaration: 'FunctionDeclaration',
    FunctionExpression: 'FunctionExpression',
    Identifier: 'Identifier',
    IfStatement: 'IfStatement',
    ImportDeclaration: 'ImportDeclaration',
    ImportDefaultSpecifier: 'ImportDefaultSpecifier',
    ImportNamespaceSpecifier: 'ImportNamespaceSpecifier',
    ImportSpecifier: 'ImportSpecifier',
    Literal: 'Literal',
    LabeledStatement: 'LabeledStatement',
    LogicalExpression: 'LogicalExpression',
    MemberExpression: 'MemberExpression',
    MetaProperty: 'MetaProperty',
    MethodDefinition: 'MethodDefinition',
    NewExpression: 'NewExpression',
    ObjectExpression: 'ObjectExpression',
    ObjectPattern: 'ObjectPattern',
    Program: 'Program',
    Property: 'Property',
    RestElement: 'RestElement',
    ReturnStatement: 'ReturnStatement',
    SequenceExpression: 'SequenceExpression',
    SpreadElement: 'SpreadElement',
    Super: 'Super',
    SwitchCase: 'SwitchCase',
    SwitchStatement: 'SwitchStatement',
    TaggedTemplateExpression: 'TaggedTemplateExpression',
    TemplateElement: 'TemplateElement',
    TemplateLiteral: 'TemplateLiteral',
    ThisExpression: 'ThisExpression',
    ThrowStatement: 'ThrowStatement',
    TryStatement: 'TryStatement',
    UnaryExpression: 'UnaryExpression',
    UpdateExpression: 'UpdateExpression',
    VariableDeclaration: 'VariableDeclaration',
    VariableDeclarator: 'VariableDeclarator',
    WhileStatement: 'WhileStatement',
    WithStatement: 'WithStatement',
    YieldExpression: 'YieldExpression'
};

function Node() {
    this.IsNeedIndexData = false; //是否需要大盘数据
    this.IsNeedLatestIndexData = false; //是否需要下载最新大盘数据
    this.IsNeedLatestData = false; //是否需要最新的个股行情数据
    this.IsNeedSymbolData = false; //是否需要下载股票数据
    this.IsNeedSymbolDayData = false; //跨周期是否下载日线数据
    this.IsNeedFinanceData = new Set(); //需要下载的财务数据
    this.IsNeedMarginData = new Set();
    this.IsNeedNewsAnalysisData = new Set(); //新闻统计数据
    this.IsNeedBlockIncreaseData = new Set(); //是否需要市场涨跌股票数据统计
    this.IsNeedSymbolExData = new Set(); //下载股票行情的其他数据
    this.IsNeedHK2SHSZData = new Set(); //下载北上资金数据
    this.IsNeedSectionFinance = new Map(); //下载截面财务数据 { key= 报告期 , Set() 字段}

    this.IsCustomFunction = []; //自定义函数 {Name, ID, Args:}    
    this.IsCustomVariant = []; //自定义变量 {Name, ID}

    this.GetDataJobList = function () //下载数据任务列表
    {
        var jobs = [];
        if (this.IsNeedSymbolData) jobs.push({ ID: JS_EXECUTE_JOB_ID.JOB_DOWNLOAD_SYMBOL_DATA });
        if (this.IsNeedIndexData) jobs.push({ ID: JS_EXECUTE_JOB_ID.JOB_DOWNLOAD_INDEX_DATA });
        if (this.IsNeedLatestData) jobs.push({ ID: JS_EXECUTE_JOB_ID.JOB_DOWNLOAD_SYMBOL_LATEST_DATA });
        if (this.IsNeedSymbolDayData) jobs.push({ ID: JS_EXECUTE_JOB_ID.JOB_DOWNLOAD_SYMBOL_DAY_DATA });
        if (this.IsNeedLatestIndexData) jobs.push({ ID: JS_EXECUTE_JOB_ID.JOB_DOWNLOAD_LATEST_INDEX_DATA });

        //涨跌停家数统计
        var _iteratorNormalCompletion = true;
        var _didIteratorError = false;
        var _iteratorError = undefined;

        try {
            for (var _iterator = this.IsNeedBlockIncreaseData[Symbol.iterator](), _step; !(_iteratorNormalCompletion = (_step = _iterator.next()).done); _iteratorNormalCompletion = true) {
                var blockSymbol = _step.value;

                jobs.push({ ID: JS_EXECUTE_JOB_ID.JOB_DOWNLOAD_INDEX_INCREASE_DATA, Symbol: blockSymbol });
            }

            //加载财务数据
        } catch (err) {
            _didIteratorError = true;
            _iteratorError = err;
        } finally {
            try {
                if (!_iteratorNormalCompletion && _iterator.return) {
                    _iterator.return();
                }
            } finally {
                if (_didIteratorError) {
                    throw _iteratorError;
                }
            }
        }

        var _iteratorNormalCompletion2 = true;
        var _didIteratorError2 = false;
        var _iteratorError2 = undefined;

        try {
            for (var _iterator2 = this.IsNeedFinanceData[Symbol.iterator](), _step2; !(_iteratorNormalCompletion2 = (_step2 = _iterator2.next()).done); _iteratorNormalCompletion2 = true) {
                var jobID = _step2.value;

                if (jobID == JS_EXECUTE_JOB_ID.JOB_DOWNLOAD_DIVIDEND_YIELD_DATA) //股息率 需要总市值
                    jobs.push({ ID: JS_EXECUTE_JOB_ID.JOB_DOWNLOAD_MARKETVALUE_DATA });

                jobs.push({ ID: jobID });
            }

            //加载融资融券
        } catch (err) {
            _didIteratorError2 = true;
            _iteratorError2 = err;
        } finally {
            try {
                if (!_iteratorNormalCompletion2 && _iterator2.return) {
                    _iterator2.return();
                }
            } finally {
                if (_didIteratorError2) {
                    throw _iteratorError2;
                }
            }
        }

        var _iteratorNormalCompletion3 = true;
        var _didIteratorError3 = false;
        var _iteratorError3 = undefined;

        try {
            for (var _iterator3 = this.IsNeedMarginData[Symbol.iterator](), _step3; !(_iteratorNormalCompletion3 = (_step3 = _iterator3.next()).done); _iteratorNormalCompletion3 = true) {
                var jobID = _step3.value;

                jobs.push({ ID: jobID });
            }

            //加载北上资金
        } catch (err) {
            _didIteratorError3 = true;
            _iteratorError3 = err;
        } finally {
            try {
                if (!_iteratorNormalCompletion3 && _iterator3.return) {
                    _iterator3.return();
                }
            } finally {
                if (_didIteratorError3) {
                    throw _iteratorError3;
                }
            }
        }

        var _iteratorNormalCompletion4 = true;
        var _didIteratorError4 = false;
        var _iteratorError4 = undefined;

        try {
            for (var _iterator4 = this.IsNeedHK2SHSZData[Symbol.iterator](), _step4; !(_iteratorNormalCompletion4 = (_step4 = _iterator4.next()).done); _iteratorNormalCompletion4 = true) {
                var jobID = _step4.value;

                jobs.push({ ID: jobID });
            }

            //加载新闻统计
        } catch (err) {
            _didIteratorError4 = true;
            _iteratorError4 = err;
        } finally {
            try {
                if (!_iteratorNormalCompletion4 && _iterator4.return) {
                    _iterator4.return();
                }
            } finally {
                if (_didIteratorError4) {
                    throw _iteratorError4;
                }
            }
        }

        var _iteratorNormalCompletion5 = true;
        var _didIteratorError5 = false;
        var _iteratorError5 = undefined;

        try {
            for (var _iterator5 = this.IsNeedNewsAnalysisData[Symbol.iterator](), _step5; !(_iteratorNormalCompletion5 = (_step5 = _iterator5.next()).done); _iteratorNormalCompletion5 = true) {
                var jobID = _step5.value;

                jobs.push({ ID: jobID });
            }

            //行情其他数据
        } catch (err) {
            _didIteratorError5 = true;
            _iteratorError5 = err;
        } finally {
            try {
                if (!_iteratorNormalCompletion5 && _iterator5.return) {
                    _iterator5.return();
                }
            } finally {
                if (_didIteratorError5) {
                    throw _iteratorError5;
                }
            }
        }

        var _iteratorNormalCompletion6 = true;
        var _didIteratorError6 = false;
        var _iteratorError6 = undefined;

        try {
            for (var _iterator6 = this.IsNeedSymbolExData[Symbol.iterator](), _step6; !(_iteratorNormalCompletion6 = (_step6 = _iterator6.next()).done); _iteratorNormalCompletion6 = true) {
                var jobID = _step6.value;

                jobs.push({ ID: jobID });
            }

            //获取截面数据下载任务
        } catch (err) {
            _didIteratorError6 = true;
            _iteratorError6 = err;
        } finally {
            try {
                if (!_iteratorNormalCompletion6 && _iterator6.return) {
                    _iterator6.return();
                }
            } finally {
                if (_didIteratorError6) {
                    throw _iteratorError6;
                }
            }
        }

        var _iteratorNormalCompletion7 = true;
        var _didIteratorError7 = false;
        var _iteratorError7 = undefined;

        try {
            for (var _iterator7 = this.IsNeedSectionFinance[Symbol.iterator](), _step7; !(_iteratorNormalCompletion7 = (_step7 = _iterator7.next()).done); _iteratorNormalCompletion7 = true) {
                var item = _step7.value;

                jobs.push({ ID: JS_EXECUTE_JOB_ID.JOB_DOWNLOAD_SECTION_SF, SF: item });
            }
        } catch (err) {
            _didIteratorError7 = true;
            _iteratorError7 = err;
        } finally {
            try {
                if (!_iteratorNormalCompletion7 && _iterator7.return) {
                    _iterator7.return();
                }
            } finally {
                if (_didIteratorError7) {
                    throw _iteratorError7;
                }
            }
        }

        for (var i in this.IsCustomVariant) {
            var item = this.IsCustomVariant[i];
            jobs.push({ ID: item.ID, Name: item.Name });
        }

        for (var i in this.IsCustomFunction) {
            var item = this.IsCustomFunction[i];
            jobs.push({ ID: item.ID, Name: item.Name, Args: item.Args });
        }

        return jobs;
    };

    this.VerifySymbolVariable = function (varName) {
        var setIndexName = new Set(['INDEXA', 'INDEXC', 'INDEXH', 'INDEXL', "INDEXO", "INDEXV", 'INDEXDEC', 'INDEXADV']);
        if (setIndexName.has(varName)) {
            this.IsNeedIndexData = true;
            return;
        }

        var setSymbolDataName = new Set(['CLOSE', 'C', 'VOL', 'V', 'OPEN', 'O', 'HIGH', 'H', 'LOW', 'L', 'AMOUNT']);
        if (setSymbolDataName.has(varName)) {
            this.IsNeedSymbolData = true;
            return;
        }

        if (varName === 'VOLR') {
            if (!this.IsNeedSymbolExData.has(JS_EXECUTE_JOB_ID.JOB_DOWNLOAD_VOLR_DATA)) this.IsNeedSymbolExData.add(JS_EXECUTE_JOB_ID.JOB_DOWNLOAD_VOLR_DATA);
        }

        if (varName == 'HYBLOCK' || varName == 'DYBLOCK' || varName == 'GNBLOCK') {
            if (!this.IsNeedSymbolExData.has(JS_EXECUTE_JOB_ID.JOB_DOWNLOAD_GROUP_DATA)) this.IsNeedSymbolExData.add(JS_EXECUTE_JOB_ID.JOB_DOWNLOAD_GROUP_DATA);
        }

        //流通股本（手）
        if (varName === 'CAPITAL') {
            if (!this.IsNeedFinanceData.has(JS_EXECUTE_JOB_ID.JOB_DOWNLOAD_CAPITAL_DATA)) this.IsNeedFinanceData.add(JS_EXECUTE_JOB_ID.JOB_DOWNLOAD_CAPITAL_DATA);
        }

        if (varName === 'EXCHANGE') {
            if (!this.IsNeedFinanceData.has(JS_EXECUTE_JOB_ID.JOB_DOWNLOAD_EXCHANGE_DATA)) this.IsNeedFinanceData.add(JS_EXECUTE_JOB_ID.JOB_DOWNLOAD_EXCHANGE_DATA);
        }

        if (varName == 'FROMOPEN') {
            this.IsNeedLatestIndexData = true;
            return;
        }

        if (g_JSComplierResource.IsCustomVariant(varName)) //自定义函数( 不过滤了, 调一次就写一次)
            {
                var item = { Name: varName, ID: JS_EXECUTE_JOB_ID.JOB_CUSTOM_VARIANT_DATA };
                this.IsCustomVariant.push(item);
            }
    };

    this.VerifySymbolFunction = function (callee, args) {
        if (callee.Name == 'DYNAINFO') {
            this.IsNeedLatestData = true;
            return;
        }

        //财务函数
        if (callee.Name == 'FINANCE') {
            var jobID = JS_EXECUTE_JOB_ID.GetFinnanceJobID(args[0].Value);
            if (jobID && !this.IsNeedFinanceData.has(jobID)) this.IsNeedFinanceData.add(jobID);
            return;
        }

        if (callee.Name === 'MARGIN') {
            var _jobID = JS_EXECUTE_JOB_ID.GetMarginJobID(args[0].Value);
            if (_jobID && !this.IsNeedMarginData.has(_jobID)) this.IsNeedMarginData.add(_jobID);
            return;
        }

        if (callee.Name === 'NEWS') {
            var _jobID2 = JS_EXECUTE_JOB_ID.GetNewsAnalysisID(args[0].Value);
            if (_jobID2 && !this.IsNeedNewsAnalysisData.has(_jobID2)) this.IsNeedNewsAnalysisData.add(_jobID2);
            return;
        }

        if (callee.Name === 'HK2SHSZ') //北上资金
            {
                var _jobID3 = JS_EXECUTE_JOB_ID.GetHK2SHSZJobID(args[0].Value);
                if (_jobID3 && !this.IsNeedHK2SHSZData.has(_jobID3)) this.IsNeedHK2SHSZData.add(_jobID3);
                return;
            }

        if (callee.Name == 'COST' || callee.Name == 'WINNER') //筹码都需要换手率
            {
                if (!this.IsNeedFinanceData.has(JS_EXECUTE_JOB_ID.JOB_DOWNLOAD_EXCHANGE_DATA)) this.IsNeedFinanceData.add(JS_EXECUTE_JOB_ID.JOB_DOWNLOAD_EXCHANGE_DATA);
                return;
            }

        if (callee.Name === 'BETA') //beta需要下载上证指数
            {
                this.IsNeedIndexData = true;
                return;
            }

        if (callee.Name == 'UPCOUNT' || callee.Name == 'DOWNCOUNT') //上涨下跌个数
            {
                var blockSymbol = args[0].Value;
                if (!this.IsNeedBlockIncreaseData.has(blockSymbol)) this.IsNeedBlockIncreaseData.add(blockSymbol);
                return;
            }

        if (callee.Name == 'COVER_C' || callee.Name == 'COVER_O' || callee.Name == 'COVER_H' || callee.Name == 'COVER_L' || callee.Name == 'COVER_A' || callee.Name == 'COVER_V') //跨周期函数
            {
                var periodName = args[0].Value;
                if (periodName == 'DAY' || periodName == 'WEEK' || periodName == 'YEAR' || periodName == 'MONTH') this.IsNeedSymbolDayData = true;
                return;
            }

        if (callee.Name == 'SF') //Section finance
            {
                var period = JS_EXECUTE_JOB_ID.GetSectionReportPeriod(args[0].Value, args[1].Value); //报告期
                if (!period) return;
                var _jobID4 = JS_EXECUTE_JOB_ID.GetSectionFinanceID(args[2].Value);
                if (!_jobID4) return;
                var sfkey = period.Year + '-' + period.Quarter;

                if (!this.IsNeedSectionFinance.has(sfkey)) {
                    var finacne = { Period: period, Fields: new Set([_jobID4]) };
                    this.IsNeedSectionFinance.set(sfkey, finacne);
                } else {
                    var finacne = this.IsNeedSectionFinance.get(sfkey);
                    if (!finacne.Fields.has(_jobID4)) finacne.Fields.add(_jobID4);
                }

                return;
            }

        if (g_JSComplierResource.IsCustomFunction(callee.Name)) //自定义函数( 不过滤了, 调一次就写一次)
            {
                var item = { Name: callee.Name, ID: JS_EXECUTE_JOB_ID.JOB_CUSTOM_FUNCTION_DATA, Args: args };
                this.IsCustomFunction.push(item);
            }
    };

    this.ExpressionStatement = function (expression) {
        return { Type: Syntax.ExpressionStatement, Expression: expression };
    };

    this.Script = function (body) {
        return { Type: Syntax.Program, Body: body, SourceType: '通达信脚本' };
    };

    this.SequenceExpression = function (expression) {
        return { Type: Syntax.SequenceExpression, Expression: expression };
    };

    this.BinaryExpression = function (operator, left, right) {
        var logical = operator === '||' || operator === '&&' || operator == 'AND' || operator == 'OR';
        var type = logical ? Syntax.LogicalExpression : Syntax.BinaryExpression;

        return { Type: type, Operator: operator, Left: left, Right: right };
    };

    this.Literal = function (value, raw) {
        return { Type: Syntax.Literal, Value: value, Raw: raw };
    };

    this.Identifier = function (name) {
        this.VerifySymbolVariable(name);

        return { Type: Syntax.Identifier, Name: name };
    };

    this.AssignmentExpression = function (operator, left, right) {
        return { Type: Syntax.AssignmentExpression, Operator: operator, Left: left, Right: right };
    };

    this.UnaryExpression = function (operator, argument) {
        return { Type: Syntax.UnaryExpression, Operator: operator, Argument: argument, Prefix: true };
    };

    this.EmptyStatement = function () {
        return { Type: Syntax.EmptyStatement };
    };

    this.CallExpression = function (callee, args) {
        this.VerifySymbolFunction(callee, args);

        return { Type: Syntax.CallExpression, Callee: callee, Arguments: args };
    };
}

function JSParser(code) {
    this.ErrorHandler = new ErrorHandler();
    this.Scanner = new Scanner(code, this.ErrorHandler);
    this.Node = new Node(); //节点创建

    this.LookAhead = { Type: 2, Value: '', LineNumber: this.Scanner.LineNumber, LineStart: 0, Start: 0, End: 0 };
    this.HasLineTerminator = false;
    this.Context = {
        IsModule: false,
        await: false,
        allowIn: true,
        allowStrictDirective: true,
        allowYield: true,
        FirstCoverInitializedNameError: null,
        IsAssignmentTarget: false,
        IsBindingElement: false,
        InFunctionBody: false,
        inIteration: false,
        inSwitch: false,
        labelSet: {},
        Strict: false
    };

    this.PeratorPrecedence = {
        ')': 0,
        ';': 0,
        ',': 0,
        ']': 0,
        '||': 1,
        'OR': 1,
        '&&': 2,
        'AND': 2,
        '|': 3,
        '^': 4,
        '&': 5,
        '==': 6,
        '=': 6,
        '!=': 6,
        '<>': 6,
        '===': 6,
        '!==': 6,
        '<': 7,
        '>': 7,
        '<=': 7,
        '>=': 7,
        '<<': 8,
        '>>': 8,
        '>>>': 8,
        '+': 9,
        '-': 9,
        '*': 11,
        '/': 11,
        '%': 11
    };

    this.StartMarker = { Index: 0, Line: this.Scanner.LineNumber, Column: 0 };
    this.LastMarker = { Index: 0, Line: this.Scanner.LineNumber, Column: 0 };

    this.Initialize = function () {
        this.NextToken();
        this.LastMarker = { Index: this.Scanner.Index, Line: this.Scanner.LineNumber, Column: this.Scanner.Index - this.Scanner.LineStart };
    };

    this.CreateNode = function () {
        return { Index: this.StartMarker.Index, Line: this.StartMarker.Line, Column: this.StartMarker.Column };
    };

    this.StartNode = function (token, lastLineStart) {
        if (lastLineStart == void 0) {
            lastLineStart = 0;
        }

        var column = token.Start - token.LineStart;
        var line = token.LineNumber;
        if (column < 0) {
            column += lastLineStart;
            line--;
        }

        return { Index: token.Start, Line: line, Column: column };
    };

    this.Match = function (value) {
        return this.LookAhead.Type == 7 /*Punctuator*/ && this.LookAhead.Value == value;
    };

    this.Expect = function (value) {
        var token = this.NextToken();
        if (token.Type != 7 /*Punctuator*/ || token.Value != value) this.ThrowUnexpectedToken(token);
    };

    //是否是赋值操作符
    this.MatchAssign = function () {
        if (this.LookAhead.Type != 7 /*Punctuator*/) return false;
        var op = this.LookAhead.Value;

        return op == ':' || op == ':=';
    };

    this.GetTokenRaw = function (token) {
        return this.Scanner.Source.slice(token.Start, token.End);
    };

    this.NextToken = function () {
        var token = this.LookAhead;
        this.LastMarker.Index = this.Scanner.Index;
        this.LastMarker.Line = this.Scanner.LineNumber;
        this.LastMarker.Column = this.Scanner.Index - this.Scanner.LineStart;
        this.CollectComments(); //过滤注释 空格

        if (this.Scanner.Index !== this.StartMarker.Index) {
            this.StartMarker.Index = this.Scanner.Index;
            this.StartMarker.Line = this.Scanner.LineNumber;
            this.StartMarker.Column = this.Scanner.Index - this.Scanner.LineStart;
        }

        var next = this.Scanner.Lex();
        this.HasLineTerminator = token.LineNumber != next.LineNumber;
        if (next && this.Context.Strict && next.Type == 3 /*Identifier */) {
                //TODO:
            }

        this.LookAhead = next;

        return token;
    };

    this.CollectComments = function () {
        this.Scanner.ScanComments();
    };

    this.ParseScript = function () {
        var node = this.CreateNode();
        var body = this.ParseDirectivePrologues();

        while (this.LookAhead.Type != 2 /*EOF*/) {
            body.push(this.ParseStatementListItem());
        }

        return this.Finalize(node, this.Node.Script(body));
    };

    //https://tc39.github.io/ecma262/#sec-directive-prologues-and-the-use-strict-directive
    this.ParseDirective = function () {
        var token = this.LookAhead;
        var node = this.CreateNode();
        var expr = this.ParseExpression();
    };

    this.ParseDirectivePrologues = function () {
        var firstRestricted = null;
        var body = [];
        while (true) {
            var token = this.LookAhead;
            if (token.Type != 8 /*StringLiteral*/) break;

            var statement = this.ParseDirective();
            body.push(statement);
        }

        return body;
    };

    // https://tc39.github.io/ecma262/#sec-block
    this.ParseStatementListItem = function () {
        var statement = void 0;
        this.Context.IsAssignmentTarget = true;
        this.Context.IsBindingElement = true;
        if (this.LookAhead.Type == 4 /*Keyword*/) {} else {
            statement = this.ParseStatement();
        }

        return statement;
    };

    // https://tc39.github.io/ecma262/#sec-ecmascript-language-statements-and-declarations
    this.ParseStatement = function () {
        var statement = void 0;
        switch (this.LookAhead.Type) {
            case 1 /* BooleanLiteral */:
            case 5 /* NullLiteral */:
            case 6 /* NumericLiteral */:
            case 8 /* StringLiteral */:
            case 10 /* Template */:
            case 9 /* RegularExpression */:
                statement = this.ParseExpressionStatement();
                break;
            case 7 /* Punctuator */:
                var value = this.LookAhead.Value;
                if (value === '(') statement = this.ParseExpressionStatement();else if (value === ';') statement = this.ParseEmptyStatement();else statement = this.ParseExpressionStatement();
                break;
            case 3 /* Identifier */:
                statement = this.ParseLabelledStatement();
                break;
            case 4 /* Keyword */:
                break;
            default:
                statement = "error";
        }

        return statement;
    };

    // https://tc39.github.io/ecma262/#sec-empty-statement
    this.ParseEmptyStatement = function () {
        var node = this.CreateNode();
        this.Expect(';');
        return this.Finalize(node, this.Node.EmptyStatement());
    };

    //https://tc39.github.io/ecma262/#sec-labelled-statements
    this.ParseLabelledStatement = function () {
        var node = this.CreateNode();
        var expr = this.ParseExpression();
        this.ConsumeSemicolon();
        var statement = new this.Node.ExpressionStatement(expr);

        return this.Finalize(node, statement);
    };

    // https://tc39.github.io/ecma262/#sec-comma-operator
    this.ParseExpression = function () {
        var startToken = this.LookAhead;
        var expr = this.IsolateCoverGrammar(this.ParseAssignmentExpression);
        if (this.Match(',')) {
            var expressions = [];
            expressions.push(expr);
            while (this.LookAhead.Type != 2 /*EOF*/) {
                if (!this.Match(',')) break;
                this.NextToken();
                expressions.push(this.IsolateCoverGrammar(this.ParseAssignmentExpression));
            }

            expr = this.Finalize(this.StartNode(startToken), this.Node.SequenceExpression(expressions));
        }

        return expr;
    };

    this.ParseAssignmentExpression = function () {
        var expr = void 0;

        var startToken = this.LookAhead;
        var token = startToken;
        expr = this.ParseConditionalExpression();

        if (this.MatchAssign()) {
            if (!this.Context.IsAssignmentTarget) {
                var marker = expr.Marker;
                this.ThrowUnexpectedError(marker.Index, marker.Line, marker.Column, Messages.InvalidLHSInAssignment);
            }

            if (!this.Match('=') && !this.Match(':')) {
                this.Context.IsAssignmentTarget = false;
                this.Context.IsBindingElement = false;
            } else {
                this.ReinterpretExpressionAsPattern(expr);
            }

            token = this.NextToken();
            var operator = token.Value;
            var right = this.IsolateCoverGrammar(this.ParseAssignmentExpression);
            expr = this.Finalize(this.StartNode(startToken), this.Node.AssignmentExpression(operator, expr, right));
            this.Context.FirstCoverInitializedNameError = null;
        }

        return expr;
    };

    this.ParseConditionalExpression = function () {
        var startToken = this.LookAhead;
        var expr = this.InheritCoverGrammar(this.ParseBinaryExpression);

        return expr;
    };

    this.ParseBinaryExpression = function () {
        var startToken = this.LookAhead;
        var expr = this.InheritCoverGrammar(this.ParseExponentiationExpression);
        var token = this.LookAhead;
        var prec = this.BinaryPrecedence(token);
        if (prec > 0) {
            this.NextToken();
            this.Context.IsAssignmentTarget = false;
            this.Context.IsBindingElement = false;
            var markers = [startToken, this.LookAhead];
            var left = expr;
            var right = this.IsolateCoverGrammar(this.ParseExponentiationExpression);
            var stack = [left, token.Value, right];
            var precedences = [prec];
            while (true) {
                prec = this.BinaryPrecedence(this.LookAhead);
                if (prec <= 0) break;

                while (stack.length > 2 && prec <= precedences[precedences.length - 1]) {
                    right = stack.pop();
                    var operator = stack.pop();
                    precedences.pop();
                    left = stack.pop();
                    markers.pop();
                    var node = this.StartNode(markers[markers.length - 1]);
                    stack.push(this.Finalize(node, this.Node.BinaryExpression(operator, left, right)));
                }

                //Shift
                stack.push(this.NextToken().Value);
                precedences.push(prec);
                markers.push(this.LookAhead);
                stack.push(this.IsolateCoverGrammar(this.ParseExponentiationExpression));
            }

            var _i = stack.length - 1;
            expr = stack[_i];
            var lastMarker = markers.pop();
            while (_i > 1) {
                var marker = markers.pop();
                var lastLineStart = lastMarker && lastMarker.LineStart;
                var _node = this.StartNode(marker, lastLineStart);
                var _operator = stack[_i - 1];
                expr = this.Finalize(_node, this.Node.BinaryExpression(_operator, stack[_i - 2], expr));
                _i -= 2;
                lastMarker = marker;
            }
        }

        return expr;
    };

    this.ParseExponentiationExpression = function () {
        var startToken = this.LookAhead;
        var expr = this.InheritCoverGrammar(this.ParseUnaryExpression);

        return expr;
    };

    this.ParseUnaryExpression = function () {
        var expr = void 0;
        if (this.Match('+') || this.Match('-')) {
            var node = this.StartNode(this.LookAhead);
            var token = this.NextToken();
            expr = this.InheritCoverGrammar(this.ParseUnaryExpression);
            expr = this.Finalize(node, this.Node.UnaryExpression(token.Value, expr));
            this.Context.IsAssignmentTarget = false;
            this.Context.IsBindingElement = false;
        } else {
            expr = this.ParseUpdateExpression();
        }

        return expr;
    };

    // https://tc39.github.io/ecma262/#sec-update-expressions
    this.ParseUpdateExpression = function () {
        var expr = void 0;
        var startToken = this.LookAhead;
        expr = this.InheritCoverGrammar(this.ParseLeftHandSideExpressionAllowCall);

        return expr;
    };

    this.ParseLeftHandSideExpressionAllowCall = function () {
        var startToken = this.LookAhead;
        var expr = void 0;
        expr = this.InheritCoverGrammar(this.ParsePrimaryExpression);

        while (true) {
            if (this.Match('(')) {
                this.Context.IsBindingElement = false;
                this.Context.IsAssignmentTarget = false;
                var args = this.ParseArguments(); //解析 调用参数
                expr = this.Finalize(this.StartNode(startToken), this.Node.CallExpression(expr, args));
            } else {
                break;
            }
        }

        return expr;
    };

    // https://tc39.github.io/ecma262/#sec-left-hand-side-expressions
    this.ParseArguments = function () {
        this.Expect('(');
        var args = [];
        if (!this.Match(')')) {
            while (true) {
                var expr = this.IsolateCoverGrammar(this.ParseAssignmentExpression);
                args.push(expr);

                if (this.Match(')')) break;

                this.ExpectCommaSeparator();

                if (this.Match(')')) break;
            }
        }

        this.Expect(')');
        return args;
    };

    // Quietly expect a comma when in tolerant mode, otherwise delegates to expect().
    this.ExpectCommaSeparator = function () {
        this.Expect(',');
    };

    // https://tc39.github.io/ecma262/#sec-primary-expression
    this.ParsePrimaryExpression = function () {
        var node = this.CreateNode();
        var expr = void 0;
        var token, raw;
        switch (this.LookAhead.Type) {
            case 3:
                /* Identifier */
                expr = this.Finalize(node, this.Node.Identifier(this.NextToken().Value));
                break;
            case 6: /* NumericLiteral */
            case 8:
                /* StringLiteral */
                this.Context.IsAssignmentTarget = false;
                this.Context.IsBindingElement = false;
                token = this.NextToken();
                raw = this.GetTokenRaw(token);
                expr = this.Finalize(node, this.Node.Literal(token.Value, raw));
                break;
            case 7:
                /* Punctuator */
                switch (this.LookAhead.Value) {
                    case '(':
                        this.Context.IsBindingElement = false;
                        expr = this.InheritCoverGrammar(this.ParseGroupExpression);
                        break;
                    default:
                        expr = this.ThrowUnexpectedToken(this.NextToken());
                }
                break;
            default:
                expr = this.ThrowUnexpectedToken(this.NextToken());
        }

        return expr;
    };

    this.ParseGroupExpression = function () {
        var expr = void 0;
        this.Expect('(');
        if (this.Match(')')) {
            this.NextToken();
        } else {
            var startToken = this.LookAhead;
            var params = [];
            var arrow = false;
            this.Context.IsBindingElement = true;
            expr = this.InheritCoverGrammar(this.ParseAssignmentExpression);
            if (this.Match(',')) {
                var expressions = [];
                this.Context.IsAssignmentTarget = false;
                expressions.push(expr);
                while (this.LookAhead.Type != 2 /* EOF */) {
                    if (!this.Match(',')) break;

                    this.NextToken();
                    if (this.Match(')')) {}
                }
            }

            if (!arrow) {
                this.Expect(')');
                this.Context.IsBindingElement = false;
            }
        }

        return expr;
    };

    // https://tc39.github.io/ecma262/#sec-expression-statement
    this.ParseExpressionStatement = function () {
        var node = this.CreateNode();
        var expr = this.ParseExpression();
        this.ConsumeSemicolon();

        return this.Finalize(node, this.Node.ExpressionStatement(expr));
    };

    this.ConsumeSemicolon = function () {
        if (this.Match(';')) {
            this.NextToken();
        } else if (!this.HasLineTerminator) {
            //if (this.LookAhead.Type!=2/*EOF*/ && !this.Match('}'))

            this.LastMarker.Index = this.StartMarker.Index;
            this.LastMarker.Line = this.StartMarker.Line;
            this.LastMarker.Column = this.StartMarker.Column;
        }
    };

    this.ReinterpretExpressionAsPattern = function (expr) {
        switch (expr.Type) {
            case Syntax.Identifier:
            case Syntax.AssignmentExpression:
                break;
            default:
                break;
        }
    };

    this.Finalize = function (marker, node) {
        node.Marker = { Line: marker.Line, Column: marker.Column, Index: marker.Index };
        return node;
    };

    this.BinaryPrecedence = function (token) {
        var op = token.Value;
        var precedence = void 0;

        if (token.Type === 7 /* Punctuator */) precedence = this.PeratorPrecedence[op] || 0;else precedence = 0;

        return precedence;
    };

    this.IsolateCoverGrammar = function (parseFunction) {
        var previousIsBindingElement = this.Context.IsBindingElement;
        var previousIsAssignmentTarget = this.Context.IsAssignmentTarget;
        var previousFirstCoverInitializedNameError = this.Context.FirstCoverInitializedNameError;

        this.Context.IsBindingElement = true;
        this.Context.IsAssignmentTarget = true;
        this.Context.FirstCoverInitializedNameError = null;
        var result = parseFunction.call(this);

        if (this.Context.FirstCoverInitializedNameError != null) {
            //错误 this.throwUnexpectedToken(this.context.firstCoverInitializedNameError);
        }

        this.Context.IsBindingElement = previousIsBindingElement;
        this.Context.IsAssignmentTarget = previousIsAssignmentTarget;
        this.Context.FirstCoverInitializedNameError = previousFirstCoverInitializedNameError;

        return result;
    };

    this.InheritCoverGrammar = function (parseFunction) {
        var previousIsBindingElement = this.Context.IsBindingElement;
        var previousIsAssignmentTarget = this.Context.IsAssignmentTarget;
        var previousFirstCoverInitializedNameError = this.Context.FirstCoverInitializedNameError;
        this.Context.IsBindingElement = true;
        this.Context.IsAssignmentTarget = true;
        this.Context.FirstCoverInitializedNameError = null;

        var result = parseFunction.call(this);

        this.Context.IsBindingElement = this.Context.IsBindingElement && previousIsBindingElement;
        this.Context.IsAssignmentTarget = this.Context.IsAssignmentTarget && previousIsAssignmentTarget;
        this.Context.FirstCoverInitializedNameError = previousFirstCoverInitializedNameError || this.Context.FirstCoverInitializedNameError;

        return result;
    };

    this.ThrowUnexpectedToken = function (token, message) {
        throw this.UnexpectedTokenError(token, message);
    };

    this.ThrowUnexpectedError = function (index, line, column, message) {
        var msg = message || "执行异常";

        return this.ErrorHandler.ThrowError(index, line, column, msg);
    };

    this.UnexpectedTokenError = function (token, message) {
        var msg = message || Messages.UnexpectedToken;
        var value = 'ILLEGAL';
        if (token) {
            if (!message) {}
            value = token.Value;
        }

        msg = msg.replace("%0", value);
        if (token && typeof token.LineNumber == 'number') {
            var index = token.Start;
            var _line = token.LineNumber;
            var lastMarkerLineStart = this.LastMarker.Index - this.LastMarker.Column;
            var column = token.Start - lastMarkerLineStart + 1;
            return this.ErrorHandler.CreateError(index, _line, column, msg);
        } else {
            var _index = this.LastMarker.Index;
            var _line2 = this.LastMarker.Line;
            var _column = this.LastMarker.Column + 1;
            return this.ErrorHandler.CreateError(_index, _line2, _column, msg);
        }
    };
}

/*
    算法类
*/
function JSAlgorithm(errorHandler, symbolData) {
    this.ErrorHandler = errorHandler;
    this.SymbolData = symbolData; //股票数据

    //相加
    this.Add = function (data, data2) {
        var isNumber = typeof data == 'number';
        var isNumber2 = typeof data2 == 'number';

        //单数值相加
        if (isNumber && isNumber2) return data + data2;

        //都是数组相加
        var result = [];
        if (!isNumber && !isNumber2) {
            var count = Math.max(data.length, data2.length);
            for (var _i2 = 0; _i2 < count; ++_i2) {
                result[_i2] = null; //初始化

                if (_i2 < data.length && _i2 < data2.length) {
                    if (this.IsNumber(data[_i2]) && this.IsNumber(data2[_i2])) result[_i2] = data[_i2] + data2[_i2];
                }
            }

            return result;
        }

        //单数据和数组相加
        var value = void 0;
        var aryData = void 0;
        if (isNumber) {
            value = data;
            aryData = data2;
        } else {
            value = data2;
            aryData = data;
        }

        for (var _i3 in aryData) {
            result[_i3] = null;
            if (this.IsNumber(aryData[_i3]) && this.IsNumber(value)) result[_i3] = value + aryData[_i3];
        }

        return result;
    };

    //相减
    this.Subtract = function (data, data2) {
        var isNumber = typeof data == 'number';
        var isNumber2 = typeof data2 == 'number';

        //单数值相减
        if (isNumber && isNumber2) return data - data2;

        //都是数组相减
        var result = [];
        if (!isNumber && !isNumber2) {
            var count = Math.max(data.length, data2.length);
            for (var _i4 = 0; _i4 < count; ++_i4) {
                result[_i4] = null; //初始化

                if (_i4 < data.length && _i4 < data2.length) {
                    if (this.IsNumber(data[_i4]) && this.IsNumber(data2[_i4])) result[_i4] = data[_i4] - data2[_i4];
                }
            }

            return result;
        }

        if (isNumber) //单数据-数组
            {
                for (var _i5 in data2) {
                    result[_i5] = null;
                    if (this.IsNumber(data) && this.IsNumber(data2[_i5])) result[_i5] = data - data2[_i5];
                }
            } else //数组-单数据
            {
                for (var _i6 in data) {
                    result[_i6] = null;
                    if (this.IsNumber(data[_i6]) && this.IsNumber(data2)) result[_i6] = data[_i6] - data2;
                }
            }

        return result;
    };

    //相乘
    this.Multiply = function (data, data2) {
        var isNumber = typeof data == 'number';
        var isNumber2 = typeof data2 == 'number';

        //单数值相乘
        if (isNumber && isNumber2) return data * data2;

        //都是数组相乘
        var result = [];
        if (!isNumber && !isNumber2) {
            var count = Math.max(data.length, data2.length);
            for (var _i7 = 0; _i7 < count; ++_i7) {
                result[_i7] = null; //初始化

                if (_i7 < data.length && _i7 < data2.length) {
                    if (this.IsNumber(data[_i7]) && this.IsNumber(data2[_i7])) result[_i7] = data[_i7] * data2[_i7];
                }
            }

            return result;
        }

        //单数据和数组相乘
        var value = void 0;
        var aryData = void 0;
        if (isNumber) {
            value = data;
            aryData = data2;
        } else {
            value = data2;
            aryData = data;
        }

        for (var _i8 in aryData) {
            result[_i8] = null;
            if (this.IsNumber(aryData[_i8]) && this.IsNumber(value)) result[_i8] = value * aryData[_i8];
        }

        return result;
    };

    //相除
    this.Divide = function (data, data2) {
        var isNumber = typeof data == 'number';
        var isNumber2 = typeof data2 == 'number';

        //单数值相除
        if (isNumber && isNumber2) {
            if (data2 == 0) return null; //除0判断
            return data / data2;
        }

        //都是数组相除
        var result = [];
        if (!isNumber && !isNumber2) {
            var count = Math.max(data.length, data2.length);
            for (var _i9 = 0; _i9 < count; ++_i9) {
                result[_i9] = null; //初始化

                if (_i9 < data.length && _i9 < data2.length) {
                    if (this.IsNumber(data[_i9]) && this.IsDivideNumber(data2[_i9])) result[_i9] = data[_i9] / data2[_i9];
                }
            }

            return result;
        }

        if (isNumber) //单数据-数组
            {
                for (var _i10 in data2) {
                    result[_i10] = null;
                    if (this.IsNumber(data) && this.IsDivideNumber(data2[_i10])) result[_i10] = data / data2[_i10];
                }
            } else //数组-单数据
            {
                for (var _i11 in data) {
                    result[_i11] = null;
                    if (this.IsNumber(data[_i11]) && this.IsDivideNumber(data2)) result[_i11] = data[_i11] / data2;
                }
            }

        return result;
    };

    //大于
    this.GT = function (data, data2) {
        var isNumber = typeof data == 'number';
        var isNumber2 = typeof data2 == 'number';

        //单数值比较
        if (isNumber && isNumber2) return data > data2 ? 1 : 0;

        //都是数组比较
        var result = [];
        if (!isNumber && !isNumber2) {
            var count = Math.max(data.length, data2.length);
            for (var _i12 = 0; _i12 < count; ++_i12) {
                result[_i12] = null; //初始化

                if (_i12 < data.length && _i12 < data2.length) {
                    if (!isNaN(data[_i12]) && !isNaN(data2[_i12])) result[_i12] = data[_i12] > data2[_i12] ? 1 : 0;
                }
            }

            return result;
        }

        if (isNumber) //单数据-数组
            {
                for (var _i13 in data2) {
                    result[_i13] = null;
                    if (!isNaN(data) && !isNaN(data2[_i13])) result[_i13] = data > data2[_i13] ? 1 : 0;
                }
            } else //数组-单数据
            {
                for (var _i14 in data) {
                    result[_i14] = null;
                    if (!isNaN(data[_i14]) && !isNaN(data2)) result[_i14] = data[_i14] > data2 ? 1 : 0;
                }
            }

        return result;
    };

    //大于等于
    this.GTE = function (data, data2) {
        var isNumber = typeof data == 'number';
        var isNumber2 = typeof data2 == 'number';

        //单数值比较
        if (isNumber && isNumber2) return data >= data2 ? 1 : 0;

        //都是数组比较
        var result = [];
        if (!isNumber && !isNumber2) {
            var count = Math.max(data.length, data2.length);
            for (var _i15 = 0; _i15 < count; ++_i15) {
                result[_i15] = null; //初始化

                if (_i15 < data.length && _i15 < data2.length) {
                    if (!isNaN(data[_i15]) && !isNaN(data2[_i15])) result[_i15] = data[_i15] >= data2[_i15] ? 1 : 0;
                }
            }

            return result;
        }

        if (isNumber) //单数据-数组
            {
                for (var _i16 in data2) {
                    result[_i16] = null;
                    if (!isNaN(data) && !isNaN(data2[_i16])) result[_i16] = data >= data2[_i16] ? 1 : 0;
                }
            } else //数组-单数据
            {
                for (var _i17 in data) {
                    result[_i17] = null;
                    if (!isNaN(data[_i17]) && !isNaN(data2)) result[_i17] = data[_i17] >= data2 ? 1 : 0;
                }
            }

        return result;
    };

    //小于
    this.LT = function (data, data2) {
        var isNumber = typeof data == 'number';
        var isNumber2 = typeof data2 == 'number';

        //单数值比较
        if (isNumber && isNumber2) return data < data2 ? 1 : 0;

        //都是数组比较
        var result = [];
        if (!isNumber && !isNumber2) {
            var count = Math.max(data.length, data2.length);
            for (var _i18 = 0; _i18 < count; ++_i18) {
                result[_i18] = null; //初始化

                if (_i18 < data.length && _i18 < data2.length) {
                    if (!isNaN(data[_i18]) && !isNaN(data2[_i18])) result[_i18] = data[_i18] < data2[_i18] ? 1 : 0;
                }
            }

            return result;
        }

        if (isNumber) //单数据-数组
            {
                for (var _i19 in data2) {
                    result[_i19] = null;
                    if (!isNaN(data) && !isNaN(data2[_i19])) result[_i19] = data < data2[_i19] ? 1 : 0;
                }
            } else //数组-单数据
            {
                for (var _i20 in data) {
                    result[_i20] = null;
                    if (!isNaN(data[_i20]) && !isNaN(data2)) result[_i20] = data[_i20] < data2 ? 1 : 0;
                }
            }

        return result;
    };

    //小于等于
    this.LTE = function (data, data2) {
        var isNumber = typeof data == 'number';
        var isNumber2 = typeof data2 == 'number';

        //单数值比较
        if (isNumber && isNumber2) return data >= data2 ? 1 : 0;

        //都是数组比较
        var result = [];
        if (!isNumber && !isNumber2) {
            var count = Math.max(data.length, data2.length);
            for (var _i21 = 0; _i21 < count; ++_i21) {
                result[_i21] = null; //初始化

                if (_i21 < data.length && _i21 < data2.length) {
                    if (!isNaN(data[_i21]) && !isNaN(data2[_i21])) result[_i21] = data[_i21] <= data2[_i21] ? 1 : 0;
                }
            }

            return result;
        }

        if (isNumber) //单数据-数组
            {
                for (var _i22 in data2) {
                    result[_i22] = null;
                    if (!isNaN(data) && !isNaN(data2[_i22])) result[_i22] = data <= data2[_i22] ? 1 : 0;
                }
            } else //数组-单数据
            {
                for (var _i23 in data) {
                    result[_i23] = null;
                    if (!isNaN(data[_i23]) && !isNaN(data2)) result[_i23] = data[_i23] <= data2 ? 1 : 0;
                }
            }

        return result;
    };

    //等于
    this.EQ = function (data, data2) {
        var isNumber = typeof data == 'number';
        var isNumber2 = typeof data2 == 'number';

        //单数值比较
        if (isNumber && isNumber2) return data == data2 ? 1 : 0;

        //都是数组比较
        var result = [];
        if (!isNumber && !isNumber2) {
            var count = Math.max(data.length, data2.length);
            for (var _i24 = 0; _i24 < count; ++_i24) {
                result[_i24] = null; //初始化

                if (_i24 < data.length && _i24 < data2.length) {
                    if (!isNaN(data[_i24]) && !isNaN(data2[_i24])) result[_i24] = data[_i24] == data2[_i24] ? 1 : 0;
                }
            }

            return result;
        }

        if (isNumber) //单数据-数组
            {
                for (var _i25 in data2) {
                    result[_i25] = null;
                    if (!isNaN(data) && !isNaN(data2[_i25])) result[_i25] = data == data2[_i25] ? 1 : 0;
                }
            } else //数组-单数据
            {
                for (var _i26 in data) {
                    result[_i26] = null;
                    if (!isNaN(data[_i26]) && !isNaN(data2)) result[_i26] = data[_i26] == data2 ? 1 : 0;
                }
            }

        return result;
    };

    //不等于
    this.NEQ = function (data, data2) {
        var isNumber = typeof data == 'number';
        var isNumber2 = typeof data2 == 'number';

        //单数值比较
        if (isNumber && isNumber2) return data != data2 ? 1 : 0;

        //都是数组比较
        var result = [];
        if (!isNumber && !isNumber2) {
            var count = Math.max(data.length, data2.length);
            for (var _i27 = 0; _i27 < count; ++_i27) {
                result[_i27] = null; //初始化

                if (_i27 < data.length && _i27 < data2.length) {
                    if (!isNaN(data[_i27]) && !isNaN(data2[_i27])) result[_i27] = data[_i27] != data2[_i27] ? 1 : 0;
                }
            }

            return result;
        }

        if (isNumber) //单数据-数组
            {
                for (var _i28 in data2) {
                    result[_i28] = null;
                    if (!isNaN(data) && !isNaN(data2[_i28])) result[_i28] = data != data2[_i28] ? 1 : 0;
                }
            } else //数组-单数据
            {
                for (var _i29 in data) {
                    result[_i29] = null;
                    if (!isNaN(data[_i29]) && !isNaN(data2)) result[_i29] = data[_i29] != data2 ? 1 : 0;
                }
            }

        return result;
    };

    //AND  &&
    this.And = function (data, data2) {
        var isNumber = typeof data == 'number';
        var isNumber2 = typeof data2 == 'number';

        //单数值 &&
        if (isNumber && isNumber2) return data && data2 ? 1 : 0;

        //都是数组 &&
        var result = [];
        if (!isNumber && !isNumber2) {
            var count = Math.max(data.length, data2.length);
            for (var _i30 = 0; _i30 < count; ++_i30) {
                result[_i30] = null; //初始化

                if (_i30 < data.length && _i30 < data2.length) {
                    if (!isNaN(data[_i30]) && !isNaN(data2[_i30])) result[_i30] = data[_i30] && data2[_i30] ? 1 : 0;
                }
            }

            return result;
        }

        if (isNumber) //单数据-数组
            {
                for (var _i31 in data2) {
                    result[_i31] = null;
                    if (!isNaN(data) && !isNaN(data2[_i31])) result[_i31] = data && data2[_i31] ? 1 : 0;
                }
            } else //数组-单数据
            {
                for (var _i32 in data) {
                    result[_i32] = null;
                    if (!isNaN(data[_i32]) && !isNaN(data2)) result[_i32] = data[_i32] && data2 ? 1 : 0;
                }
            }

        return result;
    };

    //OR  ||
    this.Or = function (data, data2) {
        var isNumber = typeof data == 'number';
        var isNumber2 = typeof data2 == 'number';

        //单数值 &&
        if (isNumber && isNumber2) return data || data2 ? 1 : 0;

        //都是数组 &&
        var result = [];
        if (!isNumber && !isNumber2) {
            var count = Math.max(data.length, data2.length);
            for (var _i33 = 0; _i33 < count; ++_i33) {
                result[_i33] = null; //初始化

                if (_i33 < data.length && _i33 < data2.length) {
                    if (!isNaN(data[_i33]) && !isNaN(data2[_i33])) result[_i33] = data[_i33] || data2[_i33] ? 1 : 0;
                }
            }

            return result;
        }

        if (isNumber) //单数据-数组
            {
                for (var _i34 in data2) {
                    result[_i34] = null;
                    if (!isNaN(data) && !isNaN(data2[_i34])) result[_i34] = data || data2[_i34] ? 1 : 0;
                }
            } else //数组-单数据
            {
                for (var _i35 in data) {
                    result[_i35] = null;
                    if (!isNaN(data[_i35]) && !isNaN(data2)) result[_i35] = data[_i35] || data2 ? 1 : 0;
                }
            }

        return result;
    };

    this.IF = function (data, trueData, falseData) {
        var isNumber = typeof data == 'number';
        var isNumber2 = typeof trueData == 'number';
        var isNumber3 = typeof falseData == 'number';

        //单数值
        if (isNumber) {
            if (isNumber2 && isNumber3) return data ? trueData : falseData;

            return data ? trueData : falseData;
        }

        //都是数组
        var result = [];
        for (var _i36 in data) {
            if (data[_i36]) {
                if (isNumber2) result[_i36] = trueData;else result[_i36] = trueData[_i36];
            } else {
                if (isNumber3) result[_i36] = falseData;else result[_i36] = falseData[_i36];
            }
        }

        return result;
    };

    /*
    根据条件求不同的值,同IF判断相反.
    用法: IFN(X,A,B)若X不为0则返回B,否则返回A
    例如: IFN(CLOSE>OPEN,HIGH,LOW)表示该周期收阴则返回最高值,否则返回最低值
    */

    this.IFN = function (data, trueData, falseData) {
        return this.IF(data, falseData, trueData);
    };

    //指标函数 函数名全部大写
    this.REF = function (data, n) {
        var result = [];
        if (typeof n == 'number') {
            if (data.length <= 0) return result;
            if (n >= data.length) return result;

            result = data.slice(0, data.length - n);

            for (var _i37 = 0; _i37 < n; ++_i37) {
                result.unshift(null);
            }
        } else //n 为数组的情况
            {
                for (var _i38 = 0; _i38 < data.length; ++_i38) {
                    result[_i38] = null;
                    if (_i38 >= n.length) continue;
                    var value = n[_i38];
                    if (value > 0 && value <= _i38) result[_i38] = data[_i38 - value];else if (_i38) result[_i38] = result[_i38 - 1];else result[_i38] = data[_i38];
                }
            }

        return result;
    };

    this.MAX = function (data, data2) {
        var isNumber = typeof data == 'number';
        var isNumber2 = typeof data2 == 'number';

        //单数值
        if (isNumber && isNumber2) return Math.max(data, data2);

        //都是数组
        var result = [];
        if (!isNumber && !isNumber2) {
            var count = Math.max(data.length, data2.length);
            for (var _i39 = 0; _i39 < count; ++_i39) {
                result[_i39] = null; //初始化

                if (_i39 < data.length && _i39 < data2.length) {
                    if (!isNaN(data[_i39]) && !isNaN(data2[_i39])) result[_i39] = Math.max(data[_i39], data2[_i39]);
                }
            }

            return result;
        }

        if (isNumber) //单数据-数组
            {
                for (var _i40 in data2) {
                    result[_i40] = null;
                    if (!isNaN(data) && !isNaN(data2[_i40])) result[_i40] = Math.max(data, data2[_i40]);
                }
            } else //数组-单数据
            {
                for (var _i41 in data) {
                    result[_i41] = null;
                    if (!isNaN(data[_i41]) && !isNaN(data2)) result[_i41] = Math.max(data[_i41], data2);
                }
            }

        return result;
    };

    this.MIN = function (data, data2) {
        var isNumber = typeof data == 'number';
        var isNumber2 = typeof data2 == 'number';

        //单数值
        if (isNumber && isNumber2) return Math.min(data, data2);

        //都是数组
        var result = [];
        if (!isNumber && !isNumber2) {
            var count = Math.max(data.length, data2.length);
            for (var _i42 = 0; _i42 < count; ++_i42) {
                result[_i42] = null; //初始化

                if (_i42 < data.length && _i42 < data2.length) {
                    if (!isNaN(data[_i42]) && !isNaN(data2[_i42])) result[_i42] = Math.min(data[_i42], data2[_i42]);
                }
            }

            return result;
        }

        if (isNumber) //单数据-数组
            {
                for (var _i43 in data2) {
                    result[_i43] = null;
                    if (!isNaN(data) && !isNaN(data2[_i43])) result[_i43] = Math.min(data, data2[_i43]);
                }
            } else //数组-单数据
            {
                for (var _i44 in data) {
                    result[_i44] = null;
                    if (!isNaN(data[_i44]) && !isNaN(data2)) result[_i44] = Math.min(data[_i44], data2);
                }
            }

        return result;
    };

    //取正数
    this.ABS = function (data) {
        var result = [];

        for (var _i45 in data) {
            result[_i45] = null;
            if (!isNaN(data[_i45])) result[_i45] = Math.abs(data[_i45]);
        }

        return result;
    };

    this.MA = function (data, dayCount) {
        var result = [];
        if (!Array.isArray(dayCount)) {
            if (dayCount <= 0) dayCount = 1;
            if (!data || !data.length) return result;

            for (var i = 0; i < data.length; ++i) {
                result[i] = null;
                if (this.IsNumber(data[i])) break;
            }

            var data = data.slice(0); //复制一份数据出来
            for (var days = 0; i < data.length; ++i, ++days) {
                if (days < dayCount - 1) {
                    result[i] = null;
                    continue;
                }

                var preValue = data[i - (dayCount - 1)];
                var _sum = 0;
                for (var _j = dayCount - 1; _j >= 0; --_j) {
                    var value = data[i - _j];
                    if (!this.IsNumber(value)) {
                        value = preValue; //空数据就取上一个数据
                        data[i - _j] = value;
                    } else {
                        preValue = value;
                    }
                    _sum += value;
                }

                result[i] = _sum / dayCount;
            }
        } else {
            for (var i = 0; i < data.length; ++i) {
                result[i] = null;
                if (i >= dayCount.length) continue;
                var sumCount = dayCount[i];
                if (!this.IsNumber(sumCount)) continue;
                if (sumCount <= 0) sumCount = i; //0计算当前所有的

                var sum = 0;
                for (var j = i, k = 0; j >= 0 && k < sumCount; --j, ++k) {
                    sum += data[j];
                }

                result[i] = sum / sumCount;
            }
        }

        return result;
    };

    //指数平均数指标 EMA(close,10)
    this.EMA = function (data, dayCount) {
        var result = [];

        var offset = 0;
        if (offset >= data.length) return result;

        //取首个有效数据
        for (; offset < data.length; ++offset) {
            if (data[offset] != null && !isNaN(data[offset])) break;
        }

        var p1Index = offset;
        var p2Index = offset + 1;

        result[p1Index] = data[p1Index];
        for (var i = offset + 1; i < data.length; ++i, ++p1Index, ++p2Index) {
            result[p2Index] = (2 * data[p2Index] + (dayCount - 1) * result[p1Index]) / (dayCount + 1);
        }

        return result;
    };

    /* 
        SMA 移动平均
        返回移动平均。
        用法：　SMA(X，N，M)　X的M日移动平均，M为权重，如Y=(X*M+Y'*(N-M))/N 
    */
    this.SMA = function (data, n, m) {
        var result = [];

        var i = 0;
        var lastData = null;
        for (; i < data.length; ++i) {
            if (data[i] == null || isNaN(data[i])) continue;
            lastData = data[i];
            result[i] = lastData; //第一天的数据
            break;
        }

        for (++i; i < data.length; ++i) {
            result[i] = (m * data[i] + (n - m) * lastData) / n;
            lastData = result[i];
        }

        return result;
    };

    /*
    求动态移动平均.
    用法: DMA(X,A),求X的动态移动平均.
    算法: 若Y=DMA(X,A)则 Y=A*X+(1-A)*Y',其中Y'表示上一周期Y值,A必须小于1.
    例如:DMA(CLOSE,VOL/CAPITAL)表示求以换手率作平滑因子的平均价
    */
    this.DMA = function (data, data2) {
        var result = [];
        if (data.length < 0 || data.length != data2.length) return result;

        var index = 0;
        for (; index < data.length; ++index) {
            if (data[index] != null && !isNaN(data[index]) && data2[index] != null && !isNaN(data2[index])) {
                result[index] = data[index];
                break;
            }
        }

        for (index = index + 1; index < data.length; ++index) {
            if (data[index] == null || data2[index] == null) result[index] = null;else {
                if (data2[index] < 1) result[index] = data2[index] * data[index] + (1 - data2[index]) * result[index - 1];else result[index] = data[index];
            }
        }

        return result;
    };
    /*
    返回加权移动平均
    用法:WMA(X,N):X的N日加权移动平均.
    算法:Yn=(1*X1+2*X2+...+n*Xn)/(1+2+...+n)
     */
    this.WMA = function (data, dayCount) {
        var result = [];
        if (!data || !data.length) return result;
        if (dayCount < 1) dayCount = 1;
        var i = 0;
        for (i = 0; i < data.length && !this.IsNumber(data[i]); ++i) {
            result[i] = null;
        }
        var data = data.slice(0);
        for (var days = 0; i < data.length; ++i, ++days) {
            if (days < dayCount - 1) {
                result[i] = null;
                continue;
            }
            var preValue = data[i - (dayCount - 1)];
            var sum = 0;
            var count = 0;
            for (var j = dayCount - 1; j >= 0; --j) {
                var value = data[i - j];
                if (!this.IsNumber(value)) {
                    value = preValue;
                    data[i - j] = value;
                } else preValue = value;

                count += dayCount - j;
                sum += value * (dayCount - j);
            }
            result[i] = sum / count;
        }
        return result;
    };
    /*
    返回平滑移动平均
    用法:MEMA(X,N):X的N日平滑移动平均,如Y=(X+Y'*(N-1))/N
    MEMA(X,N)相当于SMA(X,N,1)
    */
    this.MEMA = function (data, dayCount) {
        var result = [];
        if (!data || !data.length) return result;
        var i = 0,
            j = 0;
        for (j = 0; j < data.length && !this.IsNumber(data[j]); ++j) {
            result[j] = null;
        }
        i = j;
        if (dayCount < 1 || i + dayCount >= data.length) return result;
        var sum = 0;
        var data = data.slice(0);
        for (; i < j + dayCount; ++i) {
            result[i] = null;
            if (!this.IsNumber(data[i]) && i - 1 >= 0) data[i] = data[i - 1];
            sum += data[i];
        }
        result[i - 1] = sum / dayCount;
        for (; i < data.length; ++i) {
            if (this.IsNumber(result[i - 1]) && this.IsNumber(data[i])) result[i] = (data[i] + result[i - 1] * (dayCount - 1)) / dayCount;else if (i - 1 > -1 && this.IsNumber(result[i - 1])) result[i] = result[i - 1];else result[i] = null;
        }
        return result;
    };
    /*
    加权移动平均
    返回加权移动平均
    用法:EXPMA(X,M):X的M日加权移动平均
    EXPMA[i]=buffer[i]*para+(1-para)*EXPMA[i-1] para=2/(1+__para)
    */
    this.EXPMA = function (data, dayCount) {
        var result = [];
        if (dayCount >= data.length) return result;

        var i = dayCount;
        for (; i < data.length; ++i) //获取第1个有效数据
        {
            if (data[i] != null) {
                result[i] = data[i];
                break;
            }
        }

        for (i = i + 1; i < data.length; ++i) {
            if (result[i - 1] != null && data[i] != null) result[i] = (2 * data[i] + (dayCount - 1) * result[i - 1]) / (dayCount + 1);else if (result[i - 1] != null) result[i] = result[i - 1];
        }

        return result;
    };

    //加权平滑平均,MEMA[i]=SMA[i]*para+(1-para)*SMA[i-1] para=2/(1+__para)
    this.EXPMEMA = function (data, dayCount) {
        var result = [];
        if (dayCount >= data.length) return result;

        var index = 0;
        for (; index < data.length; ++index) {
            if (data[index] && !isNaN(data[index])) break;
        }

        var sum = 0;
        for (var i = 0; index < data.length && i < dayCount; ++i, ++index) {
            if (data[index] && !isNaN(data[index])) sum += data[index];else sum += data[index - 1];
        }

        result[index - 1] = sum / dayCount;
        for (; index < data.length; ++index) {
            if (result[index - 1] != null && data[index] != null) result[index] = (2 * data[index] + (dayCount - 1) * result[index - 1]) / (dayCount + 1);else if (result[index - 1] != null) result[index] = result[index - 1];
        }

        return result;
    };
    /*
    向前累加到指定值到现在的周期数.
    用法:SUMBARS(X,A):将X向前累加直到大于等于A,返回这个区间的周期数
    例如:SUMBARS(VOL,CAPITAL)求完全换手到现在的周期数
     */
    this.SUMBARS = function (data, data2) {
        var result = [];
        if (!data || !data.length || !data2 || !data2.length) return result;
        var start = 0,
            i = 0,
            j = 0;
        for (; start < data.length && !this.IsNumber(data[start]); ++start) {
            result[start] = null;
        }

        for (i = data.length - 1; i >= start; --i) {
            var total = 0;
            for (j = i, total = 0; j >= start && total < data2[i]; --j) {
                total += data[j];
            }if (j < start) result[i] = null;else result[i] = i - j;
        }
        for (i = start + 1; i < data.length; ++i) {
            if (result[i] == null) result[i] = result[i - 1];
        }
        return result;
    };
    /*
    求相反数.
    用法:REVERSE(X)返回-X.
    例如:REVERSE(CLOSE)返回-CLOSE
     */
    this.REVERSE = function (data) {
        var result = [];
        var i = 0;
        for (; i < data.length && !this.isNumber(data[i]); ++i) {
            result[i] = null;
        }
        for (; i < data.length; ++i) {
            if (!this.isNumber(data[i])) result[i] = null;else result[i] = 0 - data[i];
        }
        return result;
    };
    this.COUNT = function (data, n) {
        var result = [];

        for (var _i46 = 0; _i46 < data.length; ++_i46) {
            var count = 0;
            for (var _j2 = 0; _j2 < n && _i46 - _j2 >= 0; ++_j2) {
                if (data[_i46 - _j2]) ++count;
            }

            result[_i46] = count;
        }

        return result;
    };

    /*
    HHV 最高值
    求最高值。
    用法：　HHV(X，N)　求N周期内X最高值，N=0则从第一个有效值开始。
    例如：　HHV(HIGH,30)　表示求30日最高价。
    */
    this.HHV = function (data, n) {
        var result = [];
        if (Array.isArray(n)) {
            var max = null;
            for (var i = 0, j = 0; i < data.length; ++i) {
                result[i] = null;
                if (i >= n.length) continue;

                max = null;
                var count = n[i];
                if (count > 0 && count <= i) {
                    for (j = i - count; j <= i; ++j) {
                        if (max == null || max < data[j]) max = data[j];
                    }
                } else {
                    count = i;
                    for (j = 0; j <= i; ++j) {
                        if (max == null || max < data[j]) max = data[j];
                    }
                }

                result[i] = max;
            }
        } else {
            if (n > data.length) return result;
            if (n <= 0) n = data.length - 1;

            var nMax = 0;
            for (nMax = 0; nMax < data.length; ++nMax) {
                if (this.IsNumber(data[nMax])) break;
            }

            if (nMax < data.length) result[nMax] = data[nMax];
            for (var i = nMax + 1, j = 2; i < data.length && j < n; ++i, ++j) {
                if (data[i] >= data[nMax]) nMax = i;
                result[i] = data[nMax];
            }

            for (; i < data.length; ++i) {
                if (i - nMax < n) {
                    nMax = data[i] < data[nMax] ? nMax : i;
                } else {
                    for (j = nMax = i - n + 2; j <= i; ++j) {
                        nMax = data[j] < data[nMax] ? nMax : j;
                    }
                }

                result[i] = data[nMax];
            }
        }

        return result;
    };

    /*
    LLV 最低值
    求最低值。
    用法：　LLV(X，N)　求N周期内X最低值，N=0则从第一个有效值开始。
    例如：　LLV(LOW，0)　表示求历史最低价。
    */
    this.LLV = function (data, n) {
        var result = [];
        if (Array.isArray(n)) {
            for (var i = 0; i < data.length; ++i) {
                result[i] = null;
                if (i >= n.length) continue;

                var min = null;
                var count = n[i];
                if (count > 0 && count <= i) {
                    for (var j = i - count; j <= i; ++j) {
                        if (min == null || min > data[j]) min = data[j];
                    }
                } else {
                    count = i;
                    for (var j = 0; j <= i; ++j) {
                        if (min == null || min > data[j]) min = data[j];
                    }
                }

                result[i] = min;
            }
        } else {
            if (n > data.length) return result;
            if (n <= 0) n = data.length - 1;

            var min = null;
            for (var i = n; i < data.length; ++i, ++j) {
                if (min == null || i < n + min) //最小值是否在当前周期里面
                    {
                        min = data[i] > data[min] ? min : i;
                    } else {
                    for (var j = (min = i - n + 1) + 1; j <= i; ++j) {
                        if (data[j] < data[min]) min = j;
                    }
                }
                result[i] = data[min];
            }
        }

        return result;
    };

    this.STD = function (data, n) {
        var result = [];

        var total = 0;
        var averageData = []; //平均值
        for (var i = n - 1; i < data.length; ++i) {
            total = 0;
            for (var j = 0; j < n; ++j) {
                total += data[i - j];
            }

            averageData[i] = total / n;
        }

        for (var i = n - 1; i < data.length; ++i) {
            total = 0;
            for (var j = 0; j < n; ++j) {
                total += Math.pow(data[i - j] - averageData[i], 2);
            }

            result[i] = Math.sqrt(total / n);
        }

        return result;
    };

    //平均绝对方差
    this.AVEDEV = function (data, n) {
        var result = [];

        var total = 0;
        var averageData = []; //平均值
        for (var i = n - 1; i < data.length; ++i) {
            total = 0;
            for (var j = 0; j < n; ++j) {
                total += data[i - j];
            }

            averageData[i] = total / n;
        }

        for (var i = n - 1; i < data.length; ++i) {
            total = 0;
            for (var j = 0; j < n; ++j) {
                total += Math.abs(data[i - j] - averageData[i]);
            }

            result[i] = total / n;
        }

        return result;
    };

    //上穿
    this.CROSS = function (data, data2) {
        var result = [];

        if (Array.isArray(data) && Array.isArray(data2)) {
            if (data.length != data2.length) return result = [];

            var index = 0;
            for (; index < data.length; ++index) {
                if (this.IsNumber(data[index]) && this.IsNumber(data2[index])) break;
            }

            for (++index; index < data.length; ++index) {
                result[index] = data[index] > data2[index] && data[index - 1] < data2[index - 1] ? 1 : 0;
            }
        } else if (Array.isArray(data) && typeof data2 == 'number') {
            var index = 0;
            for (; index < data.length; ++index) {
                if (this.IsNumber(data[index])) break;
            }

            for (++index; index < data.length; ++index) {
                result[index] = data[index] > data2 && data[index - 1] < data2 ? 1 : 0;
            }
        } else if (typeof data == 'number' && Array.isArray(data2)) {
            var index = 0;
            for (; index < data2.length; ++index) {
                if (this.IsNumber(data2[index])) break;
            }

            for (++index; index < data2.length; ++index) {
                result[index] = data2[index] < data && data2[index - 1] > data ? 1 : 0;
            }
        }

        return result;
    };

    //累乘
    this.MULAR = function (data, n) {
        var result = [];
        if (data.length < n) return result;

        var index = n;
        for (; index < data.length; ++index) {
            if (data[index] != null && !isNaN(data[index])) {
                result[index] = data[index];
                break;
            }
        }

        for (++index; index < data.length; ++index) {
            result[index] = result[index - 1] * data[index];
        }

        return result;
    };

    this.SUM = function (data, n) {
        var result = [];

        if (!Array.isArray(n)) {
            if (n == 0) {
                result[0] = data[0];

                for (var i = 1; i < data.length; ++i) {
                    result[i] = result[i - 1] + data[i];
                }
            } else {
                for (var i = n - 1, j = 0; i < data.length; ++i, ++j) {
                    for (var k = 0; k < n; ++k) {
                        if (k == 0) result[i] = data[k + j];else result[i] += data[k + j];
                    }
                }
            }
        } else {
            for (var i = 0; i < data.length; ++i) {
                result[i] = null;
                var totalCount = n[i];
                if (!(totalCount > 0)) continue;

                for (var j = i, k = 0; j >= 0 && k < totalCount; --j, ++k) {
                    if (k == 0) result[i] = data[j];else result[i] += data[j];
                }
            }
        }

        return result;
    };

    /*
        BARSCOUNT 有效数据周期数
        求总的周期数。
        用法：　BARSCOUNT(X)　第一个有效数据到当前的天数。
        例如：　BARSCOUNT(CLOSE)　对于日线数据取得上市以来总交易日数，对于分笔成交取得当日成交笔数，对于1分钟线取得当日交易分钟数。
    */
    this.BARSCOUNT = function (data) {
        var result = [];
        var days = null;
        for (var _i47 in data) {
            result[_i47] = 0;
            if (days == null) {
                if (!this.IsNumber(data[_i47])) continue;

                days = 0;
            }

            result[_i47] = days;
            ++days;
        }

        return result;
    };

    //DEVSQ 数据偏差平方和
    //DEVSQ(X，N) 　返回数据偏差平方和。
    this.DEVSQ = function (data, n) {
        var result = [];
        if (typeof n != 'number') n = parseInt(n); //字符串的转成数值型
        var num = n;
        var datanum = data.length;
        var i = 0,
            j = 0,
            k = 0;
        var E = 0,
            DEV = 0;
        for (i = 0; i < datanum && !this.isNumber(data[i]); ++i) {
            result[i] = null;
        }
        if (num < 1 || i + num > datanum) return result;
        for (E = 0; i < datanum && j < num; ++i, ++j) {
            E += data[i] / num;
        }if (j == num) {
            DEV = 0;
            for (i--; k < num; k++) {
                DEV += (data[i - k] - E) * (data[i - k] - E);
            }result[i] = DEV;
            i++;
        }
        for (; i < datanum; ++i) {
            E += (data[i] - data[i - num]) / num;
            for (DEV = 0, k = 0; k < num; ++k) {
                DEV += (data[i - k] - E) * (data[i - k] - E);
            }result[i] = DEV;
        }
        return result;
    };

    //NOT 取反
    //求逻辑非。
    //用法：　NOT(X)　返回非X，即当X=0时返回1，否则返回0。
    //例如：　NOT(ISUP)　表示平盘或收阴。
    this.NOT = function (data) {
        var isNumber = typeof data == 'number';
        if (isNumber) return data ? 0 : 1;

        var result = [];
        for (var _i48 in data) {
            result[_i48] = null;
            if (this.IsNumber(data[_i48])) result[_i48] = data[_i48] ? 0 : 1;
        }

        return result;
    };

    //FORCAST 线性回归预测值
    //FORCAST(X，N)　 返回线性回归预测值。
    this.FORCAST = function (data, n) {
        var result = [];
        if (typeof n != 'number') n = parseInt(n); //字符串的转成数值型
        var num = n;
        var datanum = data.length;
        if (num < 1 || num >= datanum) return result;
        var Ex = 0,
            Ey = 0,
            Sxy = 0,
            Sxx = 0,
            Const,
            Slope;
        var i, j;
        for (j = 0; j < datanum && !this.IsNumber(data[j]); ++j) {
            result[j] = null;
        }
        for (i = j + num - 1; i < datanum; ++i) {
            Ex = Ey = Sxy = Sxx = 0;
            for (j = 0; j < num && j <= i; ++j) {
                Ex += i - j;
                Ey += data[i - j];
            }
            Ex /= num;
            Ey /= num;
            for (j = 0; j < num && j <= i; ++j) {
                Sxy += (i - j - Ex) * (data[i - j] - Ey);
                Sxx += (i - j - Ex) * (i - j - Ex);
            }
            Slope = Sxy / Sxx;
            Const = Ey - Ex * Slope / num;
            result[i] = Slope * num + Const;
        }

        return result;
    };

    //SLOPE 线性回归斜率
    //SLOPE(X，N)　 返回线性回归斜率。
    this.SLOPE = function (data, n) {
        var result = [];
        if (typeof n != 'number') n = parseInt(n); //字符串的转成数值型
        if (n < 1 || !data.length) return result;
        if (n >= data.length) return result;

        var start = 0;
        for (var _i49 = 0; _i49 < data.length; ++_i49, ++start) {
            result[_i49] = null;
            if (this.IsNumber(data[_i49])) break;
        }

        var x = void 0,
            y = void 0,
            xy = void 0,
            xx = void 0;
        for (var _i50 = start + n - 1; _i50 < data.length; ++_i50) {
            result[_i50] = null;
            x = y = xy = xx = 0;
            for (var j = 0; j < n && j <= _i50; ++j) {
                x += _i50 - j; //数据索引相加
                y += data[_i50 - j]; //数据相加
            }

            x = x / n;y = y / n;
            for (j = 0; j < n && j <= _i50; ++j) {
                xy += (_i50 - j - x) * (data[_i50 - j] - y);
                xx += (_i50 - j - x) * (_i50 - j - x);
            }

            if (xx) result[_i50] = xy / xx;else if (_i50) result[_i50] = result[_i50 - 1];
        }

        return result;
    };

    //STDP 总体标准差
    //STDP(X，N)　 返回总体标准差。
    this.STDP = function (data, n) {
        var result = [];
        if (typeof n != 'number') n = parseInt(n); //字符串的转成数值型
        var num = n;
        var datanum = data.length;
        if (num < 1 || num >= datanum) return result;
        var i = 0,
            j = 0;
        for (i = 0; i < datanum && !this.IsNumber(data[i]); ++i) {
            result[i] = null;
        }
        var SigmaPowerX = 0,
            SigmaX = 0,
            MidResult;
        for (; i < datanum && j < num; ++i, ++j) {
            SigmaPowerX += data[i] * data[i];
            SigmaX += data[i];
        }
        if (j == num) {
            MidResult = num * SigmaPowerX - SigmaX * SigmaX;
            result[i - 1] = Math.sqrt(MidResult) / num;
        }
        for (; i < datanum; ++i) {
            SigmaPowerX += data[i] * data[i] - data[i - num] * data[i - num];
            SigmaX += data[i] - data[i - num];
            MidResult = num * SigmaPowerX - SigmaX * SigmaX;
            result[i] = Math.sqrt(MidResult) / num;
        }

        return result;
    };

    //VAR 估算样本方差
    //VAR(X，N)　 返回估算样本方差。
    this.VAR = function (data, n) {
        var result = [];
        if (typeof n != 'number') n = parseInt(n); //字符串的转成数值型
        var num = n;
        var datanum = data.length;
        if (num <= 1 || num >= datanum) return result;
        var i, j;
        for (i = 0; i < datanum && !this.IsNumber(data[i]); ++i) {
            result[i] = null;
        }
        var SigmaPowerX, SigmaX;
        for (j = 0, i = i + num - 1; i < datanum; ++i) {
            SigmaPowerX = SigmaX = 0;
            for (j = 0; j < num && j <= i; ++j) {
                SigmaPowerX += data[i - j] * data[i - j];
                SigmaX += data[i - j];
            }
            result[i] = (num * SigmaPowerX - SigmaX * SigmaX) / num * (num - 1);
        }

        return result;
    };

    //VARP 总体样本方差
    //VARP(X，N)　 返回总体样本方差 。
    this.VARP = function (data, n) {
        var result = [];
        if (typeof n != 'number') n = parseInt(n); //字符串的转成数值型
        var num = n;
        var datanum = data.length;
        if (num < 1 || num >= datanum) return result;
        var i = 0,
            j = 0;
        for (i = 0; i < datanum && !this.IsNumber(data[i]); ++i) {
            result[i] = null;
        }
        var SigmaPowerX = 0,
            SigmaX = 0;
        for (; i < datanum && j < num; ++i, ++j) {
            SigmaPowerX += data[i] * data[i];
            SigmaX += data[i];
        }
        if (j == num) result[i - 1] = (num * SigmaPowerX - SigmaX * SigmaX) / (num * num);
        for (; i < datanum; ++i) {
            SigmaPowerX += data[i] * data[i] - data[i - num] * data[i - num];
            SigmaX += data[i] - data[i - num];
            result[i] = (num * SigmaPowerX - SigmaX * SigmaX) / (num * num);
        }

        return result;
    };

    //RANGE(A,B,C)表示A>B AND A<C;
    this.RANGE = function (data, range, range2) {
        var isNumber = typeof data == 'number';
        var isNumber2 = typeof range == 'number';
        var isNumber3 = typeof range2 == 'number';

        if (isNumber && isNumber2 && isNumber3) {
            if (data > Math.min(range, range2) && data < Math.max(range, range2)) return 1;else return 0;
        }

        var result = [];
        var value = void 0,
            rangeValue = void 0,
            rangValue2 = void 0;
        for (var _i51 = 0; _i51 < data.length; ++_i51) {
            result[_i51] = null;
            value = data[_i51];
            if (!this.IsNumber(value)) continue;

            if (!isNumber2) {
                if (_i51 >= range.length) continue;

                rangeValue = range[_i51];
            } else {
                rangeValue = range;
            }
            if (!this.IsNumber(rangeValue)) continue;

            if (!isNumber3) {
                if (_i51 >= range2.length) continue;

                rangeValue2 = range2[_i51];
            } else {
                rangeValue2 = range2;
            }
            if (!this.IsNumber(rangeValue2)) continue;

            result[_i51] = value > Math.min(rangeValue, rangeValue2) && value < Math.max(rangeValue, rangeValue2) ? 1 : 0;
        }

        return result;
    };

    this.EXIST = function (data, n) {
        n = parseInt(n);

        if (typeof data == 'number') return 0;

        var latestID = null; //最新满足条件的数据索引
        var result = [];
        var value;
        for (var _i52 = 0; _i52 < data.length; ++_i52) {
            result[_i52] = null;
            value = data[_i52];
            if (this.IsNumber(value) && value > 0) latestID = _i52;

            if (latestID != null && _i52 - latestID < n) result[_i52] = 1;else result[_i52] = 0;
        }

        return result;
    };

    this.TFILTER = function (data, data2, n) {
        n = parseInt(n);

        var result = [];

        var isNumber = typeof data == 'number';
        var isNumber2 = typeof range == 'number';

        var count = Math.max(data.length, data2.length);
        for (var _i53 = 0; _i53 < count; ++_i53) {}

        return result;
    };

    /*
    过滤连续出现的信号.
    用法:FILTER(X,N):X满足条件后,将其后N周期内的数据置为0,N为常量.
    例如:
    FILTER(CLOSE>OPEN,5)查找阳线,5天内再次出现的阳线不被记录在内
    */
    this.FILTER = function (data, n) {
        var result = [];
        for (var _i54 = 0, _j3 = 0; _i54 < data.length; ++_i54) {
            if (data[_i54]) {
                result[_i54] = 1;
                for (_j3 = 0; _j3 < n && _j3 + _i54 + 1 < data.length; ++_j3) {
                    result[_j3 + _i54 + 1] = 0;
                }
                _i54 += n;
            } else {
                result[_i54] = 0;
            }
        }

        return result;
    };

    this.BARSLAST = function (data) {
        var result = [];
        if (!data) return result;

        var day = null;
        for (var _i55 = 0; _i55 < data.length; ++_i55) {
            result[_i55] = null;

            if (data[_i55] > 0) day = 0;else if (day != null) ++day;

            if (day != null) result[_i55] = day;
        }

        return result;
    };

    /*
    N周期内第一个条件成立到当前的周期数.
    用法:
    BARSSINCEN(X,N):N周期内第一次X不为0到现在的天数,N为常量
    例如:
    BARSSINCEN(HIGH>10,10)表示10个周期内股价超过10元时到当前的周期数
    */
    this.BARSSINCEN = function (data, n) {
        var result = [];
        var day = null;
        for (var _i56 = 0; _i56 < data.length; ++_i56) {
            result[_i56] = null;
            if (day == null) {
                if (data[_i56]) day = 0;
            } else {
                if (data[_i56]) {
                    if (day + 1 < n) ++day;
                } else {
                    day = null;
                }
            }

            if (day) result[_i56] = day;
        }

        return result;
    };

    /*
    第一个条件成立到当前的周期数.
    用法:
    BARSSINCE(X):第一次X不为0到现在的天数
    例如:
    BARSSINCE(HIGH>10)表示股价超过10元时到当前的周期数
    */
    this.BARSSINCE = function (data) {
        var result = [];
        var day = null;

        for (var _i57 = 0; _i57 < data.length; ++_i57) {
            result[_i57] = null;
            if (day == null) {
                if (data[_i57]) day = 0;
            } else {
                ++day;
            }

            if (day) result[_i57] = day;
        }

        return result;
    };

    /*三角函数调用 func 三角函数 
    反正切值. 用法: ATAN(X)返回X的反正切值
    反余弦值. 用法: ACOS(X)返回X的反余弦值
    反正弦值. 用法: ASIN(X)返回X的反正弦值
    余弦值.  用法: COS(X)返回X的余弦值
    正弦值.  用法: SIN(X)返回X的正弦值
    正切值.  用法: TAN(X)返回X的正切值
      求自然对数. 用法: LN(X)以e为底的对数 例如: LN(CLOSE)求收盘价的对数
    求10为底的对数. 用法: LOG(X)取得X的对数 例如: LOG(100)等于2
    指数. 用法: EXP(X)为e的X次幂 例如: EXP(CLOSE)返回e的CLOSE次幂
    开平方. 用法: SQRT(X)为X的平方根 例如: SQRT(CLOSE)收盘价的平方根
    */
    this.Trigonometric = function (data, func) {
        if (!Array.isArray(data)) {
            if (this.IsNumber(data)) return func(data);

            return null;
        } else {
            var result = [];
            for (var _i58 in data) {
                var item = data[_i58];
                if (this.IsNumber(item)) result[_i58] = func(item);else result[_i58] = null;
            }

            return result;
        }
    };

    /*
    LAST(X,A,B):持续存在.
    用法:
    LAST(CLOSE>OPEN,10,5) 
    表示从前10日到前5日内一直阳线
    若A为0,表示从第一天开始,B为0,表示到最后日止
    */
    this.LAST = function (data, n, n2) {
        var result = [];
        if (n2 <= 0) n2 = data.length - 1;
        if (n2 > n) return result;

        var day = 0;

        for (var _i59 = 0, _j4 = 0; _i59 < data.length; ++_i59) {
            result[_i59] = 0;
            day = 0;
            var start = _i59 - n;
            var end = _i59 - n2;
            if (start < 0 || end < 0) continue;

            for (_j4 = start; _j4 < data.length && _j4 <= end; ++_j4, ++day) {
                if (!data[_j4]) break;
            }

            if (day == end - start + 1) //[start,end]
                result[_i59] = 1;
        }

        return result;
    };

    /*
    属于未来函数,之字转向.
    用法: ZIG(K,N),当价格变化量超过N%时转向,K表示0:开盘价,1:最高价,2:最低价,3:收盘价,其余:数组信息
    例如: ZIG(3,5)表示收盘价的5%的ZIG转向
    */
    this.ZIG = function (data, n) {
        var hisData = this.SymbolData.Data;
        var result = [];
        if (typeof data == 'number') {
            switch (data) {
                case 0:
                    data = hisData.GetOpen();
                    break;
                case 1:
                    data = hisData.GetHigh();
                    break;
                case 2:
                    data = hisData.GetLow();
                    break;
                case 3:
                    data = hisData.GetClose();
                    break;
                default:
                    return result;
            }
        }

        var bFirstPoint = false;
        var bSecondPont = false;
        var firstData = {},
            secondData = {},
            thridData = {};
        var lastData = {};
        for (var _i60 in data) {
            result[_i60] = null;
            var item = data[_i60];
            if (!this.IsNumber(item)) continue;

            if (bFirstPoint == false) {
                bFirstPoint = true;
                firstData = { ID: parseInt(_i60), Value: item }; //第1个点
            } else if (bFirstPoint == true && bSecondPont == false) {
                var temp = (item - firstData.Value) / firstData.Value * 100;
                if (temp > n) {
                    secondData = { ID: parseInt(_i60), Value: item, Up: true };
                    lastData = { ID: parseInt(_i60), Value: item };
                    bSecondPont = true;
                } else if (temp < -n) {
                    secondData = { ID: parseInt(_i60), Value: item, Up: false };
                    lastData = { ID: parseInt(_i60), Value: item };
                    bSecondPont = true;
                }
            } else if (bFirstPoint == true && bSecondPont == true) {
                var temp = (item - lastData.Value) / lastData.Value * 100;
                if (secondData.Up == true) //找下跌的点
                    {
                        if (temp < -n) {
                            thridData = { ID: parseInt(_i60), Value: item, Up: false };
                            this.CalculateZIGLine(firstData, secondData, thridData, data, result);
                            lastData = { ID: parseInt(_i60), Value: item };
                        } else {
                            if (item > lastData.Value) lastData = { ID: parseInt(_i60), Value: item };
                        }
                    } else {
                    if (temp > n) {
                        thridData = { ID: parseInt(_i60), Value: item, Up: true };
                        this.CalculateZIGLine(firstData, secondData, thridData, data, result);
                        lastData = { ID: parseInt(_i60), Value: item };
                    } else {
                        if (item < lastData.Value) lastData = { ID: parseInt(_i60), Value: item };
                    }
                }
            }
        }

        //计算最后1组数据
        thridData = { ID: data.length - 1, Value: data[data.length - 1], Up: !secondData.Up };
        this.CalculateZIGLine(firstData, secondData, thridData, data, result);

        return result;
    };

    this.JSDraw = null;
    this.CalculateZIGLine = function (firstData, secondData, thridData, data, result) {
        if (this.JSDraw == null) this.JSDraw = new JSDraw(this.ErrorHandler);
        var isUp = secondData.Up;
        var findData = firstData;
        if (isUp) {
            for (var i = firstData.ID + 1; i < thridData.ID; ++i) //查找最高点
            {
                var subItem = data[i];
                if (!this.IsNumber(subItem)) continue;
                if (findData.Value < subItem) findData = { ID: i, Value: subItem };
            }
        } else {
            for (var i = firstData.ID + 1; i < thridData.ID; ++i) //查找最低点
            {
                var subItem = data[i];
                if (!this.IsNumber(subItem)) continue;
                if (findData.Value > subItem) findData = { ID: i, Value: subItem };
            }
        }

        secondData.Value = findData.Value;
        secondData.ID = findData.ID;

        var lineCache = { Start: { ID: firstData.ID, Value: firstData.Value }, End: { ID: secondData.ID, Value: secondData.Value } };
        var lineData = this.JSDraw.CalculateDrawLine(lineCache); //计算2个点的线上 其他点的数值
        for (var i in lineData) {
            var lineItem = lineData[i];
            result[lineItem.ID] = lineItem.Value;
        }

        if (thridData.ID == data.length - 1) //最后一组数据
            {
                //最后2个点的数据连成线
                lineCache = { Start: { ID: secondData.ID, Value: secondData.Value }, End: { ID: thridData.ID, Value: thridData.Value } };
                lineData = this.JSDraw.CalculateDrawLine(lineCache); //计算2个点的线上 其他点的数值
                for (var i in lineData) {
                    var lineItem = lineData[i];
                    result[lineItem.ID] = lineItem.Value;
                }
            } else {
            firstData.ID = secondData.ID;
            firstData.Value = secondData.Value;

            secondData.ID = thridData.ID;
            secondData.Value = thridData.Value;
            secondData.Up = firstData.Value < secondData.Value;
        }
    };

    /*
    属于未来函数,前M个ZIG转向波谷到当前距离.
    用法:
    TROUGHBARS(K,N,M)表示之字转向ZIG(K,N)的前M个波谷到当前的周期数,M必须大于等于1
    例如:
    TROUGHBARS(2,5,2)表示%5最低价ZIG转向的前2个波谷到当前的周期数
    */
    this.TROUGHBARS = function (data, n, n2) {
        var zigData = this.ZIG(data, n); //计算ZIG
        var i = 0,
            result = [];
        for (i = 0; i < zigData.length; ++i) {
            result[i] = null;
            if (this.IsNumber(zigData[i])) break;
        }

        var trough = [];
        var start = i,
            j = 0;
        for (; i < zigData.length; ++i) //第1个波谷
        {
            if (i + 1 < zigData.length && i - 1 >= 0 && zigData[i] < zigData[i - 1] && zigData[i] < zigData[i + 1]) //波谷
                {
                    trough[0] = i;
                    break;
                }
        }

        for (i += 1; i < zigData.length; ++i) {
            result[i] = null;
            if (i + 1 < zigData.length && i - 1 >= 0 && zigData[i] < zigData[i - 1] && zigData[i] < zigData[i + 1]) //波谷
                {
                    JSConsole.Complier.Log('[TROUGHBARS] i', i, zigData[i]);
                    ++j;
                    trough[j] = i;
                    if (j + 1 == n2) {
                        result[i] = i - start;
                    } else if (j + 1 > n2) {
                        trough.shift(); //大于计算的波谷数,去掉第1个波谷
                        start = trough[0];
                        --j;
                        result[i] = i - start;
                    }
                } else {
                if (j + 1 === n2) result[i] = i - start;
            }
        }

        return result;
    };

    /*
    属于未来函数,前M个ZIG转向波峰到当前距离.
    用法:
    PEAKBARS(K,N,M)表示之字转向ZIG(K,N)的前M个波峰到当前的周期数,M必须大于等于1
    例如:
    PEAKBARS(0,5,1)表示%5开盘价ZIG转向的上一个波峰到当前的周期数
    */
    this.PEAKBARS = function (data, n, n2) {
        var zigData = this.ZIG(data, n); //计算ZIG
        var i = 0,
            result = [];
        for (i = 0; i < zigData.length; ++i) {
            result[i] = null;
            if (this.IsNumber(zigData[i])) break;
        }

        var trough = [];
        var start = i,
            j = 0;
        for (; i < zigData.length; ++i) //第1个波峰
        {
            if (i + 1 < zigData.length && i - 1 >= 0 && zigData[i] > zigData[i - 1] && zigData[i] > zigData[i + 1]) //波峰
                {
                    trough[0] = i;
                    break;
                }
        }

        for (i += 1; i < zigData.length; ++i) {
            result[i] = null;
            if (i + 1 < zigData.length && i - 1 >= 0 && zigData[i] > zigData[i - 1] && zigData[i] > zigData[i + 1]) //波峰
                {
                    JSConsole.Complier.Log('[TROUGHBARS] i', i, zigData[i]);
                    ++j;
                    trough[j] = i;
                    if (j + 1 == n2) {
                        result[i] = i - start;
                    } else if (j + 1 > n2) {
                        trough.shift(); //大于计算的波谷数,去掉第1个波谷
                        start = trough[0];
                        --j;
                        result[i] = i - start;
                    }
                } else {
                if (j + 1 === n2) result[i] = i - start;
            }
        }

        return result;
    };

    /*
    一直存在.
    例如:
    EVERY(CLOSE>OPEN,N) 
    表示N日内一直阳线(N应大于0,小于总周期数,N支持变量)
    */
    this.EVERY = function (data, n) {
        var result = [];
        if (n < 1) return result;
        var i = 0;
        for (; i < data.length; ++i) {
            result[i] = null;
            if (this.IsNumber(data[i])) break;
        }

        var flag = 0;
        for (; i < data.length; ++i) {
            if (data[i]) flag += 1;else flag = 0;

            if (flag == n) {
                result[i] = 1;
                --flag;
            } else {
                result[i] = 0;
            }
        }
        return result;
    };

    /*
    成本分布情况.
    用法:
    COST(10),表示10%获利盘的价格是多少,即有10%的持仓量在该价格以下,其余90%在该价格以上,为套牢盘
    该函数仅对日线分析周期有效
    */
    this.COST = function (data) {
        var result = [];
        var exchangeID = 201;
        var exchangeData = this.SymbolData.GetFinanceCacheData(exchangeID); //换手率
        if (!exchangeData) return result;

        var isNumber = Array.isArray(data) ? false : true;
        var singleData = null;
        if (isNumber) singleData = parseFloat(data);
        var compareData = null;

        for (var _i61 = this.SymbolData.Data.Data.length - 1, _j5 = 0, k = 0; _i61 >= 0; --_i61) {
            result[_i61] = null;
            var chipData = this.CalculateChip(_i61, exchangeData, this.SymbolData.Data.Data, 1);
            if (chipData.Max == null || chipData.Min == null || chipData.Max <= 0 || chipData.Min <= 0) continue;

            var max = parseInt(chipData.Max * 100);
            var min = parseInt(chipData.Min * 100);

            if (singleData != null) {
                compareData = singleData;
            } else {
                if (_i61 >= data.length) continue;
                compareData = data[_i61];
            }

            var totalVol = 0,
                vol = 0;
            var aryMap = new Map();
            for (_j5 = _i61; _j5 >= 0; --_j5) {
                var item = chipData.Data[_j5];
                var start = parseInt(item.Low * 100);
                var end = parseInt(item.High * 100);
                if (end - start + 1 <= 0) continue;

                var iAverageVolume = item.Vol;
                iAverageVolume = iAverageVolume / (end - start + 1);
                if (iAverageVolume <= 0) continue;

                for (k = start; k <= end && k <= max; ++k) {
                    if (aryMap.has(k)) {
                        vol = aryMap.get(k);
                        aryMap.set(k, vol + iAverageVolume);
                    } else {
                        aryMap.set(k, iAverageVolume);
                    }
                }

                totalVol += item.Vol;
            }

            //计算获利盘
            vol = 0;
            var _iteratorNormalCompletion8 = true;
            var _didIteratorError8 = false;
            var _iteratorError8 = undefined;

            try {
                for (var _iterator8 = aryMap[Symbol.iterator](), _step8; !(_iteratorNormalCompletion8 = (_step8 = _iterator8.next()).done); _iteratorNormalCompletion8 = true) {
                    var priceData = _step8.value;

                    vol += priceData[1];
                    result[_i61] = priceData[0] / 100;
                    if (vol / totalVol * 100 > compareData) break;
                }
            } catch (err) {
                _didIteratorError8 = true;
                _iteratorError8 = err;
            } finally {
                try {
                    if (!_iteratorNormalCompletion8 && _iterator8.return) {
                        _iterator8.return();
                    }
                } finally {
                    if (_didIteratorError8) {
                        throw _iteratorError8;
                    }
                }
            }
        }

        return result;
    };

    /*
    获利盘比例.
    用法:
    WINNER(CLOSE),表示以当前收市价卖出的获利盘比例,例如返回0.1表示10%获利盘;WINNER(10.5)表示10.5元价格的获利盘比例
    该函数仅对日线分析周期有效
    ！！！！计算比较耗时间
    */
    this.WINNER = function (data) {
        var result = [];
        var exchangeID = 201;
        var exchangeData = this.SymbolData.GetFinanceCacheData(exchangeID); //换手率
        if (!exchangeData) return result;

        var isNumber = Array.isArray(data) ? false : true;
        var singleData = null;
        if (isNumber) singleData = parseInt(parseFloat(data) * 100);
        var compareData = null;

        for (var _i62 = this.SymbolData.Data.Data.length - 1, _j6 = 0, k = 0; _i62 >= 0; --_i62) {
            result[_i62] = null;
            var chipData = this.CalculateChip(_i62, exchangeData, this.SymbolData.Data.Data, 1);
            if (chipData.Max == null || chipData.Min == null || chipData.Max <= 0 || chipData.Min <= 0) continue;

            var max = parseInt(chipData.Max * 100);
            var min = parseInt(chipData.Min * 100);

            if (singleData != null) {
                compareData = singleData;
            } else {
                if (_i62 >= data.length) continue;
                compareData = parseInt(data[_i62] * 100);
            }

            var totalVol = 0,
                vol = 0;
            for (_j6 = _i62; _j6 >= 0; --_j6) {
                var item = chipData.Data[_j6];
                var start = parseInt(item.Low * 100);
                var end = parseInt(item.High * 100);
                if (end - start + 1 <= 0) continue;

                var iAverageVolume = item.Vol;
                iAverageVolume = iAverageVolume / (end - start + 1);
                if (iAverageVolume <= 0) continue;

                var profitVol = 0; //获利的成交量
                if (compareData > end) profitVol = item.Vol;else if (compareData < start) profitVol = 0;else profitVol = item.Vol * (compareData - start + 1) / (end - start + 1);

                vol += profitVol;
                totalVol += item.Vol;
            }

            if (totalVol > 0) result[_i62] = vol / totalVol;
        }

        return result;
    };

    //计算截至到某一天的历史所有筹码
    this.CalculateChip = function (index, exchangeData, hisData, dRate) {
        var result = { Min: null, Max: null, Data: [] };
        var seed = 1; //筹码历史衰减换手系数
        var max = null,
            min = null;
        for (var _i63 = index; _i63 >= 0; --_i63) {
            var _item = {}; //Vol:量 High:最高 Low:最低
            var kData = hisData[_i63];
            if (_i63 == index) _item.Vol = kData.Vol * exchangeData[_i63];else _item.Vol = kData.Vol * seed;

            _item.Date = kData.Date;
            _item.High = kData.High;
            _item.Low = kData.Low;

            if (max == null) max = _item.High;else if (max < _item.High) max = _item.High;
            if (min == null) min = _item.Low;else if (min < _item.Low) min = _item.Low;

            result.Data[_i63] = _item;

            seed *= 1 - exchangeData[_i63] / 100 * dRate; //换手率累乘
        }

        result.Max = max;
        result.Min = min;

        return result;
    };

    /*
    返回是否连涨周期数.
    用法:
    UPNDAY(CLOSE,M)
    表示连涨M个周期,M为常量
    */
    this.UPNDAY = function (data, n) {
        var result = [];
        if (n < 1) return result;
        if (data == null || n > data.length) return result;

        var days = 0;
        for (var _i64 = 0; _i64 < data.length; ++_i64) {
            result[_i64] = 0;
            if (_i64 - 1 < 0) continue;
            if (!this.IsNumber(data[_i64]) || !this.IsNumber(data[_i64 - 1])) //无效数都不算连涨
                {
                    days = 0;
                    continue;
                }

            if (data[_i64] > data[_i64 - 1]) ++days;else days = 0;

            if (days == n) {
                result[_i64] = 1;
                --days;
            }
        }

        return result;
    };

    /*
    返回是否连跌周期.
    用法:
    DOWNNDAY(CLOSE,M)
    表示连跌M个周期,M为常量
    */
    this.DOWNNDAY = function (data, n) {
        var result = [];
        if (n < 1) return result;
        if (data == null || n > data.length) return result;

        var days = 0;
        for (var _i65 = 0; _i65 < data.length; ++_i65) {
            result[_i65] = 0;
            if (_i65 - 1 < 0) continue;
            if (!this.IsNumber(data[_i65]) || !this.IsNumber(data[_i65 - 1])) //无效数都不算连涨
                {
                    days = 0;
                    continue;
                }

            if (data[_i65] < data[_i65 - 1]) ++days;else days = 0;

            if (days == n) {
                result[_i65] = 1;
                --days;
            }
        }

        return result;
    };

    /*
    返回是否持续存在X>Y
    用法:
    NDAY(CLOSE,OPEN,3)
    表示连续3日收阳线
    */
    this.NDAY = function (data, data2, n) {
        var result = [];
        if (n < 1) return result;
        if (!Array.isArray(data) && !Array.isArray(data2)) return result;
        if (data == null || data2 == null) return result;

        if (Array.isArray(data) && Array.isArray(data2)) {
            if (n >= data.length || n >= data2.length) return result;
            var count = Math.max(data.length, data2.length);
            var days = 0;
            for (var _i66 = 0; _i66 < count; ++_i66) {
                result[_i66] = 0;
                if (_i66 >= data.length || _i66 >= data2.length) continue;
                if (!this.IsNumber(data[_i66]) || !this.IsNumber(data2[_i66])) {
                    days = 0;
                    continue;
                }

                if (data[_i66] > data2[_i66]) ++days;else days = 0;

                if (days == n) {
                    result[_i66] = 1;
                    --days;
                }
            }
        } else if (Array.isArray(data) && !Array.isArray(data2)) {
            if (n >= data.length || !this.IsNumber(data2)) return;
            var days = 0;
            for (var _i67 in data) {
                result[_i67] = 0;
                if (!this.IsNumber(data[_i67])) {
                    days = 0;
                    continue;
                }

                if (data[_i67] > data2) ++days;else days = 0;

                if (days == n) {
                    result[_i67] = 1;
                    --days;
                }
            }
        } else if (!Array.isArray(data) && Array.isArray(data2)) {
            if (n >= data2.length || !this.IsNumber(data)) return;
            var days = 0;
            for (var _i68 in data2) {
                result[_i68] = 0;
                if (!this.IsNumber(data2[_i68])) {
                    days = 0;
                    continue;
                }

                if (data > data2[_i68]) ++days;else days = 0;

                if (days == n) {
                    result[_i68] = 1;
                    --days;
                }
            }
        }

        return result;
    };

    /*
    两条线维持一定周期后交叉.
    用法:LONGCROSS(A,B,N)表示A在N周期内都小于B,本周期从下方向上穿过B时返回1,否则返回0
    */
    this.LONGCROSS = function (data, data2, n) {
        var result = [];
        var count = Math.max(data.length, data2.length);
        for (var _i69 = 0; _i69 < count; ++_i69) {
            result[_i69] = 0;
            if (_i69 - 1 < 0) continue;
            if (_i69 >= data.length || _i69 >= data2.length) continue;
            if (!this.IsNumber(data[_i69]) || !this.IsNumber(data2[_i69]) || !this.IsNumber(data[_i69 - 1]) || !this.IsNumber(data2[_i69 - 1])) continue;

            if (data[_i69] > data2[_i69] && data[_i69 - 1] < data2[_i69 - 1]) result[_i69] = 1;
        }

        for (var _i70 = 0, _j7 = 0; _i70 < count; ++_i70) {
            if (!result[_i70]) continue;

            for (_j7 = 1; _j7 <= n && _i70 - _j7 >= 0; ++_j7) {
                if (data[_i70 - _j7] >= data2[_i70 - _j7]) {
                    result[_i70] = 0;
                    break;
                }
            }
        }

        return result;
    };

    /*
    EXISTR(X,A,B):是否存在(前几日到前几日间).
    例如: EXISTR(CLOSE>OPEN,10,5) 
    表示从前10日内到前5日内存在着阳线
    若A为0,表示从第一天开始,B为0,表示到最后日止
    */
    this.EXISTR = function (data, n, n2) {
        var result = [];
        if (!Array.isArray(data)) return result;

        n = parseInt(n);
        n2 = parseInt(n2);
        if (n <= 0) n = data.length;
        if (n2 <= 0) n2 = 1;
        if (n2 > n) return result;

        var result = [];
        var value;
        for (var _i71 = 0, _j8 = 0; _i71 < data.length; ++_i71) {
            result[_i71] = null;
            if (_i71 - n < 0 || _i71 - n2 < 0) continue;

            result[_i71] = 0;
            for (_j8 = n; _j8 >= n2; --_j8) {
                var value = data[_i71 - _j8];
                if (this.IsNumber(value) && value) {
                    result[_i71] = 1;
                    break;
                }
            }
        }

        return result;
    };

    /*
    RELATE(X,Y,N) 返回X和Y的N周期的相关系数
    RELATE(X,Y,N)=(∑[(Xi-Avg(X))(Yi-Avg(y))])/N ÷ √((∑(Xi-Avg(X))^2)/N * (∑(Yi-Avg(Y))^2)/N)
    其中 avg(x)表示x的N周期均值：  avg(X) = (∑Xi)/N  
    √(...)表示开平方
    */
    this.RELATE = function (data, data2, n) {
        var result = [];
        if (n < 1) n = 1;

        if (!Array.isArray(data) || !Array.isArray(data2)) return result;

        var dataAverage = this.CalculateAverage(data, n);
        var data2Average = this.CalculateAverage(data2, n);

        var count = Math.max(data.length, data2.length);
        for (var _i72 = 0, _j9 = 0; _i72 < count; ++_i72) {
            result[_i72] = null;

            if (_i72 >= data.length || _i72 >= data2.length || _i72 >= dataAverage.length || _i72 >= data2Average.length) continue;

            var average = dataAverage[_i72];
            var average2 = data2Average[_i72];

            var total = 0,
                total2 = 0,
                total3 = 0;
            for (_j9 = _i72 - n + 1; _j9 <= _i72; ++_j9) {
                total += (data[_j9] - average) * (data2[_j9] - average2); //∑[(Xi-Avg(X))(Yi-Avg(y))])
                total2 += Math.pow(data[_j9] - average, 2); //∑(Xi-Avg(X))^2
                total3 += Math.pow(data2[_j9] - average2, 2); //∑(Yi-Avg(Y))^2)
            }

            result[_i72] = total / n / (Math.sqrt(total2 / n) * Math.sqrt(total3 / n));
        }

        return result;
    };

    //计算数组n周期内的均值
    this.CalculateAverage = function (data, n) {
        var result = [];
        if (n < 1) return result;

        var total = 0;

        for (var i = 0; i < data.length; ++i) //去掉开始的无效数
        {
            if (this.IsNumber(data[i])) break;
        }

        for (; i < data.length && i < n; ++i) //计算第1个周期的数据
        {
            result[i] = null;
            var value = data[i];
            if (!this.IsNumber(value)) continue;
            total += value;
        }
        result[i - 1] = total / n;

        for (; i < data.length; ++i) //计算后面的周期数据
        {
            var value = data[i];
            var preValue = data[i - n]; //上一个周期的第1个数据
            if (!this.IsNumber(value)) value = 0;
            if (!this.IsNumber(preValue)) preValue = 0;

            total = total - preValue + value; //当前周期的数据 等于上一个周期数据 去掉上一个周期的第1个数据 加上这个周期的最后1个数据
            result[i] = total / n;
        }

        return result;
    };

    /*
    COVAR(X,Y,N) 返回X和Y的N周期的协方差
    */
    this.COVAR = function (data, data2, n) {
        var result = [];
        if (n < 1) n = 1;

        if (!Array.isArray(data) || !Array.isArray(data2)) return result;

        var dataAverage = this.CalculateAverage(data, n);
        var data2Average = this.CalculateAverage(data2, n);

        var count = Math.max(data.length, data2.length);

        var count = Math.max(data.length, data2.length);
        for (var _i73 = 0, _j10 = 0; _i73 < count; ++_i73) {
            result[_i73] = null;

            if (_i73 >= data.length || _i73 >= data2.length || _i73 >= dataAverage.length || _i73 >= data2Average.length) continue;

            var average = dataAverage[_i73];
            var average2 = data2Average[_i73];

            var total = 0;
            for (_j10 = _i73 - n + 1; _j10 <= _i73; ++_j10) {
                total += (data[_j10] - average) * (data2[_j10] - average2);
            }

            result[_i73] = total / n;
        }

        return result;
    };

    /*
    求上一高点到当前的周期数.
    用法:
    HHVBARS(X,N):求N周期内X最高值到当前周期数,N=0表示从第一个有效值开始统计
    例如:
    HHVBARS(HIGH,0)求得历史新高到到当前的周期数
    */
    this.HHVBARS = function (data, n) {
        var result = [];
        if (!Array.isArray(data)) return result;
        if (n < 1) n = data.length;

        var nMax = null; //最大值索引
        for (var i = 0; i < data.length; ++i) {
            result[i] = null;
            if (this.IsNumber(data[i])) {
                nMax = i;
                break;
            }
        }

        var j = 0;
        for (i = nMax + 1; i < data.length && j < n; ++i, ++j) //求第1个最大值
        {
            if (data[i] >= data[nMax]) nMax = i;
            if (n == data.length) result[i] = i - nMax;
        }

        for (; i < data.length; ++i) {
            if (i - nMax < n) {
                if (data[i] >= data[nMax]) nMax = i;
            } else {
                nMax = i - n + 1;
                for (j = nMax; j <= i; ++j) //计算区间最大值
                {
                    if (data[j] >= data[nMax]) nMax = j;
                }
            }

            result[i] = i - nMax;
        }

        return result;
    };

    /*
    求上一低点到当前的周期数.
    用法: LLVBARS(X,N):求N周期内X最低值到当前周期数,N=0表示从第一个有效值开始统计
    例如: LLVBARS(HIGH,20)求得20日最低点到当前的周期数
    */
    this.LLVBARS = function (data, n) {
        var result = [];
        if (!Array.isArray(data)) return result;
        if (n < 1) n = data.length;

        var nMin = null; //最小值索引
        for (var i = 0; i < data.length; ++i) {
            result[i] = null;
            if (this.IsNumber(data[i])) {
                nMin = i;
                break;
            }
        }

        var j = 0;
        for (i = nMin + 1; i < data.length && j < n; ++i, ++j) //求第1个最大值
        {
            if (data[i] <= data[nMin]) nMin = i;
            if (n == data.length) result[i] = i - nMin;
        }

        for (; i < data.length; ++i) {
            if (i - nMin < n) {
                if (data[i] <= data[nMin]) nMin = i;
            } else {
                nMin = i - n + 1;
                for (j = nMin; j <= i; ++j) //计算区间最小值
                {
                    if (data[j] <= data[nMin]) nMin = j;
                }
            }

            result[i] = i - nMin;
        }

        return result;
    };

    /*
    β(Beta)系数
    BETA(N) 返回当前证券N周期收益与对应大盘指数收益相比的贝塔系数
    需要下载上证指数历史数据
    涨幅(X)=(现价-上一个交易日收盘价）/上一个交易日收盘价
    公式=股票和指数协方差/指数方差
    */
    this.BETA = function (n) {
        var result = [];
        var stockData = this.SymbolData.Data;
        var indexData = this.SymbolData.IndexData;
        if (n <= 0) n = 1;

        var stockProfit = []; //股票涨幅
        var indexProfit = []; //指数涨幅

        for (var _i74 = 0; _i74 < stockData.Data.length; ++_i74) {
            stockProfit[_i74] = 0;
            indexProfit[_i74] = 0;

            var stockItem = stockData.Data[_i74];
            var indexItem = indexData.Data[_i74];

            if (stockItem.Close > 0 && stockItem.YClose > 0) stockProfit[_i74] = (stockItem.Close - stockItem.YClose) / stockItem.YClose;
            if (indexItem.Close > 0 && indexItem.YClose > 0) indexProfit[_i74] = (indexItem.Close - indexItem.YClose) / indexItem.YClose;
        }

        //计算均值数组
        var averageStockProfit = this.CalculateAverage(stockProfit, n);
        var averageIndexProfit = this.CalculateAverage(indexProfit, n);

        for (var i = 0, j = 0; i < stockData.Data.length; ++i) {
            result[i] = null;

            if (i >= stockProfit.length || i >= indexProfit.length || i >= averageStockProfit.length || i >= averageIndexProfit.length) continue;

            var averageStock = averageStockProfit[i];
            var averageIndex = averageIndexProfit[i];

            var covariance = 0; //协方差
            var variance = 0; //方差
            for (j = i - n + 1; j <= i; ++j) {
                var value = indexProfit[j] - averageIndex;
                var value2 = stockProfit[j] - averageStock;
                covariance += value * value2;
                variance += value * value;
            }

            if (this.IsDivideNumber(variance) && this.IsNumber(covariance)) result[i] = covariance / variance; //(covariance/n)/(variance/n)=covariance/variance;
        }

        return result;
    };

    /*
    用法:BETA2(X,Y,N)为X与Y的N周期相关放大系数,表示Y变化1%,则X将变化N%
    例如:BETA2(CLOSE,INDEXC,10)表示收盘价与大盘指数之间的10周期相关放大率
    */
    this.BETA2 = function (x, y, n) {
        var result = [];
        if (n <= 0) n = 1;

        var xProfit = [null]; //x数据的涨幅
        var yProfit = [null]; //y数据的涨幅

        var count = Math.max(x.length, y.length);

        var lastItem = { X: x[0], Y: y[0] };
        for (var i = 1; i < count; ++i) {
            xProfit[i] = 0;
            yProfit[i] = 0;

            var xItem = x[i];
            var yItem = y[i];

            if (lastItem.X > 0) xProfit[i] = (xItem - lastItem.X) / lastItem.X;
            if (lastItem.Y > 0) yProfit[i] = (yItem - lastItem.Y) / lastItem.Y;

            lastItem = { X: xItem, Y: yItem };
        }

        //计算均值数组
        var averageXProfit = this.CalculateAverage(xProfit, n);
        var averageYProfit = this.CalculateAverage(yProfit, n);

        for (var i = 0, j = 0; i < count; ++i) {
            result[i] = null;

            if (i >= xProfit.length || i >= yProfit.length || i >= averageXProfit.length || i >= averageYProfit.length) continue;

            var averageX = averageXProfit[i];
            var averageY = averageYProfit[i];

            var covariance = 0; //协方差
            var variance = 0; //方差
            for (j = i - n + 1; j <= i; ++j) {
                var value = xProfit[j] - averageX;
                var value2 = yProfit[j] - averageY;
                covariance += value * value2;
                variance += value * value;
            }

            if (this.IsDivideNumber(variance) && this.IsNumber(covariance)) result[i] = covariance / variance; //(covariance/n)/(variance/n)=covariance/variance;
        }

        return result;
    };

    /*
    抛物转向.
    用法:
    SAR(N,S,M),N为计算周期,S为步长,M为极值
    例如:
    SAR(10,2,20)表示计算10日抛物转向,步长为2%,极限值为20%
    */
    this.SAR = function (n, step, exValue) {
        var result = [];
        var stockData = this.SymbolData.Data;
        if (n >= stockData.Data.length) return result;

        var high = null,
            low = null;
        for (var i = 0; i < n; ++i) {
            var item = stockData.Data[i];
            if (high == null) high = item.High;else if (high < item.High) high = item.High;
            if (low == null) low = item.Low;else if (low > item.Low) low = item.Low;
        }

        var SAR_LONG = 0,
            SAR_SHORT = 1;
        var position = SAR_LONG;
        result[n - 1] = low;
        var nextSar = low,
            sip = stockData.Data[0].High,
            af = step / 100;
        for (var i = n; i < stockData.Data.length; ++i) {
            var ysip = sip;
            var item = stockData.Data[i];
            var yitem = stockData.Data[i - 1];

            if (position == SAR_LONG) {
                if (item.Low < result[i - 1]) {
                    position = SAR_SHORT;
                    sip = item.Low;
                    af = step / 100;
                    nextSar = Math.max(item.High, yitem.High);
                    nextSar = Math.max(nextSar, ysip + af * (sip - ysip));
                } else {
                    position = SAR_LONG;
                    if (item.High > ysip) {
                        sip = item.High;
                        af = Math.min(af + step / 100, exValue / 100);
                    }
                    nextSar = Math.min(item.Low, yitem.Low);
                    nextSar = Math.min(nextSar, result[i - 1] + af * (sip - result[i - 1]));
                }
            } else if (position == SAR_SHORT) {
                if (item.High > result[i - 1]) {
                    position = SAR_LONG;
                    sip = item.High;
                    af = step / 100;
                    nextSar = Math.min(item.Low, yitem.Low);
                    nextSar = Math.min(nextSar, result[i - 1] + af * (sip - ysip));
                } else {
                    position = SAR_SHORT;
                    if (item.Low < ysip) {
                        sip = item.Low;
                        af = Math.min(af + step / 100, exValue / 100);
                    }
                    nextSar = Math.max(item.High, yitem.High);
                    nextSar = Math.max(nextSar, result[i - 1] + af * (sip - result[i - 1]));
                }
            }

            result[i] = nextSar;
        }

        return result;
    };

    /*
    抛物转向点.
    用法:
    SARTURN(N,S,M),N为计算周期,S为步长,M为极值,若发生向上转向则返回1,若发生向下转向则返回-1,否则为0
    其用法与SAR函数相同
    */
    this.SARTURN = function (n, step, exValue) {
        var result = [];
        var sar = this.SAR(n, step, exValue);
        var stockData = this.SymbolData.Data;
        var index = 0;
        for (index = 0; index < sar.length; ++index) {
            if (this.IsNumber(sar[index])) break;
        }
        var flag = 0;
        if (index < stockData.Data.length) flag = stockData.Data[index].Close > sar[index];

        for (var i = index + 1; i < stockData.Data.length; ++i) {
            var item = stockData.Data[i];
            if (item.Close < sar[i] && flag) result[i] = -1;else result[i] = item.Close > sar[i] && !flag ? 1 : 0;

            flag = item.Close > sar[i];
        }

        return result;
    };

    /*
    属于未来函数,将当前位置到若干周期前的数据设为1.
    用法:
    BACKSET(X,N),若X非0,则将当前位置到N周期前的数值设为1.
    例如:
    BACKSET(CLOSE>OPEN,2)若收阳则将该周期及前一周期数值设为1,否则为0
    */
    this.BACKSET = function (condition, n) {
        var result = [];
        if (!condition) return result;
        var dataCount = condition.length;
        if (!this.IsNumber(dataCount) || dataCount <= 0) return result;

        for (var i = 0; i < dataCount; ++i) //初始化0
        {
            result[i] = 0;
        }

        for (var pos = 0; pos < dataCount; ++pos) {
            if (this.IsNumber(condition[pos])) break;
        }
        if (pos == dataCount) return result;

        var num = Math.min(dataCount - pos, Math.max(n, 1));

        for (var i = dataCount - 1, j = 0; i >= 0; --i) {
            var value = condition[i];
            if (this.IsNumber(value) && value) {
                for (j = i; j > i - num; --j) {
                    result[j] = 1;
                }
            }
        }

        if (condition[i]) {
            for (j = i; j >= pos; --j) {
                result[j] = 1;
            }
        }

        return result;
    };

    //用法:BETWEEN(A,B,C)表示A处于B和C之间时返回1,否则返回0 
    //例如:BETWEEN(CLOSE,MA(CLOSE,20),MA(CLOSE,10))表示收盘价介于10日均线和20日均线之间
    this.BETWEEN = function (condition, data, data2) {
        var result = [];
        var isNumber = typeof condition == 'number';
        var isNumber2 = typeof data == 'number';
        var isNumber3 = typeof data2 == 'number';

        if (isNumber && isNumber2 && isNumber3) //单数值
            {
                return condition >= data && condition <= data2 ? 1 : 0;
            }

        for (var i in condition) {
            result[i] = 0;
            var item = condition[i];
            var left = null,
                right = null;

            if (isNumber2) left = data;else if (i < data.length - 1) left = data[i];

            if (isNumber3) right = data2;else if (i < data2.length - 1) right = data2[i];

            if (left == null || right == null) continue;

            if (left > right) {
                if (item >= right && item <= left) result[i] = 1;
            } else {
                if (item <= right && item >= left) result[i] = 1;
            }
        }

        return result;
    };

    //STRCAT(A,B):将两个字符串A,B(非序列化)相加成一个字符串C.
    //用法: STRCAT('多头','开仓')将两个字符串'多头','开仓'相加成一个字符串'多头开仓'
    this.STRCAT = function (str1, str2) {
        var result = str1 + str2;
        return result;
    };

    //CON2STR(A,N):取A最后的值(非序列值)转为字符串,小数位数N.
    //用法: CON2STR(FINANCE(20),3)表示取营业收入,以3位小数转为字符串
    this.CON2STR = function (data, n) {
        var result = [];
        if (Array.isArray(data)) {
            for (var i = data.length - 1; i >= 0; --i) {
                var item = data[i];
                if (IFrameSplitOperator.IsNumber(item)) {
                    result = item.toFixed(n);
                    return result;
                }
            }
        } else {
            if (IFrameSplitOperator.IsNumber(data)) result = data.toFixed(n);
        }

        return result;
    };

    ////////////////////////////////////////////////////////////////////////
    //  跨周期函数COVER_C(), COVER_O(), COVER_H(), COVER_L(), COVER_A(), COVER_V()
    this.CoverPeriod = function (name, periodName, n) {
        var periodInfo = this.GetPeriodInfo({ Name: periodName });
        if (!periodInfo) return null;

        var curPeriodInfo = this.GetPeriodInfo({ PeriodID: this.SymbolData.Data.Period });
        if (!curPeriodInfo) return null;

        if (curPeriodInfo.Order > curPeriodInfo.Order) return null; //只能小周期转大周期

        var result;
        if (curPeriodInfo.Period == periodInfo.Period) {
            result = this.SymbolData.Data;
        } else {
            var hisData;
            if (ChartData.IsMinutePeriod(curPeriodInfo.Period, true) && ChartData.IsDayPeriod(periodInfo.Period, true)) {
                if (periodInfo.Period == 0) hisData = this.SymbolData.DayData.Data; //日线直接用
                else hisData = this.SymbolData.DayData.GetPeriodData(periodInfo.Period); //分钟数据不复权 直接算周期就可以了
            } else {
                var bindData = new ChartData();
                bindData.Data = this.SymbolData.SourceData.Data;
                bindData.Period = this.SymbolData.Period;
                bindData.Right = this.SymbolData.Right;

                if (ChartData.IsDayPeriod(periodInfo.Period, true) && bindData.Right > 0) //日线数据才复权
                    {
                        var rightData = bindData.GetRightData(bindData.Right);
                        bindData.Data = rightData;
                    }

                hisData = bindData.GetPeriodData(periodInfo.Period);
            }

            var data = this.SymbolData.Data.CoverTo(hisData, curPeriodInfo.Period, periodInfo.Period);
            result = new ChartData();
            result.Data = data;
        }

        if (IFrameSplitOperator.IsPlusNumber(n)) {
            var refResult = new ChartData();
            var data = result.GetRef(n);
            refResult.Data = data;
            result = refResult;
        }

        switch (name) {
            case 'COVER_C':
                return result.GetClose();
            case 'COVER_O':
                return result.GetOpen();
            case 'COVER_H':
                return result.GetHigh();
            case 'COVER_L':
                return result.GetLow();
            case 'COVER_A':
                return result.GetAmount();
            case 'COVER_V':
                return result.GetVol();
            default:
                return null;
        }
    };

    this.MOD = function (data, data2) {
        var result = [];
        var isNumber = typeof data == 'number';
        var isNumber2 = typeof data2 == 'number';

        //单数值
        if (isNumber && isNumber2) {
            return JSAlgorithm.MOD(data, data2);
        } else if (!isNumber && !isNumber2) //都是数组相加
            {
                var count = Math.max(data.length, data2.length);
                for (var _i75 = 0; _i75 < count; ++_i75) {
                    result[_i75] = null; //初始化

                    if (_i75 < data.length && _i75 < data2.length) {
                        if (this.IsNumber(data[_i75]) && this.IsNumber(data2[_i75])) result[_i75] = JSAlgorithm.MOD(data[_i75], data2[_i75]);
                    }
                }

                return result;
            } else if (isNumber && !isNumber2) //单数字 数组
            {
                for (var i in data2) {
                    result[i] = null;
                    if (this.IsNumber(data) && this.IsNumber(data2[i])) result[i] = JSAlgorithm.MOD(data, data2[i]);
                }

                return result;
            } else if (!isNumber && isNumber2) //数组 单数字
            {
                for (var i in data) {
                    result[i] = null;
                    if (this.IsNumber(data[i]) && this.IsNumber(data2)) result[i] = JSAlgorithm.MOD(data[i], data2);
                }

                return result;
            }

        return result;
    };

    this.POW = function (data, n) {
        var result = [];
        if (!IFrameSplitOperator.IsNumber(n)) return result;

        var isNumber = typeof data == 'number';

        if (isNumber) return Math.pow(data, n);

        for (var i in data) {
            var item = data[i];
            if (IFrameSplitOperator.IsNumber(item)) result[i] = Math.pow(item, n);else result[i] = null;
        }

        return result;
    };

    this.CEILING = function (data) {
        var isNumber = typeof data == 'number';
        if (isNumber) return parseInt(data);

        var result = [];
        for (var i in data) {
            var item = data[i];
            if (IFrameSplitOperator.IsNumber(item)) result[i] = parseInt(item);else result[i] = null;
        }

        return result;
    };

    this.FLOOR = function (data) {
        var isNumber = typeof data == 'number';
        if (isNumber) return parseInt(data - 1);

        var result = [];
        for (var i in data) {
            var item = data[i];
            if (IFrameSplitOperator.IsNumber(item)) result[i] = parseInt(item - 1);else result[i] = null;
        }

        return result;
    };

    //函数调用
    this.CallFunction = function (name, args, node, symbolData) {
        switch (name) {
            case 'MAX':
                return this.MAX(args[0], args[1]);
            case 'MIN':
                return this.MIN(args[0], args[1]);
            case 'REF':
                return this.REF(args[0], args[1]);
            case 'ABS':
                return this.ABS(args[0]);
            case 'MA':
                return this.MA(args[0], args[1]);
            case "EMA":
                return this.EMA(args[0], args[1]);
            case "SMA":
                return this.SMA(args[0], args[1], args[2]);
            case "DMA":
                return this.DMA(args[0], args[1]);
            case 'EXPMA':
                return this.EXPMA(args[0], args[1]);
            case 'EXPMEMA':
                return this.EXPMEMA(args[0], args[1]);
            case 'COUNT':
                return this.COUNT(args[0], args[1]);
            case 'LLV':
                return this.LLV(args[0], args[1]);
            case 'LLVBARS':
                return this.LLVBARS(args[0], args[1]);
            case 'HHV':
                return this.HHV(args[0], args[1]);
            case 'HHVBARS':
                return this.HHVBARS(args[0], args[1]);
            case 'MULAR':
                return this.MULAR(args[0], args[1]);
            case 'CROSS':
                return this.CROSS(args[0], args[1]);
            case 'LONGCROSS':
                return this.LONGCROSS(args[0], args[1], args[2]);
            case 'AVEDEV':
                return this.AVEDEV(args[0], args[1]);
            case 'STD':
                return this.STD(args[0], args[1]);
            case 'IF':
            case 'IFF':
                return this.IF(args[0], args[1], args[2]);
            case 'IFN':
                return this.IFN(args[0], args[1], args[2]);
            case 'NOT':
                return this.NOT(args[0]);
            case 'SUM':
                return this.SUM(args[0], args[1]);
            case 'RANGE':
                return this.RANGE(args[0], args[1], args[2]);
            case 'EXIST':
                return this.EXIST(args[0], args[1]);
            case 'EXISTR':
                return this.EXISTR(args[0], args[1], args[2]);
            case 'FILTER':
                return this.FILTER(args[0], args[1]);
            case 'TFILTER':
                return this.TFILTER(args[0], args[1], args[2]);
            case 'SLOPE':
                return this.SLOPE(args[0], args[1]);
            case 'BARSLAST':
                return this.BARSLAST(args[0]);
            case 'BARSCOUNT':
                return this.BARSCOUNT(args[0]);
            case 'BARSSINCEN':
                return this.BARSSINCEN(args[0], args[1]);
            case 'BARSSINCE':
                return this.BARSSINCE(args[0]);
            case 'LAST':
                return this.LAST(args[0], args[1], args[2]);
            case 'EVERY':
                return this.EVERY(args[0], args[1]);
            case 'DEVSQ':
                return this.DEVSQ(args[0], args[1]);
            case 'ZIG':
                return this.ZIG(args[0], args[1]);
            case 'TROUGHBARS':
                return this.TROUGHBARS(args[0], args[1], args[2]);
            case 'PEAKBARS':
                return this.PEAKBARS(args[0], args[1], args[2]);
            case 'COST':
                return this.COST(args[0]);
            case 'WINNER':
                return this.WINNER(args[0]);
            case 'FORCAST':
                return this.FORCAST(args[0], args[1]);
            case 'STDP':
                return this.STDP(args[0], args[1]);
            case 'VAR':
                return this.VAR(args[0], args[1]);
            case 'VARP':
                return this.VARP(args[0], args[1]);
            case 'UPNDAY':
                return this.UPNDAY(args[0], args[1]);
            case 'DOWNNDAY':
                return this.DOWNNDAY(args[0], args[1]);
            case 'NDAY':
                return this.NDAY(args[0], args[1], args[2]);
            case 'RELATE':
                return this.RELATE(args[0], args[1], args[2]);
            case 'COVAR':
                return this.COVAR(args[0], args[1], args[2]);
            case 'BETA':
                return this.BETA(args[0]);
            case 'BETA2':
                return this.BETA2(args[0], args[1], args[2]);
            case 'WMA':
                return this.WMA(args[0], args[1]);
            case 'MEMA':
                return this.MEMA(args[0], args[1]);
            case 'SUMBARS':
                return this.SUMBARS(args[0], args[1]);
            case 'REVERSE':
                return this.REVERSE(args[0]);
            case 'SAR':
                return this.SAR(args[0], args[1], args[2]);
            case 'SARTURN':
                return this.SARTURN(args[0], args[1], args[2]);
            case 'BACKSET':
                return this.BACKSET(args[0], args[1]);
            case 'BETWEEN':
                return this.BETWEEN(args[0], args[1], args[2]);
            case 'STRCAT':
                return this.STRCAT(args[0], args[1]);
            case 'CON2STR':
                return this.CON2STR(args[0], args[1]);
            case 'COVER_C':
            case 'COVER_O':
            case 'COVER_H':
            case 'COVER_L':
            case 'COVER_A':
            case 'COVER_V':
                if (args.length == 2) return this.CoverPeriod(name, args[0], args[1]);
                return this.CoverPeriod(name, args[0]);
            case 'MOD':
                return this.MOD(args[0], args[1]);
            case 'POW':
                return this.POW(args[0], args[1]);
            case 'CEILING':
                return this.CEILING(args[0]);
            case 'FLOOR':
                return this.FLOOR(args[0]);
            //三角函数
            case 'ATAN':
                return this.Trigonometric(args[0], Math.atan);
            case 'ACOS':
                return this.Trigonometric(args[0], Math.acos);
            case 'ASIN':
                return this.Trigonometric(args[0], Math.asin);
            case 'COS':
                return this.Trigonometric(args[0], Math.cos);
            case 'SIN':
                return this.Trigonometric(args[0], Math.sin);
            case 'TAN':
                return this.Trigonometric(args[0], Math.tan);
            case 'LN':
                return this.Trigonometric(args[0], Math.log);
            case 'LOG':
                return this.Trigonometric(args[0], Math.log10);
            case 'EXP':
                return this.Trigonometric(args[0], Math.exp);
            case 'SQRT':
                return this.Trigonometric(args[0], Math.sqrt);
            default:
                if (g_JSComplierResource.IsCustomFunction(name)) return this.CallCustomFunction(name, args, symbolData);

                this.ThrowUnexpectedNode(node, '函数' + name + '不存在');
        }
    };

    this.CallCustomFunction = function (name, args, symbolData) {
        var item = g_JSComplierResource.CustomFunction.Data.get(name);
        if (!item || !item.Invoke) return [];

        var obj = { Name: name, Args: args, Symbol: symbolData.Symbol, Period: symbolData.Period, Right: symbolData.Right, KData: symbolData.Data };
        return item.Invoke(obj);
    };

    this.ThrowUnexpectedNode = function (node, message) {
        var marker = node.Marker;
        var msg = message || "执行异常";

        return this.ErrorHandler.ThrowError(marker.Index, marker.Line, marker.Column, msg);
    };
}

JSAlgorithm.prototype.GetPeriodInfo = function (obj) //{Name:周期名字  PeriodID:周期ID}
{
    var PERIOD_LIST = [{ Name: 'MIN1', Period: 4, Order: 1 }, { Name: 'MIN5', Period: 5, Order: 2 }, { Name: 'MIN15', Period: 6, Order: 3 }, { Name: 'MIN30', Period: 7, Order: 4 }, { Name: 'MIN60', Period: 8, Order: 5 }, { Name: 'DAY', Period: 0, Order: 1000 }, { Name: 'WEEK', Period: 1, Order: 1001 }, { Name: 'MONTH', Period: 2, Order: 1002 }, { Name: "YEAR", Period: 3, Order: 1003 }];

    if (obj.Name) {
        for (var i in PERIOD_LIST) {
            if (obj.Name && PERIOD_LIST[i].Name == obj.Name) return PERIOD_LIST[i];
        }

        return null;
    }

    if (IFrameSplitOperator.IsNumber(obj.PeriodID)) {
        for (var i in PERIOD_LIST) {
            if (PERIOD_LIST[i].Period == obj.PeriodID) return PERIOD_LIST[i];
        }

        return null;
    }

    return null;
};

//是否有是有效的数字
JSAlgorithm.prototype.IsNumber = function (value) {
    if (value == null) return false;
    if (isNaN(value)) return false;

    return true;
};

//是否有是有效的除数
JSAlgorithm.prototype.IsDivideNumber = function (value) {
    if (value == null) return false;
    if (isNaN(value)) return false;
    if (value == 0) return false;

    return true;
};

//取模
JSAlgorithm.MOD = function (number, divisor) {
    if (number < 0 && divisor < 0 || number >= 0 && divisor >= 0) //同号
        {
            if (parseInt(number) == number && parseInt(divisor) == divisor) //全为整数
                {
                    return number % divisor;
                } else //被除数-(整商×除数)之后在第一位小数位进行四舍五入
                {
                    var value = parseFloat((number - Math.floor(number / divisor) * divisor).toFixed(1));
                    return value;
                }
        } else //异号
        {
            var absNumber = Math.abs(number); //绝对值
            var absDivisor = Math.abs(divisor); //绝对值
            var value = Math.abs(Math.abs(divisor) * (Math.floor(absNumber / absDivisor) + 1) - Math.abs(number));
            if (divisor < 0) value = -value;
            return value;
        }
};

/*
   绘图函数 
*/
function JSDraw(errorHandler, symbolData) {
    this.ErrorHandler = errorHandler;
    this.SymbolData = symbolData;

    this.DRAWTEXT = function (condition, price, text) {
        var drawData = [];
        var result = { DrawData: drawData, DrawType: 'DRAWTEXT', Text: text };
        if (condition.length <= 0) return result;

        var IsNumber = typeof price == "number";

        for (var i in condition) {
            drawData[i] = null;

            if (isNaN(condition[i]) || !condition[i]) continue;

            if (IsNumber) {
                drawData[i] = price;
            } else {
                if (this.IsNumber(price[i])) drawData[i] = price[i];
            }
        }

        return result;
    };

    this.DRAWTEXT_FIX = function (condition, x, y, type, text) {
        var result = { Position: null, DrawType: 'DRAWTEXT_FIX', Text: text };
        if (condition.length <= 0) return result;

        for (var i in condition) {
            if (isNaN(condition[i]) || !condition[i]) continue;

            result.Position = { X: x, Y: y, Type: type };
            return result;
        }

        return result;
    };

    //direction 文字Y轴位置 0=middle 1=价格的顶部 2=价格的底部
    //offset 文字Y轴偏移
    this.SUPERDRAWTEXT = function (condition, price, text, direction, offset) {
        var drawData = [];
        var result = { DrawData: drawData, DrawType: 'SUPERDRAWTEXT', Text: text, YOffset: offset, Direction: direction, TextAlign: 'center' };
        if (condition.length <= 0) return result;

        var IsNumber = typeof price == "number";

        for (var i in condition) {
            drawData[i] = null;

            if (isNaN(condition[i]) || !condition[i]) continue;

            if (IsNumber) {
                drawData[i] = price;
            } else {
                if (this.IsNumber(price[i])) drawData[i] = price[i];
            }
        }

        return result;
    };

    /*
    STICKLINE 绘制柱线
    在图形上绘制柱线。
    用法：　STICKLINE(COND，PRICE1，PRICE2，WIDTH，EMPTY)，当COND条件满足时，在PRICE1和PRICE2位置之间画柱状线，宽度为WIDTH(10为标准间距)，EMPTH不为0则画空心柱。
    例如：　STICKLINE(CLOSE>OPEN，CLOSE，OPEN，0.8，1)表示画K线中阳线的空心柱体部分。
    */
    this.STICKLINE = function (condition, data, data2, width, type) {
        var drawData = [];
        var result = { DrawData: drawData, DrawType: 'STICKLINE', Width: width, Type: type };

        if (condition.length <= 0) return result;

        var IsNumber = typeof data == "number";
        var IsNumber2 = typeof data2 == "number";

        for (var i in condition) {
            drawData[i] = null;

            if (isNaN(condition[i]) || !condition[i]) continue;

            if (IsNumber && IsNumber2) {
                drawData[i] = { Value: data, Value2: data2 };
            } else if (IsNumber && !IsNumber2) {
                if (isNaN(data2[i])) continue;
                drawData[i] = { Value: data, Value2: data2[i] };
            } else if (!IsNumber && IsNumber2) {
                if (isNaN(data[i])) continue;
                drawData[i] = { Value: data[i], Value2: data2 };
            } else {
                if (isNaN(data[i]) || isNaN(data2[i])) continue;
                drawData[i] = { Value: data[i], Value2: data2[i] };
            }
        }

        return result;
    };

    /*
    DRAWLINE 绘制直线段
    在图形上绘制直线段。
    用法：　DRAWLINE(COND1，PRICE1，COND2，PRICE2，EXPAND)
    当COND1条件满足时，在PRICE1位置画直线起点，当COND2条件满足时，在PRICE2位置画直线终点，EXPAND为延长类型。
    例如：　DRAWLINE(HIGH>=HHV(HIGH,20),HIGH,LOW<=LLV(LOW,20),LOW,1)　表示在创20天新高与创20天新低之间画直线并且向右延长。
    */
    this.DRAWLINE = function (condition, data, condition2, data2, expand) {
        var drawData = [];
        var result = { DrawData: drawData, DrawType: 'DRAWLINE', Expand: expand };

        if (condition.length <= 0) return result;
        var count = Math.max(condition.length, condition2.length);

        var bFirstPoint = false;
        var bSecondPont = false;
        var lineCache = { Start: {}, End: {}, List: new Array() };

        for (var _i76 = 0; _i76 < count; ++_i76) {
            drawData[_i76] = null;
            if (_i76 < condition.length && _i76 < condition2.length) {
                if (bFirstPoint == false && bSecondPont == false) {
                    if (condition[_i76] == null || !condition[_i76]) continue;

                    bFirstPoint = true;
                    lineCache.Start = { ID: _i76, Value: data[_i76] }; //第1个点
                } else if (bFirstPoint == true && bSecondPont == false) {
                    var bCondition2 = condition2[_i76] != null && condition2[_i76]; //条件2
                    if (!bCondition2) continue;

                    if (bCondition2) {
                        bSecondPont = true;
                        lineCache.End = { ID: _i76, Value: data2[_i76] }; //第2个点
                    }
                }

                if (bFirstPoint == true && bSecondPont == true) {
                    var lineData = this.CalculateDrawLine(lineCache); //计算2个点的线上 其他点的数值

                    for (var _j11 in lineData) {
                        var _item2 = lineData[_j11];
                        drawData[_item2.ID] = _item2.Value;
                    }

                    bFirstPoint = bSecondPont = false;
                    lineCache = { Start: {}, End: {} };
                }
            }
        }

        return result;
    };

    /*
    画出带状线.
    用法: DRAWBAND(VAL1,COLOR1,VAL2,COLOR2),当VAL1>VAL2时,在VAL1和VAL2之间填充COLOR1;当VAL1<VAL2时,填充COLOR2,这里的颜色均使用RGB函数计算得到.
    例如: DRAWBAND(OPEN,RGB(0,224,224),CLOSE,RGB(255,96,96));
    */
    this.DRAWBAND = function (data, color, data2, color2) {
        var drawData = [];
        var result = { DrawData: drawData, DrawType: 'DRAWBAND', Color: [color.toLowerCase(), color2.toLowerCase()] }; //颜色使用小写字符串
        var count = Math.max(data.length, data2.length);

        for (var _i77 = 0; _i77 < count; ++_i77) {
            var _item3 = { Value: null, Value2: null };
            if (_i77 < data.length) _item3.Value = data[_i77];
            if (_i77 < data2.length) _item3.Value2 = data2[_i77];

            drawData.push(_item3);
        }

        return result;
    };

    this.DRAWKLINE = function (high, open, low, close) {
        var drawData = [];
        var result = { DrawData: drawData, DrawType: 'DRAWKLINE' };
        var count = Math.max(high.length, open.length, low.length, close.length);

        for (var _i78 = 0; _i78 < count; ++_i78) {
            var _item4 = { Open: null, High: null, Low: null, Close: null };

            if (_i78 < high.length && _i78 < open.length && _i78 < low.length && _i78 < close.length) {
                _item4.Open = open[_i78];
                _item4.High = high[_i78];
                _item4.Low = low[_i78];
                _item4.Close = close[_i78];
            }

            drawData[_i78] = _item4;
        }

        return result;
    };

    //满足条件画一根K线
    this.DRAWKLINE_IF = function (condition, high, open, low, close) {
        var drawData = [];
        var result = { DrawData: drawData, DrawType: 'DRAWKLINE_IF' };
        var count = Math.max(condition.length, high.length, open.length, low.length, close.length);

        for (var _i79 = 0; _i79 < count; ++_i79) {
            var _item5 = { Open: null, High: null, Low: null, Close: null };

            if (_i79 < high.length && _i79 < open.length && _i79 < low.length && _i79 < close.length && _i79 < condition.length) {
                if (condition[_i79]) {
                    _item5.Open = open[_i79];
                    _item5.High = high[_i79];
                    _item5.Low = low[_i79];
                    _item5.Close = close[_i79];
                }
            }

            drawData[_i79] = _item5;
        }

        return result;
    };

    /*
    PLOYLINE 折线段
    在图形上绘制折线段。
    用法：　PLOYLINE(COND，PRICE)，当COND条件满足时，以PRICE位置为顶点画折线连接。
    例如：　PLOYLINE(HIGH>=HHV(HIGH,20),HIGH)表示在创20天新高点之间画折线。
    */
    this.POLYLINE = function (condition, data) {
        var drawData = [];
        var result = { DrawData: drawData, DrawType: 'POLYLINE' };
        var isNumber = typeof data == 'number';

        var bFirstPoint = false;
        var bSecondPont = false;
        if (isNumber) {
            for (var _i80 in condition) {
                drawData[_i80] = null;
                if (bFirstPoint == false) {
                    if (!condition[_i80]) continue;

                    drawData[_i80] = data;
                    bFirstPoint = true;
                } else {
                    drawData[_i80] = data;
                }
            }
        } else {
            var lineCache = { Start: {}, End: {}, List: new Array() };
            for (var _i81 in condition) {
                drawData[_i81] = null;
                if (bFirstPoint == false && bSecondPont == false) {
                    if (condition[_i81] == null || !condition[_i81]) continue;
                    if (_i81 >= data.length || !this.IsNumber(data[_i81])) continue;

                    bFirstPoint = true;
                    lineCache.Start = { ID: parseInt(_i81), Value: data[_i81] }; //第1个点
                } else if (bFirstPoint == true && bSecondPont == false) {
                    if (condition[_i81] == null || !condition[_i81]) continue;
                    if (_i81 >= data.length || !this.IsNumber(data[_i81])) continue;

                    lineCache.End = { ID: parseInt(_i81), Value: data[_i81] }; //第2个点
                    //根据起始点和结束点 计算中间各个点的数据
                    var lineData = this.CalculateDrawLine(lineCache); //计算2个点的线上 其他点的数值

                    for (var _j12 in lineData) {
                        var _item6 = lineData[_j12];
                        drawData[_item6.ID] = _item6.Value;
                    }

                    var start = { ID: lineCache.End.ID, Value: lineCache.End.Value };
                    lineCache = { Start: start, End: {} };
                }
            }
        }

        return result;
    };

    /*
    画出数字.
    用法:
    DRAWNUMBER(COND,PRICE,NUMBER),当COND条件满足时,在PRICE位置书写数字NUMBER.
    例如:
    DRAWNUMBER(CLOSE/OPEN>1.08,LOW,C)表示当日实体阳线大于8%时在最低价位置显示收盘价.
    */
    this.DRAWNUMBER = function (condition, data, data2) {
        var drawData = { Value: new Array(), Text: new Array() };
        var result = { DrawData: drawData, DrawType: 'DRAWNUMBER' };
        var isNumber = typeof data2 == 'number';
        var text = void 0;
        if (isNumber) text = data2.toFixed(2);

        for (var _i82 in condition) {
            drawData.Value[_i82] = null;
            if (!condition[_i82]) continue;
            if (_i82 >= data.length || !this.IsNumber(data[_i82])) continue;

            if (isNumber) {
                drawData.Value[_i82] = data[_i82];
                drawData.Text[_i82] = text;
            } else {
                if (_i82 >= data2.length || !data2[_i82]) continue;
                drawData.Value[_i82] = data[_i82];
                if (typeof data2[_i82] == 'number') drawData.Text[_i82] = data2[_i82].toFixed(2);else drawData.Text[_i82] = data2[_i82].toString();
            }
        }

        return result;
    };

    /*
    在图形上绘制小图标.
    用法:
    DRAWICON(COND,PRICE,TYPE),当COND条件满足时,在PRICE位置画TYPE号图标(TYPE为1--41).
    例如:
    DRAWICON(CLOSE>OPEN,LOW,1)表示当收阳时在最低价位置画1号图标.
    */
    this.DRAWICON = function (condition, data, type) {
        var icon = g_JSComplierResource.GetDrawIcon(type);

        if (!icon) {
            //图标对应的字符代码
            var mapIcon = new Map([[1, { Symbol: '↑', Color: 'rgb(238,44,44)' }], [2, { Symbol: '↓', Color: 'rgb(0,139,69)' }], [3, { Symbol: '😧' }], [4, { Symbol: '😨' }], [5, { Symbol: '😁' }], [6, { Symbol: '😱' }], [7, { Symbol: 'B', Color: 'rgb(238,44,44)' }], [8, { Symbol: 'S', Color: 'rgb(0,139,69)' }], [9, { Symbol: '💰' }], [10, { Symbol: '📪' }], [11, { Symbol: '👆' }], [12, { Symbol: '👇' }], [13, { Symbol: 'B', Color: 'rgb(178,34,34)' }], [14, { Symbol: 'S', Color: 'rgb(0,139,69)' }], [36, { Symbol: 'Χ', Color: 'rgb(238,44,44)' }], [37, { Symbol: 'X', Color: 'rgb(0,139,69)' }], [38, { Symbol: '▲', Color: 'rgb(238,44,44)' }], [39, { Symbol: '▼', Color: 'rgb(0,139,69)' }], [40, { Symbol: '◉', Color: 'rgb(238,44,44)' }], [41, { Symbol: '◈', Color: 'rgb(238,44,44)' }], [42, { Symbol: '📌' }], [43, { Symbol: '💎' }], [44, { Symbol: '🥇' }], [45, { Symbol: '🥈' }], [46, { Symbol: '🥉' }], [47, { Symbol: '🏅' }]]);

            icon = mapIcon.get(type);
        }

        if (!icon) icon = { Symbol: '🚩' };
        var drawData = [];
        var result = { DrawData: drawData, DrawType: 'DRAWICON', Icon: icon };
        if (condition.length <= 0) return result;

        var IsNumber = typeof data == "number";
        if (typeof condition == 'number') {
            if (!condition) return result;

            for (var i = 0; i < this.SymbolData.Data.Data.length; ++i) {
                if (IsNumber) {
                    drawData[i] = data;
                } else {
                    if (i < data.length && this.IsNumber(data[i])) drawData[i] = data[i];else drawData[i] = null;
                }
            }
            return result;
        }

        for (var i in condition) {
            drawData[i] = null;

            if (!condition[i]) continue;

            if (IsNumber) {
                drawData[i] = data;
            } else {
                if (this.IsNumber(data[i])) drawData[i] = data[i];
            }
        }

        return result;
    };

    /*
    绘制通道
    condition:条件
    data,data2:通道顶部和底部
    borderColor: 通道顶部和底部线段颜色RGB(24,30,40) 不填就不画
    borderWidth: 通道顶部和底部线段宽度
    areaColor: 通道面积颜色 RGB(200,30,44) 不填使用默认颜色
    dotted: 通道顶部和底部虚线设置 '3,4' , 不填默认 3,3
    */
    this.DRAWCHANNEL = function (condition, data, data2, borderColor, borderWidth, dotted, areaColor) {
        var drawData = [];
        var result = { DrawData: drawData, DrawType: 'DRAWCHANNEL', Border: {} };
        if (condition.length <= 0) return result;

        if (borderColor) result.Border.Color = borderColor;
        if (areaColor) result.AreaColor = areaColor;
        if (borderWidth > 0) result.Border.Width = borderWidth;
        if (dotted) {
            var ary = dotted.split(',');
            result.Border.Dotted = [];
            for (var i in ary) {
                var item = ary[i];
                if (!item) continue;
                var value = parseInt(ary[i]);
                if (value <= 0) continue;
                result.Border.Dotted.push(value);
            }
            if (result.Border.Dotted.length <= 0) result.Border.Dotted = null;
        }

        var IsNumber = typeof data == "number";
        var IsNumber2 = typeof data2 == "number";
        if (typeof condition == 'number') {
            if (!condition) return result; //条件是否

            for (var i = 0; i < this.SymbolData.Data.Data.length; ++i) {
                drawData[i] = null;

                if (IsNumber && IsNumber2) {
                    drawData[i] = { Value: data, Value2: data2 };
                } else if (IsNumber && !IsNumber2) {
                    if (isNaN(data2[i])) continue;
                    drawData[i] = { Value: data, Value2: data2[i] };
                } else if (!IsNumber && IsNumber2) {
                    if (isNaN(data[i])) continue;
                    drawData[i] = { Value: data[i], Value2: data2 };
                } else {
                    if (isNaN(data[i]) || isNaN(data2[i])) continue;
                    drawData[i] = { Value: data[i], Value2: data2[i] };
                }
            }
        } else {
            for (var i = 0; i < condition.length; ++i) {
                drawData[i] = null;

                if (isNaN(condition[i]) || !condition[i]) continue;

                if (IsNumber && IsNumber2) {
                    drawData[i] = { Value: data, Value2: data2 };
                } else if (IsNumber && !IsNumber2) {
                    if (isNaN(data2[i])) continue;
                    drawData[i] = { Value: data, Value2: data2[i] };
                } else if (!IsNumber && IsNumber2) {
                    if (isNaN(data[i])) continue;
                    drawData[i] = { Value: data[i], Value2: data2 };
                } else {
                    if (isNaN(data[i]) || isNaN(data2[i])) continue;
                    drawData[i] = { Value: data[i], Value2: data2[i] };
                }
            }
        }

        return result;
    };

    this.RGB = function (r, g, b) {
        var rgb = "RGB(" + r + "," + g + "," + b + ")";
        return rgb;
    };

    this.RGBA = function (r, g, b, a) {
        var rgba = "RGB(" + r + "," + g + "," + b + "," + a + ")";
        return rgba;
    };

    //用法:PARTLINE(PRICE,COND1,COLOR1,COND2,COLOR2...),
    //绘制PRICE线,当COND1条件满足时,用COLOR1颜色,当COND2条件满足时,用COLOR2颜色,否则不绘制,从COLOR1之后的参数均可以省略,最多可以有10组条件
    //例如:PARTLINE(CLOSE,CLOSE>OPEN,RGB(255,0,0),CLOSE<OPEN,RGB(0,255,0),1,RGB(0,0,255))
    //表示画收盘价线,阳线时用红色,阴线时用绿色,平盘用蓝色.注意最后一个条件为1,表示前面都不满足时必然满足这个条件
    this.PARTLINE = function (args) {
        var drawData = [];
        var result = { DrawData: drawData, DrawType: 'PARTLINE' };
        if (args.length < 3) return result;

        var data = args[0];
        var condition = [];
        for (var i = 1; i < args.length && i + 1 < args.length; i += 2) {
            condition.push({ Cond: args[i], RGB: args[i + 1] });
        }

        for (var i in data) {
            var drawItem = { Value: null, RGB: null };
            drawData[i] = drawItem;

            var value = data[i];
            if (!this.IsNumber(value)) continue;

            var rgb = null;
            for (var j in condition) {
                var item = condition[j];
                if (i < item.Cond.length && item.Cond[i]) {
                    rgb = item.RGB;
                    break;
                }
            }

            if (rgb) {
                drawItem.Value = value;
                drawItem.RGB = rgb;
            }
        }

        return result;
    };

    //填充背景.
    //用法: DRAWGBK(COND,COLOR1,COLOR2,colorAngle)  colorAngle=渐近色角度
    //例如: DRAWGBK(O>C,RGB(0,255,0),RGB(255,0,0),0);
    this.DRAWGBK = function (condition, color, color2, colorAngle) {
        var drawData = { Color: [], Angle: colorAngle };
        if (color) drawData.Color.push(color);
        if (color2) drawData.Color.push(color2);

        var result = { DrawData: null, DrawType: 'DRAWGBK' };
        if (Array.isArray(condition)) {
            for (var i in condition) {
                var item = condition[i];
                if (item) {
                    result.DrawData = drawData;
                    break;
                }
            }
        } else {
            if (condition) result.DrawData = drawData;
        }

        return result;
    };

    //画文字 及线段
    this.DRAWTEXT_LINE = function (condition, price, text, textcolor, fontSize, linetype, linecolor) {
        var drawData = { Text: { Title: text, Color: textcolor }, Line: { Type: linetype, Color: linecolor } };
        if (fontSize <= 0) fontSize = 12;
        drawData.Text.Font = fontSize * GetDevicePixelRatio() + 'px 微软雅黑';

        var result = { DrawData: null, DrawType: 'DRAWTEXT_LINE' };

        if (Array.isArray(condition)) {
            for (var i in condition) {
                var item = condition[i];
                if (item) {
                    if (Array.isArray(price)) drawData.Price = price[i];else drawData.Price = price;
                    result.DrawData = drawData;

                    break;
                }
            }
        } else {
            if (condition) {
                if (Array.isArray(price)) drawData.Price = price[0];else drawData.Price = price;
                result.DrawData = drawData;
            }
        }

        return result;
    };

    // 相对位置上画矩形.
    //用法: DRAWRECTREL(LEFT,TOP,RIGHT,BOTTOM,COLOR),以图形窗口(LEFT,TOP)为左上角,(RIGHT,BOTTOM)为右下角绘制矩形,坐标单位是窗口沿水平和垂直方向的1/1000,取值范围是0—999,超出范围则可能显示在图形窗口外,矩形中间填充颜色COLOR,COLOR为0表示不填充.
    //例如: DRAWRECTREL(0,0,500,500,RGB(255,255,0))表示在图形最左上部1/4位置用黄色绘制矩形
    this.DRAWRECTREL = function (left, top, right, bottom, color) {

        var drawData = { Rect: { Left: left, Top: top, Right: right, Bottom: bottom }, Color: color };
        if (color == 0) drawData.Color = null;
        var result = { DrawData: drawData, DrawType: 'DRAWRECTREL' };

        return result;
    };
}

JSDraw.prototype.CalculateDrawLine = function (lineCache) {
    lineCache.List = [];
    for (var _i83 = lineCache.Start.ID; _i83 <= lineCache.End.ID; ++_i83) {
        lineCache.List.push(_i83);
    }var height = Math.abs(lineCache.Start.Value - lineCache.End.Value);
    var width = lineCache.List.length - 1;

    var result = [];
    result.push({ ID: lineCache.Start.ID, Value: lineCache.Start.Value }); //第1个点

    if (lineCache.Start.Value > lineCache.End.Value) {
        for (var _i84 = 1; _i84 < lineCache.List.length - 1; ++_i84) {
            var value = height * (lineCache.List.length - 1 - _i84) / width + lineCache.End.Value;
            result.push({ ID: lineCache.List[_i84], Value: value });
        }
    } else {
        for (var _i85 = 1; _i85 < lineCache.List.length - 1; ++_i85) {
            var value = height * _i85 / width + lineCache.Start.Value;
            result.push({ ID: lineCache.List[_i85], Value: value });
        }
    }

    result.push({ ID: lineCache.End.ID, Value: lineCache.End.Value }); //最后一个点

    return result;
};

//是否有是有效的数字
JSDraw.prototype.IsNumber = function (value) {
    if (value == null) return false;
    if (isNaN(value)) return false;

    return true;
};

JSDraw.prototype.IsDrawFunction = function (name) {
    var setFunctionName = new Set(["STICKLINE", "DRAWTEXT", 'SUPERDRAWTEXT', 'DRAWLINE', 'DRAWBAND', 'DRAWKLINE', 'DRAWKLINE_IF', 'PLOYLINE', 'POLYLINE', 'DRAWNUMBER', 'DRAWICON', 'DRAWCHANNEL', 'PARTLINE', 'DRAWTEXT_FIX', 'DRAWGBK', 'DRAWTEXT_LINE', 'DRAWRECTREL']);
    if (setFunctionName.has(name)) return true;

    return false;
};

//http://www.newone.com.cn/helpcontroller/index?code=zszy_pc
var DYNAINFO_ARGUMENT_ID = {
    YCLOSE: 3,
    OPEN: 4,
    HIGH: 5,
    LOW: 6,
    CLOSE: 7,
    VOL: 8,
    AMOUNT: 10,
    AMPLITUDE: 13, //振幅
    INCREASE: 14, //涨幅
    EXCHANGERATE: 37 //换手率
};

function JSSymbolData(ast, option, jsExecute) {
    this.AST = ast; //语法树
    this.Execute = jsExecute;

    this.Symbol = '600000.sh';
    this.Name;
    this.Data = null; //个股数据
    this.DayData = null; //日线数据
    this.SourceData = null; //不复权的个股数据
    this.MarketValue = null; //总市值
    this.Period = 0; //周期
    this.Right = 0; //复权
    this.DataType = 0; //默认K线数据 2=分钟走势图数据 3=多日分钟走势图
    this.IsBeforeData = false; //当日走势图数据是否包含开盘前数据
    this.DayCount; //多日分时图天数

    this.KLineApiUrl = g_JSComplierResource.Domain + "/API/KLine2"; //日线
    this.MinuteKLineApiUrl = g_JSComplierResource.Domain + '/API/KLine3'; //分钟K线
    this.RealtimeApiUrl = g_JSComplierResource.Domain + '/API/stock'; //实时行情
    this.HistoryMinuteApiUrl = g_JSChartResource.Domain + '/API/StockMinuteData'; //历史分钟数据(多日分时图)
    this.StockHistoryDayApiUrl = g_JSComplierResource.Domain + '/API/StockHistoryDay'; //历史财务数据
    this.StockHistoryDay3ApiUrl = g_JSComplierResource.Domain + '/API/StockHistoryDay3'; //历史财务数据
    this.StockNewsAnalysisApiUrl = g_JSComplierResource.CacheDomain + '/cache/newsanalyze'; //新闻分析数据
    this.HKToSHSZApiUrl = //北上资金
    [g_JSComplierResource.CacheDomain + '/cache/historyday/all/hk2shsz.json', //日线数据
    g_JSComplierResource.CacheDomain + '/cache/analyze/hk2shsz/hk2shsz.json', //最新分钟
    g_JSComplierResource.Domain + '/API/HKToSHSZMinuteData' //多日分时分钟
    ];

    this.MaxRequestDataCount = 1000;
    this.MaxRequestMinuteDayCount = 5;

    this.LatestData; //最新行情
    this.IndexData; //大盘指数
    this.LatestIndexData; //最新大盘数据
    this.FinanceData = new Map(); //财务数据
    this.MarginData = new Map(); //融资融券
    this.HKToSHSZData = new Map(); //北上资金
    this.NewsAnalysisData = new Map(); //新闻统计
    this.ExtendData = new Map(); //其他的扩展数据
    this.UserData = new Map(); //用户数据

    this.SectionFinanceData = new Map(); //截面报告数据
    this.ThrowSFPeirod = new Set(); //重新获取数据

    this.NetworkFilter; //网络请求回调 function(data, callback);


    //使用option初始化
    if (option) {
        if (option.HQDataType) this.DataType = option.HQDataType;
        if (option.Data) {
            this.Data = option.Data;
            if (this.DataType != HQ_DATA_TYPE.MINUTE_ID && this.DataType != HQ_DATA_TYPE.MULTIDAY_MINUTE_ID && this.DataType != HQ_DATA_TYPE.HISTORY_MINUTE_ID) //分钟走势图数据 没有周期和复权
                {
                    this.Period = option.Data.Period; //周期
                    this.Right = option.Data.Right; //复权
                }
            //this.Data=null;
        }

        if (option.SourceData) this.SourceData = option.SourceData;
        if (option.Symbol) this.Symbol = option.Symbol;
        if (option.Name) this.Name = option.Name;
        if (option.MaxRequestDataCount > 0) this.MaxRequestDataCount = option.MaxRequestDataCount;
        if (option.MaxRequestMinuteDayCount > 0) this.MaxRequestMinuteDayCount = option.MaxRequestMinuteDayCount;
        if (option.KLineApiUrl) this.KLineApiUrl = option.KLineApiUrl;
        if (option.Right) this.Right = option.Right;
        if (option.Period) this.Period = option.Period;
        if (option.IsBeforeData === true) this.IsBeforeData = option.IsBeforeData;
        if (option.NetworkFilter) this.NetworkFilter = option.NetworkFilter;
        if (option.DayCount > 0) this.DayCount = option.DayCount;
    }

    this.RecvError = function (request) {
        JSConsole.Complier.Log('[JSSymbolData::RecvError] ajax error.', request.status);
        throw { FunctionName: 'RecvError', Request: request };
    };

    //最新行情
    this.GetLatestData = function () {
        if (this.LatestData) return this.Execute.RunNextJob();

        var self = this;

        if (this.NetworkFilter) {
            var obj = {
                Name: 'JSSymbolData::GetLatestData', //类名::
                Explain: 'DYNAINFO()',
                Request: { Url: self.RealtimeApiUrl, Type: 'POST',
                    Data: { symbol: [this.Symbol], field: ["name", "symbol", "yclose", "open", "price", "high", "low", "vol", "amount", "date", "time", "increase", "exchangerate", "amplitude"] } },
                Self: this,
                PreventDefault: false
            };
            this.NetworkFilter(obj, function (data) {
                self.RecvLatestData(data);
                self.Execute.RunNextJob();
            });

            if (obj.PreventDefault == true) return; //已被上层替换,不调用默认的网络请求
        }

        JSNetwork.HttpRequest({
            url: self.RealtimeApiUrl,
            data: {
                "field": ["name", "symbol", "yclose", "open", "price", "high", "low", "vol", "amount", "date", "time", "increase", "exchangerate", "amplitude"],
                "symbol": [this.Symbol]
            },
            type: "post",
            dataType: "json",
            async: true,
            success: function success(recvData) {
                self.RecvLatestData(recvData);
                self.Execute.RunNextJob();
            },
            error: function error(request) {
                self.RecvError(request);
            }
        });
    };

    this.RecvLatestData = function (data) {
        if (!data.stock || data.stock.length != 1) return;

        var stock = data.stock[0];
        this.LatestData = { Symbol: stock.symbol, Name: stock.name, Date: stock.date, Time: stock.time,
            YClose: stock.yclose, Price: stock.price, Open: stock.open, High: stock.high, Low: stock.low, Vol: stock.vol, Amount: stock.amount,
            Increase: stock.increase, Exchangerate: stock.exchangerate, Amplitude: stock.amplitude };

        JSConsole.Complier.Log('[JSSymbolData::RecvLatestData]', this.LatestData);
    };

    this.GetLatestCacheData = function (dataname) {
        if (!this.LatestData) return null;

        switch (dataname) {
            case DYNAINFO_ARGUMENT_ID.YCLOSE:
                return this.LatestData.YClose;
            case DYNAINFO_ARGUMENT_ID.OPEN:
                return this.LatestData.Open;
            case DYNAINFO_ARGUMENT_ID.HIGH:
                return this.LatestData.High;
            case DYNAINFO_ARGUMENT_ID.LOW:
                return this.LatestData.Low;
            case DYNAINFO_ARGUMENT_ID.VOL:
                return this.LatestData.Vol;
            case DYNAINFO_ARGUMENT_ID.AMOUNT:
                return this.LatestData.Amount;
            case DYNAINFO_ARGUMENT_ID.INCREASE:
                return this.LatestData.Increase;
            case DYNAINFO_ARGUMENT_ID.EXCHANGERATE:
                return this.LatestData.Exchangerate;
            case DYNAINFO_ARGUMENT_ID.AMPLITUDE:
                return this.LatestData.Amplitude;
            case DYNAINFO_ARGUMENT_ID.CLOSE:
                return this.LatestData.Price;
            default:
                return null;
        }
    };

    this.GetLatestIndexData = function () {
        if (this.LatestIndexData) return this.Execute.RunNextJob();

        var self = this;

        if (this.NetworkFilter) {
            var obj = {
                Name: 'JSSymbolData::GetLatestIndexData', //类名:: 函数
                Explain: '最新大盘数据',
                Request: { Url: self.RealtimeApiUrl, Type: 'POST',
                    Data: { symbol: ['000001.sh'], field: ["name", "symbol", "yclose", "open", "price", "high", "low", "vol", "amount", "date", "time", "increase", "amplitude"] } },
                Self: this,
                PreventDefault: false
            };
            this.NetworkFilter(obj, function (data) {
                self.RecvLatestIndexData(data);
                self.Execute.RunNextJob();
            });

            if (obj.PreventDefault == true) return; //已被上层替换,不调用默认的网络请求
        }

        JSNetwork.HttpRequest({
            url: self.RealtimeApiUrl,
            data: {
                "field": ["name", "symbol", "yclose", "open", "price", "high", "low", "vol", "amount", "date", "time", "increase", "amplitude"],
                "symbol": ['000001.sh']
            },
            type: "post",
            dataType: "json",
            async: true,
            success: function success(recvData) {
                self.RecvLatestIndexData(recvData);
                self.Execute.RunNextJob();
            },
            error: function error(request) {
                self.RecvError(request);
            }
        });
    };

    this.RecvLatestIndexData = function (data) {
        if (!data.stock || data.stock.length != 1) return;

        var stock = data.stock[0];
        this.LatestIndexData = { Symbol: stock.symbol, Name: stock.name, Date: stock.date, Time: stock.time,
            YClose: stock.yclose, Price: stock.price, Open: stock.open, High: stock.high, Low: stock.low, Vol: stock.vol, Amount: stock.amount,
            Increase: stock.increase, Amplitude: stock.amplitude };

        JSConsole.Complier.Log('[JSSymbolData::RecvLatestIndexData]', this.LatestIndexData);
    };

    this.GetLatestIndexCacheData = function (dataname) {
        if (!this.LatestIndexData) return null;

        switch (dataname) {
            case 'FROMOPEN':
                {
                    var value = parseInt(this.LatestIndexData.Time / 100);
                    var time = parseInt(value / 100) * 60 + value % 100; //转化为分钟个数
                    var index = 0;
                    if (time <= 9 * 60 + 25) index = 0;else if (time < 9 * 60 + 30) index = 1;else if (time <= 11 * 60 + 30) index = time - (9 * 60 + 30) + 1;else if (time < 13 * 60) index = 121;else if (time <= 15 * 60) index = 122 + time - 13 * 60;else index = 242;
                    return index;
                }
            default:
                return null;
        }
    };

    this.GetVolRateData = function (job, node) {
        var volrKey = job.ID.toString() + '-VolRate-' + this.Symbol;
        if (this.ExtendData.has(volrKey)) return this.Execute.RunNextJob();

        var self = this;
        JSNetwork.HttpRequest({
            url: self.RealtimeApiUrl,
            data: {
                "field": ["name", "symbol", "avgvol5", 'date'],
                "symbol": [this.Symbol]
            },
            type: "post",
            dataType: "json",
            async: true,
            success: function success(recvData) {
                self.RecvVolRateData(recvData, volrKey);
                self.Execute.RunNextJob();
            },
            error: function error(request) {
                self.RecvError(request);
            }
        });
    };

    this.RecvVolRateData = function (data, key) {
        if (!data.stock || data.stock.length != 1) return;
        var avgVol5 = data.stock[0].avgvol5;
        var date = data.stock[0].date;
        var item = { AvgVol5: avgVol5, Date: date };
        this.ExtendData.set(key, item);

        JSConsole.Complier.Log('[JSSymbolData::RecvVolRateData]', item);
    };

    this.GetVolRateCacheData = function (node) {
        var key = JS_EXECUTE_JOB_ID.JOB_DOWNLOAD_VOLR_DATA.toString() + '-VolRate-' + this.Symbol;
        if (!key || !this.ExtendData.has(key)) this.Execute.ThrowUnexpectedNode(node, '不支持VOLR');

        var result = [];
        var value = this.ExtendData.get(key);
        var avgVol5 = value.AvgVol5 / 241;
        var totalVol = 0;
        //5日成交总量只取了最新一天的,历史的暂时没有取,所以数据计算的时候只计算最新的一天, 其他都空
        for (var i = 0, j = 0; i < this.Data.Data.length; ++i) {
            result[i] = null;
            var item = this.Data.Data[i];
            var dateTime = item.DateTime; //日期加时间
            if (!dateTime) continue;
            var aryValue = dateTime.split(' ');
            if (aryValue.length != 2) continue;
            var date = parseInt(aryValue[0]);
            if (date != value.Date) continue;

            totalVol += item.Vol;
            if (avgVol5 > 0) result[i] = totalVol / (j + 1) / avgVol5 * 100;
            ++j;
        }

        return result;
    };

    this.GetGroupData = function (job) {
        var key = job.ID.toString() + '-Group-' + this.Symbol;
        if (this.ExtendData.has(key)) return this.Execute.RunNextJob();

        var self = this;

        if (this.NetworkFilter) {
            var obj = {
                Name: 'JSSymbolData::GetGroupData', //类名::
                Explain: 'HYBLOCK|DYBLOCK',
                Request: { Url: self.RealtimeApiUrl, Type: 'POST',
                    Data: { symbol: [this.Symbol], field: ["name", "symbol", "industry", 'concept', 'region'] } },
                Self: this,
                PreventDefault: false
            };
            this.NetworkFilter(obj, function (data) {
                self.RecvGroupData(data, key);
                self.Execute.RunNextJob();
            });

            if (obj.PreventDefault == true) return; //已被上层替换,不调用默认的网络请求
        }

        JSNetwork.HttpRequest({
            url: self.RealtimeApiUrl,
            data: {
                "field": ["name", "symbol", "industry", 'concept', 'region'],
                "symbol": [this.Symbol]
            },
            type: "post",
            dataType: "json",
            async: true,
            success: function success(recvData) {
                self.RecvGroupData(recvData, key);
                self.Execute.RunNextJob();
            },
            error: function error(request) {
                self.RecvError(request);
            }
        });
    };

    this.RecvGroupData = function (data, key) {
        if (!data.stock) return;
        if (data.stock.length != 1) return;
        var stock = data.stock[0];
        var industry = stock.industry;
        var concept = stock.concept;
        var region = stock.region;

        var groupData = { Industry: [], Concept: [], Region: [] };
        if (industry) {
            for (var i in industry) {
                var item = industry[i];
                groupData.Industry.push({ Name: item.name, Symbol: item.symbol });
            }
        }

        if (concept) {
            for (var i in concept) {
                var item = concept[i];
                groupData.Concept.push({ Name: item.name, Symbol: item.symbol });
            }
        }

        if (region) {
            for (var i in region) {
                var item = region[i];
                groupData.Region.push({ Name: item.name, Symbol: item.symbol });
            }
        }

        this.ExtendData.set(key, groupData);
    };

    //获取大盘指数数据
    this.GetIndexData = function () {
        if (this.IndexData) return this.Execute.RunNextJob();

        var self = this;
        if (ChartData.IsDayPeriod(this.Period, true)) //请求日线数据 9=季线
            {
                if (this.NetworkFilter) {
                    var obj = {
                        Name: 'JSSymbolData::GetIndexData', //类名::
                        Explain: '大盘数据',
                        Period: self.Period,
                        Request: { Url: self.KLineApiUrl, Type: 'POST',
                            Data: { field: ["name", "symbol", "yclose", "open", "price", "high", "low", "vol", 'up', 'down', 'stop', 'unchanged'],
                                symbol: '000001.sh', start: -1, count: self.MaxRequestDataCount + 500 } },
                        Self: this,
                        PreventDefault: false
                    };
                    this.NetworkFilter(obj, function (data) {
                        self.RecvIndexHistroyData(data);
                        self.Execute.RunNextJob();
                    });

                    if (obj.PreventDefault == true) return; //已被上层替换,不调用默认的网络请求
                }

                JSNetwork.HttpRequest({
                    url: self.KLineApiUrl,
                    data: {
                        "field": ["name", "symbol", "yclose", "open", "price", "high", "low", "vol", 'up', 'down', 'stop', 'unchanged'],
                        "symbol": '000001.sh',
                        "start": -1,
                        "count": self.MaxRequestDataCount + 500 //多请求2年的数据 确保股票剔除停牌日期以后可以对上
                    },
                    type: "post",
                    dataType: "json",
                    async: true,
                    success: function success(recvData) {
                        self.RecvIndexHistroyData(recvData);
                        self.Execute.RunNextJob();
                    },
                    error: function error(request) {
                        self.RecvError(request);
                    }
                });
            } else if (ChartData.IsMinutePeriod(this.Period, true)) //请求分钟数据
            {
                if (this.NetworkFilter) {
                    var obj = {
                        Name: 'JSSymbolData::GetIndexData', //类名::
                        Explain: '大盘数据',
                        Period: self.Period,
                        Request: { Url: self.MinuteKLineApiUrl, Type: 'POST',
                            Data: { field: ["name", "symbol", "yclose", "open", "price", "high", "low", "vol"],
                                symbol: '000001.sh', start: -1, count: self.MaxRequestDataCount + 5 } },
                        Self: this,
                        PreventDefault: false
                    };
                    this.NetworkFilter(obj, function (data) {
                        self.RecvIndexMinuteHistroyData(data);
                        self.Execute.RunNextJob();
                    });

                    if (obj.PreventDefault == true) return; //已被上层替换,不调用默认的网络请求
                }

                JSNetwork.HttpRequest({
                    url: self.MinuteKLineApiUrl,
                    data: {
                        "field": ["name", "symbol", "yclose", "open", "price", "high", "low", "vol"],
                        "symbol": '000001.sh',
                        "start": -1,
                        "count": self.MaxRequestMinuteDayCount + 5
                    },
                    type: "post",
                    dataType: "json",
                    async: true,
                    success: function success(data) {
                        self.RecvIndexMinuteHistroyData(data);
                        self.Execute.RunNextJob();
                    },
                    error: function error(request) {
                        self.RecvError(request);
                    }
                });
            }
    };

    this.RecvIndexHistroyData = function (data) {
        JSConsole.Complier.Log('[JSSymbolData::RecvIndexHistroyData] recv data', data);

        var hisData = this.JsonDataToHistoryData(data);
        this.IndexData = new ChartData();
        this.IndexData.DataType = 0; /*日线数据 */
        this.IndexData.Data = hisData;

        var aryOverlayData = this.SourceData.GetOverlayData(this.IndexData.Data); //和主图数据拟合以后的数据
        this.IndexData.Data = aryOverlayData;

        if (ChartData.IsDayPeriod(this.Period, false)) //周期数据
            {
                var periodData = this.IndexData.GetPeriodData(this.Period);
                this.IndexData.Data = periodData;
            }
    };

    this.RecvIndexMinuteHistroyData = function (data) {
        JSConsole.Complier.Log('[JSSymbolData::RecvIndexMinuteHistroyData] recv data', data);

        var hisData = this.JsonDataToMinuteHistoryData(data);
        this.IndexData = new ChartData();
        this.IndexData.DataType = 1; /*分钟线数据 */
        this.IndexData.Data = hisData;

        if (ChartData.IsMinutePeriod(this.Period, false)) //周期数据
            {
                var periodData = this.IndexData.GetPeriodData(this.Period);
                this.IndexData.Data = periodData;
            }
    };

    //获取大盘指数缓存数据
    this.GetIndexCacheData = function (dataName) {
        if (!this.IndexData) return new Array();

        switch (dataName) {
            case 'INDEXA':
                return this.IndexData.GetAmount();
            case 'INDEXC':
                return this.IndexData.GetClose();
            case 'INDEXH':
                return this.IndexData.GetHigh();
            case 'INDEXL':
                return this.IndexData.GetLow();
            case 'INDEXO':
                return this.IndexData.GetOpen();
            case 'INDEXV':
                return this.IndexData.GetVol();
            case 'INDEXADV':
                return this.IndexData.GetUp();
            case 'INDEXDEC':
                return this.IndexData.GetDown();
        }
    };

    //分钟涨幅股票个数统计数据下载
    this.GetIndexIncreaseData = function (job) {
        var upKey = job.ID.toString() + '-UpCount-' + job.Symbol;
        var downKey = job.ID.toString() + '-DownCount-' + job.Symbol;
        if (this.ExtendData.has(upKey) && this.ExtendData.has(downKey)) return this.Execute.RunNextJob();

        var symbol = job.Symbol;
        symbol = symbol.replace('.CI', '.ci');
        var self = this;
        if (this.DataType === HQ_DATA_TYPE.MINUTE_ID || this.DataType === HQ_DATA_TYPE.MULTIDAY_MINUTE_ID) //走势图数据
            {
                var apiUrl = g_JSComplierResource.CacheDomain + '/cache/analyze/increaseanalyze/' + symbol + '.json';
                JSConsole.Complier.Log('[JSSymbolData::GetIndexIncreaseData] minute Get url=', apiUrl);
                JSNetwork.HttpRequest({
                    url: apiUrl,
                    type: "get",
                    dataType: "json",
                    async: true,
                    success: function success(data) {
                        self.RecvMinuteIncreaseData(data, { UpKey: upKey, DownKey: downKey });
                        self.Execute.RunNextJob();
                    },
                    error: function error(request) {
                        self.RecvError(request);
                    }
                });
            } else if (this.DataType === HQ_DATA_TYPE.KLINE_ID && this.Period === 0) //K线图 日线
            {
                JSConsole.Complier.Log('[JSSymbolData::GetIndexIncreaseData] K day Get url=', self.KLineApiUrl);
                JSNetwork.HttpRequest({
                    url: self.KLineApiUrl,
                    data: {
                        "symbol": symbol,
                        "start": -1,
                        "count": self.MaxRequestDataCount
                    },
                    type: "post",
                    dataType: "json",
                    async: true,
                    success: function success(data) {
                        self.RecvHistoryIncreaseData(data, { UpKey: upKey, DownKey: downKey });
                        self.Execute.RunNextJob();
                    },
                    error: function error(request) {
                        self.RecvError(request);
                    }
                });
            }
    };

    this.RecvHistoryIncreaseData = function (data, key) {
        JSConsole.Complier.Log('[JSSymbolData::RecvHistoryIncreaseData] recv data', data);

        var upData = [],
            downData = [];
        for (var i in data.data) {
            var item = data.data[i];
            var upItem = new SingleData();
            var downItem = new SingleData();
            upItem.Date = item[0];
            upItem.Value = item[8];
            upData[i] = upItem;
            downItem.Date = item[0];
            downItem.Value = item[9];
            downData[i] = downItem;
        }

        var aryFixedData = this.Data.GetFittingData(upData);
        var bindData = new ChartData();
        bindData.Data = aryFixedData;
        bindData.Period = this.Period; //只支持日线
        var data = bindData.GetValue();
        this.ExtendData.set(key.UpKey, data);

        aryFixedData = this.Data.GetFittingData(downData);
        bindData = new ChartData();
        bindData.Data = aryFixedData;
        bindData.Period = this.Period; //只支持日线
        data = bindData.GetValue();
        this.ExtendData.set(key.DownKey, data);
    };

    this.RecvMinuteIncreaseData = function (data, key) {
        JSConsole.Complier.Log('[JSSymbolData::RecvMinuteIncreaseData] recv data', data);
        if (!data.minute) return;
        var minuteData = data.minute;
        if (!minuteData.time || !minuteData.up || !minuteData.down) return;
        var upData = [],
            downData = [];

        if (this.IsBeforeData) {
            for (var i = 0, j = 0; i < this.Data.Data.length; ++i) {
                upData[i] = null;
                downData[i] = null;
                var item = this.Data.Data[i];
                if (item.Before) continue; //盘前数据
                var dateTime = item.DateTime; //日期加时间
                if (!dateTime) continue;
                var aryValue = dateTime.split(' ');
                if (aryValue.length != 2) continue;
                var date = parseInt(aryValue[0]);
                if (date != data.date) continue;

                upData[i] = minuteData.up[j];
                downData[i] = minuteData.down[j];
                ++j;
            }
        } else {
            for (var i = 0, j = 0; i < this.Data.Data.length; ++i) {
                upData[i] = null;
                downData[i] = null;
                var item = this.Data.Data[i];
                var dateTime = item.DateTime; //日期加时间
                if (!dateTime) continue;
                var aryValue = dateTime.split(' ');
                if (aryValue.length != 2) continue;
                var date = parseInt(aryValue[0]);
                if (date != data.date) continue;

                upData[i] = minuteData.up[j];
                downData[i] = minuteData.down[j];
                ++j;
            }
        }

        this.ExtendData.set(key.UpKey, upData);
        this.ExtendData.set(key.DownKey, downData);
    };

    //分钟涨幅股票个数统计数据
    this.GetIndexIncreaseCacheData = function (funcName, symbol, node) {
        var key;
        if (funcName == 'UPCOUNT') key = JS_EXECUTE_JOB_ID.JOB_DOWNLOAD_INDEX_INCREASE_DATA.toString() + '-UpCount-' + symbol;else if (funcName == 'DOWNCOUNT') key = JS_EXECUTE_JOB_ID.JOB_DOWNLOAD_INDEX_INCREASE_DATA.toString() + '-DownCount-' + symbol;

        if (!key || !this.ExtendData.has(key)) this.Execute.ThrowUnexpectedNode(node, '不支持函数' + funcName + '(' + symbol + ')');

        return this.ExtendData.get(key);
    };

    this.GetSymbolData = function () {
        if (this.Data) return this.Execute.RunNextJob();

        var self = this;

        if (this.DataType === HQ_DATA_TYPE.MINUTE_ID) //当天分钟数据
            {
                JSNetwork.HttpRequest({
                    url: self.RealtimeApiUrl,
                    data: {
                        "field": ["name", "symbol", "yclose", "open", "price", "high", "low", "vol", "amount", "date", "minute", "time", "minutecount"],
                        "symbol": [self.Symbol],
                        "start": -1
                    },
                    type: "post",
                    dataType: "json",
                    async: true,
                    success: function success(recvData) {
                        self.RecvMinuteData(recvData);
                        self.Execute.RunNextJob();
                    }
                });
                return;
            }

        if (this.DataType === HQ_DATA_TYPE.MULTIDAY_MINUTE_ID) {
            JSNetwork.HttpRequest({
                url: self.HistoryMinuteApiUrl,
                data: {
                    "symbol": self.Symbol,
                    "daycount": self.DayCount
                },
                type: "post",
                dataType: "json",
                async: true,
                success: function success(recvData) {
                    self.RecvMultiDayMinuteData(recvData);
                    self.Execute.RunNextJob();
                }
            });
            return;
        }

        if (ChartData.IsDayPeriod(this.Period, true)) //请求日线数据
            {
                JSNetwork.HttpRequest({
                    url: self.KLineApiUrl,
                    data: {
                        "field": ["name", "symbol", "yclose", "open", "price", "high", "low", "vol"],
                        "symbol": self.Symbol,
                        "start": -1,
                        "count": self.MaxRequestDataCount
                    },
                    type: "post",
                    dataType: "json",
                    async: true,
                    success: function success(recvData) {
                        self.RecvHistroyData(recvData);
                        self.Execute.RunNextJob();
                    },
                    error: function error(request) {
                        self.RecvError(request);
                    }
                });
            } else if (ChartData.IsMinutePeriod(this.Period, true)) //请求分钟数据
            {
                JSNetwork.HttpRequest({
                    url: this.MinuteKLineApiUrl,
                    data: {
                        "field": ["name", "symbol", "yclose", "open", "price", "high", "low", "vol"],
                        "symbol": self.Symbol,
                        "start": -1,
                        "count": self.MaxRequestMinuteDayCount
                    },
                    type: "post",
                    dataType: "json",
                    async: true,
                    success: function success(data) {
                        self.RecvMinuteHistroyData(data);
                        self.Execute.RunNextJob();
                    },
                    error: function error(request) {
                        self.RecvError(request);
                    }
                });
            }
    };

    this.RecvHistroyData = function (data) {
        JSConsole.Complier.Log('[JSSymbolData::RecvHistroyData] recv data', data);

        var hisData = this.JsonDataToHistoryData(data);
        this.Data = new ChartData();
        this.Data.DataType = 0; /*日线数据 */
        this.Data.Data = hisData;
        this.SourceData = new ChartData();
        this.SourceData.Data = hisData;

        if (this.Right > 0) //复权
            {
                var rightData = this.Data.GetRightDate(this.Right);
                this.Data.Data = rightData;
            }

        if (ChartData.IsDayPeriod(this.Period, false)) //周期数据
            {
                var periodData = this.Data.GetPeriodData(this.Period);
                this.Data.Data = periodData;
            }

        this.Data.Right = this.Right;
        this.Data.Period = this.Period;
        this.Name = data.name;
    };

    this.RecvMinuteHistroyData = function (data) {
        JSConsole.Complier.Log('[JSSymbolData::RecvMinuteHistroyData] recv data', data);

        var hisData = this.JsonDataToMinuteHistoryData(data);
        this.Data = new ChartData();
        this.Data.DataType = 1; /*分钟线数据 */
        this.Data.Data = hisData;
        this.SourceData = new ChartData();
        this.SourceData.Data = hisData;

        if (ChartData.IsMinutePeriod(this.Period, false)) //周期数据
            {
                var periodData = this.Data.GetPeriodData(this.Period);
                this.Data.Data = periodData;
            }

        this.Data.Period = this.Period;
        this.Name = data.name;
    };

    //最新的分钟数据走势图
    this.RecvMinuteData = function (data) {
        JSConsole.Complier.Log('[JSSymbolData::RecvMinuteData] recv data', data);

        var aryMinuteData = this.JsonDataToMinuteData(data);
        this.Data = new ChartData();
        this.Data.DataType = 2; /*分钟走势图数据 */
        this.Data.Data = aryMinuteData;

        this.Name = data.stock[0].name;
    };

    this.RecvMultiDayMinuteData = function (data) {
        var aryMinuteData = this.JsonDataToMultiDayMinuteData(data);
        this.Data = new ChartData();
        this.Data.DataType = 2; /*分钟走势图数据 */
        this.Data.Data = aryMinuteData;

        this.Name = data.name;
    };

    this.GetSymbolCacheData = function (dataName) {
        if (!this.Data) return new Array();

        switch (dataName) {
            case 'CLOSE':
            case 'C':
                return this.Data.GetClose();
            case 'VOL':
            case 'V':
                return this.Data.GetVol();
            case 'OPEN':
            case 'O':
                return this.Data.GetOpen();
            case 'HIGH':
            case 'H':
                return this.Data.GetHigh();
            case 'LOW':
            case 'L':
                return this.Data.GetLow();
            case 'AMOUNT':
                return this.Data.GetAmount();
        }
    };

    this.GetSymbolDayData = function () {
        //分钟周期才需要下载日线
        if (this.DataType != HQ_DATA_TYPE.KLINE_ID || !ChartData.IsMinutePeriod(this.Period, true) || this.DayData) return this.Execute.RunNextJob();

        var self = this;
        JSNetwork.HttpRequest({
            url: self.KLineApiUrl,
            data: {
                "field": ["name", "symbol", "yclose", "open", "price", "high", "low", "vol"],
                "symbol": self.Symbol,
                "start": -1,
                "count": self.MaxRequestDataCount
            },
            type: "post",
            dataType: "json",
            async: true,
            success: function success(recvData) {
                self.RecvSymbolDayData(recvData);
                self.Execute.RunNextJob();
            },
            error: function error(request) {
                self.RecvError(request);
            }
        });
    };

    this.RecvSymbolDayData = function (data) {
        JSConsole.Complier.Log('[JSSymbolData::RecvSymbolDayData] recv data', data);

        var hisData = this.JsonDataToHistoryData(data);
        var dayData = new ChartData();
        dayData.DataType = 0; /*日线数据 */
        dayData.Data = hisData; //保存原始数据 不复权

        dayData.Right = this.Right;
        dayData.Period = this.Period;

        this.DayData = dayData;
    };

    this.GetCurrBarsCount = function () {
        var result = [];
        if (!this.Data || !this.Data.Data || !this.Data.Data.length) return result;

        var lCount = this.Data.Data.length;
        for (var _i86 = lCount - 1; _i86 >= 0; --_i86) {
            result.push(_i86);
        }return result;
    };

    this.GetIsLastBar = function () {
        var result = [];
        if (!this.Data || !this.Data.Data || !this.Data.Data.length) return result;

        var lCount = this.Data.Data.length;
        for (var _i87 = 0; _i87 < lCount; ++_i87) {
            if (_i87 == lCount - 1) result.push(1);else result.push(0);
        }

        return result;
    };

    //财务函数
    this.GetFinanceCacheData = function (id, node) {
        var jobID = JS_EXECUTE_JOB_ID.GetFinnanceJobID(id);
        if (!jobID) this.Execute.ThrowUnexpectedNode(node, '不支持FINANCE(' + id + ')');
        if (this.FinanceData.has(jobID)) return this.FinanceData.get(jobID);

        return [];
    };

    this.GetCompanyReleaseDate = function (jobID) {
        if (this.FinanceData.has(jobID)) return this.Execute.RunNextJob();

        var self = this;
        JSNetwork.HttpRequest({
            url: self.RealtimeApiUrl,
            data: {
                "field": ["name", "symbol", "company.releasedate"], //公司上司日期
                "symbol": [this.Symbol]
            },
            type: "post",
            dataType: "json",
            async: true,
            success: function success(recvData) {
                self.RecvCompanyReleaseDate(jobID, recvData);
                self.Execute.RunNextJob();
            },
            error: function error(request) {
                self.RecvError(request);
            }
        });
    };

    this.RecvCompanyReleaseDate = function (jobID, data) {
        if (!data.stock || data.stock.length != 1) return;
        var value = data.stock[0].company.releasedate;
        var releaseDate = new Date(value / 10000, value % 10000 / 100, value % 100);

        var aryData = [];
        for (var _i88 in this.Data.Data) {
            value = this.Data.Data[_i88].Date;
            var _date = new Date(value / 10000, value % 10000 / 100, value % 100);
            var _item7 = new SingleData();
            _item7.Date = value;
            _item7.Value = parseInt((_date - releaseDate) / (1000 * 60 * 60 * 24));
            aryData[_i88] = _item7;
        }

        JSConsole.Complier.Log('[JSSymbolData::RecvCompanyReleaseDate] jobID=', jobID, aryData);

        var bindData = new ChartData();
        bindData.Data = aryData;
        bindData.Period = this.Period; //周期

        if (bindData.Period > 0) //周期数据
            {
                var periodData = bindData.GetPeriodSingleData(bindData.Period);
                bindData.Data = periodData;
            }

        var stockData = bindData.GetValue();
        this.FinanceData.set(jobID, stockData);
    };

    //下载财务数据
    this.GetFinanceData = function (jobID) {
        if (this.FinanceData.has(jobID)) return this.Execute.RunNextJob();

        JSConsole.Complier.Log('[JSSymbolData::GetFinanceData] jobID=', jobID);
        var self = this;
        var fieldList = ["name", "date", "symbol"];

        switch (jobID) {
            case JS_EXECUTE_JOB_ID.JOB_DOWNLOAD_FLOW_EQUITY_DATA: //流通股本（万股）
            case JS_EXECUTE_JOB_ID.JOB_DOWNLOAD_CAPITAL_DATA: //流通股本（手）
            case JS_EXECUTE_JOB_ID.JOB_DOWNLOAD_FLOW_MARKETVALUE_DATA: //流通市值
            case JS_EXECUTE_JOB_ID.JOB_DOWNLOAD_EXCHANGE_DATA:
                //换手率
                fieldList.push("capital.a");
                break;
            case JS_EXECUTE_JOB_ID.JOB_DOWNLOAD_TOTAL_EQUITY_DATA: //总股本（万股）
            case JS_EXECUTE_JOB_ID.JOB_DOWNLOAD_MARKETVALUE_DATA:
                //总市值
                fieldList.push('capital.total');
                break;
            case JS_EXECUTE_JOB_ID.JOB_DOWNLOAD_PER_U_PROFIT_DATA:
                fieldList.push('finance.peruprofit');
                break;
            case JS_EXECUTE_JOB_ID.JOB_DOWNLOAD_PER_NETASSET_DATA:
                fieldList.push('finance.pernetasset');
                break;
            case JS_EXECUTE_JOB_ID.JOB_DOWNLOAD_PER_C_RESERVE_DATA:
                fieldList.push('finance.percreserve');
                break;
            case JS_EXECUTE_JOB_ID.JOB_DOWNLOAD_PER_S_EARNING_DATA:
            case JS_EXECUTE_JOB_ID.JOB_DOWNLOAD_PER_S_EARNING2_DATA:
                fieldList.push('finance.persearning');
                break;
            case JS_EXECUTE_JOB_ID.JOB_DOWNLOAD_N_PROFIT_DATA:
                fieldList.push('finance.nprofit');
                break;
            case JS_EXECUTE_JOB_ID.JOB_DOWNLOAD_AL_RATIO_DATA:
                fieldList.push('finance.alratio');
                break;
            case JS_EXECUTE_JOB_ID.JOB_DOWNLOAD_PROFIT_YOY_DATA:
                fieldList.push('finance.profityoy');
                break;
            case JS_EXECUTE_JOB_ID.JOB_DOWNLOAD_DIVIDEND_YIELD_DATA:
                //过去4个季度现金分红总额
                fieldList.push('execdividend.quarter4');
                break;
            case JS_EXECUTE_JOB_ID.JOB_DOWNLOAD_SHAREHOLDER_DATA:
                fieldList.push('shareholder'); //股东信息
                break;
        }

        if (this.NetworkFilter) {
            var obj = {
                Name: 'JSSymbolData::GetFinanceData', //类名::
                Explain: '财务数据',
                JobID: jobID,
                Request: { Url: self.StockHistoryDayApiUrl, Type: 'POST',
                    Data: { field: fieldList, symbol: [this.Symbol], orderfield: "date" } },
                Self: this,
                PreventDefault: false
            };
            this.NetworkFilter(obj, function (data) {
                self.RecvFinanceData(data, jobID);
                self.Execute.RunNextJob();
            });

            if (obj.PreventDefault == true) return; //已被上层替换,不调用默认的网络请求
        }

        //请求数据
        JSNetwork.HttpRequest({
            url: this.StockHistoryDayApiUrl,
            data: {
                "field": fieldList,
                "symbol": [this.Symbol],
                "orderfield": "date"
            },
            type: "post",
            dataType: "json",
            async: true,
            success: function success(recvData) {
                self.RecvFinanceData(recvData, jobID);
                self.Execute.RunNextJob();
            }
        });
    };

    this.RecvFinanceData = function (recvData, jobID) {
        switch (jobID) {
            case JS_EXECUTE_JOB_ID.JOB_DOWNLOAD_FLOW_EQUITY_DATA:
            case JS_EXECUTE_JOB_ID.JOB_DOWNLOAD_CAPITAL_DATA:
            case JS_EXECUTE_JOB_ID.JOB_DOWNLOAD_TOTAL_EQUITY_DATA:
            case JS_EXECUTE_JOB_ID.JOB_DOWNLOAD_PER_U_PROFIT_DATA:
            case JS_EXECUTE_JOB_ID.JOB_DOWNLOAD_PER_NETASSET_DATA:
            case JS_EXECUTE_JOB_ID.JOB_DOWNLOAD_PER_C_RESERVE_DATA:
            case JS_EXECUTE_JOB_ID.JOB_DOWNLOAD_PER_S_EARNING_DATA:
            case JS_EXECUTE_JOB_ID.JOB_DOWNLOAD_PER_S_EARNING2_DATA:
            case JS_EXECUTE_JOB_ID.JOB_DOWNLOAD_N_PROFIT_DATA:
            case JS_EXECUTE_JOB_ID.JOB_DOWNLOAD_FLOW_MARKETVALUE_DATA:
            case JS_EXECUTE_JOB_ID.JOB_DOWNLOAD_MARKETVALUE_DATA:
            case JS_EXECUTE_JOB_ID.JOB_DOWNLOAD_EXCHANGE_DATA:
            case JS_EXECUTE_JOB_ID.JOB_DOWNLOAD_AL_RATIO_DATA:
            case JS_EXECUTE_JOB_ID.JOB_DOWNLOAD_PROFIT_YOY_DATA:
            case JS_EXECUTE_JOB_ID.JOB_DOWNLOAD_DIVIDEND_YIELD_DATA:
            case JS_EXECUTE_JOB_ID.JOB_DOWNLOAD_SHAREHOLDER_DATA:
                return this.RecvStockDayData(recvData, jobID);
        }
    };

    this.RecvStockDayData = function (recvData, jobID) {
        //JSConsole.Complier.Log(recvData);
        if (!recvData.stock || recvData.stock.length != 1) return;

        var stock = recvData.stock[0];
        var bFinanceData = false; //是否是定期的财务数据
        var bMarketValue = false; //是否计算市值
        var aryData = new Array();
        for (var _i89 in stock.stockday) {
            var item = stock.stockday[_i89];
            var indexData = new SingleData();
            indexData.Date = item.date;

            switch (jobID) {
                case JS_EXECUTE_JOB_ID.JOB_DOWNLOAD_CAPITAL_DATA:
                    var financeData = item.capital;
                    if (!financeData) continue;
                    if (!this.IsNumber(financeData.a)) continue;
                    indexData.Value = financeData.a / 100; //流通股本（手）
                    bFinanceData = true;
                    break;
                case JS_EXECUTE_JOB_ID.JOB_DOWNLOAD_TOTAL_EQUITY_DATA:
                    var financeData = item.capital;
                    if (!financeData) continue;
                    if (!this.IsNumber(financeData.total)) continue;
                    indexData.Value = financeData.total / 10000; //总股本（万股）
                    bFinanceData = true;
                    break;
                case JS_EXECUTE_JOB_ID.JOB_DOWNLOAD_FLOW_EQUITY_DATA:
                    var financeData = item.capital;
                    if (!financeData) continue;
                    if (!this.IsNumber(financeData.a)) continue;
                    indexData.Value = financeData.a / 10000; //流通股本（万股）
                    bFinanceData = true;
                    break;
                case JS_EXECUTE_JOB_ID.JOB_DOWNLOAD_FLOW_MARKETVALUE_DATA:
                    //流通市值
                    var financeData = item.capital;
                    if (!financeData) continue;
                    if (!this.IsNumber(financeData.a)) continue;
                    indexData.Value = financeData.a; //流通股本
                    bMarketValue = true;
                    break;
                case JS_EXECUTE_JOB_ID.JOB_DOWNLOAD_MARKETVALUE_DATA:
                    //总市值
                    var financeData = item.capital;
                    if (!financeData) continue;
                    if (!this.IsNumber(financeData.total)) continue;
                    indexData.Value = financeData.total; //总股本
                    bMarketValue = true;
                    break;
                case JS_EXECUTE_JOB_ID.JOB_DOWNLOAD_EXCHANGE_DATA:
                    //换手率
                    var financeData = item.capital;
                    if (!financeData) continue;
                    if (!this.IsNumber(financeData.a)) continue;
                    indexData.Value = financeData.a; //流通股本
                    bFinanceData = true;
                    break;
                case JS_EXECUTE_JOB_ID.JOB_DOWNLOAD_PER_U_PROFIT_DATA:
                    var financeData = this.JsonDataToFinance(item);
                    if (!financeData) continue;
                    if (!this.IsNumber(financeData.Finance.peruprofit)) continue;
                    indexData.Value = financeData.Finance.peruprofit; //每股未分配利润
                    bFinanceData = true;
                    break;
                case JS_EXECUTE_JOB_ID.JOB_DOWNLOAD_PER_NETASSET_DATA:
                    var financeData = this.JsonDataToFinance(item);
                    if (!financeData) continue;
                    if (!this.IsNumber(financeData.Finance.pernetasset)) continue;
                    indexData.Value = financeData.Finance.pernetasset; //每股净资产
                    bFinanceData = true;
                    break;
                case JS_EXECUTE_JOB_ID.JOB_DOWNLOAD_PER_C_RESERVE_DATA:
                    var financeData = this.JsonDataToFinance(item);
                    if (!financeData) continue;
                    if (!this.IsNumber(financeData.Finance.percreserve)) continue;
                    indexData.Value = financeData.Finance.percreserve; //每股资本公积金
                    bFinanceData = true;
                    break;
                case JS_EXECUTE_JOB_ID.JOB_DOWNLOAD_PER_S_EARNING_DATA:
                    var financeData = this.JsonDataToFinance(item);
                    if (!financeData) continue;
                    if (!this.IsNumber(financeData.Finance.persearning)) continue;
                    indexData.Value = financeData.Finance.persearning; //每股收益
                    bFinanceData = true;
                    break;
                case JS_EXECUTE_JOB_ID.JOB_DOWNLOAD_PER_S_EARNING2_DATA:
                    var financeData = this.JsonDataToFinance(item);
                    if (!financeData) continue;
                    if (!this.IsNumber(financeData.Finance.persearning)) continue;
                    indexData.Value = financeData.Finance.persearning / financeData.Announcement.quarter * 4; //每股收益(折算为全年收益)  报告期每股收益/报告期*4
                    bFinanceData = true;
                    break;
                case JS_EXECUTE_JOB_ID.JOB_DOWNLOAD_N_PROFIT_DATA:
                    var financeData = this.JsonDataToFinance(item);
                    if (!financeData) continue;
                    if (!this.IsNumber(financeData.Finance.nprofit)) continue;
                    indexData.Value = financeData.Finance.nprofit; //净利润
                    bFinanceData = true;
                    break;
                case JS_EXECUTE_JOB_ID.JOB_DOWNLOAD_AL_RATIO_DATA:
                    var financeData = this.JsonDataToFinance(item);
                    if (!financeData) continue;
                    if (!this.IsNumber(financeData.Finance.alratio)) continue;
                    indexData.Value = financeData.Finance.alratio; //资产负债率
                    bFinanceData = true;
                    break;
                case JS_EXECUTE_JOB_ID.JOB_DOWNLOAD_PROFIT_YOY_DATA:
                    var financeData = this.JsonDataToFinance(item);
                    if (!financeData) continue;
                    if (!this.IsNumber(financeData.Finance.profityoy)) continue;
                    indexData.Value = financeData.Finance.profityoy; //净利润同比增长率
                    bFinanceData = true;
                    break;
                case JS_EXECUTE_JOB_ID.JOB_DOWNLOAD_DIVIDEND_YIELD_DATA:
                    var financeData = item.execdividend;
                    if (!financeData) continue;
                    if (!this.IsNumber(financeData.quarter4)) continue;
                    indexData.Value = financeData.quarter4; //过去4个季度现金分红总额
                    bFinanceData = true;
                    break;
                case JS_EXECUTE_JOB_ID.JOB_DOWNLOAD_SHAREHOLDER_DATA:
                    //股东人数
                    var shareHolder = item.shareholder;
                    if (!shareHolder || !this.IsNumber(shareHolder.count)) continue;
                    indexData.Value = shareHolder.count;
                    bFinanceData = true;
                    break;
                default:
                    continue;
            }

            aryData.push(indexData);
        }

        var aryFixedData = void 0;
        if (bFinanceData) {
            if (this.SourceData) aryFixedData = this.SourceData.GetFittingFinanceData(aryData);else aryFixedData = this.Data.GetFittingFinanceData(aryData);
        } else if (bMarketValue) {
            if (this.SourceData) aryFixedData = this.SourceData.GetFittingMarketValueData(aryData); //总市值用不复权的价格计算
            else aryFixedData = this.Data.GetFittingMarketValueData(aryData);

            if (jobID == JS_EXECUTE_JOB_ID.JOB_DOWNLOAD_MARKETVALUE_DATA) this.MarketValue = aryFixedData; //总市值保存下 算其他数据可能要用
        } else if (jobID == JS_EXECUTE_JOB_ID.JOB_DOWNLOAD_DIVIDEND_YIELD_DATA) //股息率TTM：过去4个季度现金分红总额/总市值 * 100%
            {
                aryFixedData = this.CalculateDividendYield(aryData);
            } else {
            aryFixedData = this.SourceData.GetFittingData(aryData);
        }

        //JSConsole.Complier.Log('[JSSymbolData::RecvStockDayData] jobID=' + jobID, aryFixedData)

        var bindData = new ChartData();
        bindData.Data = aryFixedData;
        bindData.Period = this.Period; //周期

        if (bindData.Period > 0) //周期数据
            {
                var periodData = bindData.GetPeriodSingleData(bindData.Period);
                bindData.Data = periodData;
            }

        if (jobID == JS_EXECUTE_JOB_ID.JOB_DOWNLOAD_EXCHANGE_DATA) //计算换手率 成交量/流通股本*100
            {
                for (var _i90 in this.Data.Data) {
                    if (!bindData.Data[_i90]) continue;

                    if (this.IsNumber(bindData.Data[_i90].Value) && bindData.Data[_i90].Value != 0) bindData.Data[_i90].Value = this.Data.Data[_i90].Vol / bindData.Data[_i90].Value * 100;else bindData.Data[_i90].Value = 1;
                }
            }

        var data = bindData.GetValue();
        this.FinanceData.set(jobID, data);
    };

    //计算股息率 股息率TTM：过去4个季度现金分红总额/总市值 * 100%
    this.CalculateDividendYield = function (cashData) {
        var dividendYield = [];
        if (!this.MarketValue) return dividendYield;

        for (var _i91 = 0, _j13 = 0; _i91 < this.Data.Data.length; ++_i91) {
            var day = this.Data.Data[_i91];
            var market = this.MarketValue[_i91];
            if (!day || !market) continue;

            var _item8 = new SingleData();
            _item8.Date = day.Date;
            _item8.Value = 0;

            if (_j13 + 1 < cashData.length) {
                if (cashData[_j13].Date < day.Date && cashData[_j13 + 1].Date <= day.Date) {
                    ++_j13;
                    --_i91;
                    continue;
                }
            }

            if (_j13 < cashData.length) {
                var cash = cashData[_j13];
                var endDate = cash.Date + 10000; //1年有效

                if (day.Date >= cash.Date && day.Date <= endDate && this.IsDivideNumber(market.Value) && this.IsNumber(cash.Value)) {
                    _item8.Value = cash.Value / market.Value * 100;
                }
            }

            dividendYield.push(_item8);
        }

        return dividendYield;
    };

    //融资融券函数
    this.GetMarginCacheData = function (id, node) {
        var jobID = JS_EXECUTE_JOB_ID.GetMarginJobID(id);
        if (!jobID) this.Execute.ThrowUnexpectedNode(node, '不支持MARGIN(' + id + ')');
        if (this.MarginData.has(jobID)) return this.MarginData.get(jobID);

        return [];
    };

    //下融资融券
    this.GetMarginData = function (jobID) {
        if (this.MarginData.has(jobID)) return this.Execute.RunNextJob();

        JSConsole.Complier.Log('[JSSymbolData::GetMarginData] jobID=', jobID);
        var self = this;
        var fieldList = ["name", "date", "symbol"];

        switch (jobID) {
            case JS_EXECUTE_JOB_ID.JOB_DOWNLOAD_MARGIN_BALANCE:
                //融资融券余额
                fieldList.push("margin.balance");
                break;
            case JS_EXECUTE_JOB_ID.JOB_DOWNLOAD_MARGIN_RATE:
                //融资占比
                fieldList.push("margin.rate");
                break;

            case JS_EXECUTE_JOB_ID.JOB_DOWNLOAD_MARGIN_BUY_BALANCE: //买入信息-融资余额
            case JS_EXECUTE_JOB_ID.JOB_DOWNLOAD_MARGIN_BUY_AMOUNT: //买入信息-买入额
            case JS_EXECUTE_JOB_ID.JOB_DOWNLOAD_MARGIN_BUY_REPAY: //买入信息-偿还额
            case JS_EXECUTE_JOB_ID.JOB_DOWNLOAD_MARGIN_BUY_NET:
                //买入信息-融资净买入
                fieldList.push("margin.buy");
                break;

            case JS_EXECUTE_JOB_ID.JOB_DOWNLOAD_MARGIN_SELL_BALANCE: //卖出信息-融券余量
            case JS_EXECUTE_JOB_ID.JOB_DOWNLOAD_MARGIN_SELL_VOLUME: //卖出信息-卖出量
            case JS_EXECUTE_JOB_ID.JOB_DOWNLOAD_MARGIN_SELL_REPAY: //卖出信息-偿还量
            case JS_EXECUTE_JOB_ID.JOB_DOWNLOAD_MARGIN_SELL_NET:
                //卖出信息-融券净卖出
                fieldList.push("margin.sell");
                break;
        }

        //请求数据
        JSNetwork.HttpRequest({
            url: this.StockHistoryDayApiUrl,
            data: {
                "field": fieldList,
                "symbol": [this.Symbol],
                "orderfield": "date"
            },
            type: "post",
            dataType: "json",
            async: true,
            success: function success(recvData) {
                self.RecvMarginData(recvData, jobID);
                self.Execute.RunNextJob();
            }
        });
    };

    this.RecvMarginData = function (data, jobID) {
        JSConsole.Complier.Log(data);
        if (!data.stock || data.stock.length != 1) return;

        var stock = data.stock[0];
        var aryData = new Array();
        var aryData2 = [],
            aryData3 = [],
            aryData4 = []; //其他3个数据
        for (var _i92 in stock.stockday) {
            var item = stock.stockday[_i92];
            var marginData = item.margin;
            if (!marginData) continue;

            var indexData = new SingleData();
            indexData.Date = item.date;

            switch (jobID) {
                case JS_EXECUTE_JOB_ID.JOB_DOWNLOAD_MARGIN_BALANCE:
                    if (!this.IsNumber(marginData.balance)) continue;
                    indexData.Value = marginData.balance; //融资融券余额
                    aryData.push(indexData);
                    break;
                case JS_EXECUTE_JOB_ID.JOB_DOWNLOAD_MARGIN_RATE:
                    if (!this.IsNumber(marginData.rate)) continue;
                    indexData.Value = marginData.rate; //融资占比
                    aryData.push(indexData);
                    break;

                case JS_EXECUTE_JOB_ID.JOB_DOWNLOAD_MARGIN_BUY_BALANCE: //买入信息-融资余额
                case JS_EXECUTE_JOB_ID.JOB_DOWNLOAD_MARGIN_BUY_AMOUNT: //买入信息-买入额
                case JS_EXECUTE_JOB_ID.JOB_DOWNLOAD_MARGIN_BUY_REPAY: //买入信息-偿还额
                case JS_EXECUTE_JOB_ID.JOB_DOWNLOAD_MARGIN_BUY_NET:
                    //买入信息-融资净买入
                    var buyData = marginData.buy;
                    if (!buyData) continue;
                    if (!this.IsNumber(buyData.balance) || !this.IsNumber(buyData.amount) || !this.IsNumber(buyData.repay) || !this.IsNumber(buyData.net)) continue;

                    indexData.Value = buyData.balance;
                    var indexData2 = new SingleData();
                    indexData2.Date = item.date;
                    indexData2.Value = buyData.amount;
                    var indexData3 = new SingleData();
                    indexData3.Date = item.date;
                    indexData3.Value = buyData.repay;
                    var indexData4 = new SingleData();
                    indexData4.Date = item.date;
                    indexData4.Value = buyData.net;

                    aryData.push(indexData);
                    aryData2.push(indexData2);
                    aryData3.push(indexData3);
                    aryData4.push(indexData4);
                    break;
                case JS_EXECUTE_JOB_ID.JOB_DOWNLOAD_MARGIN_SELL_BALANCE: //卖出信息-融券余量
                case JS_EXECUTE_JOB_ID.JOB_DOWNLOAD_MARGIN_SELL_VOLUME: //卖出信息-卖出量
                case JS_EXECUTE_JOB_ID.JOB_DOWNLOAD_MARGIN_SELL_REPAY: //卖出信息-偿还量
                case JS_EXECUTE_JOB_ID.JOB_DOWNLOAD_MARGIN_SELL_NET:
                    //卖出信息-融券净卖出
                    var sellData = marginData.sell;
                    if (!sellData) continue;
                    if (!this.IsNumber(sellData.balance) || !this.IsNumber(sellData.volume) || !this.IsNumber(sellData.repay) || !this.IsNumber(sellData.net)) continue;

                    indexData.Value = buyData.balance;
                    var indexData2 = new SingleData();
                    indexData2.Date = item.date;
                    indexData2.Value = buyData.volume;
                    var indexData3 = new SingleData();
                    indexData3.Date = item.date;
                    indexData3.Value = buyData.repay;
                    var indexData4 = new SingleData();
                    indexData4.Date = item.date;
                    indexData4.Value = buyData.net;

                    aryData.push(indexData);
                    aryData2.push(indexData2);
                    aryData3.push(indexData3);
                    aryData4.push(indexData4);
                    break;
                default:
                    continue;
            }
        }

        var allData = [];
        if (jobID === JS_EXECUTE_JOB_ID.JOB_DOWNLOAD_MARGIN_BALANCE || jobID === JS_EXECUTE_JOB_ID.JOB_DOWNLOAD_MARGIN_RATE) {
            allData.push({ JobID: jobID, Data: aryData });
        } else if (jobID === JS_EXECUTE_JOB_ID.JOB_DOWNLOAD_MARGIN_BUY_BALANCE || jobID === JS_EXECUTE_JOB_ID.JOB_DOWNLOAD_MARGIN_BUY_AMOUNT || jobID === JS_EXECUTE_JOB_ID.JOB_DOWNLOAD_MARGIN_BUY_REPAY || jobID === JS_EXECUTE_JOB_ID.JOB_DOWNLOAD_MARGIN_BUY_NET) {
            allData.push({ JobID: JS_EXECUTE_JOB_ID.JOB_DOWNLOAD_MARGIN_BUY_BALANCE, Data: aryData });
            allData.push({ JobID: JS_EXECUTE_JOB_ID.JOB_DOWNLOAD_MARGIN_BUY_AMOUNT, Data: aryData2 });
            allData.push({ JobID: JS_EXECUTE_JOB_ID.JOB_DOWNLOAD_MARGIN_BUY_REPAY, Data: aryData3 });
            allData.push({ JobID: JS_EXECUTE_JOB_ID.JOB_DOWNLOAD_MARGIN_BUY_NET, Data: aryData4 });
        } else if (jobID === JS_EXECUTE_JOB_ID.JOB_DOWNLOAD_MARGIN_SELL_BALANCE || jobID === JS_EXECUTE_JOB_ID.JOB_DOWNLOAD_MARGIN_SELL_VOLUME || jobID === JS_EXECUTE_JOB_ID.JOB_DOWNLOAD_MARGIN_SELL_REPAY || jobID === JS_EXECUTE_JOB_ID.JOB_DOWNLOAD_MARGIN_SELL_NET) {
            allData.push({ JobID: JS_EXECUTE_JOB_ID.JOB_DOWNLOAD_MARGIN_SELL_BALANCE, Data: aryData });
            allData.push({ JobID: JS_EXECUTE_JOB_ID.JOB_DOWNLOAD_MARGIN_SELL_VOLUME, Data: aryData2 });
            allData.push({ JobID: JS_EXECUTE_JOB_ID.JOB_DOWNLOAD_MARGIN_SELL_REPAY, Data: aryData3 });
            allData.push({ JobID: JS_EXECUTE_JOB_ID.JOB_DOWNLOAD_MARGIN_SELL_NET, Data: aryData4 });
        }

        for (var _i93 in allData) {
            var aryFixedData = this.SourceData.GetFittingData(allData[_i93].Data);

            var bindData = new ChartData();
            bindData.Data = aryFixedData;
            bindData.Period = this.Period; //周期

            if (bindData.Period > 0) //周期数据
                {
                    var periodData = bindData.GetPeriodSingleData(bindData.Period);
                    bindData.Data = periodData;
                }

            var _data = bindData.GetValue();
            this.MarginData.set(allData[_i93].JobID, _data);
        }
    };

    this.GetNewsAnalysisCacheData = function (id, node) {

        var jobID = JS_EXECUTE_JOB_ID.GetNewsAnalysisID(id);
        if (!jobID) this.Execute.ThrowUnexpectedNode(node, '不支持NEWS(' + id + ')');
        if (this.NewsAnalysisData.has(jobID)) return this.NewsAnalysisData.get(jobID);

        return [];
    };

    this.GetHKToSHSZData = function (jobID) {
        if (this.HKToSHSZData.has(jobID)) return this.Execute.RunNextJob();

        var url,
            dataType = 0;
        if (this.DataType === HQ_DATA_TYPE.MINUTE_ID) dataType = 1;else if (this.DataType == HQ_DATA_TYPE.MULTIDAY_MINUTE_ID) dataType = 2;else if (this.Period <= 3) dataType = 0;

        url = this.HKToSHSZApiUrl[dataType];
        var self = this;
        JSConsole.Complier.Log("[JSSymbolData::GetHKToSHSZData] jobID=" + jobID + ", url=" + url + ", dataType=" + dataType);

        if (dataType === 2) {
            //请求数据
            JSNetwork.HttpRequest({
                url: url,
                data: {
                    "symbol": this.Symbol,
                    'daycount': this.MaxRequestMinuteDayCount
                },
                type: "post",
                dataType: "json",
                async: true,
                success: function success(recvData) {
                    self.RecvMulitMinuteHKToSHSZData(recvData, jobID);
                    self.Execute.RunNextJob();
                }
            });
        } else {
            //请求数据
            JSNetwork.HttpRequest({
                url: url,
                type: "get",
                dataType: "json",
                async: true,
                success: function success(recvData) {
                    if (dataType == 0) self.RecvHKToSHSZData(recvData, jobID);else if (dataType == 1) self.RecvMinuteHKToSHSZData(recvData, jobID);
                    self.Execute.RunNextJob();
                }
            });
        }
    };

    this.RecvMinuteHKToSHSZData = function (data, jobID) {
        var arySHSZData = [],
            arySHData = [],
            arySZData = [];
        if (this.IsBeforeData) {
            for (i in this.SourceData.Data) {
                var item = this.SourceData.Data[i];
                if (item.Before) {
                    arySHSZData.push(null);
                    arySHData.push(null);
                    arySZData.push(null);
                } else {
                    break;
                }
            }

            for (var i = 0; i < data.time.length; ++i) {
                var time = data.time[i];
                if (time === 925) continue;

                var SHValue = data.hk2sh[i];
                var SZValue = data.hk2sz[i];
                var total = SHValue + SZValue;

                arySHSZData.push(total);
                arySHData.push(SHValue);
                arySZData.push(SZValue);
            }
        } else {
            for (var i = 0; i < data.time.length; ++i) {
                var time = data.time[i];
                var SHValue = data.hk2sh[i];
                var SZValue = data.hk2sz[i];
                var total = SHValue + SZValue;

                arySHSZData.push(total);
                arySHData.push(SHValue);
                arySZData.push(SZValue);
            }
        }

        var allData = [{ Data: arySHSZData, JobID: JS_EXECUTE_JOB_ID.JOB_DOWNLOAD_HK_TO_SH_SZ }, { Data: arySHData, JobID: JS_EXECUTE_JOB_ID.JOB_DOWNLOAD_HK_TO_SH }, { Data: arySZData, JobID: JS_EXECUTE_JOB_ID.JOB_DOWNLOAD_HK_TO_SZ }];

        for (var i in allData) {
            var item = allData[i];
            this.HKToSHSZData.set(item.JobID, item.Data);
        }
    };

    this.RecvHKToSHSZData = function (data, jobID) {
        var arySHSZData = [],
            arySHData = [],
            arySZData = [];
        for (var i = 0; i < data.date.length; ++i) {
            var date = data.date[i];
            var SHValue = data.hk2sh[i] * 1000000; //单位是百万
            var SZValue = data.hk2sz[i] * 1000000; //单位是百万
            var total = SHValue + SZValue;

            var itemSHSZData = new SingleData();
            itemSHSZData.Date = date;
            itemSHSZData.Value = total;

            var itemSHData = new SingleData();
            itemSHData.Date = date;
            itemSHData.Value = SHValue;

            var itemSZData = new SingleData();
            itemSZData.Date = date;
            itemSZData.Value = SZValue;

            arySHSZData.push(itemSHSZData);
            arySHData.push(itemSHData);
            arySZData.push(itemSZData);
        }

        var allData = [{ Data: arySHSZData, JobID: JS_EXECUTE_JOB_ID.JOB_DOWNLOAD_HK_TO_SH_SZ }, { Data: arySHData, JobID: JS_EXECUTE_JOB_ID.JOB_DOWNLOAD_HK_TO_SH }, { Data: arySZData, JobID: JS_EXECUTE_JOB_ID.JOB_DOWNLOAD_HK_TO_SZ }];

        for (var _i94 in allData) {
            var aryFixedData = this.Data.GetFittingData(allData[_i94].Data);

            var bindData = new ChartData();
            bindData.Data = aryFixedData;
            bindData.Period = this.Period; //周期

            if (bindData.Period > 0) //周期数据
                {
                    var periodData = bindData.GetPeriodSingleData(bindData.Period);
                    bindData.Data = periodData;
                }

            var _data2 = bindData.GetValue();
            this.HKToSHSZData.set(allData[_i94].JobID, _data2);
        }
    };

    this.RecvMulitMinuteHKToSHSZData = function (data, jobID) //多日分时图北上资金
    {
        if (!data.data || data.data.length <= 0) return;

        var arySHSZData = [],
            arySHData = [],
            arySZData = [];
        for (var i = 0, j = 0; i < this.Data.Data.length && j < data.data.length;) {
            arySHSZData[i] = null;
            arySHData[i] = null;
            arySZData[i] = null;
            var item = this.Data.Data[i];
            var dateTime = item.DateTime; //日期加时间
            if (!dateTime) {
                ++i;
                continue;
            }
            var aryValue = dateTime.split(' ');
            if (aryValue.length != 2) {
                ++i;
                continue;
            }
            var date = parseInt(aryValue[0]);
            var day = data.data[j];
            if (!day.minute || day.minute.length <= 0) {
                ++j;
                continue;
            }

            if (day.date > date) {
                ++i;
                continue;
            } else if (day.date < date) {
                ++j;
                continue;
            }

            for (var k in day.minute) {
                var timeItem = day.minute[k];

                var SHValue = timeItem[1];
                var SZValue = timeItem[2];
                var total = SHValue + SZValue;

                arySHSZData[i] = total;
                arySHData[i] = SHValue;
                arySZData[i] = SZValue;
                ++i;
            }

            ++j;
        }

        var allData = [{ Data: arySHSZData, JobID: JS_EXECUTE_JOB_ID.JOB_DOWNLOAD_HK_TO_SH_SZ }, { Data: arySHData, JobID: JS_EXECUTE_JOB_ID.JOB_DOWNLOAD_HK_TO_SH }, { Data: arySZData, JobID: JS_EXECUTE_JOB_ID.JOB_DOWNLOAD_HK_TO_SZ }];

        for (var i in allData) {
            var item = allData[i];
            this.HKToSHSZData.set(item.JobID, item.Data);
        }
    };

    //北上资金函数
    this.GetHKToSHSZCacheData = function (id, node) {
        var jobID = JS_EXECUTE_JOB_ID.GetHK2SHSZJobID(id);
        if (!jobID) this.Execute.ThrowUnexpectedNode(node, '不支持HK2SHSZ(' + id + ')');
        if (this.HKToSHSZData.has(jobID)) return this.HKToSHSZData.get(jobID);

        return [];
    };

    //下载新闻统计
    this.GetNewsAnalysisData = function (jobID) {
        if (this.NewsAnalysisData.has(jobID)) return this.Execute.RunNextJob();

        var self = this;
        var mapFolder = new Map([[JS_EXECUTE_JOB_ID.JOB_DOWNLOAD_NEWS_ANALYSIS_NEGATIVE, "negative"], [JS_EXECUTE_JOB_ID.JOB_DOWNLOAD_NEWS_ANALYSIS_RESEARCH, 'research'], [JS_EXECUTE_JOB_ID.JOB_DOWNLOAD_NEWS_ANALYSIS_INTERACT, 'interact'], [JS_EXECUTE_JOB_ID.JOB_DOWNLOAD_NEWS_ANALYSIS_HOLDERCHANGE, 'holderchange'], //NEWS(4)   股东增持
        [JS_EXECUTE_JOB_ID.JOB_DOWNLOAD_NEWS_ANALYSIS_HOLDERCHANGE2, 'holderchange'], //NEWS(5)   股东减持
        [JS_EXECUTE_JOB_ID.JOB_DOWNLOAD_NEWS_ANALYSIS_TRUSTHOLDER, 'trustholder'], //NEWS(6)   信托持股
        [JS_EXECUTE_JOB_ID.JOB_DOWNLOAD_NEWS_ANALYSIS_BLOCKTRADING, 'Blocktrading'], //NEWS(7)   大宗交易
        [JS_EXECUTE_JOB_ID.JOB_DOWNLOAD_NEWS_ANALYSIS_COMPANYNEWS, 'companynews'], //NEWS(8)   官网新闻
        [JS_EXECUTE_JOB_ID.JOB_DOWNLOAD_NEWS_ANALYSIS_TOPMANAGERS, 'topmanagers'], //NEWS(9)   高管要闻
        [JS_EXECUTE_JOB_ID.JOB_DOWNLOAD_NEWS_ANALYSIS_PLEDGE, 'Pledge']]);

        if (!mapFolder.has(jobID)) {
            this.Execute.RunNextJob();
            return;
        }
        var folderName = mapFolder.get(jobID);
        var url = this.StockNewsAnalysisApiUrl + '/' + folderName + '/' + this.Symbol + '.json';

        //请求数据
        JSNetwork.HttpRequest({
            url: url,
            type: "get",
            dataType: "json",
            async: true,
            success: function success(recvData) {
                self.RecvNewsAnalysisData(recvData, jobID);
                self.Execute.RunNextJob();
            },
            error: function error(request, textStatus) {
                self.RecvNewsAnalysisDataError(request, textStatus, jobID);
                self.Execute.RunNextJob();
            }
        });
    };

    this.RecvNewsAnalysisDataError = function (request, textStatus, jobID) {
        JSConsole.Complier.Log('[JSSymbolData::RecvNewsAnalysisDataError] ajax error.', request.status);

        //没有新闻使用0数据填充
        var aryData = [];
        for (var i = 0; i < this.Data.Data.length; ++i) {
            var item = new SingleData();
            item.Date = this.Data.Data[i].Date;
            item.Value = 0;
            aryData.push(item);
        }

        var bindData = new ChartData();
        bindData.Data = aryData;
        this.NewsAnalysisData.set(jobID, bindData.GetValue());
    };

    this.RecvNewsAnalysisData = function (data, jobID) {
        if (!data.data || !data.date) return;
        if (data.data.length <= 0 || data.data.length != data.date.length) return;

        JSConsole.Complier.Log('[JSSymbolData::RecvNewsAnalysisData] jobID', jobID, data.update);
        if (jobID == JS_EXECUTE_JOB_ID.JOB_DOWNLOAD_NEWS_ANALYSIS_HOLDERCHANGE || jobID == JS_EXECUTE_JOB_ID.JOB_DOWNLOAD_NEWS_ANALYSIS_HOLDERCHANGE2) {
            var aryData = [],
                aryData2 = [];
            for (var i = 0; i < data.data.length; ++i) {
                var item = new SingleData();
                item.Date = data.date[i];
                item.Value = data.data[i];
                if (this.IsNumber(item.Value)) aryData.push(item);

                if (i < data.data2.length) {
                    item = new SingleData();
                    item.Date = data.date[i];
                    item.Value = data.data2[i];
                    if (this.IsNumber(item.Value)) aryData2.push(item);
                }
            }

            var aryFixedData = this.Data.GetFittingData2(aryData, 0);
            var bindData = new ChartData();
            bindData.Data = aryFixedData;
            this.NewsAnalysisData.set(JS_EXECUTE_JOB_ID.JOB_DOWNLOAD_NEWS_ANALYSIS_HOLDERCHANGE, bindData.GetValue());

            aryFixedData = this.Data.GetFittingData2(aryData2, 0);
            bindData = new ChartData();
            bindData.Data = aryFixedData;
            this.NewsAnalysisData.set(JS_EXECUTE_JOB_ID.JOB_DOWNLOAD_NEWS_ANALYSIS_HOLDERCHANGE2, bindData.GetValue());
        } else {
            var aryData = [];
            for (var i = 0; i < data.data.length; ++i) {
                var item = new SingleData();
                item.Date = data.date[i];
                item.Value = data.data[i];
                aryData.push(item);
            }

            var _aryFixedData = this.Data.GetFittingData2(aryData, 0);
            var bindData = new ChartData();
            bindData.Data = _aryFixedData;
            this.NewsAnalysisData.set(jobID, bindData.GetValue());
        }
    };

    this.GetSectionFinanceData = function (job) {
        var sfKey = job.SF[0];
        var period = job.SF[1].Period;
        if (this.SectionFinanceData.has(sfKey)) return this.Execute.RunNextJob();

        JSConsole.Complier.Log("[JSSymbolData::GetSectionFinanceData] " + period.Year + "-" + period.Quarter);
        var fieldName = 'announcement' + period.Quarter;
        var self = this;
        var fieldList = ["name", "date", "symbol", fieldName, 'finance' + period.Quarter];
        var cond = [fieldName + '.year', "int32", "eq", period.Year.toString()];

        //请求数据
        JSNetwork.HttpRequest({
            url: this.StockHistoryDayApiUrl,
            data: {
                "field": fieldList,
                "symbol": [this.Symbol],
                "condition": [{ item: cond }]
            },
            type: "post",
            dataType: "json",
            async: true,
            success: function success(recvData) {
                self.RecvSectionFinanceData(recvData, job);
                self.Execute.RunNextJob();
            }
        });
    };

    this.RecvSectionFinanceData = function (data, job) {
        if (!data.stock || data.stock.length != 1) return;
        var stockItem = data.stock[0];
        if (!stockItem.stockday || stockItem.stockday.length <= 0) return;
        var dayItem = stockItem.stockday[0];
        var period = job.SF[1].Period;
        var finance = null;
        if (period.Quarter === 1) finance = dayItem.finance1;else if (period.Quarter === 2) finance = dayItem.finance2;else if (period.Quarter === 3) finance = dayItem.finance3;else if (period.Quarter === 4) finance = dayItem.finance4;
        if (!finance) return;

        var data = new Map([[JS_EXECUTE_JOB_ID.JOB_DOWNLOAD_SECTION_F_01, finance.currentassets], [JS_EXECUTE_JOB_ID.JOB_DOWNLOAD_SECTION_F_02, finance.monetaryfunds], [JS_EXECUTE_JOB_ID.JOB_DOWNLOAD_SECTION_F_03, finance.inventory], [JS_EXECUTE_JOB_ID.JOB_DOWNLOAD_SECTION_F_04, finance.currentliabilities], [JS_EXECUTE_JOB_ID.JOB_DOWNLOAD_SECTION_F_05, finance.ncurrentliabilities], [JS_EXECUTE_JOB_ID.JOB_DOWNLOAD_SECTION_F_06, finance.expenses3], [JS_EXECUTE_JOB_ID.JOB_DOWNLOAD_SECTION_F_07, finance.investmentincome], [JS_EXECUTE_JOB_ID.JOB_DOWNLOAD_SECTION_F_08, finance.pcnprofit], [JS_EXECUTE_JOB_ID.JOB_DOWNLOAD_SECTION_F_09, finance.nnetprofit], [JS_EXECUTE_JOB_ID.JOB_DOWNLOAD_SECTION_F_10, finance.npersearning], [JS_EXECUTE_JOB_ID.JOB_DOWNLOAD_SECTION_F_11, finance.woewa], [JS_EXECUTE_JOB_ID.JOB_DOWNLOAD_SECTION_F_12, finance.inprocess], [JS_EXECUTE_JOB_ID.JOB_DOWNLOAD_SECTION_F_13, finance.accdepreciation], [JS_EXECUTE_JOB_ID.JOB_DOWNLOAD_SECTION_F_14, finance.mholderprofit], [JS_EXECUTE_JOB_ID.JOB_DOWNLOAD_SECTION_F_15, finance.lossexchange], [JS_EXECUTE_JOB_ID.JOB_DOWNLOAD_SECTION_F_16, finance.baddebts], [JS_EXECUTE_JOB_ID.JOB_DOWNLOAD_SECTION_F_17, finance.fixedassets], [JS_EXECUTE_JOB_ID.JOB_DOWNLOAD_SECTION_F_18, finance.curdepreciation], [JS_EXECUTE_JOB_ID.JOB_DOWNLOAD_SECTION_F_19, finance.orevenues], [JS_EXECUTE_JOB_ID.JOB_DOWNLOAD_SECTION_F_20, finance.moprofit], [JS_EXECUTE_JOB_ID.JOB_DOWNLOAD_SECTION_F_21, finance.oprofit], [JS_EXECUTE_JOB_ID.JOB_DOWNLOAD_SECTION_F_22, finance.nprofit], [JS_EXECUTE_JOB_ID.JOB_DOWNLOAD_SECTION_F_23, finance.areceivable], [JS_EXECUTE_JOB_ID.JOB_DOWNLOAD_SECTION_F_24, finance.financialcost], [JS_EXECUTE_JOB_ID.JOB_DOWNLOAD_SECTION_F_25, finance.ccfo], [JS_EXECUTE_JOB_ID.JOB_DOWNLOAD_SECTION_F_26, finance.totalassets], [JS_EXECUTE_JOB_ID.JOB_DOWNLOAD_SECTION_F_27, finance.totalliabilities], [JS_EXECUTE_JOB_ID.JOB_DOWNLOAD_SECTION_F_28, finance.totalownersequity], [JS_EXECUTE_JOB_ID.JOB_DOWNLOAD_SECTION_F_29, finance.grossmargin], [JS_EXECUTE_JOB_ID.JOB_DOWNLOAD_SECTION_F_30, finance.percreserve], [JS_EXECUTE_JOB_ID.JOB_DOWNLOAD_SECTION_F_31, finance.peruprofit], [JS_EXECUTE_JOB_ID.JOB_DOWNLOAD_SECTION_F_32, finance.persearning], [JS_EXECUTE_JOB_ID.JOB_DOWNLOAD_SECTION_F_33, finance.pernetasset], [JS_EXECUTE_JOB_ID.JOB_DOWNLOAD_SECTION_F_34, finance.perccfo], [JS_EXECUTE_JOB_ID.JOB_DOWNLOAD_SECTION_F_38, finance.alratio], [JS_EXECUTE_JOB_ID.JOB_DOWNLOAD_SECTION_F_39, finance.profityoy]]);

        if (period.Quarter === 4) //年报才有的数据
            {
                data.set(JS_EXECUTE_JOB_ID.JOB_DOWNLOAD_SECTION_F_35, finance.nnprofitincrease);
                data.set(JS_EXECUTE_JOB_ID.JOB_DOWNLOAD_SECTION_F_36, finance.nnprofitspeed);
                data.set(JS_EXECUTE_JOB_ID.JOB_DOWNLOAD_SECTION_F_37, finance.nprofitincrease);
            }

        var sfKey = job.SF[0];
        this.SectionFinanceData.set(sfKey, data);
    };

    this.GetSectionFinanceCacheData = function (year, quarter, fieldName, node) {
        var period = JS_EXECUTE_JOB_ID.GetSectionReportPeriod(year, quarter);
        if (!period) this.Execute.ThrowUnexpectedNode(node, "\u4E0D\u652F\u6301FS(" + year + ", " + quarter + ", '" + fieldName + "') \u62A5\u544A\u671F\u9519\u8BEF");
        var id = JS_EXECUTE_JOB_ID.GetSectionFinanceID(fieldName);
        if (!id) this.Execute.ThrowUnexpectedNode(node, "\u4E0D\u652F\u6301FS(" + year + ", " + quarter + ",'" + fieldName + "') \u8D22\u52A1\u6570\u636E\u5B57\u6BB5\u540D\u79F0\u9519\u8BEF");

        var sfKey = period.Year + '-' + period.Quarter;
        if (!this.SectionFinanceData.has(sfKey) && !this.ThrowSFPeirod.has(sfKey)) //动态下载的数据, 抛异常以后重新下载执行
            {
                this.ThrowSFPeirod.add(sfKey); //抛过的异常就不抛了
                var job = { ID: JS_EXECUTE_JOB_ID.JOB_DOWNLOAD_SECTION_SF, SF: [sfKey, { Period: period, Fields: new Set([id]) }] };
                this.Execute.ThrowDownloadSF(node, job, "FS(" + year + ", " + quarter + ", '" + fieldName + "') \u52A8\u6001\u4E0B\u8F7D");
            }

        if (!this.SectionFinanceData.has(sfKey)) return this.Execute.ThrowUnexpectedNode(node, "\u4E0D\u652F\u6301FS(" + year + ", " + quarter + ", '" + fieldName + "') \u6CA1\u6709\u8FD9\u671F\u8D22\u52A1\u6570\u636E");

        var financeData = this.SectionFinanceData.get(sfKey);
        if (!financeData.has(id)) this.Execute.ThrowUnexpectedNode(node, "\u4E0D\u652F\u6301FS(" + year + ", " + quarter + ", '" + fieldName + "') \u6CA1\u6709\u8FD9\u671F\u8D22\u52A1\u6570\u636E\u5B57\u6BB5");

        return financeData.get(id);
    };

    this.DownloadCustomVariantData = function (job) {
        var item = g_JSComplierResource.CustomVariant.Data.get(job.Name);
        if (!item || !item.Download) return this.Execute.RunNextJob();

        var self = this;
        var obj = { Name: job.Name, Symbol: this.Symbol, KData: this.Data, Period: this.Period, Right: this.Right,
            Success: function Success() {
                self.Execute.RunNextJob();
            } };

        item.Download(obj);
    };

    this.DownloadCustomFunctionData = function (job) {
        var item = g_JSComplierResource.CustomFunction.Data.get(job.Name);
        if (!item || !item.Download) return this.Execute.RunNextJob();

        var self = this;
        var obj = { Symbol: this.Symbol, KData: this.Data, Period: this.Period, Right: this.Right, Args: job.Args, Name: job.Name,
            Success: function Success() {
                self.Execute.RunNextJob();
            } };

        item.Download(obj);
    };

    this.JsonDataToHistoryData = function (data) {
        var list = data.data;
        var aryDayData = new Array();
        var date = 0,
            yclose = 1,
            open = 2,
            high = 3,
            low = 4,
            close = 5,
            vol = 6,
            amount = 7;
        var up = 8,
            down = 9,
            stop = 10,
            unchanged = 11;
        for (var i = 0; i < list.length; ++i) {
            var item = new HistoryData();

            item.Date = list[i][date];
            item.Open = list[i][open];
            item.YClose = list[i][yclose];
            item.Close = list[i][close];
            item.High = list[i][high];
            item.Low = list[i][low];
            item.Vol = list[i][vol]; //原始单位股
            item.Amount = list[i][amount];
            //上涨 下跌家数
            if (list[i].length > up) item.Up = list[i][up];
            if (list[i].length > down) item.Down = list[i][down];
            if (list[i].length > stop) item.Stop = list[i][stop];
            if (list[i].length > unchanged) item.Unchanged = list[i][unchanged];

            if (isNaN(item.Open) || item.Open <= 0) continue; //停牌的数据剔除

            aryDayData.push(item);
        }

        return aryDayData;
    };

    this.JsonDataToMinuteHistoryData = function (data) {
        var list = data.data;
        var aryDayData = new Array();
        var date = 0,
            yclose = 1,
            open = 2,
            high = 3,
            low = 4,
            close = 5,
            vol = 6,
            amount = 7,
            time = 8;
        for (var i = 0; i < list.length; ++i) {
            var _item9 = new HistoryData();

            _item9.Date = list[i][date];
            _item9.Open = list[i][open];
            _item9.YClose = list[i][yclose];
            _item9.Close = list[i][close];
            _item9.High = list[i][high];
            _item9.Low = list[i][low];
            _item9.Vol = list[i][vol]; //原始单位股
            _item9.Amount = list[i][amount];
            _item9.Time = list[i][time];

            // if (isNaN(item.Open) || item.Open<=0) continue; //停牌的数据剔除
            aryDayData.push(_item9);
        }

        // 无效数据处理
        for (var _i95 = 0; _i95 < aryDayData.length; ++_i95) {
            var minData = aryDayData[_i95];
            if (minData == null) coninue;
            if (isNaN(minData.Open) || minData.Open <= 0 || isNaN(minData.High) || minData.High <= 0 || isNaN(minData.Low) || minData.Low <= 0 || isNaN(minData.Close) || minData.Close <= 0 || isNaN(minData.YClose) || minData.YClose <= 0) {
                if (_i95 == 0) {
                    if (minData.YClose > 0) {
                        minData.Open = minData.YClose;
                        minData.High = minData.YClose;
                        minData.Low = minData.YClose;
                        minData.Close = minData.YClose;
                    }
                } else // 用前一个有效数据填充
                    {
                        for (var _j14 = _i95 - 1; _j14 >= 0; --_j14) {
                            var minData2 = aryDayData[_j14];
                            if (minData2 == null) coninue;
                            if (minData2.Open > 0 && minData2.High > 0 && minData2.Low > 0 && minData2.Close > 0) {
                                if (minData.YClose <= 0) minData.YClose = minData2.Close;
                                minData.Open = minData2.Open;
                                minData.High = minData2.High;
                                minData.Low = minData2.Low;
                                minData.Close = minData2.Close;
                                break;
                            }
                        }
                    }
            }
        }
        return aryDayData;
    };

    //API 返回数据 转化为array[]
    this.JsonDataToMinuteData = function (data) {
        var aryMinuteData = new Array();
        for (var i in data.stock[0].minute) {
            var jsData = data.stock[0].minute[i];
            var item = new MinuteData();

            item.Close = jsData.price;
            item.Open = jsData.open;
            item.High = jsData.high;
            item.Low = jsData.low;
            item.Vol = jsData.vol; //股
            item.Amount = jsData.amount;
            if (i == 0) //第1个数据 写死9：25
                item.DateTime = data.stock[0].date.toString() + " 0925";else item.DateTime = data.stock[0].date.toString() + " " + jsData.time.toString();
            item.Date = data.stock[0].date;
            item.Time = jsData.time;
            item.Increase = jsData.increase;
            item.Risefall = jsData.risefall;
            item.AvPrice = jsData.avprice;

            aryMinuteData[i] = item;
        }

        return aryMinuteData;
    };

    //多日日线数据API 转化成array[];
    this.JsonDataToMultiDayMinuteData = function (data) {
        var symbol = data.symbol;
        var upperSymbol = symbol.toUpperCase();
        var isSHSZ = MARKET_SUFFIX_NAME.IsSHSZ(upperSymbol);
        var isFutures = MARKET_SUFFIX_NAME.IsChinaFutures(upperSymbol);
        var result = [];
        for (var i in data.data) {
            var minuteData = [];
            var dayData = data.data[i];
            var date = dayData.date;
            var yClose = dayData.yclose; //前收盘 计算涨幅
            var preClose = yClose; //前一个数据价格
            var preAvPrice = null; //上一个均价
            //var preAvPrice=data.stock[0].yclose;    //前一个均价
            for (var j in dayData.minute) {
                var jsData = dayData.minute[j];
                if (jsData[2]) preClose = jsData[2]; //保存上一个收盘数据
                var item = new MinuteData();
                item.Close = jsData[2];
                item.Open = jsData[1];
                item.High = jsData[3];
                item.Low = jsData[4];
                item.Vol = jsData[5] / 100; //原始单位股
                item.Amount = jsData[6];
                if (7 < jsData.length && jsData[7] > 0) {
                    item.AvPrice = jsData[7]; //均价
                    preAvPrice = jsData[7];
                }
                item.DateTime = date.toString() + " " + jsData[0].toString();
                item.Date = date;
                item.Time = jsData[0];

                if (!item.Close) //当前没有价格 使用上一个价格填充
                    {
                        item.Close = preClose;
                        item.Open = item.High = item.Low = item.Close;
                    }

                if (!item.AvPrice && preAvPrice) item.AvPrice = preAvPrice;

                if (item.Close && yClose) item.Increase = (item.Close - yClose) / yClose * 100;else item.Increase = null;
                if (j == 0) //第1个数据 写死9：25
                    {
                        if (isSHSZ) item.DateTime = date.toString() + " 0925";
                        item.IsFristData = true;
                    }

                //价格是0的 都用空
                if (item.Open <= 0) item.Open = null;
                if (item.Close <= 0) item.Close = null;
                if (item.AvPrice <= 0) item.AvPrice = null;
                if (item.High <= 0) item.High = null;
                if (item.Low <= 0) item.Low = null;
                if (item.AvPrice <= 0) item.AvPrice = null;

                minuteData[j] = item;
            }

            var newData = new ChartData();
            newData.Data = minuteData;
            newData.YClose = yClose;
            newData.Close = dayData.close;
            newData.Date = date;

            result.push(newData);
        }

        var minuteData = [];
        for (var i = result.length - 1; i >= 0; --i) {
            var item = result[i];
            for (var j in item.Data) {
                minuteData.push(item.Data[j]);
            }
        }

        return minuteData;
    };

    //CODELIKE 模糊股票代码
    this.CODELIKE = function (value) {
        if (this.Symbol && this.Symbol.indexOf(value) == 0) return 1;
        return 0;
    };

    this.NAMELIKE = function (value) {
        if (this.Name && this.Name.indexOf(value) == 0) return 1;
        return 0;
    };

    /*
    SETCODE 市场类型
    0:深圳 1:上海,47:中金所期货 28:郑州商品 29:大连商品 30:上海商品,27:香港指数 31:香港主板,48:香港创业板... 
    */
    this.SETCODE = function () {
        if (this.Symbol.indexOf('.sh')) return 1;
        if (this.Symbol.indexOf('.sz')) return 0;

        return 0;
    };

    this.GetSymbol = function () {
        return this.Symbol;
    };

    this.GetName = function () {
        return this.Name;
    };

    this.GetIndustry = function () {
        var key = JS_EXECUTE_JOB_ID.JOB_DOWNLOAD_GROUP_DATA.toString() + '-Group-' + this.Symbol;
        if (!this.ExtendData.has(key)) return '';

        var group = this.ExtendData.get(key);
        if (!group.Industry || group.Industry.length <= 0) return '';

        var result = '';
        for (var i in group.Industry) {
            var item = group.Industry[i];
            if (result.length > 0) result += ' ';
            result += item.Name;
        }

        return result;
    };

    this.GetRegion = function () {
        var key = JS_EXECUTE_JOB_ID.JOB_DOWNLOAD_GROUP_DATA.toString() + '-Group-' + this.Symbol;
        if (!this.ExtendData.has(key)) return '';

        var group = this.ExtendData.get(key);
        if (!group.Region || group.Region.length <= 0) return '';

        var result = '';
        for (var i in group.Region) {
            var item = group.Region[i];
            if (result.length > 0) result += ' ';
            result += item.Name;
        }

        return result;
    };

    this.GetConcept = function () {
        var key = JS_EXECUTE_JOB_ID.JOB_DOWNLOAD_GROUP_DATA.toString() + '-Group-' + this.Symbol;
        if (!this.ExtendData.has(key)) return '';

        var group = this.ExtendData.get(key);
        if (!group.Concept || group.Concept.length <= 0) return '';

        var result = '';
        for (var i in group.Concept) {
            var item = group.Concept[i];
            if (result.length > 0) result += ' ';
            result += item.Name;
        }

        return result;
    };

    this.DATE = function () {
        var result = [];
        if (!this.Data || !this.Data.Data || !this.Data.Data.length) return result;

        for (var _i96 in this.Data.Data) {
            var item = this.Data.Data[_i96];
            result[_i96] = item.Date;
        }

        return result;
    };

    this.YEAR = function () {
        var result = [];
        if (!this.Data || !this.Data.Data || !this.Data.Data.length) return result;

        for (var _i97 in this.Data.Data) {
            var item = this.Data.Data[_i97];
            if (this.IsNumber(item.Date)) result[_i97] = parseInt(item.Date / 10000);else result[_i97] = null;
        }

        return result;
    };

    this.MONTH = function () {
        var result = [];
        if (!this.Data || !this.Data.Data || !this.Data.Data.length) return result;

        for (var _i98 in this.Data.Data) {
            var item = this.Data.Data[_i98];
            if (this.IsNumber(item.Date)) result[_i98] = parseInt(item.Date % 10000 / 100);else result[_i98] = null;
        }

        return result;
    };

    //星期 1-7
    this.WEEK = function () {
        var result = [];
        if (!this.Data || !this.Data.Data || !this.Data.Data.length) return result;

        var tempDate = new Date();
        for (var _i99 in this.Data.Data) {
            var item = this.Data.Data[_i99];
            result[_i99] = null;
            if (!this.IsNumber(item.Date)) continue;

            var year = parseInt(item.Date / 10000);
            var month = parseInt(item.Date % 10000 / 100);
            var day = item.Date % 100;

            tempDate.setFullYear(year);
            tempDate.setMonth(month - 1);
            tempDate.setDate(day);

            result[_i99] = tempDate.getDay();
        }

        return result;
    };

    this.REFDATE = function (data, date) {
        var result = null;
        var index = null;
        for (var _i100 in this.Data.Data) //查找日期对应的索引
        {
            if (this.Data.Data[_i100].Date == date) {
                index = parseInt(_i100);
                break;
            }
        }

        if (index == null || index >= data.length) return null;

        return data[index];
    };

    //用法:结果从0到11,依次分别是1/5/15/30/60分钟,日/周/月,多分钟,多日,季,年
    this.PERIOD = function () {
        //Period周期 0=日线 1=周线 2=月线 3=年线 9=季线 4=1分钟 5=5分钟 6=15分钟 7=30分钟 8=60分钟
        var PERIOD_MAP = [5, 6, 7, 11, 0, 1, 2, 3, 4, 5, 9];
        if (this.Period >= 0 && this.Period <= PERIOD_MAP.length - 1) return PERIOD_MAP[this.Period];

        return this.Period;
    };

    this.GetDrawNull = function () {
        var result = [];
        if (!this.Data || !this.Data.Data || !this.Data.Data.length) return result;

        for (var _i101 in this.Data.Data) {
            result[_i101] = null;
        }

        return result;
    };
}

//是否有是有效的数字
JSSymbolData.prototype.IsNumber = function (value) {
    if (value == null) return false;
    if (isNaN(value)) return false;

    return true;
};

JSSymbolData.prototype.IsDivideNumber = function (value) {
    if (value == null) return false;
    if (isNaN(value)) return false;
    if (value == 0) return false;

    return true;
};

JSSymbolData.prototype.JsonDataToFinance = function (data) {
    var financeData;

    for (var _i102 = 1; _i102 <= 4; ++_i102) {
        switch (_i102) {
            case 1:
                var finance = data.finance1;
                var announcement = data.announcement1;
                break;
            case 2:
                var finance = data.finance2;
                var announcement = data.announcement2;
                break;
            case 3:
                var finance = data.finance3;
                var announcement = data.announcement3;
                break;
            case 4:
                var finance = data.finance4;
                var announcement = data.announcement4;
                break;
            default:
                break;
        }

        if (!finance || !announcement || !this.IsNumber(announcement.year) || !this.IsNumber(announcement.quarter)) continue;
        if (financeData) //如果存在1天公布多个报告期数据 只取最新的一个公告期数据
            {
                if (financeData.Announcement.year < announcement.year) financeData = { Date: data.date, Finance: finance, Announcement: announcement };
            } else {
            financeData = { Date: data.date, Finance: finance, Announcement: announcement };
        }
    }

    return financeData;
};

var JS_EXECUTE_DEBUG_LOG = false;

var JS_EXECUTE_JOB_ID = {
    JOB_DOWNLOAD_SYMBOL_DATA: 1, //下载股票的K线数据
    JOB_DOWNLOAD_INDEX_DATA: 2, //下载大盘的K线数据
    JOB_DOWNLOAD_SYMBOL_LATEST_DATA: 3, //最新的股票行情数据
    JOB_DOWNLOAD_INDEX_INCREASE_DATA: 4, //涨跌股票个数统计数据
    JOB_DOWNLOAD_VOLR_DATA: 5, //5日量比均量下载量比数据
    JOB_DOWNLOAD_GROUP_DATA: 6, //所属行业|地区|概念
    JOB_DOWNLOAD_SYMBOL_DAY_DATA: 7, //股票日线数据
    JOB_DOWNLOAD_LATEST_INDEX_DATA: 8, //下载最新大盘数据

    //财务函数
    JOB_DOWNLOAD_TOTAL_EQUITY_DATA: 100, //总股本（万股）
    JOB_DOWNLOAD_FLOW_EQUITY_DATA: 101, //流通股本（万股）
    JOB_DOWNLOAD_PER_U_PROFIT_DATA: 102, //每股未分配利润
    JOB_DOWNLOAD_PER_NETASSET_DATA: 103, //每股净资产
    JOB_DOWNLOAD_PER_C_RESERVE_DATA: 104, //每股资本公积金
    JOB_DOWNLOAD_PER_S_EARNING_DATA: 105, //每股收益 
    JOB_DOWNLOAD_PER_S_EARNING2_DATA: 106, //每股收益(折算为全年收益),对于沪深品种有效
    JOB_DOWNLOAD_RELEASE_DATE_DATA: 107, //上市的天数
    JOB_DOWNLOAD_N_PROFIT_DATA: 108, //净利润
    JOB_DOWNLOAD_FLOW_MARKETVALUE_DATA: 109, //流通市值
    JOB_DOWNLOAD_MARKETVALUE_DATA: 110, //总市值
    JOB_DOWNLOAD_PROFIT_YOY_DATA: 111, //利润同比 (Profit year on year)
    JOB_DOWNLOAD_AL_RATIO_DATA: 112, //资产负债率 (asset-liability ratio)
    JOB_DOWNLOAD_DIVIDEND_YIELD_DATA: 113, //股息率
    JOB_DOWNLOAD_SHAREHOLDER_DATA: 114, //股东人数    


    JOB_DOWNLOAD_CAPITAL_DATA: 200, //流通股本（手）
    JOB_DOWNLOAD_EXCHANGE_DATA: 201, //换手率 成交量/流通股本*100


    JOB_DOWNLOAD_MARGIN_BALANCE: 1000, //融资融券余额
    JOB_DOWNLOAD_MARGIN_RATE: 1001, //融资占比

    JOB_DOWNLOAD_MARGIN_BUY_BALANCE: 1010, //买入信息-融资余额
    JOB_DOWNLOAD_MARGIN_BUY_AMOUNT: 1011, //买入信息-买入额
    JOB_DOWNLOAD_MARGIN_BUY_REPAY: 1012, //买入信息-偿还额
    JOB_DOWNLOAD_MARGIN_BUY_NET: 1013, //买入信息-融资净买入

    JOB_DOWNLOAD_MARGIN_SELL_BALANCE: 1020, //卖出信息-融券余量
    JOB_DOWNLOAD_MARGIN_SELL_VOLUME: 1021, //卖出信息-卖出量
    JOB_DOWNLOAD_MARGIN_SELL_REPAY: 1022, //卖出信息-偿还量
    JOB_DOWNLOAD_MARGIN_SELL_NET: 1023, //卖出信息-融券净卖出

    JOB_DOWNLOAD_NEWS_ANALYSIS_NEGATIVE: 2000, //负面新闻统计
    JOB_DOWNLOAD_NEWS_ANALYSIS_RESEARCH: 2001, //机构调研
    JOB_DOWNLOAD_NEWS_ANALYSIS_INTERACT: 2002, //互动易
    JOB_DOWNLOAD_NEWS_ANALYSIS_HOLDERCHANGE: 2003, //股东增持
    JOB_DOWNLOAD_NEWS_ANALYSIS_HOLDERCHANGE2: 2004, //股东减持
    JOB_DOWNLOAD_NEWS_ANALYSIS_TRUSTHOLDER: 2005, //信托持股
    JOB_DOWNLOAD_NEWS_ANALYSIS_BLOCKTRADING: 2006, //大宗交易
    JOB_DOWNLOAD_NEWS_ANALYSIS_COMPANYNEWS: 2007, //官网新闻
    JOB_DOWNLOAD_NEWS_ANALYSIS_TOPMANAGERS: 2008, //高管要闻
    JOB_DOWNLOAD_NEWS_ANALYSIS_PLEDGE: 2009, //股权质押

    JOB_DOWNLOAD_HK_TO_SH: 2050, //北上流入上证
    JOB_DOWNLOAD_HK_TO_SZ: 2051, //北上流入深证
    JOB_DOWNLOAD_HK_TO_SH_SZ: 2052, //北上流总的

    JOB_CUSTOM_FUNCTION_DATA: 6000, //自定义函数
    JOB_CUSTOM_VARIANT_DATA: 6001, //自定义变量
    //截面数据
    //财务数据 SF(公告期,数据名称)   如: SF(201901,"流动资产");
    JOB_DOWNLOAD_SECTION_SF: 20000,

    JOB_DOWNLOAD_SECTION_F_01: 20001, //currentassets 流动资产
    JOB_DOWNLOAD_SECTION_F_02: 20002, //monetaryfunds 货币资金
    JOB_DOWNLOAD_SECTION_F_03: 20003, //inventory 存货
    JOB_DOWNLOAD_SECTION_F_04: 20004, //currentliabilities 流动负债
    JOB_DOWNLOAD_SECTION_F_05: 20005, //ncurrentliabilities 非流动负债
    JOB_DOWNLOAD_SECTION_F_06: 20006, //3expenses 三项费用
    JOB_DOWNLOAD_SECTION_F_07: 20007, //investmentincome 投资收益
    JOB_DOWNLOAD_SECTION_F_08: 20008, //pcnprofit 归母净利润
    JOB_DOWNLOAD_SECTION_F_09: 20009, //nnetprofit 扣非净利润
    JOB_DOWNLOAD_SECTION_F_10: 20010, //npersearning 扣非每股收益
    JOB_DOWNLOAD_SECTION_F_11: 20011, //woewa 加权平均净资产收益
    JOB_DOWNLOAD_SECTION_F_12: 20012, //inprocess 在建工程
    JOB_DOWNLOAD_SECTION_F_13: 20013, //accdepreciation 累计折旧
    JOB_DOWNLOAD_SECTION_F_14: 20014, //mholderprofit 少数股东利润
    JOB_DOWNLOAD_SECTION_F_15: 20015, //lossexchange 汇兑损益
    JOB_DOWNLOAD_SECTION_F_16: 20016, //baddebts 坏账计提
    JOB_DOWNLOAD_SECTION_F_17: 20017, //fixedassets 固定资产
    JOB_DOWNLOAD_SECTION_F_18: 20018, //curdepreciation 当期折旧
    JOB_DOWNLOAD_SECTION_F_19: 20019, //orevenues 营业总收入
    JOB_DOWNLOAD_SECTION_F_20: 20020, //moprofit 主营业务利润
    JOB_DOWNLOAD_SECTION_F_21: 20021, //oprofit 营业利润
    JOB_DOWNLOAD_SECTION_F_22: 20022, //nprofit 净利润
    JOB_DOWNLOAD_SECTION_F_23: 20023, //areceivable 应收账款
    JOB_DOWNLOAD_SECTION_F_24: 20024, //financialcost 财务费用
    JOB_DOWNLOAD_SECTION_F_25: 20025, //ccfo 经营性现金流
    JOB_DOWNLOAD_SECTION_F_26: 20026, //totalassets 资产总计
    JOB_DOWNLOAD_SECTION_F_27: 20027, //totalliabilities 负债总计
    JOB_DOWNLOAD_SECTION_F_28: 20028, //totalownersequity 所有者权益总计
    JOB_DOWNLOAD_SECTION_F_29: 20029, //grossmargin 毛利率
    JOB_DOWNLOAD_SECTION_F_30: 20030, //percreserve 每股资本公积金
    JOB_DOWNLOAD_SECTION_F_31: 20031, //peruprofit 每股未分配利润
    JOB_DOWNLOAD_SECTION_F_32: 20032, //persearning 每股收益
    JOB_DOWNLOAD_SECTION_F_33: 20033, //pernetasset 每股净资产
    JOB_DOWNLOAD_SECTION_F_34: 20034, //perccfo 每股经营性现金流
    JOB_DOWNLOAD_SECTION_F_35: 20035, //nnprofitincrease finance4特有,扣非净利润涨幅
    JOB_DOWNLOAD_SECTION_F_36: 20036, //nnprofitspeed finance4特有,扣非净利润涨速
    JOB_DOWNLOAD_SECTION_F_37: 20037, //nprofitincrease finance4特有,净利润涨幅
    JOB_DOWNLOAD_SECTION_F_38: 20038, //alratio 资产负债率（数值乘以100）
    JOB_DOWNLOAD_SECTION_F_39: 20039, //profityoy 利润同比%（数值乘以100）


    JOB_RUN_SCRIPT: 10000, //执行脚本

    GetFinnanceJobID: function GetFinnanceJobID(value) {
        var dataMap = new Map([[1, JS_EXECUTE_JOB_ID.JOB_DOWNLOAD_TOTAL_EQUITY_DATA], //FINANCE(1)   总股本（万股）
        [7, JS_EXECUTE_JOB_ID.JOB_DOWNLOAD_FLOW_EQUITY_DATA], //FINANCE(7)   流通股本（万股）
        [9, JS_EXECUTE_JOB_ID.JOB_DOWNLOAD_AL_RATIO_DATA], //FINANCE(9)   资产负债率 (asset-liability ratio)
        [18, JS_EXECUTE_JOB_ID.JOB_DOWNLOAD_PER_C_RESERVE_DATA], //FINANCE(18)  每股公积金
        [30, JS_EXECUTE_JOB_ID.JOB_DOWNLOAD_N_PROFIT_DATA], //FINANCE(30)  净利润
        [32, JS_EXECUTE_JOB_ID.JOB_DOWNLOAD_PER_U_PROFIT_DATA], //FINANCE(32)  每股未分配利润
        [33, JS_EXECUTE_JOB_ID.JOB_DOWNLOAD_PER_S_EARNING2_DATA], //FINANCE(33)  每股收益(折算为全年收益),对于沪深品种有效
        [34, JS_EXECUTE_JOB_ID.JOB_DOWNLOAD_PER_NETASSET_DATA], //FINANCE(34)  每股净资产
        [38, JS_EXECUTE_JOB_ID.JOB_DOWNLOAD_PER_S_EARNING_DATA], //FINANCE(38)  每股收益(最近一期季报)
        [40, JS_EXECUTE_JOB_ID.JOB_DOWNLOAD_FLOW_MARKETVALUE_DATA], //FINANCE(40)  流通市值 
        [41, JS_EXECUTE_JOB_ID.JOB_DOWNLOAD_MARKETVALUE_DATA], //FINANCE(41)  总市值
        [42, JS_EXECUTE_JOB_ID.JOB_DOWNLOAD_RELEASE_DATE_DATA], //FINANCE(42)  上市的天数
        [43, JS_EXECUTE_JOB_ID.JOB_DOWNLOAD_PROFIT_YOY_DATA], //FINANCE(43)  利润同比 (Profit year on year)
        [45, JS_EXECUTE_JOB_ID.JOB_DOWNLOAD_DIVIDEND_YIELD_DATA], //FINANCE(45)  股息率

        [100, JS_EXECUTE_JOB_ID.JOB_DOWNLOAD_SHAREHOLDER_DATA], //FINANCE(100) 股东人数

        [200, JS_EXECUTE_JOB_ID.JOB_DOWNLOAD_CAPITAL_DATA], //流通股本（手）
        [201, JS_EXECUTE_JOB_ID.JOB_DOWNLOAD_EXCHANGE_DATA] //换手率 成交量/流通股本

        ]);

        if (dataMap.has(value)) return dataMap.get(value);

        return null;
    },

    //融资融券
    GetMarginJobID: function GetMarginJobID(value) {
        var dataMap = new Map([[1, JS_EXECUTE_JOB_ID.JOB_DOWNLOAD_MARGIN_BALANCE], //MARGIN(1)   融资融券余额
        [2, JS_EXECUTE_JOB_ID.JOB_DOWNLOAD_MARGIN_RATE], //MARGIN(2)   融资占比

        [3, JS_EXECUTE_JOB_ID.JOB_DOWNLOAD_MARGIN_BUY_BALANCE], //MARGIN(3)   买入信息-融资余额
        [4, JS_EXECUTE_JOB_ID.JOB_DOWNLOAD_MARGIN_BUY_AMOUNT], //MARGIN(4)   买入信息-买入额
        [5, JS_EXECUTE_JOB_ID.JOB_DOWNLOAD_MARGIN_BUY_REPAY], //MARGIN(5)   买入信息-偿还额
        [6, JS_EXECUTE_JOB_ID.JOB_DOWNLOAD_MARGIN_BUY_NET], //MARGIN(6)   买入信息-融资净买入

        [7, JS_EXECUTE_JOB_ID.JOB_DOWNLOAD_MARGIN_SELL_BALANCE], //MARGIN(7)   卖出信息-融券余量
        [8, JS_EXECUTE_JOB_ID.JOB_DOWNLOAD_MARGIN_SELL_VOLUME], //MARGIN(8)   卖出信息-卖出量
        [9, JS_EXECUTE_JOB_ID.JOB_DOWNLOAD_MARGIN_SELL_REPAY], //MARGIN(9)   卖出信息-偿还量
        [10, JS_EXECUTE_JOB_ID.JOB_DOWNLOAD_MARGIN_SELL_NET]]);

        if (dataMap.has(value)) return dataMap.get(value);

        return null;
    },

    //北上资金
    GetHK2SHSZJobID: function GetHK2SHSZJobID(value) {
        var dataMap = new Map([[1, JS_EXECUTE_JOB_ID.JOB_DOWNLOAD_HK_TO_SH_SZ], //HK2SHSZ(1)   北上流总的
        [2, JS_EXECUTE_JOB_ID.JOB_DOWNLOAD_HK_TO_SH], //HK2SHSZ(2)   北上流入上证
        [3, JS_EXECUTE_JOB_ID.JOB_DOWNLOAD_HK_TO_SZ]]);

        if (dataMap.has(value)) return dataMap.get(value);

        return null;
    },

    GetNewsAnalysisID: function GetNewsAnalysisID(value) {
        var dataMap = new Map([[1, JS_EXECUTE_JOB_ID.JOB_DOWNLOAD_NEWS_ANALYSIS_NEGATIVE], //NEWS(1)   负面新闻统计
        [2, JS_EXECUTE_JOB_ID.JOB_DOWNLOAD_NEWS_ANALYSIS_RESEARCH], //NEWS(2)   机构调研统计
        [3, JS_EXECUTE_JOB_ID.JOB_DOWNLOAD_NEWS_ANALYSIS_INTERACT], //NEWS(3)   互动易
        [4, JS_EXECUTE_JOB_ID.JOB_DOWNLOAD_NEWS_ANALYSIS_HOLDERCHANGE], //NEWS(4)   股东增持
        [5, JS_EXECUTE_JOB_ID.JOB_DOWNLOAD_NEWS_ANALYSIS_HOLDERCHANGE2], //NEWS(5)   股东减持
        [6, JS_EXECUTE_JOB_ID.JOB_DOWNLOAD_NEWS_ANALYSIS_TRUSTHOLDER], //NEWS(6)   信托持股
        [7, JS_EXECUTE_JOB_ID.JOB_DOWNLOAD_NEWS_ANALYSIS_BLOCKTRADING], //NEWS(7)   大宗交易
        [8, JS_EXECUTE_JOB_ID.JOB_DOWNLOAD_NEWS_ANALYSIS_COMPANYNEWS], //NEWS(8)   官网新闻
        [9, JS_EXECUTE_JOB_ID.JOB_DOWNLOAD_NEWS_ANALYSIS_TOPMANAGERS], //NEWS(9)   高管要闻
        [10, JS_EXECUTE_JOB_ID.JOB_DOWNLOAD_NEWS_ANALYSIS_PLEDGE]]);

        if (dataMap.has(value)) return dataMap.get(value);

        return null;
    },

    //财务截面数据 分报告期
    GetSectionFinanceID: function GetSectionFinanceID(value) {
        var dataMap = new Map([['流动资产', JS_EXECUTE_JOB_ID.JOB_DOWNLOAD_SECTION_F_01], ['货币资金', JS_EXECUTE_JOB_ID.JOB_DOWNLOAD_SECTION_F_02], ['存货', JS_EXECUTE_JOB_ID.JOB_DOWNLOAD_SECTION_F_03], ['流动负债', JS_EXECUTE_JOB_ID.JOB_DOWNLOAD_SECTION_F_04], ['非流动负债', JS_EXECUTE_JOB_ID.JOB_DOWNLOAD_SECTION_F_05], ['三项费用', JS_EXECUTE_JOB_ID.JOB_DOWNLOAD_SECTION_F_06], ['投资收益', JS_EXECUTE_JOB_ID.JOB_DOWNLOAD_SECTION_F_07], ['归母净利润', JS_EXECUTE_JOB_ID.JOB_DOWNLOAD_SECTION_F_08], ['扣非净利润', JS_EXECUTE_JOB_ID.JOB_DOWNLOAD_SECTION_F_09], ['扣非每股收益', JS_EXECUTE_JOB_ID.JOB_DOWNLOAD_SECTION_F_10], ['加权平均净资产收益', JS_EXECUTE_JOB_ID.JOB_DOWNLOAD_SECTION_F_11], ['在建工程', JS_EXECUTE_JOB_ID.JOB_DOWNLOAD_SECTION_F_12], ['累计折旧', JS_EXECUTE_JOB_ID.JOB_DOWNLOAD_SECTION_F_13], ['少数股东利润', JS_EXECUTE_JOB_ID.JOB_DOWNLOAD_SECTION_F_14], ['汇兑损益', JS_EXECUTE_JOB_ID.JOB_DOWNLOAD_SECTION_F_15], ['坏账计提', JS_EXECUTE_JOB_ID.JOB_DOWNLOAD_SECTION_F_16], ['固定资产', JS_EXECUTE_JOB_ID.JOB_DOWNLOAD_SECTION_F_17], ['当期折旧', JS_EXECUTE_JOB_ID.JOB_DOWNLOAD_SECTION_F_18], ['营业总收入', JS_EXECUTE_JOB_ID.JOB_DOWNLOAD_SECTION_F_19], ['主营业务利润', JS_EXECUTE_JOB_ID.JOB_DOWNLOAD_SECTION_F_20], ['营业利润', JS_EXECUTE_JOB_ID.JOB_DOWNLOAD_SECTION_F_21], ['净利润', JS_EXECUTE_JOB_ID.JOB_DOWNLOAD_SECTION_F_22], ['应收账款', JS_EXECUTE_JOB_ID.JOB_DOWNLOAD_SECTION_F_23], ['财务费用', JS_EXECUTE_JOB_ID.JOB_DOWNLOAD_SECTION_F_24], ['经营性现金流', JS_EXECUTE_JOB_ID.JOB_DOWNLOAD_SECTION_F_25], ['资产总计', JS_EXECUTE_JOB_ID.JOB_DOWNLOAD_SECTION_F_26], ['负债总计', JS_EXECUTE_JOB_ID.JOB_DOWNLOAD_SECTION_F_27], ['所有者权益总计', JS_EXECUTE_JOB_ID.JOB_DOWNLOAD_SECTION_F_28], ['毛利率', JS_EXECUTE_JOB_ID.JOB_DOWNLOAD_SECTION_F_29], ['每股资本公积金', JS_EXECUTE_JOB_ID.JOB_DOWNLOAD_SECTION_F_30], ['每股未分配利润', JS_EXECUTE_JOB_ID.JOB_DOWNLOAD_SECTION_F_31], ['每股收益', JS_EXECUTE_JOB_ID.JOB_DOWNLOAD_SECTION_F_32], ['每股净资产', JS_EXECUTE_JOB_ID.JOB_DOWNLOAD_SECTION_F_33], ['每股经营性现金流', JS_EXECUTE_JOB_ID.JOB_DOWNLOAD_SECTION_F_34], ['扣非净利润涨幅', JS_EXECUTE_JOB_ID.JOB_DOWNLOAD_SECTION_F_35], ['扣非净利润涨速', JS_EXECUTE_JOB_ID.JOB_DOWNLOAD_SECTION_F_36], ['净利润涨幅', JS_EXECUTE_JOB_ID.JOB_DOWNLOAD_SECTION_F_37], ['资产负债率', JS_EXECUTE_JOB_ID.JOB_DOWNLOAD_SECTION_F_38], ['利润同比', JS_EXECUTE_JOB_ID.JOB_DOWNLOAD_SECTION_F_39]]);

        if (dataMap.has(value)) return dataMap.get(value);

        return null;
    },

    //获取报告期 2018, 1 
    GetSectionReportPeriod: function GetSectionReportPeriod(year, quarter) {
        if (year >= 2000 && quarter >= 1 && quarter <= 4) return { Year: year, Quarter: quarter };
        return null;
    }

};

function JSExecute(ast, option) {
    this.AST = ast; //语法树

    this.ErrorHandler = new ErrorHandler();
    this.VarTable = new Map(); //变量表
    this.OutVarTable = []; //输出变量
    this.Arguments = [];
    this.ErrorCallback; //执行错误回调

    //脚本自动变量表, 只读
    this.ConstVarTable = new Map([
    //个股数据
    ['CLOSE', null], ['VOL', null], ['OPEN', null], ['HIGH', null], ['LOW', null], ['AMOUNT', null], ['C', null], ['V', null], ['O', null], ['H', null], ['L', null], ['VOLR', null],

    //日期类
    ['DATE', null], ['YEAR', null], ['MONTH', null], ['PERIOD', null], ['WEEK', null],

    //大盘数据
    ['INDEXA', null], ['INDEXC', null], ['INDEXH', null], ['INDEXL', null], ['INDEXO', null], ['INDEXV', null], ['INDEXADV', null], ['INDEXDEC', null], ['FROMOPEN', null], //已开盘有多长分钟

    ['CURRBARSCOUNT', null], //到最后交易日的周期数
    ['ISLASTBAR', null], //判断是否为最后一个周期

    ['CAPITAL', null], //流通股本（手）
    ['EXCHANGE', null], //换手率
    ['SETCODE', null], //市场类型
    ['CODE', null], //品种代码
    ['STKNAME', null], //品种名称

    ['HYBLOCK', null], //所属行业板块
    ['DYBLOCK', null], //所属地域板块
    ['GNBLOCK', null], //所属概念

    ['DRAWNULL', null]]);

    this.SymbolData = new JSSymbolData(this.AST, option, this);
    this.Algorithm = new JSAlgorithm(this.ErrorHandler, this.SymbolData);
    this.Draw = new JSDraw(this.ErrorHandler, this.SymbolData);

    this.JobList = []; //执行的任务队列

    this.UpdateUICallback = null; //回调
    this.CallbackParam = null;
    this.IsSectionMode = false;

    if (option) {
        if (option.Callback) this.UpdateUICallback = option.Callback;
        if (option.CallbackParam) this.CallbackParam = option.CallbackParam;
        if (option.Arguments) this.Arguments = option.Arguments;
        if (option.IsSectionMode) this.IsSectionMode = option.IsSectionMode;
    }

    this.Execute = function () {
        JSConsole.Complier.Log('[JSExecute::Execute] JobList', this.JobList);
        this.OutVarTable = [];
        this.VarTable = new Map();
        this.RunNextJob();
    };

    this.RunNextJob = function () {
        if (this.JobList.length <= 0) return;

        var jobItem = this.JobList.shift();

        switch (jobItem.ID) {
            case JS_EXECUTE_JOB_ID.JOB_DOWNLOAD_SYMBOL_DATA:
                return this.SymbolData.GetSymbolData();
            case JS_EXECUTE_JOB_ID.JOB_DOWNLOAD_SYMBOL_DAY_DATA:
                return this.SymbolData.GetSymbolDayData();
            case JS_EXECUTE_JOB_ID.JOB_DOWNLOAD_INDEX_DATA:
                return this.SymbolData.GetIndexData();
            case JS_EXECUTE_JOB_ID.JOB_DOWNLOAD_LATEST_INDEX_DATA:
                return this.SymbolData.GetLatestIndexData();
            case JS_EXECUTE_JOB_ID.JOB_DOWNLOAD_INDEX_INCREASE_DATA:
                return this.SymbolData.GetIndexIncreaseData(jobItem);
            case JS_EXECUTE_JOB_ID.JOB_DOWNLOAD_SYMBOL_LATEST_DATA:
                return this.SymbolData.GetLatestData();
            case JS_EXECUTE_JOB_ID.JOB_DOWNLOAD_VOLR_DATA:
                //量比
                return this.SymbolData.GetVolRateData(jobItem);
            case JS_EXECUTE_JOB_ID.JOB_DOWNLOAD_GROUP_DATA:
                return this.SymbolData.GetGroupData(jobItem); //行业|概念|地区

            case JS_EXECUTE_JOB_ID.JOB_DOWNLOAD_TOTAL_EQUITY_DATA:
            case JS_EXECUTE_JOB_ID.JOB_DOWNLOAD_CAPITAL_DATA:
            case JS_EXECUTE_JOB_ID.JOB_DOWNLOAD_FLOW_EQUITY_DATA:
            case JS_EXECUTE_JOB_ID.JOB_DOWNLOAD_PER_NETASSET_DATA:
            case JS_EXECUTE_JOB_ID.JOB_DOWNLOAD_PER_U_PROFIT_DATA:
            case JS_EXECUTE_JOB_ID.JOB_DOWNLOAD_PER_C_RESERVE_DATA:
            case JS_EXECUTE_JOB_ID.JOB_DOWNLOAD_PER_S_EARNING_DATA:
            case JS_EXECUTE_JOB_ID.JOB_DOWNLOAD_PER_S_EARNING2_DATA:
            case JS_EXECUTE_JOB_ID.JOB_DOWNLOAD_N_PROFIT_DATA:
            case JS_EXECUTE_JOB_ID.JOB_DOWNLOAD_FLOW_MARKETVALUE_DATA:
            case JS_EXECUTE_JOB_ID.JOB_DOWNLOAD_MARKETVALUE_DATA:
            case JS_EXECUTE_JOB_ID.JOB_DOWNLOAD_EXCHANGE_DATA:
            case JS_EXECUTE_JOB_ID.JOB_DOWNLOAD_AL_RATIO_DATA:
            case JS_EXECUTE_JOB_ID.JOB_DOWNLOAD_PROFIT_YOY_DATA:
            case JS_EXECUTE_JOB_ID.JOB_DOWNLOAD_DIVIDEND_YIELD_DATA:
            case JS_EXECUTE_JOB_ID.JOB_DOWNLOAD_SHAREHOLDER_DATA:
                return this.SymbolData.GetFinanceData(jobItem.ID);

            case JS_EXECUTE_JOB_ID.JOB_DOWNLOAD_RELEASE_DATE_DATA:
                return this.SymbolData.GetCompanyReleaseDate(jobItem.ID);

            case JS_EXECUTE_JOB_ID.JOB_DOWNLOAD_MARGIN_BALANCE:
            case JS_EXECUTE_JOB_ID.JOB_DOWNLOAD_MARGIN_RATE:
            case JS_EXECUTE_JOB_ID.JOB_DOWNLOAD_MARGIN_BUY_BALANCE: //买入信息-融资余额
            case JS_EXECUTE_JOB_ID.JOB_DOWNLOAD_MARGIN_BUY_AMOUNT: //买入信息-买入额
            case JS_EXECUTE_JOB_ID.JOB_DOWNLOAD_MARGIN_BUY_REPAY: //买入信息-偿还额
            case JS_EXECUTE_JOB_ID.JOB_DOWNLOAD_MARGIN_BUY_NET: //买入信息-融资净买入
            case JS_EXECUTE_JOB_ID.JOB_DOWNLOAD_MARGIN_SELL_BALANCE: //卖出信息-融券余量
            case JS_EXECUTE_JOB_ID.JOB_DOWNLOAD_MARGIN_SELL_VOLUME: //卖出信息-卖出量
            case JS_EXECUTE_JOB_ID.JOB_DOWNLOAD_MARGIN_SELL_REPAY: //卖出信息-偿还量
            case JS_EXECUTE_JOB_ID.JOB_DOWNLOAD_MARGIN_SELL_NET:
                //卖出信息-融券净卖出
                return this.SymbolData.GetMarginData(jobItem.ID);

            case JS_EXECUTE_JOB_ID.JOB_DOWNLOAD_NEWS_ANALYSIS_NEGATIVE: //负面新闻
            case JS_EXECUTE_JOB_ID.JOB_DOWNLOAD_NEWS_ANALYSIS_RESEARCH: //机构调研
            case JS_EXECUTE_JOB_ID.JOB_DOWNLOAD_NEWS_ANALYSIS_INTERACT: //互动易
            case JS_EXECUTE_JOB_ID.JOB_DOWNLOAD_NEWS_ANALYSIS_HOLDERCHANGE: //股东增持
            case JS_EXECUTE_JOB_ID.JOB_DOWNLOAD_NEWS_ANALYSIS_HOLDERCHANGE2: //股东减持
            case JS_EXECUTE_JOB_ID.JOB_DOWNLOAD_NEWS_ANALYSIS_TRUSTHOLDER: //信托持股
            case JS_EXECUTE_JOB_ID.JOB_DOWNLOAD_NEWS_ANALYSIS_BLOCKTRADING: //大宗交易
            case JS_EXECUTE_JOB_ID.JOB_DOWNLOAD_NEWS_ANALYSIS_COMPANYNEWS: //官网新闻
            case JS_EXECUTE_JOB_ID.JOB_DOWNLOAD_NEWS_ANALYSIS_TOPMANAGERS: //高管要闻
            case JS_EXECUTE_JOB_ID.JOB_DOWNLOAD_NEWS_ANALYSIS_PLEDGE:
                //股权质押
                return this.SymbolData.GetNewsAnalysisData(jobItem.ID);

            case JS_EXECUTE_JOB_ID.JOB_DOWNLOAD_HK_TO_SH_SZ: //北上总的
            case JS_EXECUTE_JOB_ID.JOB_DOWNLOAD_HK_TO_SH: //北上上证
            case JS_EXECUTE_JOB_ID.JOB_DOWNLOAD_HK_TO_SZ:
                //北上深证
                return this.SymbolData.GetHKToSHSZData(jobItem.ID);

            case JS_EXECUTE_JOB_ID.JOB_DOWNLOAD_SECTION_SF:
                return this.SymbolData.GetSectionFinanceData(jobItem); //财务截面报告数据

            case JS_EXECUTE_JOB_ID.JOB_CUSTOM_VARIANT_DATA:
                return this.SymbolData.DownloadCustomVariantData(jobItem);
            case JS_EXECUTE_JOB_ID.JOB_CUSTOM_FUNCTION_DATA:
                return this.SymbolData.DownloadCustomFunctionData(jobItem);

            case JS_EXECUTE_JOB_ID.JOB_RUN_SCRIPT:
                return this.Run();
        }
    };

    this.ReadSymbolData = function (name, node) {
        switch (name) {
            case 'CLOSE':
            case 'C':
            case 'VOL':
            case 'V':
            case 'OPEN':
            case 'O':
            case 'HIGH':
            case 'H':
            case 'LOW':
            case 'L':
            case 'AMOUNT':
                return this.SymbolData.GetSymbolCacheData(name);
            case 'VOLR':
                return this.SymbolData.GetVolRateCacheData(node);

            //大盘数据
            case 'INDEXA':
            case 'INDEXC':
            case 'INDEXH':
            case 'INDEXH':
            case 'INDEXO':
            case 'INDEXV':
            case 'INDEXL':
            case 'INDEXADV':
            case 'INDEXDEC':
                return this.SymbolData.GetIndexCacheData(name);
            case 'CURRBARSCOUNT':
                return this.SymbolData.GetCurrBarsCount();
            case 'ISLASTBAR':
                return this.SymbolData.GetIsLastBar();
            case 'CAPITAL':
                return this.SymbolData.GetFinanceCacheData(JS_EXECUTE_JOB_ID.JOB_DOWNLOAD_CAPITAL_DATA);
            case 'EXCHANGE':
                return this.SymbolData.GetFinanceCacheData(JS_EXECUTE_JOB_ID.JOB_DOWNLOAD_EXCHANGE_DATA);
            case 'SETCODE':
                return this.SymbolData.SETCODE();
            case 'CODE':
                return this.SymbolData.GetSymbol();
            case 'STKNAME':
                return this.SymbolData.GetName();
            case 'HYBLOCK':
                return this.SymbolData.GetIndustry();
            case 'DYBLOCK':
                return this.SymbolData.GetRegion();
            case 'GNBLOCK':
                return this.SymbolData.GetConcept();

            case 'DATE':
                return this.SymbolData.DATE();
            case 'YEAR':
                return this.SymbolData.YEAR();
            case 'MONTH':
                return this.SymbolData.MONTH();
            case 'WEEK':
                return this.SymbolData.WEEK();
            case 'PERIOD':
                return this.SymbolData.PERIOD();
            case 'FROMOPEN':
                return this.SymbolData.GetLatestIndexCacheData('FROMOPEN');

            case 'DRAWNULL':
                return this.SymbolData.GetDrawNull();
        }
    };

    this.ReadCustomVariant = function (name, node) {
        var item = g_JSComplierResource.CustomVariant.Data.get(name);
        if (!item || !item.Read) return [];

        var obj = { Name: name, Symbol: this.SymbolData.Symbol, KData: this.SymbolData.Data, Period: this.SymbolData.Period, Right: this.SymbolData.Right };
        return item.Read(obj);
    };

    //读取变量
    this.ReadVariable = function (name, node) {
        if (this.ConstVarTable.has(name)) {
            var data = this.ConstVarTable.get(name);

            if (data == null) //动态加载,用到再加载
                {
                    data = this.ReadSymbolData(name, node);
                    this.ConstVarTable.set(name, data);
                }

            return data;
        }

        if (g_JSComplierResource.IsCustomVariant(name)) return this.ReadCustomVariant(name, node); //读取自定义变量

        if (this.VarTable.has(name)) return this.VarTable.get(name);

        this.ThrowUnexpectedNode(node, '变量' + name + '不存在');
        return null;
    };

    //单数据转成数组 个数和历史数据一致
    this.SingleDataToArrayData = function (value) {
        var count = this.SymbolData.Data.Data.length;
        var result = [];
        for (var _i103 = 0; _i103 < count; ++_i103) {
            result[_i103] = value;
        }

        return result;
    };

    this.RunAST = function () {
        //预定义的变量
        for (var _i104 in this.Arguments) {
            var _item10 = this.Arguments[_i104];
            this.VarTable.set(_item10.Name, _item10.Value);
        }

        if (!this.AST) this.ThrowError();
        if (!this.AST.Body) this.ThrowError();

        for (var _i105 in this.AST.Body) {
            var _item11 = this.AST.Body[_i105];
            this.VisitNode(_item11);

            //输出变量
            if (_item11.Type == Syntax.ExpressionStatement && _item11.Expression) {
                if (_item11.Expression.Type == Syntax.AssignmentExpression && _item11.Expression.Operator == ':' && _item11.Expression.Left) {
                    var assignmentItem = _item11.Expression;
                    var varName = assignmentItem.Left.Name;
                    var _outVar = this.VarTable.get(varName);
                    if (!this.IsSectionMode && !Array.isArray(_outVar)) {
                        if (typeof _outVar == 'string') _outVar = this.SingleDataToArrayData(parseFloat(_outVar));else _outVar = this.SingleDataToArrayData(_outVar);
                    }

                    this.OutVarTable.push({ Name: varName, Data: _outVar, Type: 0 });
                } else if (_item11.Expression.Type == Syntax.CallExpression) {
                    var callItem = _item11.Expression;
                    if (this.Draw.IsDrawFunction(callItem.Callee.Name)) {
                        var draw = callItem.Draw;
                        draw.Name = callItem.Callee.Name;
                        this.OutVarTable.push({ Name: draw.Name, Draw: draw, Type: 1 });
                    }
                } else if (_item11.Expression.Type == Syntax.SequenceExpression) {
                    var _varName = void 0;
                    var _draw = void 0;
                    var color = void 0;
                    var lineWidth = void 0;
                    var colorStick = false;
                    var pointDot = false;
                    var circleDot = false;
                    var lineStick = false;
                    var stick = false;
                    var volStick = false;
                    var isShow = true;
                    var isExData = false;
                    var isDotLine = false;
                    for (var _j15 in _item11.Expression.Expression) {
                        var itemExpression = _item11.Expression.Expression[_j15];
                        if (itemExpression.Type == Syntax.AssignmentExpression && itemExpression.Operator == ':' && itemExpression.Left) {
                            _varName = itemExpression.Left.Name;
                            var varValue = this.VarTable.get(_varName);
                            if (!Array.isArray(varValue)) {
                                varValue = this.SingleDataToArrayData(varValue);
                                this.VarTable.set(_varName, varValue); //把常量放到变量表里
                            }
                        } else if (itemExpression.Type == Syntax.Identifier) {
                            var value = itemExpression.Name;
                            if (value === 'COLORSTICK') colorStick = true;else if (value === 'POINTDOT') pointDot = true;else if (value === 'CIRCLEDOT') circleDot = true;else if (value == 'DOTLINE') isDotLine = true;else if (value === 'LINESTICK') lineStick = true;else if (value === 'STICK') stick = true;else if (value === 'VOLSTICK') volStick = true;else if (value.indexOf('COLOR') == 0) color = value;else if (value.indexOf('LINETHICK') == 0) lineWidth = value;else if (value.indexOf('NODRAW') == 0) isShow = false;else if (value.indexOf('EXDATA') == 0) isExData = true; //扩展数据, 不显示再图形里面
                        } else if (itemExpression.Type == Syntax.Literal) //常量
                            {
                                var aryValue = this.SingleDataToArrayData(itemExpression.Value);
                                _varName = itemExpression.Value.toString();
                                this.VarTable.set(_varName, aryValue); //把常量放到变量表里
                            } else if (itemExpression.Type == Syntax.CallExpression && this.Draw.IsDrawFunction(itemExpression.Callee.Name)) {
                            _draw = itemExpression.Draw;
                            _draw.Name = itemExpression.Callee.Name;
                        }
                    }

                    if (pointDot && _varName) //圆点
                        {
                            var _outVar2 = this.VarTable.get(_varName);
                            if (!Array.isArray(_outVar2)) _outVar2 = this.SingleDataToArrayData(_outVar2);
                            var _value = { Name: _varName, Data: _outVar2, Radius: 2, Type: 3 };
                            if (color) _value.Color = color;
                            if (lineWidth) _value.LineWidth = lineWidth;
                            this.OutVarTable.push(_value);
                        } else if (circleDot && _varName) //圆点
                        {
                            var _outVar3 = this.VarTable.get(_varName);
                            if (!Array.isArray(_outVar3)) _outVar3 = this.SingleDataToArrayData(_outVar3);
                            var _value2 = { Name: _varName, Data: _outVar3, Radius: 1.3, Type: 3 };
                            if (color) _value2.Color = color;
                            if (lineWidth) _value2.LineWidth = lineWidth;
                            this.OutVarTable.push(_value2);
                        } else if (lineStick && _varName) //LINESTICK  同时画出柱状线和指标线
                        {
                            var _outVar4 = this.VarTable.get(_varName);
                            var _value3 = { Name: _varName, Data: _outVar4, Type: 4 };
                            if (color) _value3.Color = color;
                            if (lineWidth) _value3.LineWidth = lineWidth;
                            this.OutVarTable.push(_value3);
                        } else if (stick && _varName) //STICK 画柱状线
                        {
                            var _outVar5 = this.VarTable.get(_varName);
                            var _value4 = { Name: _varName, Data: _outVar5, Type: 5 };
                            if (color) _value4.Color = color;
                            if (lineWidth) _value4.LineWidth = lineWidth;
                            this.OutVarTable.push(_value4);
                        } else if (volStick && _varName) //VOLSTICK   画彩色柱状线
                        {
                            var _outVar6 = this.VarTable.get(_varName);
                            var _value5 = { Name: _varName, Data: _outVar6, Type: 6 };
                            if (color) _value5.Color = color;
                            this.OutVarTable.push(_value5);
                        } else if (_varName && color) {
                        var _outVar7 = this.VarTable.get(_varName);
                        if (!Array.isArray(_outVar7)) _outVar7 = this.SingleDataToArrayData(_outVar7);
                        var _value6 = { Name: _varName, Data: _outVar7, Color: color, Type: 0 };
                        if (lineWidth) _value6.LineWidth = lineWidth;
                        if (isShow == false) _value6.IsShow = false;
                        if (isExData == true) _value6.IsExData = true;
                        if (isDotLine == true) _value6.IsDotLine = true;
                        this.OutVarTable.push(_value6);
                    } else if (_draw) //画图函数
                        {
                            var outVar = { Name: _draw.Name, Draw: _draw, Type: 1 };
                            if (color) outVar.Color = color;
                            if (lineWidth) outVar.LineWidth = lineWidth;
                            this.OutVarTable.push(outVar);
                        } else if (colorStick && _varName) //CYW: SUM(VAR4,10)/10000, COLORSTICK; 画上下柱子
                        {
                            var _outVar8 = this.VarTable.get(_varName);
                            var _value7 = { Name: _varName, Data: _outVar8, Color: color, Type: 2 };
                            this.OutVarTable.push(_value7);
                        } else if (_varName) {
                        var _outVar9 = this.VarTable.get(_varName);
                        var _value8 = { Name: _varName, Data: _outVar9, Type: 0 };
                        if (color) _value8.Color = color;
                        if (lineWidth) _value8.LineWidth = lineWidth;
                        if (isShow == false) _value8.IsShow = false;
                        if (isExData == true) _value8.IsExData = true;
                        if (isDotLine == true) _value8.IsDotLine = true;
                        this.OutVarTable.push(_value8);
                    }
                }
            }
        }

        JSConsole.Complier.Log('[JSExecute::Run]', this.VarTable);

        return this.OutVarTable;
    };

    this.Run = function () {
        try {
            var data = this.RunAST(); //执行脚本
            JSConsole.Complier.Log('[JSComplier.Run] execute finish', data);

            if (this.UpdateUICallback) {
                JSConsole.Complier.Log('[JSComplier.Run] invoke UpdateUICallback.');
                if (this.CallbackParam && this.CallbackParam.Self && this.CallbackParam.Self.ClassName === 'ScriptIndexConsole') this.CallbackParam.JSExecute = this;
                this.UpdateUICallback(data, this.CallbackParam);
            }
        } catch (error) {
            JSConsole.Complier.Log(error);
            if (error.Job) {
                JSConsole.Complier.Log('[JSComplier.Run] download job and reexectue', error.Job);
                this.JobList.push(error.Job);
                this.JobList.push({ ID: JS_EXECUTE_JOB_ID.JOB_RUN_SCRIPT });
                this.Execute();
            } else if (this.ErrorCallback) {
                this.ErrorCallback(error);
            }
        }
    };

    this.VisitNode = function (node) {
        switch (node.Type) {
            case Syntax.SequenceExpression:
                this.VisitSequenceExpression(node);
                break;
            case Syntax.ExpressionStatement:
                this.VisitNode(node.Expression);
                break;
            case Syntax.AssignmentExpression:
                this.VisitAssignmentExpression(node);
                break;
            case Syntax.BinaryExpression:
            case Syntax.LogicalExpression:
                this.VisitBinaryExpression(node);
                break;
            case Syntax.CallExpression:
                this.VisitCallExpression(node);
                break;
        }
    };

    this.VisitSequenceExpression = function (node) {
        for (var _i106 in node.Expression) {
            var _item12 = node.Expression[_i106];
            this.VisitNode(_item12);
        }
    };

    //函数调用
    this.VisitCallExpression = function (node) {
        var funcName = node.Callee.Name;
        var args = [];
        for (var _i107 in node.Arguments) {
            var _item13 = node.Arguments[_i107];
            var value = void 0;
            if (_item13.Type == Syntax.BinaryExpression || _item13.Type == Syntax.LogicalExpression) value = this.VisitBinaryExpression(_item13);else if (_item13.Type == Syntax.CallExpression) value = this.VisitCallExpression(_item13);else value = this.GetNodeValue(_item13);
            args.push(value);
        }

        if (JS_EXECUTE_DEBUG_LOG) JSConsole.Complier.Log('[JSExecute::VisitCallExpression]', funcName, '(', args.toString(), ')');

        switch (funcName) {
            case 'DYNAINFO':
                //行情最新数据
                node.Out = this.SymbolData.GetLatestCacheData(args[0]);
                break;
            case 'STICKLINE':
                node.Draw = this.Draw.STICKLINE(args[0], args[1], args[2], args[3], args[4]);
                node.Out = [];
                break;
            case 'DRAWTEXT':
                node.Draw = this.Draw.DRAWTEXT(args[0], args[1], args[2]);
                node.Out = [];
                break;
            case 'DRAWTEXT_FIX':
                node.Draw = this.Draw.DRAWTEXT_FIX(args[0], args[1], args[2], args[3], args[4]);
                node.Out = [];
                break;
            case 'SUPERDRAWTEXT':
                node.Draw = this.Draw.SUPERDRAWTEXT(args[0], args[1], args[2], args[3], args[4]);
                node.Out = [];
                break;
            case 'DRAWICON':
                node.Draw = this.Draw.DRAWICON(args[0], args[1], args[2]);
                node.Out = [];
                break;
            case 'DRAWLINE':
                node.Draw = this.Draw.DRAWLINE(args[0], args[1], args[2], args[3], args[4]);
                node.Out = node.Draw.DrawData;
                break;
            case 'DRAWBAND':
                node.Draw = this.Draw.DRAWBAND(args[0], args[1], args[2], args[3]);
                node.Out = [];
                break;
            case 'DRAWKLINE':
                node.Draw = this.Draw.DRAWKLINE(args[0], args[1], args[2], args[3]);
                node.Out = [];
                break;
            case 'DRAWKLINE_IF':
                node.Draw = this.Draw.DRAWKLINE_IF(args[0], args[1], args[2], args[3], args[4]);
                node.Out = [];
                break;
            case 'PLOYLINE':
            case 'POLYLINE':
                node.Draw = this.Draw.POLYLINE(args[0], args[1]);
                node.Out = node.Draw.DrawData;
                break;
            case 'DRAWNUMBER':
                node.Draw = this.Draw.DRAWNUMBER(args[0], args[1], args[2]);
                node.Out = node.Draw.DrawData.Value;
                break;
            case "DRAWCHANNEL":
                node.Draw = this.Draw.DRAWCHANNEL(args[0], args[1], args[2], args[3], args[4], args[5], args[6]);
                node.Out = [];
                break;
            case 'RGB':
                node.Out = this.Draw.RGB(args[0], args[1], args[2]);
                break;
            case 'PARTLINE':
                node.Draw = this.Draw.PARTLINE(args);
                node.Out = [];
                break;
            case 'DRAWGBK':
                node.Draw = this.Draw.DRAWGBK(args[0], args[1], args[2], args[3]);
                node.Out = [];
                break;
            case 'DRAWTEXT_LINE':
                node.Draw = this.Draw.DRAWTEXT_LINE(args[0], args[1], args[2], args[3], args[4], args[5], args[6]);
                node.Out = [];
                break;
            case 'DRAWRECTREL':
                node.Draw = this.Draw.DRAWRECTREL(args[0], args[1], args[2], args[3], args[4]);
                node.Out = [];
                break;
            case 'CODELIKE':
                node.Out = this.SymbolData.CODELIKE(args[0]);
                break;
            case 'NAMELIKE':
                node.Out = this.SymbolData.NAMELIKE(args[1]);
                break;
            case 'REFDATE':
                node.Out = this.SymbolData.REFDATE(args[0], args[1]);
                break;
            case 'FINANCE':
                node.Out = this.SymbolData.GetFinanceCacheData(args[0], node);
                break;
            case "MARGIN":
                node.Out = this.SymbolData.GetMarginCacheData(args[0], node);
                break;
            case "HK2SHSZ":
                node.Out = this.SymbolData.GetHKToSHSZCacheData(args[0], node);
                break;
            case "NEWS":
                node.Out = this.SymbolData.GetNewsAnalysisCacheData(args[0], node);
                break;
            case 'UPCOUNT':
            case 'DOWNCOUNT':
                node.Out = this.SymbolData.GetIndexIncreaseCacheData(funcName, args[0], node);
                break;
            case 'SF':
                node.Out = this.SymbolData.GetSectionFinanceCacheData(args[0], args[1], args[2], node);
                break;
            default:
                node.Out = this.Algorithm.CallFunction(funcName, args, node, this.SymbolData);
                break;
        }

        return node.Out;
    };

    //赋值
    this.VisitAssignmentExpression = function (node) {
        var left = node.Left;
        if (left.Type != Syntax.Identifier) this.ThrowUnexpectedNode(node);

        var varName = left.Name;

        var right = node.Right;
        var value = null;
        if (right.Type == Syntax.BinaryExpression || right.Type == Syntax.LogicalExpression) value = this.VisitBinaryExpression(right);else if (right.Type == Syntax.CallExpression) value = this.VisitCallExpression(right);else if (right.Type == Syntax.Literal) value = right.Value;else if (right.Type == Syntax.Identifier) //右值是变量
            value = this.ReadVariable(right.Name, right);

        if (JS_EXECUTE_DEBUG_LOG) JSConsole.Complier.Log('[JSExecute::VisitAssignmentExpression]', varName, ' = ', value);
        this.VarTable.set(varName, value);
    };

    //逻辑运算
    this.VisitBinaryExpression = function (node) {
        var stack = [];
        stack.push(node);
        var temp = null;

        while (stack.length != 0) {
            temp = stack[stack.length - 1];
            if (temp.Left && node != temp.Left && node != temp.Right) {
                stack.push(temp.Left);
            } else if (temp.Right && node != temp.Right) {
                stack.push(temp.Right);
            } else {
                var value = stack.pop();
                if (value.Type == Syntax.BinaryExpression) //只遍历操作符就可以
                    {
                        var leftValue = this.GetNodeValue(value.Left);
                        var rightValue = this.GetNodeValue(value.Right);

                        if (JS_EXECUTE_DEBUG_LOG) JSConsole.Complier.Log('[JSExecute::VisitBinaryExpression] BinaryExpression', value, leftValue, rightValue);
                        value.Out = null; //保存中间值

                        switch (value.Operator) {
                            case '-':
                                value.Out = this.Algorithm.Subtract(leftValue, rightValue);
                                break;
                            case '*':
                                value.Out = this.Algorithm.Multiply(leftValue, rightValue);
                                break;
                            case '/':
                                value.Out = this.Algorithm.Divide(leftValue, rightValue);
                                break;
                            case '+':
                                value.Out = this.Algorithm.Add(leftValue, rightValue);
                                break;
                            case '>':
                                value.Out = this.Algorithm.GT(leftValue, rightValue);
                                break;
                            case '>=':
                                value.Out = this.Algorithm.GTE(leftValue, rightValue);
                                break;
                            case '<':
                                value.Out = this.Algorithm.LT(leftValue, rightValue);
                                break;
                            case '<=':
                                value.Out = this.Algorithm.LTE(leftValue, rightValue);
                                break;
                            case '==':
                            case '=':
                                //= 比较
                                value.Out = this.Algorithm.EQ(leftValue, rightValue);
                                break;
                            case '!=':
                            case '<>':
                                value.Out = this.Algorithm.NEQ(leftValue, rightValue);
                                break;
                        }

                        if (JS_EXECUTE_DEBUG_LOG) JSConsole.Complier.Log('[JSExecute::VisitBinaryExpression] BinaryExpression', value);
                    } else if (value.Type == Syntax.LogicalExpression) {
                    var _leftValue = this.GetNodeValue(value.Left);
                    var _rightValue = this.GetNodeValue(value.Right);

                    if (JS_EXECUTE_DEBUG_LOG) JSConsole.Complier.Log('[JSExecute::VisitBinaryExpression] LogicalExpression', value, _leftValue, _rightValue);
                    value.Out = null; //保存中间值

                    switch (value.Operator) {
                        case '&&':
                        case 'AND':
                            value.Out = this.Algorithm.And(_leftValue, _rightValue);
                            break;
                        case '||':
                        case 'OR':
                            value.Out = this.Algorithm.Or(_leftValue, _rightValue);
                            break;
                    }

                    if (JS_EXECUTE_DEBUG_LOG) JSConsole.Complier.Log('[JSExecute::VisitBinaryExpression] LogicalExpression', value);
                }

                node = temp;
            }
        }

        return node.Out;
    };

    this.GetNodeValue = function (node) {
        switch (node.Type) {
            case Syntax.Literal:
                //数字
                return node.Value;
            case Syntax.UnaryExpression:
                if (node.Operator == '-') {
                    var _value9 = this.GetNodeValue(node.Argument);
                    return this.Algorithm.Subtract(0, _value9);
                }
                return node.Argument.Value;
            case Syntax.Identifier:
                var value = this.ReadVariable(node.Name, node);
                return value;
            case Syntax.BinaryExpression:
            case Syntax.LogicalExpression:
                return node.Out;
            case Syntax.CallExpression:
                return this.VisitCallExpression(node);
            default:
                this.ThrowUnexpectedNode(node);
        }
    };

    this.ThrowUnexpectedNode = function (node, message) {
        var marker = node.Marker;
        var msg = message || "执行异常";

        return this.ErrorHandler.ThrowError(marker.Index, marker.Line, marker.Column, msg);
    };

    this.ThrowDownloadSF = function (node, job, message) {
        var marker = node.Marker;
        var msg = message;

        return this.ErrorHandler.ThrowDownloadJob(marker.Index, marker.Line, marker.Column, msg, job);
    };

    this.ThrowError = function () {};
}

//对外导出类
function JSComplier() {}

//词法分析
JSComplier.Tokenize = function (code) {
    JSConsole.Complier.Log('[JSComplier.Tokenize]', code);
    var tokenizer = new Tokenizer(code);
    var tokens = [];
    try {
        while (true) {
            var token = tokenizer.GetNextToken();
            if (!token) break;

            tokens.push(token);
        }
    } catch (e) {}

    return tokens;
};

//语法解析 生成抽象语法树(Abstract Syntax Tree)
JSComplier.Parse = function (code) {
    JSConsole.Complier.Log('[JSComplier.Parse]', code);

    var parser = new JSParser(code);
    parser.Initialize();
    var program = parser.ParseScript();
    var ast = program;
    return ast;
};

/*
    执行
    option.Symbol=股票代码
    option.Name=股票名称
    option.Data=这个股票的ChartData
    option.Right=复权
    option.MaxReqeustDataCount=请求数据的最大个数
*/

function timeout(ms) {
    return new Promise(function (resolve) {
        setTimeout(resolve, ms);
    });
}

JSComplier.Execute = function (code, option, errorCallback) {
    //异步调用
    //var asyncExecute= async function() es5不能执行 去掉异步
    var asyncExecute = function asyncExecute() {
        try {
            JSConsole.Complier.Log('[JSComplier.Execute]', code, option);

            JSConsole.Complier.Log('[JSComplier.Execute] parser .....');
            var parser = new JSParser(code);
            parser.Initialize();
            var program = parser.ParseScript();

            var ast = program;
            JSConsole.Complier.Log('[JSComplier.Execute] parser finish.', ast);

            JSConsole.Complier.Log('[JSComplier.Execute] execute .....');
            var execute = new JSExecute(ast, option);
            execute.ErrorCallback = errorCallback; //执行错误回调
            execute.JobList = parser.Node.GetDataJobList();
            if (option.ClassName == 'ScriptIndexConsole') execute.JobList.unshift({ ID: JS_EXECUTE_JOB_ID.JOB_DOWNLOAD_SYMBOL_DATA });
            execute.JobList.push({ ID: JS_EXECUTE_JOB_ID.JOB_RUN_SCRIPT });

            var _result = execute.Execute();
        } catch (error) {
            JSConsole.Complier.Log(error);

            if (errorCallback) errorCallback(error);
        }
    };

    asyncExecute();

    JSConsole.Complier.Log('[JSComplier.Execute] async execute.');
};

JSComplier.SetDomain = function (domain, cacheDomain) //修改API地址
{
    if (domain) g_JSComplierResource.Domain = domain;
    if (cacheDomain) g_JSComplierResource.CacheDomain = cacheDomain;
};

JSComplier.AddIcon = function (obj) //添加一个obj={ID:, Text:, Color, Family: }
{
    g_JSComplierResource.CustomDrawIcon.Data.set(obj.ID, obj);
};

JSComplier.AddFunction = function (obj) //添加函数
{
    if (!obj || !obj) return;

    var ID = obj.Name.toUpperCase();
    g_JSComplierResource.CustomFunction.Data.set(ID, obj);
};

JSComplier.AddVariant = function (obj) {
    if (!obj || !obj) return;

    var ID = obj.Name.toUpperCase();
    g_JSComplierResource.CustomVariant.Data.set(ID, obj);
};

var HQ_DATA_TYPE = {
    KLINE_ID: 0, //K线
    MINUTE_ID: 2, //当日走势图
    HISTORY_MINUTE_ID: 3, //历史分钟走势图
    MULTIDAY_MINUTE_ID: 4 //多日走势图
};

//脚本指标
//name=指标名字 args=参数名字 参数值
function ScriptIndex(name, script, args, option) {
    this.newMethod = BaseIndex; //派生
    this.newMethod(name);
    delete this.newMethod;

    this.Script = script;
    this.Arguments = [];
    this.OutVar = [];
    this.ID; //指标ID
    this.FloatPrecision = 2; //小数位数
    this.StringFormat;
    this.KLineType = null; //K线显示类型
    this.InstructionType; //五彩K线, 交易指标
    this.YSpecificMaxMin = null; //最大最小值
    this.YSplitScale = null; //固定刻度
    this.Condition = null; //限制条件

    //指标上锁配置信息
    this.IsLocked = false; //是否锁住指标
    this.LockCallback = null;
    this.LockID = null;
    this.LockBG = null; //锁背景色
    this.LockTextColor = null;
    this.LockText = null;
    this.LockFont = null;
    this.LockCount = 20;
    this.LockMinWidth = null;
    this.TitleFont = g_JSChartResource.TitleFont; //标题字体
    this.IsShortTitle = false; //是否显示指标参数

    if (option) {
        if (option.FloatPrecision >= 0) this.FloatPrecision = option.FloatPrecision;
        if (option.StringFormat > 0) this.StringFormat = option.StringFormat;
        if (option.ID) this.ID = option.ID;
        if (option.KLineType >= 0 || option.KLineType === -1) this.KLineType = option.KLineType;
        if (option.InstructionType) this.InstructionType = option.InstructionType;
        if (option.YSpecificMaxMin) this.YSpecificMaxMin = option.YSpecificMaxMin;
        if (option.YSplitScale) this.YSplitScale = option.YSplitScale;
        if (option.Condition) this.Condition = option.Condition;
        if (option.TitleFont) this.TitleFont = option.TitleFont;
        if (option.IsShortTitle) this.IsShortTitle = option.IsShortTitle;
    }

    if (option && option.Lock) {
        if (option.Lock.IsLocked == true) this.IsLocked = true; //指标上锁
        if (option.Lock.Callback) this.LockCallback = option.Lock.Callback; //锁回调
        if (option.Lock.ID) this.LockID = option.Lock.ID; //锁ID
        if (option.Lock.BG) this.LockBG = option.Lock.BG;
        if (option.Lock.TextColor) this.LockTextColor = option.Lock.TextColor;
        if (option.Lock.Text) this.LockText = option.Lock.Text;
        if (option.Lock.Font) this.LockFont = option.Lock.Font;
        if (option.Lock.Count) this.LockCount = option.Lock.Count;
        if (option.Lock.MinWidth) this.LockMinWidth = option.Lock.MinWidth * GetDevicePixelRatio();
    }

    if (args) this.Arguments = args;

    this.SetLock = function (lockData) {
        if (lockData.IsLocked == true) {
            this.IsLocked = true; //指标上锁
            if (lockData.Callback) this.LockCallback = lockData.Callback; //锁回调
            if (lockData.ID) this.LockID = lockData.ID; //锁ID
            if (lockData.BG) this.LockBG = lockData.BG;
            if (lockData.TextColor) this.LockTextColor = lockData.TextColor;
            if (lockData.Text) this.LockText = lockData.Text;
            if (lockData.Font) this.LockFont = lockData.Font;
            if (lockData.Count) this.LockCount = lockData.Count;
            if (lockData.MinWidth) this.LockMinWidth = lockData.MinWidth * GetDevicePixelRatio();
        } else {
            //清空锁配置信息
            this.IsLocked = false; //是否锁住指标
            this.LockCallback = null;
            this.LockID = null;
            this.LockBG = null; //锁背景色
            this.LockTextColor = null;
            this.LockText = null;
            this.LockFont = null;
            this.LockCount = 20;
        }
    };

    this.ExecuteScript = function (hqChart, windowIndex, hisData) {
        this.OutVar = [];
        var self = this;
        var param = {
            HQChart: hqChart,
            WindowIndex: windowIndex,
            HistoryData: hisData,
            Self: this
        };

        //数据类型
        var hqDataType = HQ_DATA_TYPE.KLINE_ID; //默认K线
        if (hqChart.ClassName === 'MinuteChartContainer' || hqChart.ClassName === 'MinuteChartHScreenContainer') {
            if (hqChart.DayCount > 1) hqDataType = HQ_DATA_TYPE.MULTIDAY_MINUTE_ID; //多日分钟
            else hqDataType = HQ_DATA_TYPE.MINUTE_ID; //分钟数据
        } else if (hqChart.ClassName === 'HistoryMinuteChartContainer') {
            hqDataType = HQ_DATA_TYPE.HISTORY_MINUTE_ID; //历史分钟
        }
        var option = {
            HQDataType: hqDataType,
            Symbol: hqChart.Symbol,
            Name: hqChart.Name,
            Data: hisData,
            SourceData: hqChart.SourceData,
            Callback: this.RecvResultData, CallbackParam: param,
            Async: true,
            MaxRequestDataCount: hqChart.MaxReqeustDataCount,
            MaxRequestMinuteDayCount: hqChart.MaxRequestMinuteDayCount,
            Arguments: this.Arguments,
            Condition: this.Condition,
            IsBeforeData: hqChart.IsBeforeData
        };

        if (hqDataType === HQ_DATA_TYPE.HISTORY_MINUTE_ID) option.TrateDate = hqChart.TradeDate;
        if (hqDataType === HQ_DATA_TYPE.MULTIDAY_MINUTE_ID) option.DayCount = hqChart.DayCount;
        if (hqChart.NetworkFilter) option.NetworkFilter = hqChart.NetworkFilter;

        if (this.Condition && !this.IsMeetCondition(param, option)) {
            this.ShowConditionError(param);
            return;
        }

        var code = this.Script;
        var run = JSComplier.Execute(code, option, hqChart.ScriptErrorCallback);
    };

    //是否符合限制条件
    this.IsMeetCondition = function (param, option) {
        JSConsole.Complier.Log('[ScriptIndex::IsMeetCondition] ', this.Condition);
        if (this.Condition.Period) //周期是否满足
            {
                if (!this.IsMeetPeriodCondition(param, option)) return false;
            }

        return true;
    };

    //周期是否满足条件
    this.IsMeetPeriodCondition = function (param, option) {
        if (!this.Condition.Period) return true;

        for (var i in this.Condition.Period) {
            var item = this.Condition.Period[i];
            switch (item) {
                case CONDITION_PERIOD.MINUTE_ID:
                    if (option.HQDataType == HQ_DATA_TYPE.MINUTE_ID) return true;
                    break;
                case CONDITION_PERIOD.MULTIDAY_MINUTE_ID:
                    if (option.HQDataType == HQ_DATA_TYPE.MULTIDAY_MINUTE_ID) return true;
                    break;
                case CONDITION_PERIOD.KLINE_DAY_ID:
                case CONDITION_PERIOD.KLINE_WEEK_ID:
                case CONDITION_PERIOD.KLINE_MONTH_ID:
                case CONDITION_PERIOD.KLINE_YEAR_ID:

                case CONDITION_PERIOD.KLINE_MINUTE_ID:
                case CONDITION_PERIOD.KLINE_5_MINUTE_ID:
                case CONDITION_PERIOD.KLINE_15_MINUTE_ID:
                case CONDITION_PERIOD.KLINE_30_MINUTE_ID:
                case CONDITION_PERIOD.KLINE_60_MINUTE_ID:
                    if (param.HQChart.Period == item) return true;
                    break;
            }
        }

        return false;
    };

    //显示指标不符合条件
    this.ShowConditionError = function (param) {
        var hqChart = param.HQChart;
        var windowIndex = param.WindowIndex;

        hqChart.DeleteIndexPaint(windowIndex);
        if (windowIndex == 0) hqChart.ShowKLine(true);

        var message = '指标不支持当前品种或周期';

        var line = new ChartLine();
        line.Canvas = hqChart.Canvas;
        line.ChartBorder = hqChart.Frame.SubFrame[windowIndex].Frame.ChartBorder;
        line.ChartFrame = hqChart.Frame.SubFrame[windowIndex].Frame;
        line.NotSupportMessage = message;
        hqChart.ChartPaint.push(line);
        hqChart.Draw();
    };

    this.RecvResultData = function (outVar, param) {
        var hqChart = param.HQChart;
        var windowIndex = param.WindowIndex;
        var hisData = param.HistoryData;
        param.Self.OutVar = outVar;
        param.Self.BindData(hqChart, windowIndex, hisData);

        if (param.Self.IsLocked == false) //不上锁
            {
                param.HQChart.Frame.SubFrame[windowIndex].Frame.SetLock(null);
            } else //上锁
            {
                var lockData = { IsLocked: true, Callback: param.Self.LockCallback, IndexName: param.Self.Name, ID: param.Self.LockID,
                    BG: param.Self.LockBG, Text: param.Self.LockText, TextColor: param.Self.LockTextColor, Font: param.Self.LockFont, Count: param.Self.LockCount, MinWidth: param.Self.LockMinWidth };
                param.HQChart.Frame.SubFrame[windowIndex].Frame.SetLock(lockData);
            }

        param.HQChart.UpdataDataoffset(); //更新数据偏移
        param.HQChart.UpdateFrameMaxMin(); //调整坐标最大 最小值
        param.HQChart.Draw();

        if (hqChart.GetIndexEvent) {
            var event = hqChart.GetIndexEvent(); //指标计算完成回调
            if (event) {
                var self = param.Self;
                var data = { OutVar: self.OutVar, WindowIndex: windowIndex, Name: self.Name, Arguments: self.Arguments, HistoryData: hisData,
                    Stock: { Symbol: hqChart.Symbol, Name: hqChart.Name } };
                event.Callback(event, data, self);
            }
        }
    };

    this.CreateLine = function (hqChart, windowIndex, varItem, id) {
        var line = new ChartLine();
        line.Canvas = hqChart.Canvas;
        line.DrawType = 1;
        line.Name = varItem.Name;
        line.ChartBorder = hqChart.Frame.SubFrame[windowIndex].Frame.ChartBorder;
        line.ChartFrame = hqChart.Frame.SubFrame[windowIndex].Frame;
        if (varItem.Color) line.Color = this.GetColor(varItem.Color);else line.Color = this.GetDefaultColor(id);

        if (varItem.LineWidth) {
            var width = parseInt(varItem.LineWidth.replace("LINETHICK", ""));
            if (!isNaN(width) && width > 0) line.LineWidth = width;
        }

        if (varItem.IsDotLine) line.IsDotLine = true; //虚线
        if (varItem.IsShow == false) line.IsShow = false;

        var titleIndex = windowIndex + 1;
        line.Data.Data = varItem.Data;
        hqChart.TitlePaint[titleIndex].Data[id] = new DynamicTitleData(line.Data, varItem.Name, line.Color);

        hqChart.ChartPaint.push(line);
    };

    //创建柱子
    this.CreateBar = function (hqChart, windowIndex, varItem, id) {
        var bar = new ChartStickLine();
        bar.Canvas = hqChart.Canvas;
        if (varItem.Draw.Width > 0) bar.LineWidth = varItem.Draw.Width;else bar.LineWidth = 1;

        bar.Name = varItem.Name;
        bar.ChartBorder = hqChart.Frame.SubFrame[windowIndex].Frame.ChartBorder;
        bar.ChartFrame = hqChart.Frame.SubFrame[windowIndex].Frame;
        if (varItem.Color) bar.Color = this.GetColor(varItem.Color);else bar.Color = this.GetDefaultColor(id);

        var titleIndex = windowIndex + 1;
        bar.Data.Data = varItem.Draw.DrawData;
        bar.BarType = varItem.Draw.Type;

        //hqChart.TitlePaint[titleIndex].Data[id]=new DynamicTitleData(bar.Data,varItem.Name,bar.Color);

        hqChart.ChartPaint.push(bar);
    };

    //创建文本
    this.CreateText = function (hqChart, windowIndex, varItem, id) {
        var chartText = new ChartSingleText();
        chartText.Canvas = hqChart.Canvas;

        chartText.Name = varItem.Name;
        chartText.ChartBorder = hqChart.Frame.SubFrame[windowIndex].Frame.ChartBorder;
        chartText.ChartFrame = hqChart.Frame.SubFrame[windowIndex].Frame;
        if (varItem.Color) chartText.Color = this.GetColor(varItem.Color);else chartText.Color = this.GetDefaultColor(id);

        var titleIndex = windowIndex + 1;
        if (varItem.Draw.Position) chartText.Position = varItem.Draw.Position; //赋值坐标
        if (varItem.Draw.DrawData) chartText.Data.Data = varItem.Draw.DrawData;
        chartText.Text = varItem.Draw.Text;
        if (varItem.Draw.Direction > 0) chartText.Direction = varItem.Draw.Direction;
        if (varItem.Draw.YOffset > 0) chartText.YOffset = varItem.Draw.YOffset;
        if (varItem.Draw.TextAlign) chartText.TextAlign = varItem.Draw.TextAlign;

        //hqChart.TitlePaint[titleIndex].Data[id]=new DynamicTitleData(bar.Data,varItem.Name,bar.Color);

        hqChart.ChartPaint.push(chartText);
    };

    //COLORSTICK 
    this.CreateMACD = function (hqChart, windowIndex, varItem, id) {
        var chartMACD = new ChartMACD();
        chartMACD.Canvas = hqChart.Canvas;

        chartMACD.Name = varItem.Name;
        chartMACD.ChartBorder = hqChart.Frame.SubFrame[windowIndex].Frame.ChartBorder;
        chartMACD.ChartFrame = hqChart.Frame.SubFrame[windowIndex].Frame;

        var titleIndex = windowIndex + 1;
        chartMACD.Data.Data = varItem.Data;
        hqChart.TitlePaint[titleIndex].Data[id] = new DynamicTitleData(chartMACD.Data, varItem.Name, this.GetDefaultColor(id));

        hqChart.ChartPaint.push(chartMACD);
    };

    this.CreatePointDot = function (hqChart, windowIndex, varItem, id) {
        var pointDot = new ChartPointDot();
        pointDot.Canvas = hqChart.Canvas;
        pointDot.Name = varItem.Name;
        pointDot.ChartBorder = hqChart.Frame.SubFrame[windowIndex].Frame.ChartBorder;
        pointDot.ChartFrame = hqChart.Frame.SubFrame[windowIndex].Frame;
        if (varItem.Color) pointDot.Color = this.GetColor(varItem.Color);else pointDot.Color = this.GetDefaultColor(id);

        if (varItem.Radius) pointDot.Radius = varItem.Radius;

        if (varItem.LineWidth) {
            var width = parseInt(varItem.LineWidth.replace("LINETHICK", ""));
            if (!isNaN(width) && width > 0) pointDot.Radius = width;
        }

        var titleIndex = windowIndex + 1;
        pointDot.Data.Data = varItem.Data;
        hqChart.TitlePaint[titleIndex].Data[id] = new DynamicTitleData(pointDot.Data, varItem.Name, pointDot.Color);

        hqChart.ChartPaint.push(pointDot);
    };

    this.CreateStick = function (hqChart, windowIndex, varItem, id) {
        var chart = new ChartStick();
        chart.Canvas = hqChart.Canvas;
        chart.Name = varItem.Name;
        chart.ChartBorder = hqChart.Frame.SubFrame[windowIndex].Frame.ChartBorder;
        chart.ChartFrame = hqChart.Frame.SubFrame[windowIndex].Frame;
        if (varItem.Color) chart.Color = this.GetColor(varItem.Color);else chart.Color = this.GetDefaultColor(id);

        if (varItem.LineWidth) {
            var width = parseInt(varItem.LineWidth.replace("LINETHICK", ""));
            if (!isNaN(width) && width > 0) chart.LineWidth = width;
        }

        var titleIndex = windowIndex + 1;
        chart.Data.Data = varItem.Data;
        hqChart.TitlePaint[titleIndex].Data[id] = new DynamicTitleData(chart.Data, varItem.Name, chart.Color);

        hqChart.ChartPaint.push(chart);
    };

    this.CreateLineStick = function (hqChart, windowIndex, varItem, id) {
        var chart = new ChartLineStick();
        chart.Canvas = hqChart.Canvas;
        chart.Name = varItem.Name;
        chart.ChartBorder = hqChart.Frame.SubFrame[windowIndex].Frame.ChartBorder;
        chart.ChartFrame = hqChart.Frame.SubFrame[windowIndex].Frame;
        if (varItem.Color) chart.Color = this.GetColor(varItem.Color);else chart.Color = this.GetDefaultColor(id);

        if (varItem.LineWidth) {
            var width = parseInt(varItem.LineWidth.replace("LINETHICK", ""));
            if (!isNaN(width) && width > 0) chart.LineWidth = width;
        }

        var titleIndex = windowIndex + 1;
        chart.Data.Data = varItem.Data;
        hqChart.TitlePaint[titleIndex].Data[id] = new DynamicTitleData(chart.Data, varItem.Name, chart.Color);

        hqChart.ChartPaint.push(chart);
    };

    this.CreateStraightLine = function (hqChart, windowIndex, varItem, id) {
        var line = new ChartLine();
        line.DrawType = 1;
        line.Canvas = hqChart.Canvas;
        line.Name = varItem.Name;
        line.ChartBorder = hqChart.Frame.SubFrame[windowIndex].Frame.ChartBorder;
        line.ChartFrame = hqChart.Frame.SubFrame[windowIndex].Frame;
        if (varItem.Color) line.Color = this.GetColor(varItem.Color);else line.Color = this.GetDefaultColor(id);

        if (varItem.LineWidth) {
            var width = parseInt(varItem.LineWidth.replace("LINETHICK", ""));
            if (!isNaN(width) && width > 0) line.LineWidth = width;
        }

        var titleIndex = windowIndex + 1;
        line.Data.Data = varItem.Draw.DrawData;
        //hqChart.TitlePaint[titleIndex].Data[id]=new DynamicTitleData(line.Data,varItem.Name,line.Color);

        hqChart.ChartPaint.push(line);
    };

    this.CreateVolStick = function (hqChart, windowIndex, varItem, id, hisData) {
        var chart = new ChartVolStick();
        chart.Canvas = hqChart.Canvas;
        chart.Name = varItem.Name;
        chart.ChartBorder = hqChart.Frame.SubFrame[windowIndex].Frame.ChartBorder;
        chart.ChartFrame = hqChart.Frame.SubFrame[windowIndex].Frame;
        chart.KLineDrawType = hqChart.KLineDrawType; //设置K线显示类型
        if (varItem.Color) chart.Color = this.GetColor(varItem.Color);else chart.Color = this.GetDefaultColor(id);

        var titleIndex = windowIndex + 1;
        chart.Data.Data = varItem.Data;
        chart.HistoryData = hisData;
        hqChart.TitlePaint[titleIndex].Data[id] = new DynamicTitleData(chart.Data, varItem.Name, chart.Color);

        hqChart.ChartPaint.push(chart);
    };

    this.CreateBand = function (hqChart, windowIndex, varItem, id) {
        var chart = new ChartBand();
        chart.Canvas = hqChart.Canvas;
        chart.Name = varItem.Name;
        chart.ChartBorder = hqChart.Frame.SubFrame[windowIndex].Frame.ChartBorder;
        chart.ChartFrame = hqChart.Frame.SubFrame[windowIndex].Frame;

        chart.FirstColor = varItem.Draw.Color[0];
        chart.SecondColor = varItem.Draw.Color[1];
        chart.Data.Data = varItem.Draw.DrawData;

        hqChart.ChartPaint.push(chart);
    };

    //创建K线图
    this.CreateKLine = function (hqChart, windowIndex, varItem, id) {
        var chart = new ChartKLine();
        chart.Canvas = hqChart.Canvas;
        chart.Name = varItem.Name;
        chart.ChartBorder = hqChart.Frame.SubFrame[windowIndex].Frame.ChartBorder;
        chart.ChartFrame = hqChart.Frame.SubFrame[windowIndex].Frame;

        chart.Data.Data = varItem.Draw.DrawData;
        chart.IsShowMaxMinPrice = false;
        chart.IsShowKTooltip = false;

        if (varItem.Color) //如果设置了颜色,使用外面设置的颜色
            chart.UnchagneColor = chart.DownColor = chart.UpColor = this.GetColor(varItem.Color);

        hqChart.ChartPaint.push(chart);
    };

    this.CreatePolyLine = function (hqChart, windowIndex, varItem, id) {
        var line = new ChartLine();
        line.Canvas = hqChart.Canvas;
        line.Name = varItem.Name;
        line.ChartBorder = hqChart.Frame.SubFrame[windowIndex].Frame.ChartBorder;
        line.ChartFrame = hqChart.Frame.SubFrame[windowIndex].Frame;
        if (varItem.Color) line.Color = this.GetColor(varItem.Color);else line.Color = this.GetDefaultColor(id);

        if (varItem.LineWidth) {
            var width = parseInt(varItem.LineWidth.replace("LINETHICK", ""));
            if (!isNaN(width) && width > 0) line.LineWidth = width;
        }

        var titleIndex = windowIndex + 1;
        line.Data.Data = varItem.Draw.DrawData;
        hqChart.TitlePaint[titleIndex].Data[id] = new DynamicTitleData(line.Data, ' ', line.Color); //给一个空的标题

        hqChart.ChartPaint.push(line);
    };

    this.CreateBackgroud = function (hqChart, windowIndex, varItem, id) {
        var chart = new ChartBackground();
        chart.Canvas = hqChart.Canvas;
        chart.Name = varItem.Name;
        chart.ChartBorder = hqChart.Frame.SubFrame[windowIndex].Frame.ChartBorder;
        chart.ChartFrame = hqChart.Frame.SubFrame[windowIndex].Frame;
        if (varItem.Draw && varItem.Draw.DrawData) {
            var drawData = varItem.Draw.DrawData;
            chart.Color = drawData.Color;
            chart.ColorAngle = drawData.Angle;
        }

        hqChart.ChartPaint.push(chart);
    };

    this.CreateTextLine = function (hqChart, windowIndex, varItem, id) {
        var chart = new ChartTextLine();
        chart.Canvas = hqChart.Canvas;
        chart.Name = varItem.Name;
        chart.ChartBorder = hqChart.Frame.SubFrame[windowIndex].Frame.ChartBorder;
        chart.ChartFrame = hqChart.Frame.SubFrame[windowIndex].Frame;
        if (varItem.Draw && varItem.Draw.DrawData) {
            var drawData = varItem.Draw.DrawData;
            chart.Text = drawData.Text;
            chart.Line = drawData.Line;
            chart.Price = drawData.Price;
        }

        hqChart.ChartPaint.push(chart);
    };

    this.CreateNumberText = function (hqChart, windowIndex, varItem, id) {
        var chartText = new ChartSingleText();
        chartText.Canvas = hqChart.Canvas;

        chartText.Name = varItem.Name;
        chartText.ChartBorder = hqChart.Frame.SubFrame[windowIndex].Frame.ChartBorder;
        chartText.ChartFrame = hqChart.Frame.SubFrame[windowIndex].Frame;
        if (varItem.Color) chartText.Color = this.GetColor(varItem.Color);else chartText.Color = this.GetDefaultColor(id);

        var titleIndex = windowIndex + 1;
        chartText.Data.Data = varItem.Draw.DrawData.Value;
        chartText.Text = varItem.Draw.DrawData.Text;

        //hqChart.TitlePaint[titleIndex].Data[id]=new DynamicTitleData(bar.Data,varItem.Name,bar.Color);

        hqChart.ChartPaint.push(chartText);
    };

    //创建图标
    this.CreateIcon = function (hqChart, windowIndex, varItem, id) {
        var chartText = new ChartSingleText();
        chartText.Canvas = hqChart.Canvas;
        chartText.TextAlign = 'center';

        chartText.Name = varItem.Name;
        chartText.ChartBorder = hqChart.Frame.SubFrame[windowIndex].Frame.ChartBorder;
        chartText.ChartFrame = hqChart.Frame.SubFrame[windowIndex].Frame;

        var titleIndex = windowIndex + 1;
        chartText.Data.Data = varItem.Draw.DrawData;
        var icon = varItem.Draw.Icon;
        if (icon.IconFont == true) {
            chartText.IconFont = { Family: icon.Family, Text: icon.Symbol, Color: icon.Color };
        } else {
            chartText.Text = icon.Symbol;
            if (varItem.Color) chartText.Color = this.GetColor(varItem.Color);else if (icon.Color) chartText.Color = icon.Color;else chartText.Color = 'rgb(0,0,0)';
        }

        //hqChart.TitlePaint[titleIndex].Data[id]=new DynamicTitleData(bar.Data,varItem.Name,bar.Color);

        hqChart.ChartPaint.push(chartText);
    };

    //创建通道
    this.CreateChannel = function (hqChart, windowIndex, varItem, id) {
        var chart = new ChartChannel();
        chart.Canvas = hqChart.Canvas;
        chart.Name = varItem.Name;
        chart.ChartBorder = hqChart.Frame.SubFrame[windowIndex].Frame.ChartBorder;
        chart.ChartFrame = hqChart.Frame.SubFrame[windowIndex].Frame;

        if (varItem.Draw.AreaColor) chart.AreaColor = varItem.Draw.AreaColor;else if (varItem.Color) chart.AreaColor = this.GetColor(varItem.Color);else chart.AreaColor = this.GetDefaultColor(id);

        if (varItem.Draw.Border.Color) chart.LineColor = varItem.Draw.Border.Color;else chart.LineColor = null;

        if (varItem.Draw.Border.Dotted) chart.LineDotted = varItem.Draw.Border.Dotted;
        if (varItem.Draw.Border.Width > 0) chart.LineWidth = varItem.Draw.Border.Width;

        //let titleIndex=windowIndex+1;
        chart.Data.Data = varItem.Draw.DrawData;
        hqChart.ChartPaint.push(chart);
    };

    this.CreatePartLine = function (hqChart, windowIndex, varItem, i) {
        var chart = new ChartPartLine();
        chart.Canvas = hqChart.Canvas;
        chart.Name = varItem.Name;
        chart.ChartBorder = hqChart.Frame.SubFrame[windowIndex].Frame.ChartBorder;
        chart.ChartFrame = hqChart.Frame.SubFrame[windowIndex].Frame;

        chart.Data.Data = varItem.Draw.DrawData;
        hqChart.ChartPaint.push(chart);
    };

    this.CreateMultiLine = function (hqChart, windowIndex, varItem, i) {
        var chart = new ChartMultiLine();
        chart.Canvas = hqChart.Canvas;
        chart.Name = varItem.Name;
        chart.ChartBorder = hqChart.Frame.SubFrame[windowIndex].Frame.ChartBorder;
        chart.ChartFrame = hqChart.Frame.SubFrame[windowIndex].Frame;

        chart.Data = hqChart.ChartPaint[0].Data; //绑定K线
        chart.Lines = varItem.Draw.DrawData;
        hqChart.ChartPaint.push(chart);
    };

    this.CreateMultiText = function (hqChart, windowIndex, varItem, i) {
        var chart = new ChartMultiText();
        chart.Canvas = hqChart.Canvas;
        chart.Name = varItem.Name;
        chart.ChartBorder = hqChart.Frame.SubFrame[windowIndex].Frame.ChartBorder;
        chart.ChartFrame = hqChart.Frame.SubFrame[windowIndex].Frame;

        chart.Data = hqChart.ChartPaint[0].Data; //绑定K线
        chart.Texts = varItem.Draw.DrawData;
        hqChart.ChartPaint.push(chart);
    };

    this.CreateMultiSVGIcon = function (hqChart, windowIndex, varItem, i) {
        var chart = new ChartMultiSVGIcon();
        chart.Canvas = hqChart.Canvas;
        chart.Name = varItem.Name;
        chart.ChartBorder = hqChart.Frame.SubFrame[windowIndex].Frame.ChartBorder;
        chart.ChartFrame = hqChart.Frame.SubFrame[windowIndex].Frame;

        chart.Data = hqChart.ChartPaint[0].Data; //绑定K线
        chart.Family = varItem.Draw.DrawData.Family;
        chart.Icon = varItem.Draw.DrawData.Icon;
        hqChart.ChartPaint.push(chart);
    };

    this.CreateRectangle = function (hqChart, windowIndex, varItem, i) {
        var chart = new ChartRectangle();
        chart.Canvas = hqChart.Canvas;
        chart.Name = varItem.Name;
        chart.ChartBorder = hqChart.Frame.SubFrame[windowIndex].Frame.ChartBorder;
        chart.ChartFrame = hqChart.Frame.SubFrame[windowIndex].Frame;

        chart.Color = [varItem.Draw.DrawData.Color];
        chart.Rect = varItem.Draw.DrawData.Rect;
        if (varItem.Color) chart.BorderColor = this.GetColor(varItem.Color);
        hqChart.ChartPaint.push(chart);
    };

    //创建K线
    this.CreateSelfKLine = function (hqChart, windowIndex, hisData) {
        var chart = new ChartKLine();
        chart.Canvas = hqChart.Canvas;
        chart.Name = "Self Kline";
        chart.ChartBorder = hqChart.Frame.SubFrame[windowIndex].Frame.ChartBorder;
        chart.ChartFrame = hqChart.Frame.SubFrame[windowIndex].Frame;

        chart.Data = hisData;
        chart.IsShowMaxMinPrice = false;
        chart.IsShowKTooltip = false;
        chart.DrawType = this.KLineType;

        hqChart.ChartPaint.push(chart);
    };

    this.BindInstructionData = function (hqChart, windowIndex, hisData) //绑定指示指标
    {
        if (this.OutVar == null || this.OutVar.length < 0) return;
        if (this.InstructionType == 2) //五彩K线
            {
                var varItem = this.OutVar[this.OutVar.length - 1]; //取最后一组数据作为指示数据
                hqChart.SetInstructionData(this.InstructionType, { Data: varItem.Data, Name: this.Name, ID: this.ID }); //设置指示数据
                return true;
            } else if (this.InstructionType == 1) //交易系统
            {
                var buyData, sellData;
                for (var i in this.OutVar) {
                    var _item14 = this.OutVar[i];
                    if (_item14.Name == 'ENTERLONG') buyData = _item14.Data;else if (_item14.Name == 'EXITLONG') sellData = _item14.Data;
                }

                hqChart.SetInstructionData(this.InstructionType, { Buy: buyData, Sell: sellData, Name: this.Name, ID: this.ID }); //设置指示数据
                return true;
            }
    };

    this.BindData = function (hqChart, windowIndex, hisData) {
        if (windowIndex == 0 && this.InstructionType) {
            this.BindInstructionData(hqChart, windowIndex, hisData);
            return;
        }

        //清空主指标图形
        hqChart.DeleteIndexPaint(windowIndex);
        if (windowIndex == 0) hqChart.ShowKLine(true);

        if (this.OutVar == null || this.OutVar.length < 0) return;

        //叠加一个K线背景
        if (this.KLineType != null) {
            if (this.KLineType === 0 || this.KLineType === 1 || this.KLineType === 2) this.CreateSelfKLine(hqChart, windowIndex, hisData);else if (this.KLineType === -1 && windowIndex == 0) hqChart.ShowKLine(false);
        }

        if (windowIndex >= 1 && hqChart.Frame) {
            hqChart.Frame.SubFrame[windowIndex].Frame.YSplitOperator.FloatPrecision = this.FloatPrecision;
            if (this.YSpecificMaxMin) hqChart.Frame.SubFrame[windowIndex].Frame.YSpecificMaxMin = this.YSpecificMaxMin; //最大最小值
            if (this.YSplitScale) hqChart.Frame.SubFrame[windowIndex].Frame.YSplitScale = this.YSplitScale; //固定刻度
        }

        /*
        if (this.Name=='MA')    //测试多线段
        {
            var point1={Point:[{Index:300, Value:15.5}, {Index:301, Value:14.2} , {Index:304, Value:14.05}], Color:'rgb(244,55,50)'};
            var point2={Point:[{Index:307, Value:14.5}, {Index:308, Value:14.2} , {Index:309, Value:14.15}], Color:'rgb(0,55,50)'};
            var testData={ Name:'MULTI_LINE', Type:1,Draw:{ DrawType:'MULTI_LINE', DrawData:[point1,point2] } };
            this.OutVar.push(testData);
        }
        */

        for (var _i108 in this.OutVar) {
            var _item15 = this.OutVar[_i108];
            if (_item15.IsExData === true) continue; //扩展数据不显示图形

            if (_item15.Type == 0) {
                this.CreateLine(hqChart, windowIndex, _item15, _i108);
            } else if (_item15.Type == 1) {
                switch (_item15.Draw.DrawType) {
                    case 'STICKLINE':
                        this.CreateBar(hqChart, windowIndex, _item15, _i108);
                        break;
                    case 'DRAWTEXT':
                    case 'SUPERDRAWTEXT':
                    case 'DRAWTEXT_FIX':
                        this.CreateText(hqChart, windowIndex, _item15, _i108);
                        break;
                    case 'DRAWLINE':
                        this.CreateStraightLine(hqChart, windowIndex, _item15, _i108);
                        break;
                    case 'DRAWBAND':
                        this.CreateBand(hqChart, windowIndex, _item15, _i108);
                        break;
                    case 'DRAWKLINE':
                        this.CreateKLine(hqChart, windowIndex, _item15, _i108);
                        break;
                    case 'DRAWKLINE_IF':
                        this.CreateKLine(hqChart, windowIndex, _item15, _i108);
                        break;
                    case 'POLYLINE':
                        this.CreatePolyLine(hqChart, windowIndex, _item15, _i108);
                        break;
                    case 'DRAWGBK':
                        this.CreateBackgroud(hqChart, windowIndex, _item15, _i108);
                        break;
                    case 'DRAWTEXT_LINE':
                        this.CreateTextLine(hqChart, windowIndex, _item15, _i108);
                        break;
                    case 'DRAWNUMBER':
                        this.CreateNumberText(hqChart, windowIndex, _item15, _i108);
                        break;
                    case 'DRAWICON':
                        this.CreateIcon(hqChart, windowIndex, _item15, _i108);
                        break;
                    case 'DRAWCHANNEL':
                        this.CreateChannel(hqChart, windowIndex, _item15, _i108);
                        break;
                    case 'PARTLINE':
                        this.CreatePartLine(hqChart, windowIndex, _item15, _i108);
                        break;
                    case 'DRAWRECTREL':
                        this.CreateRectangle(hqChart, windowIndex, _item15, _i108);
                        break;
                    case 'MULTI_LINE':
                        this.CreateMultiLine(hqChart, windowIndex, _item15, _i108);
                        break;
                    case 'MULTI_TEXT':
                        this.CreateMultiText(hqChart, windowIndex, _item15, _i108);
                        break;
                    case 'MULTI_SVGICON':
                        this.CreateMultiSVGIcon(hqChart, windowIndex, _item15, _i108);
                        break;
                }
            } else if (_item15.Type == 2) {
                this.CreateMACD(hqChart, windowIndex, _item15, _i108);
            } else if (_item15.Type == 3) {
                this.CreatePointDot(hqChart, windowIndex, _item15, _i108);
            } else if (_item15.Type == 4) {
                this.CreateLineStick(hqChart, windowIndex, _item15, _i108);
            } else if (_item15.Type == 5) {
                this.CreateStick(hqChart, windowIndex, _item15, _i108);
            } else if (_item15.Type == 6) {
                this.CreateVolStick(hqChart, windowIndex, _item15, _i108, hisData);
            }

            var titlePaint = hqChart.TitlePaint[windowIndex + 1];
            if (titlePaint && titlePaint.Data && _i108 < titlePaint.Data.length) //设置标题数值 小数位数和格式
                {
                    if (this.StringFormat > 0) titlePaint.Data[_i108].StringFormat = this.StringFormat;
                    if (this.FloatPrecision >= 0) titlePaint.Data[_i108].FloatPrecision = this.FloatPrecision;
                }
        }

        var titleIndex = windowIndex + 1;
        hqChart.TitlePaint[titleIndex].Title = this.Name;

        if (!this.IsShortTitle) {
            var indexParam = '';
            for (var _i109 in this.Arguments) {
                var _item16 = this.Arguments[_i109];
                if (indexParam.length > 0) indexParam += ',';
                indexParam += _item16.Value.toString();
            }

            if (indexParam.length > 0) hqChart.TitlePaint[titleIndex].Title = this.Name + '(' + indexParam + ')';
        }

        if (this.TitleFont) hqChart.TitlePaint[titleIndex].Font = this.TitleFont;

        return true;
    };

    //给一个默认的颜色
    this.GetDefaultColor = function (id) {
        var COLOR_ARRAY = ["rgb(255,174,0)", "rgb(25,199,255)", "rgb(175,95,162)", "rgb(236,105,65)", "rgb(68,114,196)", "rgb(229,0,79)", "rgb(0,128,255)", "rgb(252,96,154)", "rgb(42,230,215)", "rgb(24,71,178)"];

        var number = parseInt(id);
        return COLOR_ARRAY[number % (COLOR_ARRAY.length - 1)];
    };

    //获取颜色
    this.GetColor = function (colorName) {
        var COLOR_MAP = new Map([['COLORBLACK', 'rgb(0,0,0)'], ['COLORBLUE', 'rgb(18,95,216)'], ['COLORGREEN', 'rgb(25,158,0)'], ['COLORCYAN', 'rgb(0,255,198)'], ['COLORRED', 'rgb(238,21,21)'], ['COLORMAGENTA', 'rgb(255,0,222)'], ['COLORBROWN', 'rgb(149,94,15)'], ['COLORLIGRAY', 'rgb(218,218,218)'], //画淡灰色
        ['COLORGRAY', 'rgb(133,133,133)'], //画深灰色
        ['COLORLIBLUE', 'rgb(94,204,255)'], //淡蓝色
        ['COLORLIGREEN', 'rgb(183,255,190)'], //淡绿色
        ['COLORLICYAN', 'rgb(154,255,242)'], //淡青色
        ['COLORLIRED', 'rgb(255,172,172)'], //淡红色
        ['COLORLIMAGENTA', 'rgb(255,145,241)'], //淡洋红色
        ['COLORWHITE', 'rgb(255,255,255)'], //白色
        ['COLORYELLOW', 'rgb(255,198,0)']]);

        if (COLOR_MAP.has(colorName)) return COLOR_MAP.get(colorName);

        //COLOR 自定义色
        //格式为COLOR+“RRGGBB”：RR、GG、BB表示红色、绿色和蓝色的分量，每种颜色的取值范围是00-FF，采用了16进制。
        //例如：MA5：MA(CLOSE，5)，COLOR00FFFF　表示纯红色与纯绿色的混合色：COLOR808000表示淡蓝色和淡绿色的混合色。
        if (colorName.indexOf('COLOR') == 0) return '#' + colorName.substr(5);

        return 'rgb(30,144,255)';
    };
}

function OverlayScriptIndex(name, script, args, option) {
    this.newMethod = ScriptIndex; //派生
    this.newMethod(name, script, args, option);
    delete this.newMethod;

    //叠加指标
    this.OverlayIndex = null; // { IsOverlay:true, Identify:overlayFrame.Identify, WindowIndex:windowIndex, Frame:overlayFrame }

    this.BindData = function (hqChart, windowIndex, hisData) {
        if (!this.OverlayIndex || this.OverlayIndex.IsOverlay != true) return;

        this.OverlayIndex.Frame.ChartPaint = [];
        if (this.OutVar == null || this.OutVar.length < 0) return;

        /*叠加一个K线背景
        if (this.KLineType!=null)
        {
            if (this.KLineType===0 || this.KLineType===1 || this.KLineType===2) this.CreateSelfKLine(hqChart,windowIndex,hisData);
            else if (this.KLineType===-1 && windowIndex==0) hqChart.ShowKLine(false);
        }
          if (windowIndex>=1 && hqChart.Frame)
        {
            hqChart.Frame.SubFrame[windowIndex].Frame.YSplitOperator.FloatPrecision=this.FloatPrecision;
            if (this.YSpecificMaxMin)  hqChart.Frame.SubFrame[windowIndex].Frame.YSpecificMaxMin=this.YSpecificMaxMin;  //最大最小值
            if (this.YSplitScale)   hqChart.Frame.SubFrame[windowIndex].Frame.YSplitScale=this.YSplitScale;             //固定刻度
        }
        */

        //指标名字
        var titleInfo = { Data: [], Title: this.Name };
        var indexParam = '';
        for (var i in this.Arguments) {
            var _item17 = this.Arguments[i];
            if (indexParam.length > 0) indexParam += ',';
            indexParam += _item17.Value.toString();
        }
        if (indexParam.length > 0) titleInfo.Title = this.Name + '(' + indexParam + ')';

        var titleIndex = windowIndex + 1;
        var titlePaint = hqChart.TitlePaint[titleIndex];
        titlePaint.OverlayIndex.set(this.OverlayIndex.Identify, titleInfo);
        this.OverlayIndex.Frame.Frame.Title = titleInfo.Title; //给子框架设置标题

        for (var i in this.OutVar) {
            var _item18 = this.OutVar[i];
            if (_item18.IsExData === true) continue; //扩展数据不显示图形

            if (_item18.Type == 0) {
                this.CreateLine(hqChart, windowIndex, _item18, i);
            } else if (_item18.Type == 1) {
                switch (_item18.Draw.DrawType) {
                    case 'STICKLINE':
                        this.CreateBar(hqChart, windowIndex, _item18, i);
                        break;
                    case 'DRAWTEXT':
                    case 'SUPERDRAWTEXT':
                    case 'DRAWTEXT_FIX':
                        this.CreateText(hqChart, windowIndex, _item18, i);
                        break;
                    case 'DRAWLINE':
                        this.CreateStraightLine(hqChart, windowIndex, _item18, i);
                        break;
                    case 'DRAWBAND':
                        this.CreateBand(hqChart, windowIndex, _item18, i);
                        break;
                    case 'DRAWKLINE':
                        this.CreateKLine(hqChart, windowIndex, _item18, i);
                        break;
                    case 'DRAWKLINE_IF':
                        this.CreateKLine(hqChart, windowIndex, _item18, i);
                        break;
                    case 'POLYLINE':
                        this.CreatePolyLine(hqChart, windowIndex, _item18, i);
                        break;
                    case 'DRAWNUMBER':
                        this.CreateNumberText(hqChart, windowIndex, _item18, i);
                        break;
                    case 'DRAWICON':
                        this.CreateIcon(hqChart, windowIndex, _item18, i);
                        break;
                    case 'DRAWCHANNEL':
                        this.CreateChannel(hqChart, windowIndex, _item18, i);
                        break;
                }
            } else if (_item18.Type == 2) {
                this.CreateMACD(hqChart, windowIndex, _item18, i);
            } else if (_item18.Type == 3) {
                this.CreatePointDot(hqChart, windowIndex, _item18, i);
            } else if (_item18.Type == 4) {
                this.CreateLineStick(hqChart, windowIndex, _item18, i);
            } else if (_item18.Type == 5) {
                this.CreateStick(hqChart, windowIndex, _item18, i);
            } else if (_item18.Type == 6) {
                this.CreateVolStick(hqChart, windowIndex, _item18, i, hisData);
            }
        }

        /*
        hqChart.TitlePaint[titleIndex].Title=this.Name;
          let indexParam='';
        for(let i in this.Arguments)
        {
            let item=this.Arguments[i];
            if (indexParam.length>0) indexParam+=',';
            indexParam+=item.Value.toString();
        }
          if (indexParam.length>0) hqChart.TitlePaint[titleIndex].Title=this.Name+'('+indexParam+')';
        */

        return true;
    };

    //指标执行完成
    this.RecvResultData = function (outVar, param) {
        var hqChart = param.HQChart;
        var windowIndex = param.WindowIndex;
        var hisData = param.HistoryData;
        param.Self.OutVar = outVar;
        param.Self.BindData(hqChart, windowIndex, hisData);

        param.HQChart.UpdataDataoffset(); //更新数据偏移
        param.HQChart.UpdateFrameMaxMin(); //调整坐标最大 最小值
        param.HQChart.Draw();

        var event = hqChart.GetOverlayIndexEvent(); //指标计算完成回调
        if (event) {
            var self = param.Self;
            var data = { OutVar: self.OutVar, WindowIndex: windowIndex, Name: self.Name, Arguments: self.Arguments, HistoryData: hisData,
                Identify: self.OverlayIndex.Identify,
                Stock: { Symbol: hqChart.Symbol, Name: hqChart.Name } };
            event.Callback(event, data, self);
        }
    };

    //////////////////////////////////////////////////////////////////////////////////////
    //  图形创建
    /////////////////////////////////////////////////////////////////////////////////////

    this.CreateLine = function (hqChart, windowIndex, varItem, id) {
        var overlayIndex = this.OverlayIndex;
        var frame = overlayIndex.Frame;
        var chart = new ChartLine();
        chart.Canvas = hqChart.Canvas;
        chart.DrawType = 1;
        chart.Name = varItem.Name;
        chart.ChartBorder = frame.Frame.ChartBorder;
        chart.ChartFrame = frame.Frame;
        chart.Identify = overlayIndex.Identify;
        if (varItem.Color) chart.Color = this.GetColor(varItem.Color);else chart.Color = this.GetDefaultColor(id);

        if (varItem.LineWidth) {
            var width = parseInt(varItem.LineWidth.replace("LINETHICK", ""));
            if (!isNaN(width) && width > 0) line.LineWidth = width;
        }

        if (varItem.IsShow == false) line.IsShow = false;

        var titleIndex = windowIndex + 1;
        chart.Data.Data = varItem.Data;
        var titlePaint = hqChart.TitlePaint[titleIndex];
        titlePaint.OverlayIndex.get(overlayIndex.Identify).Data[id] = new DynamicTitleData(chart.Data, varItem.Name, chart.Color);

        frame.ChartPaint.push(chart);
    };

    //创建柱子
    this.CreateBar = function (hqChart, windowIndex, varItem, id) {
        var overlayIndex = this.OverlayIndex;
        var frame = overlayIndex.Frame;
        var chart = new ChartStickLine();
        chart.Canvas = hqChart.Canvas;
        if (varItem.Draw.Width > 0) chart.LineWidth = varItem.Draw.Width;else chart.LineWidth = 1;

        chart.Name = varItem.Name;
        chart.ChartBorder = frame.Frame.ChartBorder;
        chart.ChartFrame = frame.Frame;
        chart.Identify = overlayIndex.Identify;
        if (varItem.Color) chart.Color = this.GetColor(varItem.Color);else chart.Color = this.GetDefaultColor(id);

        var titleIndex = windowIndex + 1;
        chart.Data.Data = varItem.Draw.DrawData;

        //hqChart.TitlePaint[titleIndex].Data[id]=new DynamicTitleData(bar.Data,varItem.Name,bar.Color);

        frame.ChartPaint.push(chart);
    };

    //创建文本
    this.CreateText = function (hqChart, windowIndex, varItem, id) {
        var overlayIndex = this.OverlayIndex;
        var frame = overlayIndex.Frame;
        var chart = new ChartSingleText();
        chart.Canvas = hqChart.Canvas;

        chart.Name = varItem.Name;
        chart.ChartBorder = frame.Frame.ChartBorder;
        chart.ChartFrame = frame.Frame;
        chart.Identify = overlayIndex.Identify;
        if (varItem.Color) chart.Color = this.GetColor(varItem.Color);else chart.Color = this.GetDefaultColor(id);

        var titleIndex = windowIndex + 1;
        if (varItem.Draw.Position) chart.Position = varItem.Draw.Position; //赋值坐标
        if (varItem.Draw.DrawData) chart.Data.Data = varItem.Draw.DrawData;
        chart.Text = varItem.Draw.Text;
        if (varItem.Draw.Direction > 0) chart.Direction = varItem.Draw.Direction;
        if (varItem.Draw.YOffset > 0) chart.YOffset = varItem.Draw.YOffset;
        if (varItem.Draw.TextAlign) chart.TextAlign = varItem.Draw.TextAlign;

        //hqChart.TitlePaint[titleIndex].Data[id]=new DynamicTitleData(bar.Data,varItem.Name,bar.Color);

        frame.ChartPaint.push(chart);
    };

    //COLORSTICK 
    this.CreateMACD = function (hqChart, windowIndex, varItem, id) {
        var overlayIndex = this.OverlayIndex;
        var frame = overlayIndex.Frame;
        var chart = new ChartMACD();
        chart.Canvas = hqChart.Canvas;

        chart.Name = varItem.Name;
        chart.ChartBorder = frame.Frame.ChartBorder;
        chart.ChartFrame = frame.Frame;
        chart.Identify = overlayIndex.Identify;

        var titleIndex = windowIndex + 1;
        chart.Data.Data = varItem.Data;
        var titlePaint = hqChart.TitlePaint[titleIndex];
        titlePaint.OverlayIndex.get(overlayIndex.Identify).Data[id] = new DynamicTitleData(chart.Data, varItem.Name, this.GetDefaultColor(id));

        frame.ChartPaint.push(chart);
    };

    this.CreatePointDot = function (hqChart, windowIndex, varItem, id) {
        var overlayIndex = this.OverlayIndex;
        var frame = overlayIndex.Frame;
        var chart = new ChartPointDot();
        chart.Canvas = hqChart.Canvas;
        chart.Name = varItem.Name;
        chart.ChartBorder = frame.Frame.ChartBorder;
        chart.ChartFrame = frame.Frame;
        chart.Identify = overlayIndex.Identify;
        if (varItem.Color) chart.Color = this.GetColor(varItem.Color);else chart.Color = this.GetDefaultColor(id);

        if (varItem.Radius) chart.Radius = varItem.Radius;

        if (varItem.LineWidth) {
            var width = parseInt(varItem.LineWidth.replace("LINETHICK", ""));
            if (!isNaN(width) && width > 0) chart.Radius = width;
        }

        var titleIndex = windowIndex + 1;
        chart.Data.Data = varItem.Data;
        var titlePaint = hqChart.TitlePaint[titleIndex];
        titlePaint.OverlayIndex.get(overlayIndex.Identify).Data[id] = new DynamicTitleData(chart.Data, varItem.Name, chart.Color);

        frame.ChartPaint.push(chart);
    };

    this.CreateStick = function (hqChart, windowIndex, varItem, id) {
        var overlayIndex = this.OverlayIndex;
        var frame = overlayIndex.Frame;
        var chart = new ChartStick();
        chart.Canvas = hqChart.Canvas;
        chart.Name = varItem.Name;
        chart.ChartBorder = frame.Frame.ChartBorder;
        chart.ChartFrame = frame.Frame;
        chart.Identify = overlayIndex.Identify;
        if (varItem.Color) chart.Color = this.GetColor(varItem.Color);else chart.Color = this.GetDefaultColor(id);

        if (varItem.LineWidth) {
            var width = parseInt(varItem.LineWidth.replace("LINETHICK", ""));
            if (!isNaN(width) && width > 0) chart.LineWidth = width;
        }

        var titleIndex = windowIndex + 1;
        chart.Data.Data = varItem.Data;
        var titlePaint = hqChart.TitlePaint[titleIndex];
        titlePaint.OverlayIndex.get(overlayIndex.Identify).Data[id] = new DynamicTitleData(chart.Data, varItem.Name, chart.Color);

        frame.ChartPaint.push(chart);
    };

    this.CreateLineStick = function (hqChart, windowIndex, varItem, id) {
        var overlayIndex = this.OverlayIndex;
        var frame = overlayIndex.Frame;
        var chart = new ChartLineStick();
        chart.Canvas = hqChart.Canvas;
        chart.Name = varItem.Name;
        chart.ChartBorder = frame.Frame.ChartBorder;
        chart.ChartFrame = frame.Frame;
        chart.Identify = overlayIndex.Identify;
        if (varItem.Color) chart.Color = this.GetColor(varItem.Color);else chart.Color = this.GetDefaultColor(id);

        if (varItem.LineWidth) {
            var width = parseInt(varItem.LineWidth.replace("LINETHICK", ""));
            if (!isNaN(width) && width > 0) chart.LineWidth = width;
        }

        var titleIndex = windowIndex + 1;
        chart.Data.Data = varItem.Data;
        var titlePaint = hqChart.TitlePaint[titleIndex];
        titlePaint.OverlayIndex.get(overlayIndex.Identify).Data[id] = new DynamicTitleData(chart.Data, varItem.Name, chart.Color);

        frame.ChartPaint.push(chart);
    };

    this.CreateStraightLine = function (hqChart, windowIndex, varItem, id) {
        var overlayIndex = this.OverlayIndex;
        var frame = overlayIndex.Frame;
        var chart = new ChartLine();
        chart.DrawType = 1;
        chart.Canvas = hqChart.Canvas;
        chart.Name = varItem.Name;
        chart.ChartBorder = frame.Frame.ChartBorder;
        chart.ChartFrame = frame.Frame;
        chart.Identify = overlayIndex.Identify;
        if (varItem.Color) chart.Color = this.GetColor(varItem.Color);else chart.Color = this.GetDefaultColor(id);

        if (varItem.LineWidth) {
            var width = parseInt(varItem.LineWidth.replace("LINETHICK", ""));
            if (!isNaN(width) && width > 0) chart.LineWidth = width;
        }

        var titleIndex = windowIndex + 1;
        chart.Data.Data = varItem.Draw.DrawData;
        //hqChart.TitlePaint[titleIndex].Data[id]=new DynamicTitleData(line.Data,varItem.Name,line.Color);

        frame.ChartPaint.push(chart);
    };

    this.CreateVolStick = function (hqChart, windowIndex, varItem, id, hisData) {
        var overlayIndex = this.OverlayIndex;
        var frame = overlayIndex.Frame;
        var chart = new ChartVolStick();
        chart.Canvas = hqChart.Canvas;
        chart.Name = varItem.Name;
        chart.ChartBorder = frame.Frame.ChartBorder;
        chart.ChartFrame = frame.Frame;
        chart.Identify = overlayIndex.Identify;
        chart.KLineDrawType = hqChart.KLineDrawType; //设置K线显示类型
        if (varItem.Color) chart.Color = this.GetColor(varItem.Color);else chart.Color = this.GetDefaultColor(id);

        var titleIndex = windowIndex + 1;
        chart.Data.Data = varItem.Data;
        chart.HistoryData = hisData;
        var titlePaint = hqChart.TitlePaint[titleIndex];
        titlePaint.OverlayIndex.get(overlayIndex.Identify).Data[id] = new DynamicTitleData(chart.Data, varItem.Name, chart.Color);

        frame.ChartPaint.push(chart);
    };

    this.CreateBand = function (hqChart, windowIndex, varItem, id) {
        var overlayIndex = this.OverlayIndex;
        var frame = overlayIndex.Frame;
        var chart = new ChartBand();
        chart.Canvas = hqChart.Canvas;
        chart.Name = varItem.Name;
        chart.ChartBorder = frame.Frame.ChartBorder;
        chart.ChartFrame = frame.Frame;
        chart.Identify = overlayIndex.Identify;

        chart.FirstColor = varItem.Draw.Color[0];
        chart.SecondColor = varItem.Draw.Color[1];
        chart.Data.Data = varItem.Draw.DrawData;

        frame.ChartPaint.push(chart);
    };

    //创建K线图
    this.CreateKLine = function (hqChart, windowIndex, varItem, id) {
        var overlayIndex = this.OverlayIndex;
        var frame = overlayIndex.Frame;
        var chart = new ChartKLine();
        chart.Canvas = hqChart.Canvas;
        chart.Name = varItem.Name;
        chart.ChartBorder = frame.Frame.ChartBorder;
        chart.ChartFrame = frame.Frame;
        chart.Identify = overlayIndex.Identify;

        chart.Data.Data = varItem.Draw.DrawData;
        chart.IsShowMaxMinPrice = false;
        chart.IsShowKTooltip = false;

        if (varItem.Color) //如果设置了颜色,使用外面设置的颜色
            chart.UnchagneColor = chart.DownColor = chart.UpColor = this.GetColor(varItem.Color);

        frame.ChartPaint.push(chart);
    };

    this.CreatePolyLine = function (hqChart, windowIndex, varItem, id) {
        var overlayIndex = this.OverlayIndex;
        var frame = overlayIndex.Frame;
        var chart = new ChartLine();
        chart.Canvas = hqChart.Canvas;
        chart.Name = varItem.Name;
        chart.ChartBorder = frame.Frame.ChartBorder;
        chart.ChartFrame = frame.Frame;
        chart.Identify = overlayIndex.Identify;
        if (varItem.Color) chart.Color = this.GetColor(varItem.Color);else chart.Color = this.GetDefaultColor(id);

        if (varItem.LineWidth) {
            var width = parseInt(varItem.LineWidth.replace("LINETHICK", ""));
            if (!isNaN(width) && width > 0) chart.LineWidth = width;
        }

        var titleIndex = windowIndex + 1;
        chart.Data.Data = varItem.Draw.DrawData;
        var titlePaint = hqChart.TitlePaint[titleIndex];
        titlePaint.OverlayIndex.get(overlayIndex.Identify).Data[id] = new DynamicTitleData(line.Data, ' ', line.Color); //给一个空的标题

        frame.ChartPaint.push(chart);
    };

    this.CreateNumberText = function (hqChart, windowIndex, varItem, id) {
        var overlayIndex = this.OverlayIndex;
        var frame = overlayIndex.Frame;
        var chart = new ChartSingleText();
        chart.Canvas = hqChart.Canvas;

        chart.Name = varItem.Name;
        chart.ChartBorder = frame.Frame.ChartBorder;
        chart.ChartFrame = frame.Frame;
        chart.Identify = overlayIndex.Identify;
        if (varItem.Color) chart.Color = this.GetColor(varItem.Color);else chart.Color = this.GetDefaultColor(id);

        var titleIndex = windowIndex + 1;
        chart.Data.Data = varItem.Draw.DrawData.Value;
        chart.Text = varItem.Draw.DrawData.Text;

        //hqChart.TitlePaint[titleIndex].Data[id]=new DynamicTitleData(bar.Data,varItem.Name,bar.Color);

        frame.ChartPaint.push(chart);
    };

    //创建图标
    this.CreateIcon = function (hqChart, windowIndex, varItem, id) {
        var overlayIndex = this.OverlayIndex;
        var frame = overlayIndex.Frame;
        var chart = new ChartSingleText();
        chart.Canvas = hqChart.Canvas;
        chart.TextAlign = 'center';

        chart.Name = varItem.Name;
        chart.ChartBorder = frame.Frame.ChartBorder;
        chart.ChartFrame = frame.Frame;
        chart.Identify = overlayIndex.Identify;

        var titleIndex = windowIndex + 1;
        chart.Data.Data = varItem.Draw.DrawData;
        var icon = varItem.Draw.Icon;
        if (icon.IconFont == true) {
            chart.IconFont = { Family: icon.Family, Text: icon.Symbol, Color: icon.Color };
        } else {
            chart.Text = icon.Symbol;
            if (varItem.Color) chart.Color = this.GetColor(varItem.Color);else if (icon.Color) chart.Color = icon.Color;else chart.Color = 'rgb(0,0,0)';
        }

        //hqChart.TitlePaint[titleIndex].Data[id]=new DynamicTitleData(bar.Data,varItem.Name,bar.Color);

        frame.ChartPaint.push(chart);
    };

    //创建通道
    this.CreateChannel = function (hqChart, windowIndex, varItem, id) {
        var overlayIndex = this.OverlayIndex;
        var frame = overlayIndex.Frame;
        var chart = new ChartChannel();
        chart.Canvas = hqChart.Canvas;
        chart.Name = varItem.Name;
        chart.ChartBorder = frame.Frame.ChartBorder;
        chart.ChartFrame = frame.Frame;
        chart.Identify = overlayIndex.Identify;

        if (varItem.Draw.AreaColor) chart.AreaColor = varItem.Draw.AreaColor;else if (varItem.Color) chart.AreaColor = this.GetColor(varItem.Color);else chart.AreaColor = this.GetDefaultColor(id);

        if (varItem.Draw.Border.Color) chart.LineColor = varItem.Draw.Border.Color;else chart.LineColor = null;

        if (varItem.Draw.Border.Dotted) chart.LineDotted = varItem.Draw.Border.Dotted;
        if (varItem.Draw.Border.Width > 0) chart.LineWidth = varItem.Draw.Border.Width;

        //let titleIndex=windowIndex+1;
        chart.Data.Data = varItem.Draw.DrawData;
        frame.ChartPaint.push(chart);
    };

    //
    this.CreatePartLine = function (hqChart, windowIndex, varItem, i) {
        var overlayIndex = this.OverlayIndex;
        var frame = overlayIndex.Frame;
        var chart = new ChartPartLine();
        chart.Canvas = hqChart.Canvas;
        chart.Name = varItem.Name;
        chart.ChartBorder = frame.Frame.ChartBorder;
        chart.ChartFrame = frame.Frame;
        chart.Identify = overlayIndex.Identify;

        chart.Data.Data = varItem.Draw.DrawData;
        frame.ChartPaint.push(chart);
    };

    //创建K线
    this.CreateSelfKLine = function (hqChart, windowIndex, hisData) {
        var overlayIndex = this.OverlayIndex;
        var frame = overlayIndex.Frame;
        var chart = new ChartKLine();
        chart.Canvas = hqChart.Canvas;
        chart.Name = "Self Kline";
        chart.ChartBorder = frame.Frame.ChartBorder;
        chart.ChartFrame = frame.Frame;
        chart.Identify = overlayIndex.Identify;

        chart.Data = hisData;
        chart.IsShowMaxMinPrice = false;
        chart.IsShowKTooltip = false;
        chart.DrawType = this.KLineType;

        frame.ChartPaint.push(chart);
    };

    //给一个默认的颜色
    this.GetDefaultColor = function (id) {
        var COLOR_ARRAY = ["rgb(24,71,178)", "rgb(42,230,215)", "rgb(252,96,154)", "rgb(0,128,255)", "rgb(229,0,79)", "rgb(68,114,196)", "rgb(255,174,0)", "rgb(25,199,255)", "rgb(175,95,162)", "rgb(236,105,65)"];

        var number = parseInt(id);
        return COLOR_ARRAY[number % (COLOR_ARRAY.length - 1)];
    };
}

//后台执行指标
function APIScriptIndex(name, script, args, option) {
    this.newMethod = ScriptIndex; //派生
    this.newMethod(name, script, args, option);
    delete this.newMethod;

    this.ApiUrl; //指标执行api地址
    this.HQDataType;

    if (option.API) {
        if (option.API.Url) this.ApiUrl = option.API.Url;
        if (option.API.Name) this.Name = this.ID = option.API.Name;
        if (option.API.ID) this.ID = option.API.ID;
    }

    this.ExecuteScript = function (hqChart, windowIndex, hisData) {
        JSConsole.Complier.Log('[APIScriptIndex::ExecuteScript] name, Arguments ', this.Name, this.Arguments);

        //数据类型
        var hqDataType = HQ_DATA_TYPE.KLINE_ID; //默认K线
        if (hqChart.ClassName === 'MinuteChartContainer' || hqChart.ClassName === 'MinuteChartHScreenContainer') {
            if (hqChart.DayCount > 1) hqDataType = HQ_DATA_TYPE.MULTIDAY_MINUTE_ID; //多日分钟
            else hqDataType = HQ_DATA_TYPE.MINUTE_ID; //分钟数据
        } else if (hqChart.ClassName === 'HistoryMinuteChartContainer') {
            hqDataType = HQ_DATA_TYPE.HISTORY_MINUTE_ID; //历史分钟
        }

        var args = [];
        if (this.Arguments) {
            for (var i in this.Arguments) {
                var item = this.Arguments[i];
                args.push({ name: item.Name, value: item.Value });
            }
        }

        var requestCount = hqChart.GetRequestDataCount();
        var self = this;
        var postData = {
            indexname: this.ID, symbol: hqChart.Symbol, script: this.Script, args: args,
            period: hqChart.Period, right: hqChart.Right, maxdatacount: requestCount.MaxRequestDataCount, maxminutedaycount: requestCount.MaxRequestMinuteDayCount, hqdatatype: hqDataType
        };
        if (hqDataType == HQ_DATA_TYPE.MULTIDAY_MINUTE_ID || hqDataType == HQ_DATA_TYPE.MINUTE_ID) postData.daycount = hqChart.DayCount;
        this.HQDataType = hqDataType;

        if (hqChart.NetworkFilter) {
            var obj = {
                Name: 'APIScriptIndex::ExecuteScript', //类名::
                Explain: '指标计算',
                Request: { Url: self.ApiUrl, Type: 'POST', Data: postData },
                Self: this,
                HQChart: hqChart,
                PreventDefault: false
            };

            hqChart.NetworkFilter(obj, function (data) {
                self.RecvAPIData(data, hqChart, windowIndex, hisData);
            });

            if (obj.PreventDefault == true) return; //已被上层替换,不调用默认的网络请求
        }

        JSNetwork.HttpRequest({
            url: self.ApiUrl,
            data: postData,
            type: "post",
            dataType: "json",
            async: true,
            success: function success(recvData) {
                self.RecvAPIData(recvData, hqChart, windowIndex, hisData);
            },
            error: function error(request) {
                self.RecvError(request);
            }
        });
    };

    this.RecvAPIData = function (data, hqChart, windowIndex, hisData) {
        JSConsole.Complier.Log('[APIScriptIndex::RecvAPIData] recv data ', this.Name, data);
        if (data.code != 0) return;

        if (data.outdata && data.outdata.name) this.Name = data.outdata.name;

        this.Arguments = [];
        if (data.outdata.args) {
            for (var i in data.outdata.args) {
                var item = data.outdata.args[i];
                this.Arguments.push({ Name: item.name, Value: item.value });
            }
        }

        if (this.HQDataType == HQ_DATA_TYPE.KLINE_ID) {
            this.OutVar = this.FittingData(data.outdata, hqChart);
            JSConsole.Complier.Log('[APIScriptIndex::RecvAPIData] conver to OutVar ', this.OutVar);
        } else {
            this.OutVar = this.FittingMinuteData(data.outdata, hqChart); //走势图数据
        }
        this.BindData(hqChart, windowIndex, hisData);

        if (this.IsLocked == false) //不上锁
            {
                hqChart.Frame.SubFrame[windowIndex].Frame.SetLock(null);
            } else //上锁
            {
                var lockData = { IsLocked: true, Callback: this.LockCallback, IndexName: this.Name, ID: this.LockID,
                    BG: this.LockBG, Text: this.LockText, TextColor: this.LockTextColor, Font: this.LockFont, Count: this.LockCount, MinWidth: this.LockMinWidth };
                hqChart.Frame.SubFrame[windowIndex].Frame.SetLock(lockData);
            }

        hqChart.UpdataDataoffset(); //更新数据偏移
        hqChart.UpdateFrameMaxMin(); //调整坐标最大 最小值
        hqChart.Draw();

        if (hqChart.GetIndexEvent) {
            var event = hqChart.GetIndexEvent(); //指标计算完成回调
            if (event) {
                var self = param.Self;
                var data = { OutVar: self.OutVar, WindowIndex: windowIndex, Name: this.Name, Arguments: this.Arguments, HistoryData: hisData,
                    Stock: { Symbol: hqChart.Symbol, Name: hqChart.Name } };
                event.Callback(event, data, this);
            }
        }
    };

    this.FittingArray = function (sourceData, date, time, hqChart, arrayType) //arrayType 0=单值数组 1=结构体
    {
        var kdata = hqChart.ChartPaint[0].Data; //K线

        var arySingleData = [];
        for (var i in sourceData) {
            var value = sourceData[i];
            var indexItem = new SingleData(); //单列指标数据
            indexItem.Date = date[i];
            if (time && i < time.length) indexItem.Time = time[i];
            indexItem.Value = value;
            arySingleData.push(indexItem);
        }

        var aryFittingData;
        if (ChartData.IsDayPeriod(hqChart.Period)) aryFittingData = kdata.GetFittingData(arySingleData); //数据和主图K线拟合
        else aryFittingData = kdata.GetMinuteFittingData(arySingleData); //数据和主图K线拟合

        var bindData = new ChartData();
        bindData.Data = aryFittingData;
        var result;
        if (arrayType == 1) result = bindData.GetObject();else result = bindData.GetValue();
        return result;
    };

    this.FittingMultiLine = function (sourceData, date, time, hqChart) {
        var kdata = hqChart.ChartPaint[0].Data; //K线

        if (ChartData.IsDayPeriod(hqChart.Period, true)) //日线
            {
                var aryPoint = [];
                for (var i in sourceData) {
                    var item = sourceData[i];
                    for (var j in item.Point) {
                        var point = item.Point[j];
                        aryPoint.push(point);
                    }
                }

                aryPoint.sort(function (a, b) {
                    return a.Date - b.Date;
                });
                kdata.GetDateIndex(aryPoint);
                return sourceData;
            } else if (ChartData.IsMinutePeriod(hqChart.Period, true)) //分钟线
            {
                var aryPoint = [];
                for (var i in sourceData) {
                    var item = sourceData[i];
                    for (var j in item.Point) {
                        var point = item.Point[j];
                        aryPoint.push(point);
                    }
                }

                aryPoint.sort(function (a, b) {
                    if (a.Date == b.Date) return a.Time - b.Time;
                    return a.Date - b.Date;
                });

                kdata.GetDateTimeIndex(aryPoint);
                return sourceData;
            }

        return null;
    };

    this.FittingMultiText = function (sourceData, date, time, hqChart) {
        var kdata = hqChart.ChartPaint[0].Data; //K线

        if (ChartData.IsDayPeriod(hqChart.Period, true)) //日线
            {
                sourceData.sort(function (a, b) {
                    return a.Date - b.Date;
                });
                kdata.GetDateIndex(sourceData);
                return sourceData;
            } else if (ChartData.IsMinutePeriod(hqChart.Period, true)) //分钟线
            {
                sourceData.sort(function (a, b) {
                    if (a.Date == b.Date) return a.Time - b.Time;
                    return a.Date - b.Date;
                });

                kdata.GetDateTimeIndex(sourceData);
                return sourceData;
            }

        return null;
    };

    this.FittingData = function (jsonData, hqChart) {
        var outVar = jsonData.outvar;
        var date = jsonData.date;
        var time = jsonData.time;
        var kdata = hqChart.ChartPaint[0].Data;

        //把数据拟合到kdata上
        var result = [];

        for (var i in outVar) {
            var item = outVar[i];
            var indexData = [];
            var outVarItem = { Name: item.name, Type: item.type };
            if (item.color) outVarItem.Color = item.color;
            if (item.data) {
                outVarItem.Data = this.FittingArray(item.data, date, time, hqChart);

                if (item.color) outVarItem.Color = item.color;
                if (item.linewidth >= 1) outVarItem.LineWidth = item.linewidth;
                if (item.isshow == false) outVarItem.IsShow = false;
                if (item.isexdata == true) outVarItem.IsExData = true;

                result.push(outVarItem);
            } else if (item.Draw) {
                var draw = item.Draw;
                var drawItem = {};
                if (draw.DrawType == 'DRAWICON') //图标
                    {
                        drawItem.Icon = draw.Icon;
                        drawItem.Name = draw.Name;
                        drawItem.DrawType = draw.DrawType;
                        drawItem.DrawData = this.FittingArray(draw.DrawData, date, time, hqChart);
                        outVarItem.Draw = drawItem;

                        result.push(outVarItem);
                    } else if (draw.DrawType == 'DRAWTEXT') //文本
                    {
                        drawItem.Text = draw.Text;
                        drawItem.Name = draw.Name;
                        drawItem.DrawType = draw.DrawType;
                        drawItem.DrawData = this.FittingArray(draw.DrawData, date, time, hqChart);
                        outVarItem.Draw = drawItem;

                        result.push(outVarItem);
                    } else if (draw.DrawType == 'STICKLINE') //柱子
                    {
                        drawItem.Name = draw.Name;
                        drawItem.Type = draw.Type;
                        drawItem.Width = draw.Width;
                        drawItem.DrawType = draw.DrawType;
                        drawItem.DrawData = this.FittingArray(draw.DrawData, date, time, hqChart, 1);
                        outVarItem.Draw = drawItem;

                        result.push(outVarItem);
                    } else if (draw.DrawType == 'MULTI_LINE') {
                    drawItem.Text = draw.Text;
                    drawItem.Name = draw.Name;
                    drawItem.DrawType = draw.DrawType;
                    drawItem.DrawData = this.FittingMultiLine(draw.DrawData, date, time, hqChart);
                    outVarItem.Draw = drawItem;

                    result.push(outVarItem);
                } else if (draw.DrawType == 'MULTI_TEXT') {
                    drawItem.Text = draw.Text;
                    drawItem.Name = draw.Name;
                    drawItem.DrawType = draw.DrawType;
                    drawItem.DrawData = this.FittingMultiText(draw.DrawData, date, time, hqChart);
                    outVarItem.Draw = drawItem;

                    result.push(outVarItem);
                } else if (draw.DrawType == 'MULTI_SVGICON') {
                    drawItem.Text = draw.Text;
                    drawItem.Name = draw.Name;
                    drawItem.DrawType = draw.DrawType;
                    drawItem.DrawData = { Icon: this.FittingMultiText(draw.DrawData.Icon, date, time, hqChart), Family: draw.DrawData.Family };
                    outVarItem.Draw = drawItem;

                    result.push(outVarItem);
                }
            }
        }

        return result;
    };

    this.FittingMinuteData = function (jsonData, hqChart) {
        var outVar = jsonData.outvar;
        var date = jsonData.date;
        var time = jsonData.time;
        var result = [];

        for (var i in outVar) {
            item = outVar[i];
            var outVarItem = { Name: item.name, Type: item.type };
            if (item.data) {
                outVarItem.Data = this.FittingMinuteArray(item.data, date, time, hqChart);
                if (item.color) outVarItem.Color = item.color;
                if (item.linewidth >= 1) outVarItem.LineWidth = item.linewidth;
                if (item.isshow == false) outVarItem.IsShow = false;
                if (item.isexdata == true) outVarItem.IsExData = true;

                result.push(outVarItem);
            }
        }

        return result;
    };

    this.FittingMinuteArray = function (sourceData, date, time, hqChart) {
        var minutedata = hqChart.SourceData;; //分钟线

        var arySingleData = [];
        for (var i in sourceData) {
            var value = sourceData[i];
            var indexItem = new SingleData(); //单列指标数据
            indexItem.Date = date[i];
            if (time && i < time.length) indexItem.Time = time[i];
            indexItem.Value = value;
            arySingleData.push(indexItem);
        }

        var aryFittingData = minutedata.GetMinuteFittingData(arySingleData); //数据和主图K线拟合
        var bindData = new ChartData();
        bindData.Data = aryFittingData;
        var result = bindData.GetValue();
        return result;
    };
}

// 本地json数据指标
function LocalJsonDataIndex(name, args, option) {
    this.newMethod = ScriptIndex; //派生
    this.newMethod(name, null, args, null);
    delete this.newMethod;

    this.JsonData; //json格式数据
    if (option.JsonData) this.JsonData = option.JsonData;

    this.RequestData = function (hqChart, windowIndex, hisData) {
        if (!this.JsonData) {
            console.warn("[LocalJsonDataIndex::RequestData] JsonData is null");
            if (param.HQChart.ScriptErrorCallback) param.HQChart.ScriptErrorCallback('json 数据不能为空');
            return;
        }

        this.OutVar = this.FittingData(this.JsonData, hisData);
        this.BindData(hqChart, windowIndex, hisData);
    };

    this.FittingData = function (jsonData, hisData) {
        outVar = jsonData.OutVar;
        date = jsonData.Date;
        time = jsonData.Time;

        result = [];

        for (i in outVar) {
            item = outVar[i];
            JSConsole.Complier.Log('[LocalJsonDataIndex::FittingData] item', item);
            if (item.Type != 0 && item.Type != 3) //复杂的图形不支持
                {
                    console.warn("[LocalJsonDataIndex::FittingData] can't support ", item.Name, item.Type);
                    continue;
                }

            var indexData = [];
            outVarItem = { Name: item.Name, Type: item.Type };
            for (j in item.Data) {
                var indexItem = new SingleData(); //单列指标数据
                indexItem.Date = date[j];
                if (time && j < time.length) indexItem.Time = time[j];
                indexItem.Value = item.Data[j];
                indexData.push(indexItem);
            }

            if (hisData.Period < 4) var aryFittingData = hisData.GetFittingData(indexData); //数据和主图K线拟合
            else var aryFittingData = hisData.GetMinuteFittingData(indexData); //数据和主图K线拟合
            var bindData = new ChartData();
            bindData.Data = aryFittingData;
            outVarItem.Data = bindData.GetValue();
            result.push(outVarItem);
        }

        return result;
    };
}

////////////////////////////////////////////////////////////////////////
//  无UI指标执行
// obj: { Name:指标名字 , ID:指标ID , Script:指标脚本, Args:指标参数, ErrorCallback:错误回调 }
//
///////////////////////////////////////////////////////////////////////
function ScriptIndexConsole(obj) {
    this.ID; //指标ID
    this.Name; //指标名字
    this.Script; //脚本
    this.Arguments = []; //参数
    this.ClassName = 'ScriptIndexConsole';
    this.ErrorCallback; //执行错误回调
    this.FinishCallback; //执行完成回调
    this.IsSectionMode = false; //截面报表模式

    if (obj) {
        if (obj.Name) this.Name = obj.Name;
        if (obj.Script) this.Script = obj.Script;
        if (obj.ID) this.ID = obj.ID;
        if (obj.Args) this.Arguments = obj.Args;
        if (obj.ErrorCallback) this.ErrorCallback = obj.ErrorCallback;
        if (obj.FinishCallback) this.FinishCallback = obj.FinishCallback;
        if (obj.IsSectionMode) this.IsSectionMode = obj.IsSectionMode;
    }

    //执行脚本
    //obj:
    //     HQDataType:(HQ_DATA_TYPE), Period:周期, Right:复权
    //     Stock: {Name:, Symbol:}, Request: { MaxDataCount:请求K线数据个数, MaxMinuteDayCount:请求分钟数据天数, TradeDate: 历史走势图才用}, :
    //     Data: 当前计算数据(周期|复权以后), Source:  原始股票数据 
    this.ExecuteScript = function (obj) {
        this.OutVar = [];
        var param = { Self: this };

        var option = {
            HQDataType: obj.HQDataType,
            Symbol: obj.Stock.Symbol,
            Name: obj.Stock.Name,
            Period: obj.Period,
            Right: obj.Right,
            Callback: this.RecvResultData, CallbackParam: param,
            Async: true,
            MaxRequestDataCount: obj.Request.MaxDataCount,
            MaxRequestMinuteDayCount: obj.Request.MaxMinuteDayCount,
            Arguments: this.Arguments,
            IsSectionMode: this.IsSectionMode,
            ClassName: 'ScriptIndexConsole'
        };

        if (obj.HQDataType === HQ_DATA_TYPE.HISTORY_MINUTE_ID) option.TrateDate = obj.Request.TradeDate;
        if (obj.HQDataType == HQ_DATA_TYPE.MULTIDAY_MINUTE_ID) option.DayCount = obj.DayCount;

        var code = this.Script;
        var run = JSComplier.Execute(code, option, this.ErrorCallback);
    };

    this.RecvResultData = function (outVar, param) {
        var self = param.Self;
        var jsExec = param.JSExecute;

        var date = null;
        if (jsExec.SymbolData.Data) date = jsExec.SymbolData.Data.GetDate();

        var result = {
            Out: outVar,
            Date: date, //日期对应 上面的数据
            Stock: { Name: jsExec.SymbolData.Name, Symbol: jsExec.SymbolData.Symbol }, //股票信息
            Index: { Name: self.Name, ID: self.ID //指标信息
            } };

        if (jsExec.SymbolData.DataType == HQ_DATA_TYPE.MULTIDAY_MINUTE_ID || jsExec.SymbolData.DataType == HQ_DATA_TYPE.MINUTE_ID) result.Time = jsExec.SymbolData.Data.GetTime();else if (jsExec.SymbolData.DataType == HQ_DATA_TYPE.KLINE_ID && ChartData.IsMinutePeriod(jsExec.SymbolData.Period, true)) result.Time = jsExec.SymbolData.Data.GetTime();
        //JSConsole.Complier.Log('[ScriptIndexConsole::RecvResultData] outVar ', outVar);
        if (self.FinishCallback) self.FinishCallback(result, param.JSExecute);
    };
}

ScriptIndexConsole.SetDomain = function (domain, cacheDomain) //修改API地址
{
    JSComplier.SetDomain(domain, cacheDomain);
};

/* 测试例子
var code1='VARHIGH:IF(VAR1<=REF(HH,-1),REF(H,BARSLAST(VAR1>=REF(HH,1))),DRAWNULL),COLORYELLOW;';
var code2='VAR1=((SMA(MAX((CLOSE - LC),0),3,1) / SMA(ABS((CLOSE - LC)),3,1)) * 100);';
var code3='mm1=1-2*-9+20;';

JSConsole.Complier.Log(code1+code2)
var tokens=JSComplier.Tokenize(code1+code2);
var ast=JSComplier.Parse(code2+code1);

JSConsole.Complier.Log(ast);
*/